/*
 * Copyright 2016 Broadcom
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation (the "GPL").
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License version 2 (GPLv2) for more details.
 *
 * You should have received a copy of the GNU General Public License
 * version 2 (GPLv2) along with this source code.
 */

#include <common.h>
#include <asm/arch/secure_api.h>

static int do_sha256hash(cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
	unsigned int i;
	unsigned int addr;
	unsigned int size;
	int outsize = 32;
	unsigned int hashalgo;
	unsigned int usehw;
	unsigned int rc;

	/*change the value of string for other test*/


	unsigned char hash[32];
	memset(hash, 0, 32);

	/* need at least 5 arguments */
	if (argc < 5)
		return cmd_usage(cmdtp);

	addr = simple_strtoul(argv[1], NULL, 16);
	size = simple_strtoul(argv[2], NULL, 16);
	printf("input buffer @0x%08x size=0x%x", addr, size);

	/* TODO need to add other hash algorithm */
	if (strcmp(argv[3], "sha256") == 0) {
		hashalgo = 3;
		printf(" hashalgo=%d\n", hashalgo);
	} else {
		printf("Error: unsupported hash algorithm!\n");
		return cmd_usage(cmdtp);
	}

	if (strcmp(argv[4], "0") == 0) {
		usehw = 0;
	} else if (strcmp(argv[4], "1") == 0) {
		usehw = 1;
	} else {
		printf("Error: usehw: <0|1>\n");
		return cmd_usage(cmdtp);
	}

	rc = ssapi_hash_create(hash, outsize, (unsigned char *)addr,
		size, hashalgo, usehw);

	printf("ssapi_hash_create() return %d\n", rc);
	for (i = 0; i < 32; i++) {
		printf("%02x", hash[i]);
		if ((i + 1) % 32)
			printf(" ");
		else
			printf("\n");
	}

	return 0;
}

U_BOOT_CMD(
	cygnushash,	5,	0,	do_sha256hash,
	"call ssapi to generate sha256hash",
	"inBufAddr inSize hashalgo usehw\nExample:\ncygnushash 0x84000000 0x20 sha256 1"
);
