/*
 * Copyright 2014 Broadcom Corporation.
 *
 * SPDX-License-Identifier:	GPL-2.0+
 */

#define SRAM_GEN_DEFS
#include <common.h>
#include <asm/io.h>
#include <asm/arch/sram_globals.h>
#include <asm/persistent_info.h>

#define RESET_CPU_LEVEL3	0xA5A50003

#define CRMU_MAIL_BOX0		0x03024024
#define CRMU_MAIL_BOX1		0x03024028
#define CRMU_SOFT_RESET_L3_CMD0	0x00000003
#define CRMU_SOFT_RESET_L0_CMD0	0x00000005
#define CRMU_SOFT_RESET_CMD1	0x00000000

void reset_cpu(ulong level)
{
	/* Send soft reset command via Mailbox. */
	if (level == RESET_CPU_LEVEL3)
		writel(CRMU_SOFT_RESET_L3_CMD0, CRMU_MAIL_BOX0);
	else
		writel(CRMU_SOFT_RESET_L0_CMD0, CRMU_MAIL_BOX0);
	writel(CRMU_SOFT_RESET_CMD1, CRMU_MAIL_BOX1);

	while (1)
		;	/* loop forever till reset */
}

int get_reset_reason(void)
{
	struct PERSISTENT_GLOBALS *eg = PERSISTENT_GLOBALS_PTR();
	int reason = eg->reset.reason;

	if (reason >= RESET_REASON_MAX) {
		error("Invalid reset reason (%d)!\n", reason);
		reason = RESET_REASON_UNKNOWN;
	}

	return reason;
}

int get_reset_ext_reason(void)
{
	struct PERSISTENT_GLOBALS *eg = PERSISTENT_GLOBALS_PTR();

	if (eg->reset.reason == RESET_REASON_SOFTWARE)
		return eg->reset.extended_reason;
	else
		return 0;
}

void clear_reset_ext_reason(unsigned long val)
{
	struct PERSISTENT_GLOBALS *eg = PERSISTENT_GLOBALS_PTR();

	eg->reset.extended_reason = val;
}

void reboot_notifier_handler(void)
{
	unsigned long reset_ext_reason = get_reset_ext_reason();

	clear_reset_ext_reason(0xBABEFACE);

	if (reset_ext_reason == RESET_REASON_EXT_RECOVERY) {
#ifdef CONFIG_ENV_IS_IN_MMC
		printf("System started emmc recovery mode\n");
		run_command("run emmc_recovery_cmd", 0);
#else
		printf("System started nand recovery mode\n");
		run_command("run nand_recovery_cmd", 0);
#endif
	} else if (reset_ext_reason == RESET_REASON_EXT_FASTBOOT) {
		printf("System started fastboot mode\n");
		run_command("fastboot", 0);
	}
}

unsigned long get_ramdump_reason(void)
{
	struct PERSISTENT_GLOBALS *eg = PERSISTENT_GLOBALS_PTR();

	return eg->ramdump.reason;
}

void clear_ramdump_reason(unsigned long val)
{
	struct PERSISTENT_GLOBALS *eg = PERSISTENT_GLOBALS_PTR();

	eg->ramdump.reason = val;
}
