/*
 * Copyright 2016 Broadcom
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation (the "GPL").
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License version 2 (GPLv2) for more details.
 *
 * You should have received a copy of the GNU General Public License
 * version 2 (GPLv2) along with this source code.
 */

#include <common.h>
#include <asm/io.h>
#include <asm/arch/secure_api.h>
#include <asm/arch/bcm_iproc_smc.h>
#include <errno.h>
#include <linux/compiler.h>
#include <linux/compat.h>
#include <asm/types.h>
#include <asm/system.h>
#include <config.h>
#include <malloc.h>
#include <asm/dma-mapping.h>


/* Compute binary hash */
unsigned int ssapi_hash_create(unsigned char *outbuffer,
					unsigned int outsize,
					unsigned char *inbuffer,
					unsigned int insize,
					unsigned int hashalgo,
					unsigned int usehw)
{
	unsigned int result;

	if (usehw) {
		/*dispatch the call into SSB to use HW block to compute hash */
		result = bcm_ssapi_call(SSAPI_BRCM_HASH_CREATE,
					SEC_FLAG,
					outbuffer,
					outsize,
					inbuffer,
					insize,
					hashalgo,
					0, 0, 0);
	} else {
		/* calling u-boot software hash function to compute hash */
		result = calculate_hash((void *)inbuffer, insize,
				"sha256", outbuffer, &outsize);
	}

	return result;
}

/* Decrypt signature */
unsigned int ssapi_rsa_public_decrypt(unsigned char *decrypteddata,
				unsigned int *decrypteddatasize,
				unsigned char *encrypteddata,
				unsigned int encrypteddatasize,
				unsigned int keylocation,
				unsigned int modlen,
				unsigned int exp)
{
	unsigned int result;
	result = bcm_ssapi_call(SSAPI_BRCM_RSA_PUBLIC_DECRYPT,
				SEC_FLAG,
				decrypteddata,
				&decrypteddatasize,
				encrypteddata,
				encrypteddatasize,
				keylocation,
				modlen,
				exp,
				0);

	return result;
}
