/*
 * Copyright 2016 Broadcom
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation (the "GPL").
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License version 2 (GPLv2) for more details.
 *
 * You should have received a copy of the GNU General Public License
 * version 2 (GPLv2) along with this source code.
 */

/*
 * Command for Secure API rsa_decrypt.
 *
 */
#include <common.h>
#include <asm/arch/secure_api.h>

/*
 This file is dedicated to test the RSA decrypt funcion.
 Using the Secure API to decrypt RSA encrypted message, needs the modulus.

 Following is the test key used for the verification
-----BEGIN RSA PRIVATE KEY-----
MIIEpQIBAAKCAQEA1bwM5ZnOXWSi4qb0qSnv65JzWA/a+o22iLMZBSsZrFdgPybJ
MYtJNtZhxsq1JM4/WjvSbidFbAYjtBckjI31r3hixrqFiCpMmqKiIImU07lg9F1J
nNePu8GtU51kILL7xZPPuHBpgiI7ZPafbnxER5LUyMV32Oi+pDi5buD6wXZ61mt1
9qUbTsKY/voimAJH1fPzDXsfdLUBv8aLGwHvbRLUMHZbKaIh2mzfoPZfAO8fR7Dq
zBcRPcsxH6wX278MkBNJE0a3D8Yu4Jm5nf4tCTNm1PEibWrhe2puRQZrb3QxPiI7
KoYFcJy4odCtgJ0/rHDXHLo9C8iisMSpIBI6JQIDAQABAoIBAHvpkSSPvgeBL7nt
h0yHprPRbROx5Yd7H0czZh8L9M9ovMEAEIartN+Y/IjOvqQ++pWqtv3rwKC3En33
KNZf+v8m8UU7n+zocQ75mLr6aJEzGVWyu9m1U3xL9dvTFNisS7EpV7UbXt/2Tq7j
GF1qp/9CziAz856KmPQGeuMgheAga/ShpyliTI5U8oV4Y/8dyfHI13IxZfe6zGI3
N8uPLTOiNv7LW4WCgBPXYLeVxCX7eq9T1g5+rstnKmi6SfJDrg3Ri03uC9/iO6bD
SJ3XpZaRrVJBJ4zwKXb/edO7Dj8Lq0cdEZEoFPu9AAT4d1JU44OfCUFEg7ZacrCC
a0TTg4ECgYEA86vtVo2fd3YJmvptCast3RCUqesRinrrc/VNBURrpQ6MOG8lXV9e
kqMe9m82ZlcrTkCbCgXmnaMjpRgUiubPlu4ukYEkmmUwcPxFm4eIdqT85yR/jKNi
bVWOPuNsNnD3P//FBA4qonEop2BoylmDVPi5000z9bFeklGUysqzPQUCgYEA4Ixf
0Vp44DSkXnqhZR5hZZskPc0kCm8GfvDcuqL2eSMYo0ERbO4Iy7Y4SSj5XzFe9jG9
DwbFq4wBxe81zv8LZFjFhUOFlRcaILbV/BThpChfkp2we64L4yF0Otc6/VVB0Q1d
ck6TXOGXheEIZfQg/8teC0RrP0Fcw21gO8AckqECgYEAvQbqnvJOesW+vdL3M0Dv
20Y5jXclrF+et93bWfKx7uFSNZVDTyPkoayZIGM1uD53AcVsgGIDfNZJmJENnpNf
Bp7O0WF1m7BT29JXzUdevWtQJZ9Ssy8r89Ui+/ZAxBruM1+e3n5+5HUULIPFxvOZ
xhkNTrXdUQO/cZKH8qk8UmECgYEAugJuqU0pzoitFHq/UbWN9w5CQlO6JE5+cg9y
ucSfKg/xq+MN3U99x8ql66zejoWmZYkkVNtGSeoL2fxjdZp+xFxpGxCpCqz9ftYW
guCFXmf3a8vm0z2FKVygHNIIUq0afWN14mfG2O0jiMq2G0bVQzAjMnA160to3Zaw
Xi6xPKECgYEAjFJOGSL103zhpzugT1k/geAONbF0bb83TGOgMfK2nh5RnsEmv4YS
KqGvG1IoJmIIa1LlK6kLVj1DOW3ahFlrmi5KMju9HCDkeVH+gOY1v2mBtkbyjCng
XfU3EoQGhc/MdUo2RKG09VkEA4/6D6IvLPXLLGgwmaMNrzbNHrwN/30=
-----END RSA PRIVATE KEY-----

How to generate moudlus:
  1. given private key
     openssl rsa -inform PEM -in ./privkey.pem -modulus
  2. given public key
     openssl rsa -inform PEM -in ./pubkey.pem -modulus -pubin

*/
unsigned char rsatestmodulus[] = {
	0xd5, 0xbc, 0x0c, 0xe5, 0x99, 0xce, 0x5d, 0x64,
	0xa2, 0xe2, 0xa6, 0xf4, 0xa9, 0x29, 0xef, 0xeb,
	0x92, 0x73, 0x58, 0x0f, 0xda, 0xfa, 0x8d, 0xb6,
	0x88, 0xb3, 0x19, 0x05, 0x2b, 0x19, 0xac, 0x57,
	0x60, 0x3f, 0x26, 0xc9, 0x31, 0x8b, 0x49, 0x36,
	0xd6, 0x61, 0xc6, 0xca, 0xb5, 0x24, 0xce, 0x3f,
	0x5a, 0x3b, 0xd2, 0x6e, 0x27, 0x45, 0x6c, 0x06,
	0x23, 0xb4, 0x17, 0x24, 0x8c, 0x8d, 0xf5, 0xaf,
	0x78, 0x62, 0xc6, 0xba, 0x85, 0x88, 0x2a, 0x4c,
	0x9a, 0xa2, 0xa2, 0x20, 0x89, 0x94, 0xd3, 0xb9,
	0x60, 0xf4, 0x5d, 0x49, 0x9c, 0xd7, 0x8f, 0xbb,
	0xc1, 0xad, 0x53, 0x9d, 0x64, 0x20, 0xb2, 0xfb,
	0xc5, 0x93, 0xcf, 0xb8, 0x70, 0x69, 0x82, 0x22,
	0x3b, 0x64, 0xf6, 0x9f, 0x6e, 0x7c, 0x44, 0x47,
	0x92, 0xd4, 0xc8, 0xc5, 0x77, 0xd8, 0xe8, 0xbe,
	0xa4, 0x38, 0xb9, 0x6e, 0xe0, 0xfa, 0xc1, 0x76,
	0x7a, 0xd6, 0x6b, 0x75, 0xf6, 0xa5, 0x1b, 0x4e,
	0xc2, 0x98, 0xfe, 0xfa, 0x22, 0x98, 0x02, 0x47,
	0xd5, 0xf3, 0xf3, 0x0d, 0x7b, 0x1f, 0x74, 0xb5,
	0x01, 0xbf, 0xc6, 0x8b, 0x1b, 0x01, 0xef, 0x6d,
	0x12, 0xd4, 0x30, 0x76, 0x5b, 0x29, 0xa2, 0x21,
	0xda, 0x6c, 0xdf, 0xa0, 0xf6, 0x5f, 0x00, 0xef,
	0x1f, 0x47, 0xb0, 0xea, 0xcc, 0x17, 0x11, 0x3d,
	0xcb, 0x31, 0x1f, 0xac, 0x17, 0xdb, 0xbf, 0x0c,
	0x90, 0x13, 0x49, 0x13, 0x46, 0xb7, 0x0f, 0xc6,
	0x2e, 0xe0, 0x99, 0xb9, 0x9d, 0xfe, 0x2d, 0x09,
	0x33, 0x66, 0xd4, 0xf1, 0x22, 0x6d, 0x6a, 0xe1,
	0x7b, 0x6a, 0x6e, 0x45, 0x06, 0x6b, 0x6f, 0x74,
	0x31, 0x3e, 0x22, 0x3b, 0x2a, 0x86, 0x05, 0x70,
	0x9c, 0xb8, 0xa1, 0xd0, 0xad, 0x80, 0x9d, 0x3f,
	0xac, 0x70, 0xd7, 0x1c, 0xba, 0x3d, 0x0b, 0xc8,
	0xa2, 0xb0, 0xc4, 0xa9, 0x20, 0x12, 0x3a, 0x25
};

/*
00000000  53 61 6d 70 6c 65 20 6d  65 73 73 61 67 65 20 66  |Sample message f|
00000010  6f 72 20 6b 65 79 6c 65  6e 3c 62 6c 6f 63 6b 6c  |or keylen<blockl|
00000020  65 6e                                             |en|

*/
unsigned char rsatestrawdata[] = {
	0x53, 0x61, 0x6d, 0x70, 0x6c, 0x65, 0x20, 0x6d,
	0x65, 0x73, 0x73, 0x61, 0x67, 0x65, 0x20, 0x66,
	0x6f, 0x72, 0x20, 0x6b, 0x65, 0x79, 0x6c, 0x65,
	0x6e, 0x3c, 0x62, 0x6c, 0x6f, 0x63, 0x6b, 0x6c,
	0x65, 0x6e
};

/* hash of raw data
75 09 89 83 de fb 44 7a a2 47 8a 0b 68 53 f5 40
40 36 ae 16 a4 b8 f1 9e a0 cd f8 be be 6e 21 21
*/
unsigned char rsatesthashofrawdata[] = {
	0x75, 0x09, 0x89, 0x83, 0xde, 0xfb, 0x44, 0x7a,
	0xa2, 0x47, 0x8a, 0x0b, 0x68, 0x53, 0xf5, 0x40,
	0x40, 0x36, 0xae, 0x16, 0xa4, 0xb8, 0xf1, 0x9e,
	0xa0, 0xcd, 0xf8, 0xbe, 0xbe, 0x6e, 0x21, 0x21
};

/* signed with .../privkey.pem/ */
unsigned char rsatestsignature[] = {
	0x71, 0x46, 0x67, 0xd5, 0xef, 0x06, 0x8d, 0xf4,
	0xe3, 0x4d, 0x02, 0xaa, 0x7e, 0x39, 0x2a, 0xe6,
	0xd9, 0x8a, 0x50, 0x99, 0x12, 0xac, 0x79, 0x8c,
	0x29, 0xb8, 0xd0, 0x5e, 0x1a, 0xbc, 0xf3, 0x13,
	0x3d, 0xb4, 0x9b, 0x66, 0x82, 0x54, 0xd0, 0x89,
	0x2b, 0x5f, 0xc1, 0x4e, 0x7c, 0x75, 0x5a, 0xc9,
	0x6c, 0xdb, 0x20, 0x5f, 0x52, 0x5a, 0x1d, 0x15,
	0x00, 0x80, 0x37, 0x2d, 0xb6, 0xc3, 0x9f, 0x22,
	0x7a, 0x09, 0x16, 0x88, 0xfe, 0x7a, 0xf9, 0xdd,
	0x9b, 0x54, 0xce, 0xc6, 0x13, 0xf0, 0x59, 0x02,
	0x98, 0xcb, 0x9b, 0x2e, 0x3c, 0x68, 0x60, 0xab,
	0x05, 0xfc, 0xf2, 0xab, 0x68, 0x79, 0xe5, 0x09,
	0xf4, 0xe4, 0xb2, 0x12, 0xb9, 0xee, 0x70, 0x28,
	0xd9, 0xde, 0xf7, 0xe8, 0xe2, 0xbe, 0xf4, 0x4d,
	0xd4, 0xcb, 0x57, 0x4d, 0x06, 0x31, 0xe9, 0x51,
	0xe1, 0x7e, 0x22, 0x70, 0x55, 0x89, 0x9d, 0x94,
	0x7c, 0x20, 0xbc, 0xe8, 0x30, 0xed, 0x90, 0x10,
	0xff, 0x1d, 0x3b, 0x28, 0x30, 0x3b, 0x1b, 0xa9,
	0x43, 0x25, 0xde, 0x6d, 0xad, 0x4b, 0x9b, 0xf9,
	0xff, 0xbe, 0xb1, 0xae, 0xfc, 0xd4, 0x3a, 0xeb,
	0xe4, 0xb8, 0xa4, 0x67, 0x8d, 0x0c, 0xdf, 0xe4,
	0x14, 0x45, 0xd3, 0x7d, 0xc4, 0x41, 0x56, 0x95,
	0xe0, 0x52, 0x0d, 0x78, 0x1b, 0x90, 0xaa, 0x46,
	0x95, 0xc0, 0x06, 0xef, 0xe8, 0x86, 0x69, 0x49,
	0x48, 0xe9, 0x6d, 0xec, 0x9f, 0x40, 0x36, 0x48,
	0x8f, 0x90, 0x00, 0x69, 0x22, 0xf3, 0x47, 0x34,
	0x1b, 0x2f, 0x62, 0xf7, 0xd1, 0x97, 0x64, 0x4c,
	0x62, 0x04, 0x64, 0x2a, 0x75, 0xbf, 0x66, 0xce,
	0x40, 0x71, 0x75, 0xfe, 0xa9, 0x80, 0x8d, 0x27,
	0x6d, 0xd1, 0xa8, 0x2a, 0xcd, 0x98, 0x30, 0x69,
	0x57, 0xa1, 0xbc, 0xb6, 0xe9, 0x4c, 0x79, 0xfe,
	0xe2, 0x37, 0xfd, 0x8c, 0x3e, 0xf6, 0x77, 0x84
};


static int do_rsa_decrypt(cmd_tbl_t *cmdtp, int flag,
		int argc, char * const argv[])
{
	unsigned int i;
	unsigned int addr;
	unsigned int size;
	unsigned int key_addr;
	unsigned int key_size = 0;
	unsigned int modLen = 0;
	unsigned int exp = 0;
	unsigned int rc;
	unsigned int hash_offset = 0;

	unsigned char decrypted_data[2048];
	unsigned int decrypted_data_size = 256;

	memset(decrypted_data, 0, 256);

	if (argc == 1) {
		printf("=== test RSA decrypt using hardcoded data ===\n");
		rc = ssapi_rsa_public_decrypt(decrypted_data,
				&decrypted_data_size,
				rsatestsignature,
				256,
				(unsigned int)&rsatestmodulus,
				256,
				65537);

		/*
		Please note in current becr case, we only verifing
		hash value, decrpyted buffer is 256 byte, but only
		last 32 byte of is the hash value.
		*/
		hash_offset = 256 - 32;
		if (memcmp((unsigned char *)&decrypted_data[hash_offset],
			rsatesthashofrawdata,
			32) == 0) {
			printf("The decrypted signature matches.\n");
		} else {
			printf("ERROR: The decrypted signature does not match.\n");
		}
	} else if (argc == 7) {
	/* need at least 5 arguments */
		addr = simple_strtoul(argv[1], NULL, 16);
		size = simple_strtoul(argv[2], NULL, 16);
		printf("input buffer @0x%08x size=0x%x\n", addr, size);

		key_addr = simple_strtoul(argv[3], NULL, 16);
		if (key_addr == 0) {
			printf("will use internal root key\n");
		} else if (key_addr == 1) {
			printf("will use internal secondary key\n");
		} else {
			simple_strtoul(argv[3], NULL, 16);
			key_size = simple_strtoul(argv[4], NULL, 16);
			printf("key buffer @0x%08x size=0x%x\n", key_addr, key_size);

			modLen = simple_strtoul(argv[5], NULL, 16);
			exp = simple_strtoul(argv[6], NULL, 16);
			printf("mod_len = 0x%02x, exponent=0x%x\n", modLen, exp);
		}

		rc = ssapi_rsa_public_decrypt(decrypted_data,
			&decrypted_data_size,
			addr, size, key_addr, modLen, exp);
	} else {
		return cmd_usage(cmdtp);
	}
	printf("ssapi_rsa_public_decrypt() return %d, decryptedDataSize=%d\n",
		rc, decrypted_data_size);
	for (i = 0; i < decrypted_data_size; i++) {
		printf("%02x", decrypted_data[i]);
		if ((i + 1) % 32)
			printf(" ");
		else
			printf("\n");
	}

	return 0;
}

U_BOOT_CMD(
	rsa_decrypt,	7,	0,	do_rsa_decrypt,
	"call ssapi to decrypt rsa encrypted data",
	"Syntax: inBufAddr inBufSize keyBufAddr keyBufSize mod_len exp\n"\
	"        mod_len is 1/2 of modulus length in bytes, exponent=0x%x\n"\
	"Notes: the last 264 bytes of ABI image is the secondary key"\
	"       which could be passed to this command\n"\
	"Example:\n"\
	"tftp 0x84000000 sig.bin\n"\
	"tftp 0x84000100 rsa_modulus.bin\n"\
	"rsa_decrypt 0x84000000 0x100 0x84000100 0x100 0x80 0x10001\n"\
	"when key includes mod_len and exp, they could be set to 0\n"\
	"rsa_decrypt 0x84000000 0x100 0x84000100 0x108 0x0 0\n"\
	"when using internal root or secondary key, all the rest 3 parameters could be set to 0\n"\
	"using root key\n"\
	"rsa_decrypt 0x84000000 0x100 0 0 0x0 0\n"\
	"using secondary key\n"\
	"rsa_decrypt 0x84000000 0x100 1 0 0x0 0\n"
);
