/*
 * $Copyright Open Broadcom Corporation$
 */

#include <common.h>
#include <post.h>

#if defined(CONFIG_CYGNUS)
#if CONFIG_POST & CONFIG_SYS_POST_LCD
#include "../bcm/halapi/lcd_common.h"

#define LCDControl 0x180a0018
#define LCDControl__LcdBpp_L 3
#define LCDControl__LcdBpp_R 1
#define LCDControl__LcdTFT 5
#define LCDControl__BEBO 9
#define LCDTiming0 0x180a0000
#define LCDTiming0__HFP_L 23
#define LCDTiming0__HFP_R 16
#define LCDTiming0__HBP_L 31
#define LCDTiming0__HBP_R 24
#define LCDTiming0__HSW_L 15
#define LCDTiming0__HSW_R 8
#define LCDTiming0__PPL_L 7
#define LCDTiming0__PPL_R 2
#define LCDTiming1 0x180a0004
#define LCDTiming1__VFP_L 23
#define LCDTiming1__VFP_R 16
#define LCDTiming1__VSW_L 15
#define LCDTiming1__VSW_R 10
#define LCDTiming1__VBP_L 31
#define LCDTiming1__VBP_R 24
#define LCDTiming2 0x180a0008
#define LCDTiming2__CPL_L 25
#define LCDTiming2__CPL_R 16
#define LCDTiming2__PCD_LO_WIDTH 5
#define LCDTiming2__PCD_HI_L 31
#define LCDTiming2__PCD_HI_R 27
#define LCDTiming2__IHS 12
#define LCDTiming2__IVS 11
#define LCDTiming2__IPC 13
#define LCDTiming3 0x180a000c
#define LCDTiming3__LEE 16
#define LCDUPBASE 0x180a0010
#define LCDControl 0x180a0018
#define LCDControl__LcdEn 0
#define LCDControl__LcdPwr 11
#define ASIU_TOP_CLK_GATING_CTRL 0x180aa024
#define ASIU_TOP_CLK_GATING_CTRL__LCD_CLK_GATE_EN 4

static int lcd_config(lcd_param_t *lcdParamPtr, bits_per_pixel_t dataBit, uint32_t clockDivider, uint32_t Frame_buffer, uint8_t bigEndian)
{
	uint32_t data, temp;
	uint32_t lcd_pcd_lo_mask = 0;
	uint32_t i;

	/* set the bPP   and TFT panel*/
	data = (dataBit<<LCDControl__LcdBpp_R) | (1 << LCDControl__LcdTFT);
	if (bigEndian)
		data |= (1 << LCDControl__BEBO);
	CPU_WRITE_32(LCDControl, data);

	/* now set the timing0 info */
	data = ((lcdParamPtr->HFP - 1)<<LCDTiming0__HFP_R)|((lcdParamPtr->HBP - 1)<<LCDTiming0__HBP_R)|((lcdParamPtr->HSW - 1)<<LCDTiming0__HSW_R);
	temp = (lcdParamPtr->PPL)/16 - 1;
	data |= temp<<LCDTiming0__PPL_R;

	CPU_WRITE_32(LCDTiming0, data);

	/* now set the timing1 info */
	data = ((lcdParamPtr->VFP - 1)<<LCDTiming1__VFP_R)|((lcdParamPtr->VSW - 1)<<LCDTiming1__VSW_R)|((lcdParamPtr->VBP - 1)<<LCDTiming1__VBP_R);
	data |= lcdParamPtr->LPF - 1;
	CPU_WRITE_32(LCDTiming1, data);

	/* now set the timing2 info */
	for (i = 0; i < LCDTiming2__PCD_LO_WIDTH; i++)
		lcd_pcd_lo_mask |= (0x1 << i);
	data = (clockDivider & lcd_pcd_lo_mask);
	temp = (clockDivider - data)>>LCDTiming2__PCD_LO_WIDTH;/*right shift 5 bit*/
	data = ((lcdParamPtr->PPL - 1)<<LCDTiming2__CPL_R) | data | (temp<<LCDTiming2__PCD_HI_R);/*hardcoded*/
	if (lcdParamPtr->IHS)
		data |= (1 << LCDTiming2__IHS);
	if (lcdParamPtr->IVS)
		data |= (1 << LCDTiming2__IVS);
	if (lcdParamPtr->IPC)
		data |= (1 << LCDTiming2__IPC);
	/* now program the LCD panel clock frequency */
	CPU_WRITE_32(LCDTiming2, data);

/*
	// GPIO Auxulary input enable
	CPU_WRITE_32(GIO2_R_GRPF2_AUX_SEL_MEMADDR,0xFFFFFFFF);

	// Enable the SMC through the GPIO.
	CPU_WRITE_32(GIO2_R_GRPF2_AUX01_SEL_MEMADDR,0x0);
*/

	/* now programming the line end control register */
	data = (1 << LCDTiming3__LEE) | (lcdParamPtr->LED - 1);
	CPU_WRITE_32(LCDTiming3, data);

	/* now load the up base address */
	CPU_WRITE_32(LCDUPBASE, Frame_buffer);

	return 0;
}

static void lcd_power_on(void)
{
	uint32_t data, i, j;
	/*this is the power on sequence */
	post_log("Turning on LCD panel\n\r");
	data = CPU_READ_32(LCDControl);
	data |= (1 << LCDControl__LcdEn);
	CPU_WRITE_32(LCDControl, data);
	for (i = 0; i < 500; i++) {
		j = j/10;
		j = 100;
	}
	data |= (1 << LCDControl__LcdPwr);
	CPU_WRITE_32(LCDControl, data);
}

#if 0
static void lcd_power_off(void)
{
	uint32_t data, addr, i, j;
	post_log("Turning off LCD panel\n\r");
	addr = LCDControl;
	/* this is the power down sequence */
	data = CPU_READ_32(addr);
	/* Power up sequence*/
	data &= ~(1 << LCDControl__LcdPwr);
	CPU_WRITE_32(addr, data);
	for (i = 0; i < 2000; i++) {
		j = j/10;
		j = 100;
	}
	data &= ~(1 << LCDControl__LcdEn);
	CPU_WRITE_32(addr, data);
}
#endif

int LCD_shrek_post_test(int flags)
{
	lcd_param_t lcdParam;
	uint32_t  address, BufferAddr, clockDiv;
	/*uint32_t LCDPalette_RAM_SIZE = 0x200;*/
	int i = 0, j, k, m;
	clockDiv = 50;

uint32_t shrek[] = {
0x3E5A5D,
0x3D5A5E,
0x3B5A5D,
0x395C60,
0x375C62,
0x355C63,
0x335E64,
0x325F65,
0x2F5E64,
0x2F5E64,
0x2F6067,
0x2C6169,
0x2B6269,
0x2A636C,
0x29636E,
0x29636F,
0x266670,
0x25656F,
0x23666F,
0x246672,
0x236772,
0x236973,
0x216975,
0x1F6974,
0x1D6976,
0x1D6B78,
0x1D6C79,
0x1D6C7B,
0x1B6C7D,
0x1B6C7D,
0x1B6E80,
0x1B6E80,
0x1C6F81,
0x1D7082,
0x1D7082,
0x1E7085,
0x1E7085,
0x1E7085,
0x1F7186,
0x1F7186,
0x207287,
0x207287,
0x1E7388,
0x1E7589,
0x1C778A,
0x1B778C,
0x1A798F,
0x19798F,
0x1A7D83,
0x1E7A85,
0x217A8C,
0x1C7C94,
0x167F95,
0x17818F,
0x1C8189,
0x227F90,
0x237D98,
0x247D9F,
0x248099,
0x1F8594,
0x1B8794,
0x1C869C,
0x2582A1,
0x337FA3,
0x27869C,
0x27869C,
0x28879D,
0x29889E,
0x2A899F,
0x2B8AA0,
0x2B8AA0,
0x2D89A0,
0x2F8BA0,
0x308CA1,
0x318DA2,
0x318DA2,
0x328EA3,
0x328EA3,
0x338FA4,
0x3490A5,
0x38929D,
0x3891A3,
0x3592A4,
0x3293A4,
0x3395A2,
0x3893A5,
0x3F91A9,
0x3D91AB,
0x3695A7,
0x3398A4,
0x3698A5,
0x3996A8,
0x3995AE,
0x3995AE,
0x3E94AD,
0x4292A9,
0x3996A7,
0x3899A0,
0x389A9B,
0x3F98A0,
0x4394A7,
0x4193A8,
0x3E96A4,
0x3F969F,
0x4294A0,
0x4193AB,
0x3C93B1,
0x3795AE,
0x3C95A3,
0x40949F,
0x4192A3,
0x3D92A7,
0x3E90A5,
0x3E90A6,
0x3B8FA7,
0x3991A5,
0x3693A5,
0x3392A4,
0x3490A5,
0x358FA7,
0x348EA8,
0x3091A4,
0x2D91A0,
0x2D929C,
0x2F8F9B,
0x308D9E,
0x2C8D9E,
0x298EA0,
0x258F9D,
0x2F88A6,
0x3685A6,
0x31869A,
0x268B93,
0x208C99,
0x1F89A3,
0x2389A2,
0x238796,
0x258595,
0x24829E,
0x1E82A4,
0x17879B,
0x138A8E,
0x19868D,
0x227F91,
0x237E91,
0x217D94,
0x1D7D96,
0x1C7C94,
0x1C7C92,
0x1E7A91,
0x1E7893,
0x1F7694,
0x1F7691,
0x1E768A,
0x1E7588,
0x1E748B,
0x1D7291,
0x1E7294,
0x1D738E,
0x1D7487,
0x1B7486,
0x1B7486,
0x1B7486,
0x1B7285,
0x1A7184,
0x1A7184,
0x1A7081,
0x1C6F81,
0x1C6F7F,
0x1C6D7E,
0x1D6C7B,
0x1D6C7B,
0x1E6C79,
0x1D6B78,
0x1E6A78,
0x1E6A78,
0x216977,
0x216977,
0x226874,
0x236774,
0x246674,
0x256571,
0x266471,
0x28646F,
0x28646E,
0x29636E,
0x2A636C,
0x2C606B,
0x2E6069,
0x2F5F69,
0x305F69,
0x315D66,
0x325F64,
0x325F64,
0x335F62,
0x365D62,
0x375D60,
0x395B5D,
0x3B5A5D,
0x3B5A5C,
0x3D5B5D,
0x3C595D,
0x395B5D,
0x395C60,
0x375C62,
0x345D63,
0x325D64,
0x325F65,
0x305F65,
0x2F6065,
0x2E6067,
0x2C6169,
0x2C626C,
0x2A636C,
0x29636E,
0x28646F,
0x25656F,
0x25656F,
0x236571,
0x226671,
0x236772,
0x236973,
0x216975,
0x206A77,
0x1D6976,
0x1D6B78,
0x1D6C7B,
0x1D6C7B,
0x1B6C7D,
0x1B6C7D,
0x1B6E80,
0x1B6E80,
0x197083,
0x1A7184,
0x1B7283,
0x1C7281,
0x1C7281,
0x1C7384,
0x1C7386,
0x1C748A,
0x1C748A,
0x1A758A,
0x1B7689,
0x1B7688,
0x1C7887,
0x1D7988,
0x1D7986,
0x1E7A87,
0x1D798E,
0x1F7A8D,
0x207B8E,
0x1F7C8E,
0x1B7E91,
0x1A8194,
0x1B8198,
0x207F9D,
0x247D9F,
0x277D9E,
0x258198,
0x208695,
0x1D8795,
0x21869A,
0x2982A2,
0x307FA6,
0x28879D,
0x28879D,
0x29889E,
0x2A899F,
0x2B8AA0,
0x2C8BA1,
0x2F8BA2,
0x2F8BA0,
0x308CA1,
0x318DA2,
0x328EA3,
0x338FA4,
0x338FA4,
0x3490A5,
0x3590A5,
0x3691A6,
0x3493A9,
0x3992A4,
0x3C92A3,
0x3994A7,
0x3695AB,
0x3996A8,
0x3E959F,
0x41939E,
0x4096A5,
0x3C97A9,
0x3C98A7,
0x3E98A3,
0x3F98A6,
0x3D98AD,
0x3F96B2,
0x3F94B1,
0x399BB6,
0x3E99B4,
0x4296B0,
0x4096AD,
0x3C97AC,
0x3999B1,
0x3998B6,
0x3B98B7,
0x3D97B2,
0x4096AD,
0x4095A9,
0x4095AA,
0x3F95AE,
0x3E96AC,
0x3F96A9,
0x3F95A2,
0x3C95A3,
0x3D94A7,
0x3D92A7,
0x3C92AB,
0x3994A9,
0x3594A6,
0x3494A4,
0x3693A5,
0x3690A8,
0x3690AA,
0x3490A9,
0x3490A7,
0x348FA2,
0x338EA1,
0x318DA4,
0x2E8CA6,
0x26909D,
0x2C8CA2,
0x3188A3,
0x3286A0,
0x2F879D,
0x26899E,
0x218BA1,
0x228BA0,
0x268798,
0x298496,
0x2A8298,
0x268297,
0x238495,
0x228491,
0x228292,
0x218094,
0x218191,
0x207F93,
0x1F7E94,
0x1E7D93,
0x1D7C8E,
0x1C7D8E,
0x1B7B93,
0x1A7995,
0x1A7A93,
0x1A798B,
0x1B7887,
0x1C7789,
0x1B758D,
0x1B758F,
0x1A7588,
0x197783,
0x1C7587,
0x1B7486,
0x1B7486,
0x1C7386,
0x1B7285,
0x1A7184,
0x1A7081,
0x1A7081,
0x1C6F7F,
0x1C6D7E,
0x1C6E7C,
0x1D6C7B,
0x1E6B7B,
0x1E6C79,
0x1E6A78,
0x1E6A78,
0x206A77,
0x216977,
0x236877,
0x236774,
0x246674,
0x256571,
0x266471,
0x26646F,
0x28646E,
0x29636E,
0x2A636C,
0x2B616B,
0x2C6169,
0x2F5F69,
0x305F69,
0x305F69,
0x306064,
0x315E63,
0x325E61,
0x365D62,
0x375D60,
0x395B5D,
0x3A5C5E,
0x3C5B5D,
0x3C5B5D,
0x3B5A5D,
0x395A5F,
0x375C62,
0x365D62,
0x325D63,
0x315E64,
0x305F65,
0x2F6065,
0x2E6067,
0x2C6167,
0x2C636A,
0x2A636C,
0x29636E,
0x28646E,
0x26646F,
0x236571,
0x236571,
0x236571,
0x236772,
0x226872,
0x216975,
0x206A75,
0x1E6A77,
0x1E6A78,
0x1D6B78,
0x1D6C7B,
0x1B6C7D,
0x1B6C7D,
0x1B6E80,
0x1B6E80,
0x1C6F81,
0x177287,
0x177284,
0x1B7381,
0x1B737F,
0x1D7380,
0x1C7384,
0x1C7488,
0x1A768D,
0x18768F,
0x18778D,
0x18778B,
0x197989,
0x1B7887,
0x1C7988,
0x1E7A89,
0x1E7A89,
0x1C7998,
0x1D7B97,
0x207C93,
0x217C8E,
0x217E8F,
0x207F95,
0x207F9D,
0x227FA1,
0x247EA0,
0x26809B,
0x25849A,
0x228696,
0x238797,
0x258699,
0x2A85A0,
0x2C85A7,
0x2A899F,
0x2A899F,
0x2B8AA0,
0x2C8BA1,
0x2D8CA2,
0x2E8DA3,
0x318DA2,
0x318DA2,
0x328EA3,
0x338FA4,
0x3490A5,
0x3591A6,
0x3691A6,
0x3792A7,
0x3893A8,
0x3893A8,
0x3893B0,
0x3C93A7,
0x4093A5,
0x3E92AA,
0x3B95B0,
0x3B96A9,
0x3F969D,
0x41949C,
0x4293A6,
0x4193AB,
0x3F95A6,
0x4198A2,
0x4298A5,
0x4298AF,
0x4198B3,
0x4298B1,
0x3A98B2,
0x4295B5,
0x4993B6,
0x4697B4,
0x3D9CB0,
0x389DB1,
0x3A99B7,
0x3997BB,
0x3B9ABA,
0x3E9AAF,
0x4298A7,
0x4596AB,
0x4295B5,
0x3E97B7,
0x3D98AB,
0x4099A1,
0x3B98A7,
0x3E97A7,
0x3F94A9,
0x4192AD,
0x3F93AF,
0x3A95AA,
0x3696A4,
0x3696A2,
0x3893A6,
0x3B90AD,
0x3B8EB0,
0x3A8DAD,
0x3890A8,
0x3590A3,
0x348EA6,
0x358CA8,
0x2B919D,
0x2A909E,
0x2D8CA0,
0x3488A4,
0x3685A4,
0x2E88A3,
0x248DA0,
0x248E9E,
0x28879B,
0x2E849B,
0x2D839A,
0x2B8399,
0x2B8198,
0x2B7F99,
0x288098,
0x238298,
0x1E8493,
0x1E8196,
0x207E97,
0x217D94,
0x1E7D8F,
0x1B7F8E,
0x187F92,
0x157D96,
0x167C95,
0x177C90,
0x197A8A,
0x1A7A88,
0x1B7889,
0x1B7889,
0x1A7884,
0x19777F,
0x1C7587,
0x1C7587,
0x1C7587,
0x1B7486,
0x1C7386,
0x1B7285,
0x1B7182,
0x1B7182,
0x1D7082,
0x1C6F7F,
0x1C6D7E,
0x1D6C7B,
0x1D6C7B,
0x1E6C79,
0x1E6C79,
0x1D6B78,
0x206A77,
0x206A77,
0x216977,
0x226874,
0x236774,
0x246672,
0x256571,
0x25656F,
0x26646F,
0x28646E,
0x29646C,
0x2A636C,
0x2C6169,
0x2E6069,
0x2F5F69,
0x2F5F69,
0x306064,
0x315E63,
0x325E61,
0x365D62,
0x375D60,
0x385B5F,
0x3A5C5E,
0x3C5B5D,
0x3A5C5E,
0x395B5D,
0x385B5F,
0x365D62,
0x355E64,
0x315E63,
0x2F5E64,
0x2F6065,
0x2E6165,
0x2C6167,
0x2C6368,
0x2A636A,
0x29646C,
0x28646E,
0x26646F,
0x26646F,
0x236571,
0x236571,
0x236772,
0x236772,
0x236975,
0x216975,
0x206A77,
0x1E6A77,
0x1D6B78,
0x1D6C7B,
0x1D6C7B,
0x1B6C7D,
0x1A6D7D,
0x1B6E80,
0x1C6F81,
0x1D7082,
0x177287,
0x1A7286,
0x1C7283,
0x1F7282,
0x207182,
0x207287,
0x1E748B,
0x1B758F,
0x1A768F,
0x19788E,
0x19788A,
0x19788A,
0x1A798B,
0x1A7A90,
0x187A95,
0x197A9A,
0x197C99,
0x1A7C97,
0x1D7D95,
0x1F7E94,
0x237E91,
0x277E8F,
0x297F90,
0x288094,
0x238298,
0x21849B,
0x23869B,
0x268798,
0x278797,
0x288799,
0x28899C,
0x298A9D,
0x2C8BA1,
0x2C8BA1,
0x2D8CA2,
0x2D8CA2,
0x2E8DA3,
0x318DA2,
0x328EA3,
0x338FA4,
0x3490A5,
0x3591A6,
0x3792A7,
0x3893A8,
0x3893A8,
0x3994A9,
0x3994A7,
0x3A95A8,
0x3F93AB,
0x3D95AB,
0x3B95AF,
0x3D94AF,
0x3F93AD,
0x4192A7,
0x4093A5,
0x3E92AA,
0x3D92AF,
0x4192AD,
0x4492A8,
0x4694AA,
0x4495B0,
0x4397B3,
0x4798AB,
0x4B9AA1,
0x4B9BA4,
0x4C9AA7,
0x4A98AC,
0x4799AF,
0x469CAD,
0x489BA9,
0x4A9CA7,
0x479EA8,
0x419DAA,
0x409DAE,
0x429AAE,
0x4697B2,
0x4697B5,
0x4399B4,
0x4199AD,
0x4099A7,
0x3E99AE,
0x4098AC,
0x4196AA,
0x4395AB,
0x4195AD,
0x3E96AC,
0x3A97A8,
0x3797A3,
0x3A96A5,
0x3C92A9,
0x3E8FAC,
0x3C90AC,
0x3792A7,
0x3592A3,
0x3590A2,
0x378FA3,
0x3091A1,
0x2F90A1,
0x318EA0,
0x338BA3,
0x338AA5,
0x2F8BA4,
0x298CA1,
0x288B9E,
0x2B879E,
0x2E85A0,
0x2C85A5,
0x2885A7,
0x2682A7,
0x2781A4,
0x26819E,
0x23839B,
0x1B8698,
0x1D839C,
0x207F9D,
0x227D9A,
0x1F7E94,
0x1B8092,
0x178093,
0x167E97,
0x167B97,
0x187A93,
0x197A8D,
0x1A798B,
0x1C798B,
0x1C798B,
0x1E7787,
0x1E7682,
0x1D7589,
0x1D7589,
0x1C7488,
0x1C7587,
0x1B7486,
0x1C7386,
0x1B7285,
0x1B7182,
0x1B7182,
0x1C6F81,
0x1C6D7E,
0x1C6D7E,
0x1E6D7C,
0x1E6B7B,
0x1E6C79,
0x1E6C79,
0x1F6B78,
0x1E6A77,
0x206A77,
0x216977,
0x226874,
0x236774,
0x236571,
0x23666F,
0x25656F,
0x26656E,
0x28646E,
0x29646C,
0x29626B,
0x2C6169,
0x2E6069,
0x2E6069,
0x316165,
0x306064,
0x335E64,
0x355E62,
0x375C62,
0x385B5F,
0x395B5D,
0x3A5C5E,
0x395C5E,
0x385B5D,
0x365C5F,
0x355E62,
0x335E64,
0x2F5E64,
0x2E5F64,
0x2E6067,
0x2C6167,
0x2C6167,
0x2C636A,
0x29646A,
0x28646C,
0x26656E,
0x25656F,
0x25656F,
0x236772,
0x236772,
0x236772,
0x236975,
0x216975,
0x206A77,
0x216B78,
0x1F6B79,
0x1E6C79,
0x1D6C7B,
0x1B6C7D,
0x1C6D7E,
0x1B6E80,
0x1B6E80,
0x1A7081,
0x1B7182,
0x197187,
0x1C7186,
0x1F7186,
0x207184,
0x217285,
0x207288,
0x1E748B,
0x1C7690,
0x1B7790,
0x1B778E,
0x1C778C,
0x1D788D,
0x1D7990,
0x1A7995,
0x177B9B,
0x157C9F,
0x197E90,
0x1A7F93,
0x1C7E97,
0x1F7E9A,
0x227E97,
0x267F91,
0x29818B,
0x27848C,
0x238592,
0x218698,
0x24879C,
0x28879B,
0x288799,
0x288999,
0x278C98,
0x278D99,
0x2C8BA1,
0x2D8CA2,
0x2E8DA3,
0x318DA2,
0x328EA3,
0x338FA4,
0x3490A5,
0x3591A6,
0x3792A7,
0x3792A7,
0x3893A8,
0x3994A7,
0x3A95A8,
0x3A95A8,
0x3D95A9,
0x3E96AA,
0x4396A6,
0x3D97AF,
0x3897B5,
0x3B97AE,
0x4293A6,
0x4792A7,
0x4592B0,
0x4092B7,
0x4095B4,
0x4296AE,
0x4896AD,
0x4794B2,
0x4594B2,
0x4496AB,
0x4999A4,
0x4C9A9C,
0x52999D,
0x4A9DA5,
0x439EB0,
0x459DB5,
0x4D98B5,
0x5198AC,
0x50999F,
0x4B9C96,
0x4BA09B,
0x479EA8,
0x469CB5,
0x459AB7,
0x469BB0,
0x489BAB,
0x479AAC,
0x449AB1,
0x449AB5,
0x4399B0,
0x439AAB,
0x429AA8,
0x4299AA,
0x4198AB,
0x3F98AA,
0x3D99A8,
0x3E97A5,
0x3E97A5,
0x3D96A8,
0x3A95AA,
0x3894A9,
0x3695A7,
0x3595A1,
0x36949E,
0x3490A7,
0x3590A5,
0x348FA2,
0x308FA1,
0x2E8FA0,
0x2E8D9F,
0x2E899C,
0x2D889D,
0x2C88A1,
0x2A88A4,
0x2986A7,
0x2985AA,
0x2586A9,
0x2185A5,
0x21859F,
0x25839C,
0x1C879B,
0x1E839F,
0x2180A0,
0x237D9F,
0x207E98,
0x1E7F92,
0x1B7E91,
0x1D7D93,
0x1D7B94,
0x1E7A93,
0x1D7992,
0x1A7892,
0x197992,
0x1C7891,
0x1F778B,
0x207685,
0x1C778A,
0x1B7689,
0x1A7588,
0x1C7488,
0x1C7488,
0x1B7486,
0x1B7285,
0x1A7184,
0x1B7182,
0x1D7082,
0x1C6F7F,
0x1C6D7E,
0x1E6D7C,
0x1E6D7C,
0x1D6C7B,
0x1E6B7B,
0x1E6C79,
0x1D6B77,
0x1E6A77,
0x206A77,
0x206874,
0x226874,
0x226671,
0x226671,
0x23666F,
0x25656E,
0x26656E,
0x28646C,
0x29646C,
0x2B616B,
0x2C6169,
0x2C6169,
0x306166,
0x305F65,
0x325F64,
0x355E64,
0x365D62,
0x365C5F,
0x385B5F,
0x3A5C5E,
0x395C5E,
0x375D60,
0x355D5F,
0x335F62,
0x325F64,
0x2F5E64,
0x2F6067,
0x2E6067,
0x2C6167,
0x2B6267,
0x2A636A,
0x29646C,
0x28646E,
0x25656F,
0x25656F,
0x236571,
0x236774,
0x226874,
0x236975,
0x216975,
0x216977,
0x206A77,
0x1F6B79,
0x1E6C79,
0x1D6C7B,
0x1B6C7D,
0x1C6D7E,
0x1B6E80,
0x1C6F81,
0x1A7081,
0x1B7084,
0x1C7185,
0x1B7286,
0x1D7287,
0x1F7186,
0x207385,
0x1D7485,
0x1A7587,
0x187789,
0x16798E,
0x18788E,
0x1B778E,
0x1F778F,
0x217790,
0x207890,
0x1E7A91,
0x1A7D94,
0x197D95,
0x217D88,
0x1F7E90,
0x1F7E9A,
0x217F9B,
0x22809A,
0x1F839B,
0x1C869C,
0x20879A,
0x278496,
0x2C8498,
0x2B879E,
0x2989A1,
0x298A9D,
0x2B8B9B,
0x2C8B9D,
0x2A8DA2,
0x2F8BA0,
0x308CA1,
0x318DA2,
0x338FA4,
0x338FA4,
0x3490A5,
0x3692A7,
0x3893A8,
0x3893A8,
0x3994A7,
0x3A95A8,
0x3D95A9,
0x3E96AA,
0x3E96AA,
0x3F97AB,
0x4098AC,
0x3F98AA,
0x3D97AF,
0x3C97AC,
0x3F97A5,
0x4496A2,
0x4694A8,
0x4794B0,
0x4895B1,
0x4896AC,
0x4798AB,
0x4798B3,
0x4697B2,
0x4799A7,
0x479AA0,
0x449CA6,
0x439EB3,
0x4F9AAD,
0x479EB9,
0x40A1C4,
0x44A0C7,
0x4C9BC2,
0x4D9BBF,
0x4A9EBA,
0x49A0B1,
0x4D9FAA,
0x4F9DA9,
0x4E9CB2,
0x479DB4,
0x469FAF,
0x489EAB,
0x499CAE,
0x499AB5,
0x459CAF,
0x459CAF,
0x439CAE,
0x419DAC,
0x409CA9,
0x429BA9,
0x4399AA,
0x4298A9,
0x4199A7,
0x3D99A6,
0x3999A9,
0x3B97AC,
0x3A96AF,
0x3A94AC,
0x3895A6,
0x3595A1,
0x3492AB,
0x3792A7,
0x3691A4,
0x3192A3,
0x2F90A1,
0x338C9C,
0x368896,
0x31869A,
0x2B89A2,
0x278BA3,
0x2A899D,
0x2F889A,
0x2C8799,
0x27889B,
0x26869C,
0x26859B,
0x228598,
0x238399,
0x25809B,
0x257F9A,
0x237F96,
0x227F91,
0x217E8F,
0x227E8D,
0x217D8C,
0x207B8D,
0x1C7B91,
0x187A97,
0x167999,
0x177898,
0x1A798F,
0x1C7789,
0x1C778A,
0x1B7689,
0x1A7588,
0x1A7588,
0x1C7488,
0x1C7488,
0x1B7486,
0x1B7285,
0x1C7185,
0x1B7182,
0x1C6F81,
0x1C6F7F,
0x1D6E7F,
0x1C6D7E,
0x1E6D7C,
0x1D6C7B,
0x1D6C79,
0x1D6C79,
0x1D6B78,
0x1E6A77,
0x1F6976,
0x206874,
0x226874,
0x236772,
0x22666F,
0x23666F,
0x25656E,
0x26656E,
0x29646C,
0x29626B,
0x2B616B,
0x2B616B,
0x2F6065,
0x2F6065,
0x2F5F63,
0x335E64,
0x355E62,
0x375C62,
0x395C60,
0x395C5E,
0x375D60,
0x365E60,
0x355E62,
0x335E64,
0x305F65,
0x2F6065,
0x2E6067,
0x2C6167,
0x2B6267,
0x2A636A,
0x29646A,
0x28646C,
0x26656E,
0x25656F,
0x236571,
0x236571,
0x226874,
0x236975,
0x216975,
0x216977,
0x206A77,
0x1E6A78,
0x1E6C79,
0x1E6B7B,
0x1D6C7B,
0x1C6D7E,
0x1C6F81,
0x1C6F81,
0x1A7081,
0x1B7084,
0x1B7084,
0x1C7185,
0x197487,
0x1B7486,
0x1D7487,
0x1D7487,
0x1B7688,
0x177888,
0x157B8A,
0x127C8C,
0x157C8F,
0x177A8F,
0x1C7891,
0x207890,
0x21798F,
0x207B8E,
0x1E7E8E,
0x1B7F8E,
0x297C8A,
0x237E93,
0x208099,
0x22809A,
0x22829B,
0x1A86A2,
0x1889AB,
0x1F88A8,
0x2C839E,
0x328299,
0x2F879F,
0x2A8AA3,
0x2B8BA1,
0x2F8A9D,
0x308AA4,
0x2E8BAC,
0x308CA1,
0x328EA3,
0x3490A5,
0x3591A6,
0x3591A6,
0x3691A6,
0x3792A7,
0x3893A6,
0x3A95A8,
0x3D95A9,
0x3E96AA,
0x3F97AB,
0x4098AC,
0x4198AB,
0x4299AC,
0x439AAD,
0x3D99B0,
0x4198AB,
0x4796A3,
0x4896A0,
0x4599A3,
0x439BA9,
0x459BAC,
0x4B99A6,
0x4D97A4,
0x4C97AA,
0x459AAE,
0x459DAB,
0x479EA5,
0x499EA5,
0x449FB4,
0x3D9FC4,
0x4B9CAF,
0x4B9DB5,
0x4B9CB9,
0x4C9CB5,
0x4B9CAF,
0x489FB3,
0x46A1C0,
0x45A1C8,
0x4A9FC4,
0x4E9EB7,
0x509FAE,
0x4EA0AC,
0x4AA1B2,
0x48A0B4,
0x489EB5,
0x4B9DB5,
0x43A1A9,
0x469FAF,
0x469CB3,
0x449BB6,
0x429DB2,
0x419DAC,
0x449AA7,
0x4799A7,
0x4699AB,
0x4199AD,
0x3D99B0,
0x3D97B1,
0x4095B2,
0x4094B0,
0x3D95AD,
0x3996A8,
0x3595AB,
0x3695A7,
0x3594A6,
0x3690A8,
0x378DA6,
0x398A9D,
0x378995,
0x338996,
0x2E8B9D,
0x2B8C9F,
0x2B8B97,
0x2E8B92,
0x308A93,
0x2F8898,
0x2C8799,
0x298697,
0x298697,
0x288597,
0x278297,
0x268098,
0x248095,
0x238092,
0x238091,
0x22808C,
0x217F87,
0x207E88,
0x1C7D8E,
0x187C94,
0x167B99,
0x167B97,
0x167B8F,
0x167B87,
0x1C798B,
0x1C778A,
0x1B7689,
0x1A7588,
0x1B7689,
0x1A7588,
0x1B7387,
0x1A7385,
0x1B7285,
0x1B7084,
0x1B7182,
0x1C6F81,
0x1C6F7F,
0x1D6E7F,
0x1C6D7E,
0x1C6D7E,
0x1D6C79,
0x1D6C79,
0x1E6C79,
0x1E6A78,
0x206A77,
0x216977,
0x226874,
0x226872,
0x226671,
0x23666F,
0x23666F,
0x26656E,
0x28646E,
0x29646C,
0x29626B,
0x2B616B,
0x2E6067,
0x2F6065,
0x2F5E64,
0x325F64,
0x355E64,
0x365D62,
0x375D60,
0x375D60,
0x385E61,
0x365E60,
0x355E62,
0x335E64,
0x305F65,
0x2F6067,
0x2E6067,
0x2D626A,
0x2C636A,
0x2A636A,
0x29646C,
0x28646C,
0x26656E,
0x25656F,
0x236571,
0x226671,
0x206874,
0x216975,
0x216977,
0x206A77,
0x1E6A78,
0x1E6A78,
0x1D6A7A,
0x1D6C7B,
0x1B6C7D,
0x1B6E7E,
0x1C6F81,
0x1B7182,
0x1B7084,
0x1B7084,
0x1B7285,
0x1B7285,
0x167686,
0x197687,
0x1C7488,
0x1E7589,
0x1E7589,
0x1B7689,
0x18798A,
0x157B8A,
0x157C8B,
0x167B8D,
0x1A7B8E,
0x1E7A91,
0x1F7B94,
0x1D7D96,
0x1C7E97,
0x1A8097,
0x267F91,
0x228195,
0x20839A,
0x21849B,
0x218499,
0x23869B,
0x24869F,
0x2886A0,
0x2D859D,
0x2F859E,
0x2E88A2,
0x2E89A4,
0x308AA2,
0x328AA0,
0x348BA6,
0x328DAA,
0x328EA3,
0x3490A5,
0x3692A7,
0x3792A7,
0x3792A7,
0x3792A5,
0x3792A5,
0x3A92A6,
0x3E96AA,
0x3E96AA,
0x4097AA,
0x4198AB,
0x4299AC,
0x439AAD,
0x439AAD,
0x459AAE,
0x4698AD,
0x4998A9,
0x4B97A5,
0x4C98A6,
0x489AA6,
0x429FA7,
0x42A0AA,
0x479DAE,
0x4E9BAF,
0x4E99AC,
0x499DA8,
0x46A0AB,
0x49A0B3,
0x4D9DB4,
0x4E9BAD,
0x4D9AA4,
0x5D9F83,
0x6CA47F,
0x7BAA7C,
0x7FB17E,
0x77B17E,
0x6AAD7A,
0x5FA485,
0x539EA1,
0x4DA2BF,
0x48A2C4,
0x4CA3B7,
0x52A2AD,
0x52A0B4,
0x4DA0BE,
0x4AA1BD,
0x49A1B5,
0x42A5AB,
0x48A1B3,
0x4C9BBA,
0x4E99C1,
0x489BBD,
0x469CB5,
0x469CA9,
0x499BA6,
0x499AAB,
0x4799B1,
0x4398B5,
0x4298B3,
0x4597AD,
0x4596AB,
0x4195AD,
0x3E95B0,
0x3796A8,
0x3697A8,
0x3793A8,
0x3A8EA8,
0x3D8AA6,
0x388AA0,
0x348E99,
0x348E97,
0x338B99,
0x2F8A9D,
0x278DA3,
0x218FA6,
0x228EA8,
0x298BA4,
0x2C899B,
0x2A8894,
0x2A869B,
0x29859A,
0x258498,
0x248399,
0x22829A,
0x1F819A,
0x1E8198,
0x1D8392,
0x1C818D,
0x1D7F88,
0x1D7D8B,
0x1D7C90,
0x1C7A93,
0x1B7B91,
0x187C8B,
0x167D86,
0x1C788D,
0x1B778C,
0x1A768B,
0x1A768B,
0x1B7689,
0x1B7689,
0x197487,
0x1A7286,
0x1B7285,
0x1B7285,
0x1B7084,
0x1B7182,
0x1C6F81,
0x1C6F7F,
0x1C6D7E,
0x1C6D7E,
0x1B6D7B,
0x1D6C7B,
0x1E6C79,
0x1F6B79,
0x206978,
0x206A77,
0x216975,
0x236975,
0x216771,
0x226671,
0x23666F,
0x25656F,
0x28646E,
0x29636E,
0x29626B,
0x29626B,
0x2D6268,
0x2E6067,
0x2F6065,
0x305F65,
0x335E64,
0x345D61,
0x355C61,
0x375D60,
0x365D62,
0x355C61,
0x345D63,
0x325F65,
0x316068,
0x2E6067,
0x2C6169,
0x2B6269,
0x2C636A,
0x2A636C,
0x29646C,
0x28646E,
0x26646F,
0x256571,
0x246674,
0x236774,
0x206876,
0x206A77,
0x206A77,
0x1F6B79,
0x1F6B79,
0x1E6B7B,
0x1D6C7B,
0x1C6D7E,
0x1B6C7D,
0x1B6E80,
0x1A7081,
0x1B7084,
0x1C7185,
0x1A7184,
0x1A7184,
0x1B7285,
0x177688,
0x197487,
0x1D7488,
0x217389,
0x23738C,
0x20748C,
0x1E768C,
0x1C798B,
0x18798A,
0x1A7B8B,
0x1C7B8D,
0x1C7B8F,
0x1E7C95,
0x1E809D,
0x1D7FA2,
0x1A7DA4,
0x1F8093,
0x218295,
0x218698,
0x1F8697,
0x228696,
0x2E8795,
0x30848F,
0x2D858F,
0x2B8B9B,
0x26899E,
0x2A8AA2,
0x318BA5,
0x3389A4,
0x348AA3,
0x368EA2,
0x3790A0,
0x3590A5,
0x3590A5,
0x3691A6,
0x3893A6,
0x3893A6,
0x3A92A6,
0x3A92A6,
0x3B93A7,
0x3F96A9,
0x4299AC,
0x439AAD,
0x4499AD,
0x459AAE,
0x469BAF,
0x469BAF,
0x449AAB,
0x4B97A4,
0x4A98AC,
0x4A9AB5,
0x4B98B2,
0x4C9BAC,
0x499DA8,
0x449DAD,
0x459FB9,
0x489EB9,
0x4D9EAF,
0x4DA0A8,
0x4DA3B2,
0x4E9EB7,
0x589AA6,
0x719F83,
0x92B573,
0xA6C659,
0xBDD251,
0xC5D340,
0xCAD843,
0xC8D94C,
0xC7D94F,
0xBFD356,
0x97BB64,
0x67A785,
0x4DA1AB,
0x4EA8C0,
0x52A2B9,
0x559DB5,
0x58A2BD,
0x4CA0BC,
0x4AA6BB,
0x48A5B4,
0x4EA0B5,
0x549BB9,
0x599AC0,
0x5196BF,
0x4D9CBB,
0x4BA1B2,
0x489CA7,
0x4A9CAA,
0x4B99AF,
0x4898B1,
0x4799AE,
0x449BA5,
0x439AA4,
0x4297AB,
0x4096B1,
0x3E97A9,
0x3C97AA,
0x3D95AB,
0x3E92AC,
0x3D8FA7,
0x368EA2,
0x30909C,
0x358F9A,
0x388B9B,
0x348AA3,
0x288CAC,
0x1F93B8,
0x1B91B7,
0x1F8CAB,
0x2689A0,
0x268696,
0x27869C,
0x27869A,
0x26869C,
0x20839A,
0x2385A0,
0x1D809F,
0x1F84A2,
0x1A829B,
0x1B8295,
0x1D8191,
0x217C8E,
0x257A8E,
0x26788D,
0x23788C,
0x1E798B,
0x197A8A,
0x1C788D,
0x1B778C,
0x1B778C,
0x1A768B,
0x1A768B,
0x1A758A,
0x1A7588,
0x1B7387,
0x1B7387,
0x1C7386,
0x1B7285,
0x1B7182,
0x1A7081,
0x1C6F81,
0x1B6E80,
0x1B6E7E,
0x1C6E7C,
0x1E6D7C,
0x1D6C7B,
0x1E6C79,
0x1E6A78,
0x206A77,
0x206876,
0x226874,
0x226874,
0x236772,
0x23666F,
0x25656F,
0x26646F,
0x29636E,
0x2A636E,
0x2A636E,
0x2B6269,
0x2C6167,
0x2D5F66,
0x2F5E64,
0x325F65,
0x335E64,
0x355E62,
0x365D62,
0x365D62,
0x345D61,
0x325D63,
0x325F65,
0x306168,
0x2E6069,
0x2C6169,
0x2B616B,
0x2A636C,
0x2A636C,
0x29636E,
0x28646E,
0x25656F,
0x246672,
0x236774,
0x236774,
0x1F6976,
0x206A77,
0x1E6A77,
0x1F6B79,
0x1E6C79,
0x1D6C7B,
0x1D6C7D,
0x1C6D7E,
0x1C6F81,
0x1C6F81,
0x1B7084,
0x1C7185,
0x1B7285,
0x1B7285,
0x1B7285,
0x1C7386,
0x1A748C,
0x1C748A,
0x1D7487,
0x207687,
0x207687,
0x1F768A,
0x1E768C,
0x1C788F,
0x1F7B92,
0x1F7C8E,
0x217C8E,
0x217D8C,
0x237C8E,
0x217B93,
0x237D98,
0x247F9E,
0x27839A,
0x238294,
0x238494,
0x238796,
0x238797,
0x238699,
0x27879D,
0x298C9F,
0x258B9A,
0x2B8F9F,
0x2F8EA2,
0x328CA4,
0x368DA8,
0x388EA5,
0x378E9F,
0x378F99,
0x3792A5,
0x3893A6,
0x3994A7,
0x3C94A8,
0x3C94A8,
0x3C94A8,
0x3D94A8,
0x3E95A8,
0x4097AA,
0x4398AC,
0x459AAE,
0x469CAD,
0x499CAE,
0x4A9DAF,
0x499CAE,
0x479AAC,
0x489DB1,
0x4797AE,
0x4A96AE,
0x5199AF,
0x539AAE,
0x519AAD,
0x4F9DB1,
0x4EA0B5,
0x4EA4B3,
0x4BA1AE,
0x4FA1AF,
0x54A1AB,
0x5B9A8F,
0x88B37B,
0xB0C256,
0xD4D343,
0xD2D026,
0xD7D21E,
0xDFD714,
0xE7DA12,
0xEFDE1C,
0xF2DE27,
0xEFDE2E,
0xEAE848,
0xBFD75D,
0x8ABE7E,
0x569D97,
0x56A0BB,
0x5DA3BD,
0x599DB0,
0x53A5B3,
0x4AA7B8,
0x50A3B3,
0x55A2B2,
0x59A0B2,
0x589AB2,
0x5E9FBB,
0x549BB7,
0x4B9BB2,
0x4CA2B1,
0x4B9EAE,
0x4C9DAE,
0x4B9CAD,
0x479EA8,
0x449EA6,
0x419EA6,
0x429BAD,
0x4297B4,
0x4196AA,
0x4197A8,
0x4097AB,
0x3D94AF,
0x3892AC,
0x3790A0,
0x369098,
0x378F9D,
0x378DA8,
0x3386A4,
0x38899A,
0x358689,
0x318788,
0x328B99,
0x2A899F,
0x2989A1,
0x268291,
0x298695,
0x248596,
0x22859A,
0x2688A1,
0x2083A0,
0x1F839D,
0x22869E,
0x1D8198,
0x1F7F97,
0x247C92,
0x27798F,
0x28798E,
0x24798E,
0x1F7993,
0x1A7995,
0x1C788D,
0x1C788D,
0x1B778C,
0x1B778C,
0x1A768B,
0x1A768B,
0x1A7588,
0x197487,
0x1C7488,
0x1B7387,
0x1B7285,
0x1A7184,
0x1B7182,
0x1A7081,
0x1C6F81,
0x1B6E80,
0x1C6D7E,
0x1E6D7C,
0x1D6C7B,
0x1E6B7B,
0x1F6B79,
0x206978,
0x216977,
0x206876,
0x226874,
0x236772,
0x246672,
0x25656F,
0x26646F,
0x28646F,
0x29636E,
0x2A636E,
0x2C636A,
0x2B6269,
0x2C6167,
0x2F6067,
0x305F65,
0x335E64,
0x355E64,
0x355E62,
0x365F63,
0x355E64,
0x325D63,
0x305F65,
0x2F6067,
0x2E6069,
0x2B616B,
0x2C626C,
0x2A636C,
0x29646C,
0x28646E,
0x26646F,
0x256571,
0x246674,
0x236774,
0x246777,
0x1E6A77,
0x1E6A77,
0x1F6B79,
0x1E6C79,
0x1E6B7B,
0x1D6C7B,
0x1C6D7E,
0x1C6D80,
0x1C6F81,
0x1A6F83,
0x1B7084,
0x1B7285,
0x1B7285,
0x1B7285,
0x1C7386,
0x1D7487,
0x1E718F,
0x1D738C,
0x1D7686,
0x1B7983,
0x197A83,
0x187A87,
0x16798C,
0x197992,
0x1A7892,
0x1D778F,
0x217A8C,
0x257D89,
0x287F88,
0x287F89,
0x297F8C,
0x2A808F,
0x297C9A,
0x287E97,
0x298294,
0x2B8995,
0x258A9C,
0x1C8BA7,
0x188EB2,
0x1C8CB1,
0x288CA6,
0x318EA0,
0x2F8C9E,
0x2D8CA2,
0x3391AA,
0x3691A6,
0x378EA2,
0x3C93A7,
0x3B93A7,
0x3C94A8,
0x3D95A9,
0x3E96AA,
0x3D95A9,
0x3E95A9,
0x3E95A8,
0x4095A9,
0x4297AB,
0x4398AC,
0x479AAC,
0x4A9DAF,
0x4D9EB1,
0x4D9EB1,
0x4B9CAF,
0x4A9BAE,
0x38A3C7,
0x4F9BA8,
0x6E9E90,
0x7BA693,
0x639B9C,
0x59A0B4,
0x519EB2,
0x53A0A8,
0x52A1A6,
0x56A5B6,
0x57A0B3,
0x649E8F,
0x94B96B,
0xBDCE44,
0xD0CF27,
0xD5C91D,
0xDECF1A,
0xE1D425,
0xE5D623,
0xEAD617,
0xF1D819,
0xEFD828,
0xEBD62F,
0xEDDC2A,
0xEAE026,
0xD6DC46,
0x9DBD74,
0x639C96,
0x5CA3B5,
0x5CA9B3,
0x57A8AC,
0x52A6B0,
0x5AA4AD,
0x57A5AF,
0x57A7B2,
0x53A1AD,
0x549CAB,
0x5AA1B5,
0x549FB4,
0x4FA1B7,
0x4AA1B5,
0x49A0B1,
0x49A1AD,
0x47A1AA,
0x43A1AB,
0x429FB0,
0x449CB4,
0x4598B6,
0x4499AE,
0x4399A8,
0x4198A9,
0x3D97B2,
0x3C93B1,
0x3D90A0,
0x3D9094,
0x3891A1,
0x338DAF,
0x3E879A,
0x618A6C,
0x84944B,
0x83904A,
0x5D8563,
0x39848A,
0x2487A4,
0x2E8891,
0x2A8693,
0x278797,
0x25889D,
0x1F819A,
0x20839A,
0x26869C,
0x218094,
0x248097,
0x237E99,
0x227D98,
0x227C96,
0x227D92,
0x1E7D93,
0x1C7B99,
0x187A9D,
0x1B7A8E,
0x1A798D,
0x1A798D,
0x1B778C,
0x1B778C,
0x1A768B,
0x1B768B,
0x1A7588,
0x1A7588,
0x1B7387,
0x1B7285,
0x1B7285,
0x1B7084,
0x1B7182,
0x1A7081,
0x1C6F81,
0x1C6D7E,
0x1C6D7E,
0x1D6C7D,
0x1E6B7B,
0x1F6B7B,
0x216A79,
0x216978,
0x216977,
0x236975,
0x226874,
0x236772,
0x246672,
0x26646F,
0x28646F,
0x29636F,
0x29636F,
0x2A636A,
0x2C636A,
0x2C6169,
0x2E6067,
0x305F67,
0x325F65,
0x335E64,
0x335E64,
0x345F65,
0x335E64,
0x315E64,
0x2F6067,
0x2E6069,
0x2C6169,
0x2C626C,
0x2A636C,
0x29646C,
0x28646E,
0x26656E,
0x25656F,
0x236571,
0x236774,
0x226776,
0x236877,
0x1E6A77,
0x1E6A77,
0x1E6C79,
0x1E6C79,
0x1D6C7B,
0x1D6C7D,
0x1C6D7E,
0x1B6E80,
0x1D6F84,
0x1A6F83,
0x1B7084,
0x1B7285,
0x1B7285,
0x1B7285,
0x1C7386,
0x1C7587,
0x1E728E,
0x1D758D,
0x197989,
0x167D86,
0x127D85,
0x127C89,
0x157A8E,
0x1A7A93,
0x1C7792,
0x1F7991,
0x237C8E,
0x237F8C,
0x23818B,
0x20828F,
0x218293,
0x1F8297,
0x2A82A6,
0x2B82A0,
0x2B819A,
0x2A869B,
0x24899B,
0x248A99,
0x2B8C9C,
0x29889A,
0x2C8AA3,
0x318DA6,
0x328EA3,
0x3191A1,
0x2F92A5,
0x2E92A9,
0x3391AD,
0x3D92AF,
0x3C94A8,
0x3D95A9,
0x3E96AA,
0x4097AB,
0x4097AA,
0x3F96A9,
0x4196AA,
0x4398AC,
0x4598AA,
0x4598AA,
0x4899AC,
0x4B9CAF,
0x4D9EB1,
0x4F9EAF,
0x4E9DAE,
0x4E9DAE,
0x35A4CF,
0x659E8F,
0x98984E,
0xB3AE5A,
0x8BAF83,
0x519C9F,
0x4CA4B8,
0x53A5BA,
0x56A3B7,
0x569DB3,
0x659D9E,
0x8EAF7A,
0xBBC44D,
0xC6C223,
0xCDC324,
0xCBC032,
0xCDC41F,
0xD5CD2C,
0xDFD32F,
0xE9D92B,
0xEEDE30,
0xEDE042,
0xE8E24E,
0xE5DD34,
0xEDDC12,
0xEBDA0E,
0xDDDE45,
0x9ABC72,
0x5EA094,
0x57A9B5,
0x5CACB7,
0x5AA5AB,
0x5EA8AB,
0x52A4AF,
0x50A9B9,
0x54ADBF,
0x54A7B5,
0x58A4B1,
0x59A2B3,
0x529FB9,
0x4CA2BD,
0x49A4B9,
0x48A3B5,
0x47A3B2,
0x48A1B3,
0x489EB5,
0x4A9CB4,
0x4C99B3,
0x469CAD,
0x459BAC,
0x439AAE,
0x4298B3,
0x4293AE,
0x4091A2,
0x3A929E,
0x3493A7,
0x3991A5,
0x598F82,
0xA8A95D,
0xDCB333,
0xE1B433,
0xACA755,
0x588F7A,
0x29899F,
0x2A899D,
0x28889E,
0x23869D,
0x2387A1,
0x2487A4,
0x2385A0,
0x26859B,
0x288396,
0x288096,
0x257F9A,
0x217E9D,
0x1C809A,
0x188196,
0x198093,
0x197D95,
0x197A9A,
0x1C7B8F,
0x1B7A8E,
0x1A798D,
0x1A798D,
0x1B778C,
0x1B778C,
0x1A768B,
0x1A7588,
0x1A7588,
0x1C7488,
0x1B7486,
0x1B7285,
0x1B7285,
0x1B7182,
0x1B7182,
0x1A7081,
0x1B6E7E,
0x1C6D7E,
0x1D6C7D,
0x1E6B7D,
0x1F6B7B,
0x1F6B7B,
0x206978,
0x216977,
0x216977,
0x236975,
0x236774,
0x246672,
0x256571,
0x26646F,
0x28646F,
0x29636F,
0x29646C,
0x2A636C,
0x2C636A,
0x2E6069,
0x2F6067,
0x305F65,
0x325F65,
0x345F65,
0x336065,
0x325F64,
0x2F5E64,
0x2E6067,
0x2C6169,
0x2B616B,
0x2A636C,
0x29636E,
0x28646E,
0x26656E,
0x26646F,
0x25656F,
0x246672,
0x226874,
0x236877,
0x216977,
0x1E6A77,
0x1E6C78,
0x1E6C79,
0x1E6B7B,
0x1D6C7B,
0x1C6D7E,
0x1C6D80,
0x1C6F81,
0x1D6F84,
0x1B7084,
0x1B7285,
0x1B7285,
0x1C7386,
0x1C7386,
0x1B7486,
0x1D7688,
0x1E7589,
0x1B768B,
0x18788E,
0x167A91,
0x157B91,
0x167891,
0x1C7891,
0x207890,
0x20778B,
0x237C8E,
0x208090,
0x1B8293,
0x168398,
0x1485A3,
0x1688AD,
0x1588B4,
0x1C87A7,
0x2284A1,
0x2784A3,
0x2588A7,
0x2F8994,
0x4E8A6E,
0x6E8F58,
0x689165,
0x408B86,
0x2C8CA4,
0x3290A9,
0x39939E,
0x30929F,
0x2A96B0,
0x329AB7,
0x3892AC,
0x3E95A9,
0x4097AB,
0x4299AD,
0x439AAE,
0x4299AC,
0x4398AC,
0x4499AD,
0x489AAF,
0x499CAE,
0x499AAD,
0x499AAD,
0x4D9CAD,
0x4E9DAE,
0x4F9EAF,
0x4F9EAF,
0x519EB0,
0x45A3BD,
0x87A277,
0xA8810A,
0xC69C12,
0xBFCB69,
0x5FA583,
0x4CADBE,
0x43A2D2,
0x50A0D1,
0x66A4B3,
0x84A77F,
0xAFB55D,
0xAA9F33,
0x96881D,
0x938522,
0xB8AC4A,
0xD1D03A,
0xD6D42A,
0xD9D51A,
0xD9D117,
0xE3D92C,
0xE9E13E,
0xE4E240,
0xE5E73C,
0xE5E126,
0xE8DB10,
0xDED611,
0xCBDA4B,
0x81B27A,
0x5CA5AE,
0x5FABC3,
0x61A6B5,
0x59A9B4,
0x59AABB,
0x54A6BC,
0x51A6BB,
0x56A9B9,
0x55A4B1,
0x59A5B5,
0x55A1B9,
0x52A3C0,
0x4DA4BF,
0x4AA5BA,
0x4BA3B7,
0x4FA0B5,
0x529DB2,
0x519DAD,
0x509CAA,
0x479EA8,
0x459BB2,
0x4798B6,
0x4A96AD,
0x4695A2,
0x3A95A7,
0x2F97B4,
0x3096AF,
0x408F8C,
0x779768,
0xB19C43,
0xD59B1E,
0xDEA41E,
0xB8A73D,
0x75A073,
0x328B8F,
0x2889A9,
0x2987A1,
0x2C879C,
0x2F879D,
0x2B87A0,
0x2886A2,
0x2987A3,
0x237E99,
0x29809C,
0x267F9D,
0x21809E,
0x1A829B,
0x168495,
0x188290,
0x1E7D8F,
0x227A90,
0x1C7B8F,
0x1C7B8F,
0x1B7A8E,
0x1A798D,
0x1C788D,
0x1B778C,
0x1A768B,
0x1B768B,
0x1A7588,
0x1C7488,
0x1B7387,
0x1C7386,
0x1B7285,
0x1C7185,
0x1B7182,
0x1A7081,
0x1C6F81,
0x1B6E7E,
0x1B6C7D,
0x1D6C7D,
0x1E6B7B,
0x1F6B7B,
0x1E6A78,
0x206978,
0x206A77,
0x216975,
0x226874,
0x236772,
0x236571,
0x256571,
0x26646F,
0x28646F,
0x29646C,
0x29646C,
0x2C626C,
0x2C6169,
0x2F6067,
0x305F67,
0x325F65,
0x336066,
0x316165,
0x305F65,
0x2E5F64,
0x2C6167,
0x2B6269,
0x29626B,
0x29646C,
0x28646E,
0x26656E,
0x26656E,
0x25656F,
0x236571,
0x236772,
0x226874,
0x216977,
0x206978,
0x1D6B77,
0x1E6C78,
0x1E6C79,
0x1D6C7B,
0x1D6C7B,
0x1C6D7E,
0x1D6E81,
0x1C6F81,
0x1B7084,
0x1B7084,
0x1B7285,
0x1C7386,
0x1C7386,
0x1C7386,
0x1B7486,
0x1C7587,
0x1B7784,
0x1A7789,
0x197790,
0x197896,
0x1C7698,
0x1F7694,
0x247590,
0x26778C,
0x227889,
0x1E7B8A,
0x197E90,
0x16839A,
0x1786A2,
0x1986A7,
0x1C84A7,
0x1F82A9,
0x1E8C9B,
0x228BA0,
0x278BAD,
0x2384A4,
0x468982,
0x95A159,
0xD1B43C,
0xCBBD50,
0x85AB78,
0x459699,
0x318C9F,
0x3B91A0,
0x3693A2,
0x2E97AD,
0x339CB2,
0x3A97A9,
0x4097AB,
0x4198AC,
0x439AAE,
0x459CAF,
0x469BAF,
0x469BAF,
0x4A9CB1,
0x4C9FB1,
0x4E9FB2,
0x4E9FB2,
0x4F9EAF,
0x4F9EAF,
0x509DAF,
0x519EB0,
0x529FB1,
0x53A0B2,
0x5BA1A1,
0xA0AA76,
0xB08212,
0xC6950A,
0xD2CC54,
0x8AB870,
0x59A49D,
0x54A7D1,
0x55A2CE,
0x70A498,
0xA2B462,
0xA69B2F,
0x716010,
0x665827,
0x625721,
0x6E5E11,
0xA39F22,
0xD7D431,
0xDED715,
0xE3D81A,
0xE8D928,
0xE5D625,
0xF2E932,
0xE5E535,
0xDCE13C,
0xDBDB2B,
0xCEC90B,
0xC9CC25,
0xA9C266,
0x6EA296,
0x60A6C0,
0x62AEC8,
0x58AEC7,
0x64A9B8,
0x6BA3A2,
0x78AFA9,
0x5EA2A3,
0x59A8B5,
0x56A7BC,
0x5AA5BC,
0x59A4B9,
0x54A5BA,
0x4FA6BA,
0x4FA4B9,
0x55A0B3,
0x599EAD,
0x559FAA,
0x50A0A9,
0x49A0A9,
0x489CB8,
0x4A99BA,
0x4C98A6,
0x48999C,
0x3998AA,
0x2C98BF,
0x3297B3,
0x4B9282,
0x738A52,
0xA18E3F,
0xB28627,
0xB28A1B,
0x958D2A,
0x668E59,
0x408B84,
0x348893,
0x3C8481,
0x43806E,
0x518873,
0x49887F,
0x33818D,
0x25819A,
0x2988A6,
0x2582A1,
0x26819E,
0x23819D,
0x1E8299,
0x1A8494,
0x1C8290,
0x247D8D,
0x28798C,
0x1D7C90,
0x1C7B8F,
0x1B7A8E,
0x1A798D,
0x1A798D,
0x1C788D,
0x1B778C,
0x1A768B,
0x1B7689,
0x1A7588,
0x1B7387,
0x1C7386,
0x1C7386,
0x1B7285,
0x1B7182,
0x1B7182,
0x1A7081,
0x1A7081,
0x1B6E7E,
0x1B6C7D,
0x1D6C7D,
0x1E6B7B,
0x1E6A7A,
0x1E6A78,
0x206A77,
0x206A77,
0x206874,
0x226874,
0x226671,
0x256571,
0x266471,
0x26646F,
0x28646C,
0x29646C,
0x2A636C,
0x2C6169,
0x2E6069,
0x2F6067,
0x305F65,
0x325F65,
0x316165,
0x2F6065,
0x2E6067,
0x2C6167,
0x2A636A,
0x29646C,
0x28646E,
0x26656E,
0x25646D,
0x25656E,
0x23666F,
0x236772,
0x226874,
0x216977,
0x206A77,
0x206978,
0x1D6B77,
0x1E6C79,
0x1E6C79,
0x1D6C7B,
0x1E6D7C,
0x1C6D7E,
0x1C6F81,
0x1D7082,
0x1B7084,
0x1B7084,
0x1B7285,
0x1C7386,
0x1C7386,
0x1B7285,
0x1B7486,
0x1C7587,
0x1B7782,
0x1B7688,
0x1B778E,
0x1C7794,
0x207793,
0x22768E,
0x247789,
0x237988,
0x227F90,
0x197D94,
0x177F9C,
0x1E83A1,
0x278297,
0x337E84,
0x407C72,
0x4D7E69,
0x30817A,
0x228491,
0x258EAC,
0x328897,
0x668E6C,
0xB5A84A,
0xDDB12C,
0xD9AE2C,
0xC3BA5F,
0x779D78,
0x408D93,
0x3995AE,
0x3C97AC,
0x3D93A2,
0x3E95A6,
0x3E9AAF,
0x4299AD,
0x439AAE,
0x449BAE,
0x479CB0,
0x489DB1,
0x4B9DB2,
0x4C9EB3,
0x4FA0B3,
0x50A1B4,
0x54A2B6,
0x54A3B4,
0x53A0B2,
0x529FB1,
0x54A1B3,
0x55A2B4,
0x56A1B4,
0x66A495,
0x9BAC8A,
0x947B3B,
0x8E6F14,
0xA99E35,
0xBDC669,
0x94AD73,
0x739E8B,
0x6FA597,
0x86B179,
0xA6B347,
0x978E19,
0x857A2A,
0x878755,
0x88864B,
0x837922,
0x706611,
0x9B8F21,
0xD5C639,
0xE9D735,
0xE7D52B,
0xE5D327,
0xE2D227,
0xE5DC37,
0xDDD83E,
0xD2CF36,
0xD6CF38,
0xCCC537,
0xC3C65F,
0x95B282,
0x6BACB0,
0x53ABCF,
0x5AB2D8,
0x6DA29C,
0x9AAC72,
0xB7C681,
0x7EAB84,
0x63AEB3,
0x50A6C1,
0x5EA9C0,
0x61A3AF,
0x5DA6AD,
0x55A6B7,
0x54A4BB,
0x59A2B5,
0x59A1B0,
0x52A1B0,
0x48A4B3,
0x4BA1B2,
0x4A9EB6,
0x4B9DB3,
0x4C9BA8,
0x489BA3,
0x4099AB,
0x3996B5,
0x3B95AF,
0x4B928E,
0x61855F,
0x7C7934,
0x94751B,
0x8F7111,
0x6A6514,
0x597C52,
0x4B8880,
0x64946A,
0x96B673,
0x9EA94B,
0x959B39,
0x7E944C,
0x629779,
0x33868C,
0x22869D,
0x2187A0,
0x25859D,
0x248399,
0x208396,
0x1D8495,
0x1D8294,
0x227E93,
0x267A92,
0x1E7D91,
0x1D7C90,
0x1C7B8F,
0x1B7A8E,
0x1A798D,
0x1C788D,
0x1B778C,
0x1B778C,
0x1B7689,
0x1A7588,
0x1C7488,
0x1C7587,
0x1C7386,
0x1C7386,
0x1C7185,
0x1B7182,
0x1A7182,
0x1A7081,
0x1B6E80,
0x1C6D7E,
0x1E6D7E,
0x1D6C7B,
0x1E6B7B,
0x1D6B78,
0x1E6A77,
0x206A77,
0x1F6974,
0x226874,
0x236772,
0x236571,
0x256571,
0x256571,
0x28646E,
0x29646C,
0x29626B,
0x2B6269,
0x2E6069,
0x2F6067,
0x305F67,
0x305F65,
0x306165,
0x2F6065,
0x2E6067,
0x2C636A,
0x2B646B,
0x29646C,
0x28646E,
0x26656E,
0x24646D,
0x23666F,
0x23666F,
0x236772,
0x206874,
0x206A77,
0x206A77,
0x1E6A78,
0x1D6B77,
0x1E6C78,
0x1E6C79,
0x1D6C7B,
0x1E6D7C,
0x1C6D7E,
0x1C6F81,
0x1D6F84,
0x1B7084,
0x1C7185,
0x1C7386,
0x1D7487,
0x1D7487,
0x1C7386,
0x1B7486,
0x1C7587,
0x1D7782,
0x1D7686,
0x1C778C,
0x1D788D,
0x20798B,
0x217985,
0x217B83,
0x1D7E87,
0x198093,
0x137E9E,
0x1981A8,
0x2481A2,
0x357B83,
0x4C775C,
0x708042,
0x8F913E,
0x699B82,
0x338A91,
0x238DA7,
0x3B8584,
0x72854E,
0xA39334,
0xB2931F,
0xBD9116,
0xB88E1E,
0x8C8946,
0x508D88,
0x3195B5,
0x3B96B1,
0x4F929B,
0x4E94A0,
0x3F9AB7,
0x449BAF,
0x449BAE,
0x469BAF,
0x489DB1,
0x4A9FB3,
0x4C9EB3,
0x4EA1B3,
0x50A1B4,
0x50A1B4,
0x55A3B7,
0x56A5B6,
0x55A2B4,
0x54A1B3,
0x56A3B5,
0x58A3B6,
0x57A3B3,
0x62A594,
0x6B8B7E,
0x585743,
0x5A4E24,
0x74681C,
0x988922,
0xBCAA44,
0xAEB25E,
0x90AE64,
0x91B659,
0x92A12C,
0x9B961E,
0x93932F,
0x879341,
0x8C943E,
0x94912A,
0x827443,
0x685624,
0x8F772D,
0xD6C046,
0xE9D63A,
0xDCCC2F,
0xE1D042,
0xEAD546,
0xECD841,
0xE6DA48,
0xBFB53C,
0x978A21,
0x988F2A,
0xA3B466,
0x73B0A0,
0x51B2DC,
0x58ACDA,
0x7FA68B,
0xC9BE54,
0xC1B336,
0x8DA75D,
0x67AEAA,
0x54AACB,
0x5CA8C0,
0x66A6A8,
0x65A7A6,
0x5AA8B4,
0x56A6BD,
0x59A4BB,
0x5AA3B6,
0x4EA4BB,
0x42A7C3,
0x4CA2B9,
0x4CA2B1,
0x4AA1AB,
0x48A0AE,
0x489AAF,
0x4A97AB,
0x4B94A7,
0x4793AA,
0x4793A1,
0x598C7B,
0x77803F,
0x8E730C,
0x916F0B,
0x766D2C,
0x6B856A,
0x588C88,
0x8C9F43,
0xB1AF34,
0xD5BB1E,
0xE8C626,
0xA99E22,
0x809E5E,
0x499686,
0x208895,
0x1D8A9E,
0x228799,
0x248595,
0x218595,
0x1E8598,
0x1B849A,
0x1D7F9A,
0x227D9A,
0x1E7D91,
0x1D7C90,
0x1C7B8F,
0x1B7A8E,
0x1B7A8E,
0x1C788D,
0x1C788D,
0x1B778C,
0x1B7689,
0x1B7689,
0x1C7488,
0x1C7587,
0x1D7487,
0x1C7386,
0x1C7185,
0x1C7283,
0x1A7182,
0x1B7182,
0x1A7081,
0x1B6E7E,
0x1C6D7E,
0x1E6D7C,
0x1E6B7B,
0x1D6B78,
0x1E6A78,
0x1E6A77,
0x206A75,
0x206874,
0x226872,
0x246672,
0x266672,
0x266672,
0x28646E,
0x29636E,
0x29626B,
0x2B616B,
0x2E6069,
0x2F6067,
0x305F67,
0x305F65,
0x2F6065,
0x2E5F66,
0x2E6069,
0x2C626C,
0x2A636E,
0x29636F,
0x28646F,
0x26646F,
0x256571,
0x236571,
0x236772,
0x236975,
0x216975,
0x206A77,
0x1E6A78,
0x1E6A78,
0x1C6B7C,
0x1D6C7D,
0x1C6D7E,
0x1C6D7E,
0x1B6E80,
0x1C6F81,
0x1B7182,
0x1B7084,
0x1B7285,
0x1C7386,
0x1D7487,
0x1C7587,
0x1C7488,
0x1C7488,
0x1C7488,
0x1C7488,
0x247288,
0x18788E,
0x127B8E,
0x1D788A,
0x257689,
0x1B7B93,
0x0F8297,
0x138192,
0x1F7C8D,
0x1F7D97,
0x1582A3,
0x228598,
0x477960,
0x888735,
0xCFC13E,
0xCCC72D,
0xD9BE19,
0x8EA85F,
0x3C8896,
0x3C879C,
0x658263,
0x8A7A2F,
0x9A7821,
0x947E29,
0x73772B,
0x54794E,
0x4B8F98,
0x3895B7,
0x359AAE,
0x3E9E9D,
0x449BA5,
0x4498BC,
0x429CA4,
0x439FAC,
0x45A1B0,
0x49A1AF,
0x4AA3AB,
0x4BA6A7,
0x4CA7AA,
0x52A4B2,
0x5CA0B7,
0x60A1B7,
0x5CA4B0,
0x56A7AB,
0x53A6AE,
0x59A5B3,
0x5CA4B2,
0x60A6B0,
0x59A7BE,
0x59ACBC,
0x59B1BD,
0x57AEBF,
0x5DA2B2,
0x648A7F,
0x838448,
0xA7962C,
0xBBAD33,
0x9FA038,
0x848D3A,
0x7E8134,
0x82771E,
0x817014,
0x756A1A,
0x5F5C19,
0x5E5326,
0x615525,
0x695C18,
0xA39731,
0xC6B937,
0xC5B22F,
0xD9C34B,
0xE0CB60,
0xD0BC66,
0xAD9362,
0x7E593F,
0x80552A,
0x7F6614,
0x7F8E3B,
0x76B49B,
0x55B2DD,
0x69AFB9,
0x93AE61,
0xD5C31B,
0xD0B100,
0xB2AB40,
0x8CAB99,
0x6BA4C1,
0x66A8C8,
0x64A8BB,
0x60A8B6,
0x5FA8B9,
0x5AA9BA,
0x55A8B6,
0x55A7B3,
0x59A5B5,
0x5CA0B5,
0x55A2AC,
0x4FA1B7,
0x4C9FBD,
0x519BB6,
0x5494AD,
0x4C96B3,
0x3D9ABB,
0x3797AF,
0x4C9793,
0x658B64,
0x7E813C,
0x97812C,
0x9D7E24,
0x826D1E,
0x6F7036,
0x738861,
0xA39238,
0xA69C3B,
0xA89E2D,
0xC2B02A,
0xC4B029,
0xA3A343,
0x6B8E6D,
0x3C7F8F,
0x3087A2,
0x268692,
0x238787,
0x25888E,
0x278297,
0x277D98,
0x287E97,
0x237E90,
0x1F7E94,
0x1E7D93,
0x1D7C92,
0x1C7B91,
0x1B7A90,
0x1A798D,
0x19788C,
0x18778B,
0x1B778C,
0x1B788A,
0x1B7689,
0x1C7488,
0x1C7587,
0x1C7386,
0x1B7285,
0x1A7184,
0x1A7182,
0x1B7182,
0x1D7082,
0x1C6F81,
0x1C6D80,
0x1E6D7E,
0x1E6B7D,
0x1E6B7B,
0x1D6B78,
0x1E6A78,
0x206A77,
0x216975,
0x226874,
0x236772,
0x246672,
0x246672,
0x26656E,
0x25646D,
0x27636B,
0x29646C,
0x2A636A,
0x2C6167,
0x2E6067,
0x306166,
0x2F6065,
0x2F6067,
0x2E6069,
0x2C626C,
0x2A636E,
0x29636F,
0x28646F,
0x26646F,
0x256571,
0x236571,
0x236772,
0x236975,
0x216975,
0x206A77,
0x1E6A78,
0x1F6B79,
0x1D6C7D,
0x1D6C7D,
0x1C6D7E,
0x1C6D7E,
0x1C6F81,
0x1C6F81,
0x1B7182,
0x1C7185,
0x1B7285,
0x1C7386,
0x1C7587,
0x1C7488,
0x1C7488,
0x1C7488,
0x1C7488,
0x1D7589,
0x1E748B,
0x18798C,
0x177C88,
0x207B80,
0x257984,
0x1F7A95,
0x167D9E,
0x197D95,
0x227E8B,
0x227F90,
0x228197,
0x327C7D,
0x6F8C56,
0xBDB143,
0xCEB72D,
0xCDBC30,
0xCFAC10,
0x93A04E,
0x4D8C81,
0x408789,
0x618664,
0x7B7632,
0x836819,
0x856B20,
0x6D6729,
0x5A7758,
0x51939F,
0x3F98BA,
0x3C98B1,
0x4299A2,
0x439BA7,
0x42A0BC,
0x499FB0,
0x4A9FB4,
0x4D9FB7,
0x4E9EB5,
0x4FA0B1,
0x50A1B2,
0x51A3B8,
0x50A4BE,
0x53A4BF,
0x58A6BD,
0x5AA7BB,
0x58A5B7,
0x57A6B7,
0x59A6B8,
0x60A3B6,
0x68A2B6,
0x60A7BB,
0x63A7B4,
0x64A7B0,
0x62A9BB,
0x69AFC8,
0x75AEB7,
0x72906E,
0x898A3A,
0xADA037,
0x978D2B,
0x7A7B2B,
0x6E7330,
0x747436,
0x7D773D,
0x837948,
0x776B45,
0x5C5020,
0x53451E,
0x5D5023,
0x736620,
0x958627,
0xA89634,
0x9A842F,
0x887528,
0x73651A,
0x675A16,
0x6A571C,
0x8A762D,
0xA79D3B,
0xA1B049,
0x90B774,
0x7AB096,
0x96B567,
0xBBC059,
0xC6AB2A,
0xBF9B21,
0xBBAF63,
0x93B6A0,
0x62A7B7,
0x62B0C7,
0x62AAB9,
0x63A9B5,
0x5DAABA,
0x59ACBE,
0x5BA9B6,
0x60A5B4,
0x60A4B9,
0x59A3BE,
0x5BA5AE,
0x54A7B7,
0x4EA4BB,
0x4D9DB4,
0x5199AF,
0x4C9AB1,
0x419DB6,
0x3FA0B0,
0x4B968F,
0x73936C,
0x9D904A,
0xA98227,
0xAA8626,
0x9A8F3D,
0x7A9052,
0x5C8B5D,
0x9C8E2B,
0xA59F3D,
0xA29930,
0xA38D1D,
0xA28C26,
0x7D8D4F,
0x448B87,
0x2189AE,
0x1E88B0,
0x26859B,
0x2E8A95,
0x238494,
0x1F839D,
0x2382A0,
0x237F96,
0x24808F,
0x207F95,
0x1F7E94,
0x1E7D93,
0x1D7C92,
0x1C7B91,
0x1B7A90,
0x1A798D,
0x19788C,
0x1B778C,
0x1B788A,
0x1B7689,
0x1C7488,
0x1C7587,
0x1C7386,
0x1C7386,
0x1B7285,
0x1A7182,
0x1B7182,
0x1B7182,
0x1C6F81,
0x1C6D80,
0x1E6D7E,
0x1E6B7D,
0x1E6B7B,
0x1D6A7A,
0x1E6A78,
0x206A77,
0x216975,
0x226874,
0x236772,
0x246672,
0x246672,
0x26656E,
0x25646D,
0x27636B,
0x28636B,
0x296269,
0x2A6166,
0x2E6067,
0x2F6065,
0x2F6067,
0x2E6067,
0x2C6169,
0x2C626C,
0x2A636E,
0x28646F,
0x26646F,
0x256571,
0x256571,
0x246672,
0x236772,
0x226874,
0x216975,
0x206A77,
0x1E6A78,
0x1E6C79,
0x1D6C7D,
0x1C6D7E,
0x1C6D7E,
0x1D6E7F,
0x1C6F81,
0x1D7082,
0x1B7084,
0x1C7185,
0x1C7386,
0x1C7386,
0x1C7587,
0x1D7589,
0x1D7589,
0x1D7589,
0x1D7589,
0x1D7589,
0x1A768D,
0x19788E,
0x1C7A86,
0x227C7D,
0x247B82,
0x237A96,
0x2079A3,
0x237A96,
0x267E8A,
0x1F7E90,
0x217F9B,
0x397D7E,
0x798F4E,
0xAC9A1E,
0xBFA11D,
0xC2A638,
0xC59D1A,
0x929441,
0x568769,
0x4E8C7F,
0x5F8765,
0x828445,
0x9E8533,
0x9D7E2C,
0x8D7F42,
0x6D8468,
0x509099,
0x3D92AF,
0x4398B7,
0x4C9DB2,
0x479DAE,
0x409FB1,
0x4B9DB2,
0x4B9DB5,
0x4E9EB7,
0x539FB6,
0x58A0B6,
0x5AA0BA,
0x58A3C2,
0x54A7C5,
0x52AAC0,
0x54A9BE,
0x59A5BD,
0x5AA2BB,
0x5AA5BA,
0x5CA7BA,
0x61A3BB,
0x69A2C0,
0x67A8BA,
0x70AAB5,
0x6FA6AD,
0x69A7BC,
0x5FA9CC,
0x63AFC7,
0x7AB0A0,
0x8DA25F,
0xACA43F,
0x988A1F,
0x757015,
0x67692A,
0x686A43,
0x605A40,
0x352410,
0x7A6552,
0x756B3A,
0x4D401D,
0x534721,
0x5C5014,
0x7B6F1B,
0x7E6F1E,
0x544409,
0x514418,
0x514918,
0x655F21,
0x726E25,
0x83812C,
0x868B25,
0x98A12E,
0xB4BE4E,
0xB9C15A,
0xC1C23A,
0xB0A941,
0x8F803F,
0x78693E,
0x778061,
0x7EAE9E,
0x6BBABE,
0x5BADBB,
0x69AEBD,
0x67ABB4,
0x5FAFB8,
0x5BAEBE,
0x60A9BA,
0x67A4B6,
0x63A6B9,
0x57A7BE,
0x5DA6AD,
0x58A8B3,
0x4DA3B2,
0x4F9EAD,
0x549DAC,
0x4E9BAB,
0x409BAE,
0x3E9EAC,
0x529991,
0x70895F,
0xA48A3D,
0xB98726,
0xB48E2B,
0x8B8936,
0x5E8242,
0x589461,
0x8C831E,
0x898323,
0x95872E,
0x917A2A,
0x756728,
0x587556,
0x32878C,
0x158AAB,
0x1F8FB7,
0x2885A4,
0x2C8498,
0x25889D,
0x1D87A1,
0x19819E,
0x24879E,
0x1F7E90,
0x208096,
0x1F7F95,
0x1D7D93,
0x1C7C92,
0x1B7B91,
0x1B7A90,
0x1A798D,
0x1A798D,
0x1C788D,
0x1B778C,
0x1B7689,
0x1B7689,
0x1C7488,
0x1C7587,
0x1C7386,
0x1B7285,
0x1A7182,
0x1B7182,
0x1B7182,
0x1C6F81,
0x1C6D80,
0x1E6D7E,
0x1D6C7D,
0x1E6B7B,
0x1E6B7B,
0x1F6B79,
0x1E6A77,
0x206A77,
0x236975,
0x236772,
0x246672,
0x236571,
0x27666F,
0x26656E,
0x28646E,
0x28636B,
0x296269,
0x2B6269,
0x2C6167,
0x2F6168,
0x306168,
0x2E6067,
0x2C6169,
0x2C626C,
0x29636E,
0x28646F,
0x26646F,
0x256571,
0x236571,
0x236772,
0x236772,
0x226874,
0x216977,
0x1E6A77,
0x1E6A78,
0x1E6C79,
0x1B6C7D,
0x1C6D7E,
0x1C6D7E,
0x1D6E81,
0x1C6F81,
0x1D7082,
0x1C7185,
0x1C7386,
0x1C7386,
0x1C7386,
0x1C7488,
0x1D7589,
0x1D7589,
0x1D7589,
0x1D7589,
0x1B7689,
0x1C778A,
0x197791,
0x1B7992,
0x217A88,
0x237B87,
0x237B93,
0x247A9B,
0x287A92,
0x2A7D8F,
0x197A9A,
0x1A86B5,
0x27829D,
0x5D8B67,
0x89852A,
0xA38724,
0xA68737,
0xA98622,
0x797633,
0x507355,
0x5B8B7F,
0x5E7D5E,
0x787937,
0x9A821E,
0xA38116,
0x8C7A26,
0x6D8052,
0x54918A,
0x469BAF,
0x439AB8,
0x4797B2,
0x4A9BAE,
0x4BA2AC,
0x4C9DAE,
0x489DB1,
0x4A9FB3,
0x56A2B0,
0x60A2B0,
0x60A1B5,
0x5CA4BC,
0x5AA7B9,
0x5BA9B3,
0x5DA7B2,
0x5EA5B7,
0x60A4B9,
0x66A8B8,
0x65A7B3,
0x65A9BE,
0x61AACA,
0x65A8BB,
0x6AAAB6,
0x6FADBA,
0x6AADC7,
0x62AECE,
0x68B0BF,
0x79AC8D,
0x8EA44F,
0xB8B83C,
0xCAC33E,
0xA19E29,
0x706B19,
0x847645,
0x654E2E,
0x331B00,
0x7C633B,
0x93905B,
0x46410A,
0x766D2A,
0x9C9231,
0xB2A934,
0xA69F35,
0x5A560F,
0x393510,
0x504831,
0x55472D,
0x665835,
0x847D49,
0x818437,
0x90932A,
0xBAB537,
0xCFBF36,
0x9EA94D,
0x8B9D6B,
0x8CA6A3,
0x81A5B3,
0x79AAB1,
0x6FB1AF,
0x64AFB4,
0x6AB4C1,
0x6CB0BD,
0x6AAFB6,
0x62B0B4,
0x5FAEBB,
0x61A8BE,
0x65A6BC,
0x61A9B8,
0x57ACB3,
0x5DA6AF,
0x56A5B2,
0x53A2B1,
0x5A9EA9,
0x5E9DA2,
0x559DAB,
0x479FB7,
0x449FB4,
0x57968D,
0x7C8D56,
0xA18A2D,
0xAB8219,
0xA98827,
0x8C8334,
0x607333,
0x6A9057,
0xA59A41,
0x92822D,
0x8D772E,
0x6D5D22,
0x525121,
0x6E8557,
0x669970,
0x529C83,
0x2E8688,
0x2D899E,
0x2787A0,
0x23879E,
0x248AA0,
0x1A8399,
0x2286A0,
0x1E809B,
0x208096,
0x1F7F95,
0x1E7E94,
0x1D7D93,
0x1C7C92,
0x1B7B91,
0x1B7A90,
0x1A798D,
0x1A798D,
0x1B778C,
0x1C778A,
0x1B7689,
0x1D7589,
0x1C7587,
0x1D7487,
0x1C7386,
0x1B7285,
0x1A7182,
0x1B7182,
0x1C6F81,
0x1D6E81,
0x1C6D7E,
0x1D6C7D,
0x1D6C7B,
0x1F6C7C,
0x1E6C79,
0x1E6A77,
0x206A77,
0x216975,
0x236975,
0x236772,
0x236571,
0x266670,
0x26656E,
0x26656E,
0x28646C,
0x2A636C,
0x2B6269,
0x2C6167,
0x2F6168,
0x2F6168,
0x2D6268,
0x2B6269,
0x2A636C,
0x29636E,
0x26646F,
0x256571,
0x256571,
0x236571,
0x236772,
0x226874,
0x216975,
0x206A77,
0x1E6A78,
0x1E6C79,
0x1E6B7B,
0x1B6C7D,
0x1C6D7E,
0x1C6D7E,
0x1C6F81,
0x1C6F81,
0x1B7182,
0x1C7185,
0x1C7386,
0x1C7386,
0x1B7387,
0x1C7488,
0x1D7589,
0x1D7589,
0x1C778A,
0x1C778A,
0x1C778A,
0x237489,
0x1C7792,
0x177A99,
0x1B7A96,
0x207B8E,
0x227D8F,
0x227F90,
0x257E90,
0x257D93,
0x2384A5,
0x1885AC,
0x19819C,
0x3A7E7D,
0x687E5A,
0x7E7238,
0x886D2A,
0x826718,
0x726C2E,
0x708461,
0x60816E,
0x647E61,
0x818643,
0xB09B28,
0xC6A21A,
0xC6AD37,
0x8D9853,
0x5C937E,
0x489FA8,
0x429EB3,
0x479CB1,
0x539EB1,
0x549DB0,
0x509FAE,
0x48A1B1,
0x49A2B2,
0x56A3AD,
0x62A1AA,
0x61A1AD,
0x5AA3B2,
0x5CA4B0,
0x65A4A9,
0x68A7AE,
0x60A8B4,
0x60A6B2,
0x6DA4A7,
0x72A0A0,
0x71ACB4,
0x5EAAC1,
0x69ACBF,
0x65AAB9,
0x69B1BF,
0x63AABE,
0x72B1C2,
0x80AEA1,
0x829B59,
0xAFB642,
0xBEC02E,
0xBDC027,
0xC9CC3D,
0xBFB63F,
0xB3993C,
0xA58133,
0x917426,
0x97852F,
0x929548,
0x929135,
0xC6C04E,
0xD2C93E,
0xCEC832,
0xD1D24A,
0xA3A63D,
0x4E4D0C,
0x66573A,
0x40211F,
0x3F1C20,
0xC1AC9B,
0x94915A,
0x818526,
0xB0AB33,
0xC6B833,
0x77A4A1,
0x7CB6C4,
0x71B8D6,
0x69B6D2,
0x73BCCB,
0x77BAC3,
0x73B5C3,
0x6DB0C3,
0x6BB0BF,
0x67AEB4,
0x65ACB2,
0x64A9B8,
0x62A5BF,
0x61A7C0,
0x5DACB9,
0x56B0B1,
0x56A8BD,
0x50A4C0,
0x55A2BE,
0x5C9EAE,
0x5C98A0,
0x529BAA,
0x49A0BE,
0x4E9CB2,
0x67947D,
0x8A9143,
0xA89626,
0xB9A230,
0xBDA646,
0xA99745,
0x81782D,
0x908E44,
0xAA9A43,
0xA88E41,
0x907737,
0x675E25,
0x767D39,
0xA9B053,
0xBDBC50,
0xC5CF6F,
0x82AE79,
0x388A84,
0x2389A0,
0x2489A7,
0x26869E,
0x26879A,
0x22849D,
0x2184A4,
0x218197,
0x208096,
0x1F7F95,
0x1E7E94,
0x1D7D93,
0x1C7C92,
0x1B7B91,
0x1B7A90,
0x1A798D,
0x1B778C,
0x1B778C,
0x1B7689,
0x1B7689,
0x1C7488,
0x1C7587,
0x1B7486,
0x1B7285,
0x1B7285,
0x1B7182,
0x1D7082,
0x1C6F81,
0x1C6D7E,
0x1E6D7E,
0x1E6D7C,
0x1E6D7C,
0x1E6C79,
0x1E6A78,
0x206A77,
0x216975,
0x236975,
0x236774,
0x226671,
0x25656F,
0x24646E,
0x25646D,
0x28646E,
0x29646C,
0x296269,
0x2B6269,
0x2D6268,
0x2D6268,
0x2D626A,
0x2C636A,
0x29646C,
0x28646E,
0x26646F,
0x256571,
0x236571,
0x236772,
0x236772,
0x236975,
0x216975,
0x206A77,
0x1E6A78,
0x1E6C79,
0x1D6C7B,
0x1B6C7D,
0x1C6D7E,
0x1D6E7F,
0x1C6F81,
0x1D7082,
0x1B7182,
0x1C7185,
0x1C7386,
0x1D7487,
0x1C7488,
0x1C7488,
0x1C7488,
0x1B7689,
0x1C778A,
0x1C778A,
0x1C778A,
0x287388,
0x1D788D,
0x147C95,
0x157D9A,
0x1E7C96,
0x247D8B,
0x20818A,
0x1D8392,
0x208194,
0x287C7E,
0x39816B,
0x4B8E6D,
0x4E8975,
0x517664,
0x646A3E,
0x847027,
0x8C7B2B,
0x8D8031,
0x767733,
0x7A905F,
0x5A7A53,
0x728649,
0xB4A73E,
0xD7AE20,
0xD1AB24,
0x9A9A44,
0x619177,
0x4EA1A7,
0x46A2AD,
0x4A9EA9,
0x539EB1,
0x5499B8,
0x539FB6,
0x4CA0B8,
0x4CA1B6,
0x579FAE,
0x609EAB,
0x5CA1B1,
0x58A5B9,
0x5DA4B8,
0x69A4B2,
0x60A2B0,
0x5AACB8,
0x67B4BA,
0x68928E,
0x586A5E,
0x718A84,
0x7CACB0,
0x79ADBB,
0x7BB0B8,
0x7BB0B6,
0x81B2B9,
0x82A7A0,
0x697B51,
0x94943A,
0xB9AF33,
0xB7AF26,
0xB0B127,
0xB4B72A,
0xC3BE2E,
0xD7C43A,
0xDBC146,
0xCFBB4A,
0xAEA337,
0xA7A93B,
0xB8B63B,
0xC2BA31,
0xC9BE2E,
0xCFC639,
0xD3D14A,
0xD6D557,
0xB1A93E,
0x887125,
0x714E26,
0x593421,
0xA79372,
0x9C9954,
0x848922,
0xB9BB3E,
0xC6C141,
0x7EB8D0,
0x6EB5D3,
0x6AB7D3,
0x7EBECA,
0x5D898A,
0x527B7F,
0x7BB2C6,
0x6DB3CC,
0x67B3C3,
0x66AFB6,
0x67ABB6,
0x67A8BC,
0x67A5BE,
0x65A7BF,
0x5EABBF,
0x54ADBF,
0x4FA9C3,
0x50A5C2,
0x54A3C1,
0x57A3BB,
0x54A2B6,
0x50A5BA,
0x51A2B3,
0x5F9891,
0x7B8F5A,
0x9B9330,
0xB19D20,
0xB6A229,
0xAA972E,
0xA79235,
0xA0872A,
0xA88F2B,
0xA6902B,
0x9D8638,
0x786626,
0x5E5B0E,
0xA8AE42,
0xC5C33C,
0xB49F12,
0xB9A425,
0xBFC86D,
0x629A75,
0x2B8698,
0x238AB3,
0x2684A6,
0x2C8597,
0x2A8597,
0x21839E,
0x228298,
0x218197,
0x208096,
0x1E7E94,
0x1D7D93,
0x1C7C92,
0x1B7B91,
0x1B7B91,
0x1B7A90,
0x1A798D,
0x1B778C,
0x1B778C,
0x1B7689,
0x1B7689,
0x1C7488,
0x1B7387,
0x1A7385,
0x1B7285,
0x1B7283,
0x1B7182,
0x1C6F81,
0x1D6E81,
0x1C6D7E,
0x1E6D7E,
0x1E6D7C,
0x1D6C79,
0x1E6C79,
0x1E6A77,
0x206A77,
0x216975,
0x226874,
0x236774,
0x23666F,
0x24646E,
0x25656F,
0x26656E,
0x29636E,
0x2A636C,
0x2C636A,
0x2D626A,
0x2D6268,
0x2C636A,
0x2A636C,
0x29636E,
0x26646F,
0x256571,
0x236571,
0x236772,
0x236774,
0x236975,
0x216975,
0x206A77,
0x1E6A77,
0x1D6B78,
0x1D6C7B,
0x1E6D7C,
0x1C6D7E,
0x1C6D7E,
0x1C6F81,
0x1D7082,
0x1B7182,
0x1C7185,
0x1C7386,
0x1D7487,
0x1C7488,
0x1C7488,
0x1B7387,
0x1A7588,
0x1B7689,
0x1C778A,
0x1D788B,
0x1D788B,
0x24758A,
0x1E7A89,
0x177E8F,
0x157F97,
0x1B7D96,
0x237E91,
0x218092,
0x1C8394,
0x258083,
0x5D946A,
0xB5C666,
0xC5CC5A,
0xA8BD62,
0x779559,
0x546227,
0x766E25,
0x82721A,
0x957E16,
0xBBAA3E,
0x9AA24C,
0x608349,
0x607F46,
0x999439,
0xBE9612,
0xC89A11,
0xA29839,
0x688D6C,
0x50999F,
0x479FAD,
0x489FA9,
0x509EB2,
0x539DBA,
0x569CB6,
0x549EBB,
0x549EB7,
0x59A1B0,
0x5BA3B1,
0x59A6BA,
0x59A6C4,
0x5FA4C3,
0x64A6BE,
0x61AABB,
0x5BADBB,
0x69ACB4,
0x6A8782,
0x635E4B,
0x837866,
0x807E72,
0x849698,
0x81908B,
0x7D8880,
0x828C84,
0x6B745F,
0x60652F,
0xA4A144,
0xAB9D30,
0xA99830,
0xAA9E3C,
0xA7A337,
0xACAC2A,
0xA9A51D,
0xB0A82C,
0x857D18,
0x86802A,
0x9C982A,
0x9A911C,
0xB2A228,
0xBEAB33,
0xC2B340,
0xCEC34F,
0xD3C84A,
0xD7C740,
0xC9B435,
0xBBA538,
0xA08F35,
0x90872C,
0x989828,
0xC5C742,
0xD4D645,
0xCECC3B,
0x6F9985,
0x81B3B2,
0x87B3BC,
0x6D7D7A,
0x5B5441,
0x54564B,
0x7399A6,
0x73B9D2,
0x66B3C3,
0x69B0B6,
0x6AACB8,
0x6CAAC1,
0x6DA6C3,
0x6DA7BD,
0x65A8BB,
0x5BA8BC,
0x5AA5A1,
0x66A79F,
0x63A4A0,
0x51A3B1,
0x46A8C3,
0x4DA6B6,
0x5F9D88,
0x7A965B,
0x93933B,
0xA7962C,
0xB99C27,
0xBB9A26,
0xA89022,
0x9B8D24,
0xA59229,
0xAB9122,
0xA1861B,
0x917E2F,
0x665D1A,
0x7B771C,
0xA7AA29,
0xB5B726,
0xCDC540,
0xA28F19,
0x9A9031,
0x85A170,
0x3E8891,
0x2085B1,
0x288AB1,
0x2E8493,
0x2A828C,
0x238497,
0x228298,
0x218197,
0x208096,
0x1F7F95,
0x1E7E94,
0x1D7D93,
0x1C7C92,
0x1B7B91,
0x1B7A90,
0x1A798D,
0x1C788D,
0x1B778C,
0x1C778A,
0x1B7689,
0x1A7588,
0x1C7488,
0x1B7486,
0x1B7486,
0x1B7285,
0x1B7182,
0x1D7082,
0x1C6F81,
0x1D6E7F,
0x1C6D7E,
0x1D6C7B,
0x1D6C7B,
0x1E6C79,
0x1F6B78,
0x206A77,
0x216975,
0x236975,
0x236975,
0x246672,
0x236571,
0x25656F,
0x276570,
0x29656F,
0x29646C,
0x2A636C,
0x2C636A,
0x2C636A,
0x2C636A,
0x2A636C,
0x28646E,
0x26646F,
0x256571,
0x236571,
0x236774,
0x226874,
0x236975,
0x216975,
0x206A77,
0x1E6A78,
0x1E6C79,
0x1D6C7B,
0x1C6E7C,
0x1B6E7E,
0x1C6F81,
0x1C6F81,
0x1D7082,
0x1B7182,
0x1C7185,
0x1C7386,
0x1D7487,
0x1D7589,
0x1C7488,
0x197487,
0x197487,
0x1B7689,
0x1C778A,
0x1D788B,
0x1E798C,
0x217889,
0x1D798E,
0x197C91,
0x1B7E95,
0x1F7D96,
0x207D9C,
0x207F9F,
0x267F91,
0x478671,
0xA6BC71,
0xC9BE3E,
0xC4B013,
0xC8C32B,
0xBDC74D,
0x717E2E,
0x59602A,
0x9F8B35,
0xC2A53D,
0xCBB13A,
0xAEAA3E,
0x728432,
0x77893F,
0xADA445,
0xCEA82D,
0xD1A623,
0xBDAC42,
0x82975F,
0x5B9690,
0x489DB2,
0x439EBB,
0x4B9DB3,
0x579FAD,
0x599BB3,
0x5A9EB5,
0x59A2B1,
0x57A6AB,
0x56AAAC,
0x5AA9B6,
0x5FA5BF,
0x64A1BD,
0x67A2B2,
0x6BA7AF,
0x75AEB5,
0x90B9BD,
0x82918C,
0x635A49,
0x866E56,
0x8B6B56,
0x89796C,
0x7A6756,
0x7F6A59,
0x67574A,
0x3F3A24,
0x80854F,
0x999D46,
0x999333,
0x998838,
0x978344,
0x928340,
0x86802C,
0x818023,
0x72701D,
0x605C1F,
0x625B2D,
0x5A5416,
0x85742E,
0xA18733,
0xB79937,
0xC2AA3E,
0xC7B343,
0xC6B543,
0xB9A92F,
0xC0B632,
0xBAB62E,
0xC4C43E,
0xD9D651,
0xE1DA4C,
0xE5DC41,
0xD7D230,
0xD6D52F,
0x5A6628,
0x374629,
0x6C706F,
0x695851,
0x7B5940,
0x705A42,
0x778683,
0x7DAEBD,
0x6CAAB5,
0x6EAAAA,
0x72AAAB,
0x70ABBB,
0x6DAAC7,
0x71AABE,
0x76A8A7,
0x76A593,
0x95B671,
0xADC06F,
0xA1B671,
0x71A98E,
0x55A8AC,
0x589F91,
0x7A9552,
0x9F9C31,
0xA89D31,
0xA89839,
0xAF8D33,
0xB68C28,
0xB4972F,
0x9F932F,
0xA69A36,
0xA88F28,
0xA7882E,
0x957F34,
0x7A7029,
0x828127,
0x9A9F2B,
0xA9AF33,
0xA9A736,
0xA89A39,
0x80762E,
0x718965,
0x398289,
0x288DA9,
0x278AA1,
0x2F868F,
0x2E8690,
0x25859B,
0x238399,
0x228298,
0x218197,
0x208096,
0x1F7F95,
0x1E7E94,
0x1D7D93,
0x1C7C92,
0x1B7B91,
0x1B7A90,
0x1A798D,
0x1B778C,
0x1B778C,
0x1B7689,
0x1A7588,
0x1A7588,
0x1B7486,
0x1B7486,
0x1B7285,
0x1B7285,
0x1B7182,
0x1C6F81,
0x1C6F81,
0x1D6E7F,
0x1B6D7B,
0x1D6C7B,
0x1D6C79,
0x1E6C79,
0x1E6A77,
0x206A77,
0x216975,
0x236975,
0x236772,
0x246672,
0x256571,
0x26646F,
0x27636D,
0x28626D,
0x29626B,
0x2C626C,
0x2D646B,
0x2A636A,
0x28636B,
0x28646E,
0x25656F,
0x236571,
0x226673,
0x226874,
0x226874,
0x206874,
0x206A77,
0x1E6A77,
0x1D6B78,
0x1D6C7B,
0x1C6E7C,
0x1C6D7E,
0x1C6F81,
0x1C6F81,
0x1C6F81,
0x1B7182,
0x1C7283,
0x1C7386,
0x1C7386,
0x1C7488,
0x1C7488,
0x1A7588,
0x1A7588,
0x1B7689,
0x1B7689,
0x1B788A,
0x1C798B,
0x1D7A8C,
0x1F7B88,
0x1C7A94,
0x1D7A9B,
0x237B93,
0x217C8F,
0x1C7F9E,
0x1C80A4,
0x2D7986,
0x70926D,
0xA09F4C,
0x998E1B,
0xC3BA2F,
0xCFC92D,
0xB9B629,
0x84892F,
0x45542B,
0x84712D,
0xB29844,
0xAD912D,
0xA29024,
0x998E24,
0xB8AA3F,
0xCCB13E,
0xD6B237,
0xD3B030,
0xC3AE31,
0x9EA241,
0x6A9367,
0x519FAB,
0x409DC6,
0x4A9FBE,
0x5B9FA8,
0x5CA1B1,
0x58A2AF,
0x52A6A8,
0x57AEA8,
0x5BABA8,
0x5C9FA5,
0x74A9B7,
0x94C0CB,
0xB2D6D4,
0xCAE6DA,
0xDEEFE5,
0xD8E3DD,
0x9BA29B,
0x5E5C4F,
0x6C5A46,
0x7F5F4A,
0x87694D,
0x886A50,
0x7E6352,
0x69574D,
0x46412D,
0x757E49,
0x8C963F,
0x898B28,
0x887D2E,
0x7C6B37,
0x6E5B31,
0x615324,
0x5D521C,
0x574D1A,
0x4B4219,
0x433D1B,
0x221E12,
0x55452E,
0x8F7343,
0xA48431,
0xB69A2D,
0xB9A238,
0xB1A044,
0xAB9F4D,
0x9B973E,
0xA5A640,
0x949221,
0xCBBF45,
0xD9C33C,
0xDAC52C,
0xCEC31C,
0xD0D024,
0x999454,
0x2E2506,
0x493D3F,
0x625255,
0x6D5847,
0x6E593E,
0x8C8374,
0xBCCBD0,
0xACCDD2,
0xA4CABF,
0x99BDAF,
0x82AFB4,
0x6AA6C0,
0x6EA8B4,
0x7EA07D,
0x9EA353,
0xC9BF3C,
0xD5BA2F,
0xDBC244,
0xA9B563,
0x6F9969,
0x769959,
0xA2A135,
0xB19E1B,
0xA79A28,
0xA09641,
0xA68E44,
0xBC9641,
0xB5983C,
0xAA9C43,
0xAC9C45,
0xB1963D,
0xA98C48,
0xA68E46,
0x88782D,
0x888538,
0x878935,
0x9B9837,
0xA79E37,
0x918636,
0x6D7953,
0x4E8486,
0x2F8A9C,
0x258F9C,
0x298C91,
0x2E8890,
0x2D889B,
0x2486A3,
0x24849A,
0x238399,
0x228298,
0x208096,
0x1F7F95,
0x1E7E94,
0x1D7D93,
0x1C7C92,
0x1B7B91,
0x1B7A90,
0x1A798D,
0x1C788D,
0x1B778C,
0x1B778C,
0x1B7689,
0x1B7689,
0x1A7587,
0x1B7486,
0x1B7486,
0x1B7285,
0x1B7182,
0x1C6F81,
0x1C6F81,
0x1C6F7F,
0x1C6D7E,
0x1B6D7B,
0x1D6C79,
0x1D6B78,
0x1E6A77,
0x206A77,
0x216977,
0x216975,
0x236774,
0x226671,
0x226470,
0x25656F,
0x26646F,
0x28646E,
0x29646C,
0x2A636C,
0x2A636A,
0x29646C,
0x27636D,
0x26646F,
0x256571,
0x236571,
0x226673,
0x226874,
0x206874,
0x216977,
0x206A77,
0x1E6A77,
0x1D6B78,
0x1D6C7B,
0x1C6D7E,
0x1C6D7E,
0x1C6F81,
0x1C6F81,
0x1B7182,
0x1B7182,
0x1C7185,
0x1C7386,
0x1D7487,
0x1C7488,
0x1C7488,
0x1B7689,
0x1B7689,
0x1A7789,
0x1A7789,
0x1B778C,
0x1C788D,
0x1D798E,
0x207987,
0x1D7893,
0x207B9A,
0x277E92,
0x24808B,
0x1B8497,
0x1985A1,
0x307E88,
0x678968,
0x76772B,
0x9E9634,
0xBAB84B,
0xA8A532,
0xA8A234,
0x7D7B26,
0x60662A,
0x665A20,
0x86722B,
0x9B8025,
0xB09427,
0xC5A72B,
0xD3AE2A,
0xD7AF2C,
0xD4AA31,
0xD4B035,
0xCFB32B,
0xBDB12B,
0x8B9F3E,
0x629B7B,
0x4E9EB5,
0x54A4C7,
0x559BB5,
0x5AA5B8,
0x4FA2B0,
0x58AFB8,
0x5BA6AC,
0x6DA4AB,
0xA9D0D7,
0xD1EEF2,
0xECFFFF,
0xF7FFF4,
0xFAF5E2,
0xF2E7D5,
0xDDDACB,
0x92988E,
0x42453E,
0x544A3E,
0x755B4A,
0x7C6543,
0x806A52,
0x6D594E,
0x52453D,
0x3D3C20,
0x868A4D,
0x878C2E,
0x858521,
0x7C7525,
0x695C25,
0x645327,
0x5F4F1C,
0x6C5C20,
0x776928,
0x6C6320,
0x5C5713,
0x3A3A22,
0x2E230D,
0x574322,
0x7F652C,
0x81671A,
0x897327,
0x826F34,
0x645828,
0x443A07,
0x685E21,
0x7E6C2A,
0xAB9242,
0xCAAD45,
0xC6B22B,
0xBDB419,
0xC9CA26,
0xA09859,
0x4F4124,
0x43373B,
0x56505A,
0x4D493E,
0x6B5D43,
0x7D6B57,
0xDCD5CF,
0xEAF3EE,
0xF0FDE9,
0xF4FFE8,
0xDCF3ED,
0xB8E4F1,
0x8EBBB6,
0x7C8A4C,
0xB9A22E,
0xC9AA1C,
0xD3A917,
0xDDB329,
0xBAA935,
0x919A31,
0x9EA837,
0xB8B12D,
0xB8A723,
0xADA030,
0x8D832A,
0x9D8E3B,
0xA48F34,
0x6C5900,
0x81731A,
0x86751D,
0x997F28,
0x8A772B,
0x867621,
0x857927,
0x898139,
0x928541,
0x95832F,
0x8F8226,
0x657338,
0x497D7B,
0x2F89AC,
0x248FAF,
0x248E9B,
0x2C8E97,
0x2B8A9E,
0x2887A5,
0x2283A3,
0x25859B,
0x24849A,
0x228298,
0x218197,
0x208096,
0x1F7F95,
0x1D7D93,
0x1D7D93,
0x1B7B91,
0x1A7A90,
0x1A798D,
0x1A798D,
0x1B778C,
0x1B778C,
0x1A7789,
0x1B7689,
0x1A7587,
0x1A7587,
0x1B7486,
0x1B7285,
0x1B7084,
0x1A7081,
0x1C6F81,
0x1C6F7F,
0x1B6E7E,
0x1C6E7C,
0x1D6C7B,
0x1D6C79,
0x1F6B79,
0x216B78,
0x206A77,
0x216977,
0x246875,
0x236774,
0x236571,
0x256571,
0x26646F,
0x28646E,
0x29636E,
0x2A636C,
0x2A636A,
0x29646C,
0x28646E,
0x26646F,
0x236571,
0x226673,
0x226874,
0x236975,
0x206876,
0x206A77,
0x206A77,
0x1F6B79,
0x1E6C79,
0x1D6C7B,
0x1C6D7E,
0x1B6E7E,
0x1C6F81,
0x1C6F81,
0x1B7182,
0x1B7182,
0x1B7285,
0x1C7386,
0x1C7587,
0x1C7488,
0x1B7689,
0x1B7689,
0x1B788A,
0x1B778C,
0x1B778C,
0x1B778C,
0x1C788D,
0x1D798E,
0x227A8E,
0x207B8E,
0x217D8C,
0x23818B,
0x208388,
0x1C848D,
0x1B8595,
0x278495,
0x418078,
0x6E8750,
0x8D862B,
0x9E8B2C,
0x9B9041,
0x7F7A36,
0x746B1C,
0x978725,
0x757433,
0x9C8A38,
0xC1A037,
0xD3B136,
0xD6BA35,
0xCCAB28,
0xD1A52A,
0xD8A635,
0xD5A833,
0xCCAC25,
0xC8B41F,
0xB4AE26,
0x8B9E45,
0x6B9678,
0x5F9FAB,
0x59A2C2,
0x54A2B9,
0x5BABC4,
0x59A1BA,
0x75A3BB,
0xCCDDED,
0xF1F7F5,
0xF1F7E9,
0xF6F8EA,
0xF9F1E6,
0xEEDDD3,
0xDED2C4,
0xC2C3B5,
0x8D958A,
0x494A44,
0x4C3E35,
0x775D50,
0x6E6441,
0x756855,
0x5F524A,
0x3B3223,
0x4A451B,
0x8F8D42,
0x878427,
0x7F781E,
0x6F631B,
0x65561B,
0x7B7030,
0x867D2E,
0x978E31,
0xA8A139,
0xA19B2D,
0x989426,
0x7E7C25,
0x5B5717,
0x3A3108,
0x40320F,
0x4B3913,
0x544117,
0x5C4B1D,
0x594A13,
0x655613,
0x7C6724,
0x82682F,
0x7B6128,
0x938033,
0xAEA435,
0xB9B52D,
0xC4C430,
0x99933F,
0x504518,
0x433836,
0x4D4449,
0x584E42,
0x63553B,
0x6F604D,
0xC3BAB3,
0xE3E3D9,
0xEFEDD6,
0xFDF7DD,
0xF5F7EC,
0xE9FCFA,
0xE5FBE6,
0x9F9B5B,
0xAA8D25,
0xB09925,
0xBCA233,
0xC0A83A,
0xA59A26,
0x959A19,
0xAAB52A,
0x949B11,
0xA8A429,
0x877F12,
0x9D942D,
0xA59E34,
0x958E23,
0x959126,
0x8E8923,
0x978F2A,
0x8E871D,
0x817D11,
0x777810,
0x6A6B0E,
0x675C0C,
0x7C6215,
0x846B17,
0x61641D,
0x4E7F61,
0x3A91A4,
0x268AAE,
0x29899F,
0x2F8992,
0x308D9F,
0x2586A9,
0x2485A6,
0x2C889F,
0x26869C,
0x25859B,
0x238399,
0x228298,
0x208096,
0x1F7F95,
0x1E7E94,
0x1D7D93,
0x1C7C92,
0x1B7B91,
0x1B7A8E,
0x1A798D,
0x1C788D,
0x1B778C,
0x1A768B,
0x1A7789,
0x1B7688,
0x1A7587,
0x1B7486,
0x1B7285,
0x1B7084,
0x1B7182,
0x1D7082,
0x1C6F81,
0x1C6F7F,
0x1C6D7E,
0x1B6D7B,
0x1D6C79,
0x1E6C79,
0x1F6B79,
0x206A77,
0x216977,
0x236975,
0x236774,
0x236571,
0x246672,
0x276570,
0x28646F,
0x29636E,
0x2A636C,
0x29646C,
0x29646C,
0x28646E,
0x25656F,
0x246672,
0x236774,
0x226874,
0x216977,
0x216977,
0x206A77,
0x1F6B78,
0x1E6C79,
0x1D6C7B,
0x1B6D7B,
0x1C6D7E,
0x1B6E7E,
0x1A7081,
0x1B7182,
0x1B7182,
0x1C7185,
0x1C7386,
0x1C7386,
0x1C7488,
0x1B7689,
0x1C778A,
0x1C778A,
0x1B778C,
0x1B778C,
0x1B778C,
0x1C788D,
0x1D798E,
0x1C7B8F,
0x207C95,
0x247D8B,
0x227F86,
0x1C8189,
0x1B838E,
0x21838C,
0x228491,
0x1F87A4,
0x20849E,
0x4B897C,
0x6F7837,
0x886F15,
0x876D23,
0x6D5F22,
0x998838,
0xB69828,
0x7B7F28,
0xB09D34,
0xCFA92A,
0xD4B22A,
0xC8B333,
0xC2AD3A,
0xC6A137,
0xD5A63A,
0xC89F2D,
0xC4A82A,
0xC8B229,
0xD2B929,
0xC6B131,
0xA3A447,
0x799D6F,
0x639F95,
0x5FA8AF,
0x5F9EAF,
0x7FA9C1,
0xD5E5F2,
0xF3F0E1,
0xE7DCAE,
0xE6DEA0,
0xF0E9B5,
0xFBF1D6,
0xE0D7CE,
0xD3D2CE,
0xBCC3BC,
0x8B9187,
0x504A3E,
0x4C3629,
0x715041,
0x655C3D,
0x675B4D,
0x4F423C,
0x32260E,
0x5E581E,
0x918B37,
0x877D26,
0x74661B,
0x675515,
0x8B7C3B,
0x8F8339,
0x938C34,
0xA8A33D,
0xB6B03E,
0xBEB53E,
0xC0B63B,
0xB5AD28,
0x9E9A2F,
0x78752E,
0x585522,
0x4E4617,
0x645720,
0x978B3F,
0xB0A740,
0xBDB137,
0xC2B13B,
0xB6A144,
0x83702C,
0x635A17,
0x7E7C25,
0xA4A231,
0xBAB535,
0x928D25,
0x5B5217,
0x534238,
0x483634,
0x6C5744,
0x665335,
0x584B38,
0xA8A39D,
0xDADAD0,
0xE9E3CD,
0xF8ECD4,
0xF8EFDE,
0xE9E9DD,
0xFFFFEA,
0xC2BC8A,
0x86722D,
0xA09635,
0xA19A42,
0x9B9B45,
0x818828,
0x92A12C,
0xA6B735,
0x879617,
0x8F9727,
0x80831C,
0xA6A83D,
0x91911F,
0x9B9E29,
0x979B2C,
0x92982C,
0x969F32,
0x929C2C,
0x979C24,
0x95A133,
0x8F9B39,
0x929134,
0x918121,
0x74660D,
0x64763C,
0x4C8775,
0x419094,
0x468F84,
0x5C9374,
0x5A8F73,
0x41888A,
0x2F8AA7,
0x2B8AA8,
0x2C8799,
0x28879D,
0x27869C,
0x24849A,
0x228298,
0x218197,
0x208096,
0x1E7E94,
0x1D7D93,
0x1C7C92,
0x1C7C92,
0x1B7A8E,
0x1B7A8E,
0x1A798D,
0x1B778C,
0x1B778C,
0x1A7789,
0x1B7688,
0x1A7587,
0x1C7587,
0x1C7386,
0x1C7185,
0x1B7084,
0x1D7082,
0x1D7082,
0x1C6F7F,
0x1B6E7E,
0x1B6D7B,
0x1D6C7B,
0x1E6C79,
0x1F6B79,
0x206A77,
0x206A77,
0x236975,
0x236774,
0x236774,
0x246672,
0x25656F,
0x28646F,
0x29636E,
0x2A646F,
0x2A656D,
0x28646C,
0x26656E,
0x25656F,
0x246672,
0x236774,
0x226776,
0x216977,
0x216977,
0x206A77,
0x1F6B79,
0x1E6C79,
0x1D6C7B,
0x1B6C7D,
0x1C6D7E,
0x1C6F81,
0x1B7182,
0x1B7182,
0x1B7182,
0x1B7285,
0x1C7386,
0x1C7587,
0x1D7589,
0x1B7689,
0x1C778A,
0x1B788A,
0x1C788D,
0x1C788D,
0x1C788D,
0x1B7A8E,
0x1B7A8E,
0x1C7B8F,
0x1A7C95,
0x237E90,
0x237F8E,
0x1C7F96,
0x1E849D,
0x278398,
0x2B8496,
0x2386A6,
0x1F8BB9,
0x23809F,
0x4F8273,
0x6B6F34,
0x846F1E,
0x9C832F,
0xB19743,
0xB4963E,
0x83811A,
0xB8A52D,
0xC9A721,
0xC9A823,
0xBBA633,
0xB4A23E,
0xB39734,
0xC1A132,
0xC1A831,
0xBEAD37,
0xBFAC34,
0xCDAC2B,
0xDEB72C,
0xDAC038,
0xB0B445,
0x85A248,
0x6AA181,
0x9DC2B0,
0xE0F1E1,
0xDFE1BC,
0xB7B465,
0xBBB440,
0xC5BA3C,
0xBAAD44,
0xD1C887,
0xDCDBBF,
0xCDD2CC,
0xB9BFBD,
0x929489,
0x4F4835,
0x4C3724,
0x644737,
0x5D4D34,
0x59483E,
0x40362D,
0x312A0D,
0x6A6626,
0x95903C,
0x857A28,
0x6E5D15,
0x7E6D27,
0x918038,
0x877833,
0x8A7838,
0x85742F,
0x8A772A,
0x9C882F,
0xB4A041,
0xC5B440,
0xD3C75B,
0xC9C15E,
0xA7A23E,
0xAAA237,
0xC1B843,
0xC9C442,
0xCECC39,
0xCFCD2B,
0xCBC527,
0xC5BB38,
0xA09539,
0x6B6622,
0x696621,
0x959035,
0xB7AF42,
0x818129,
0x594F1C,
0x6D5B4F,
0x43322B,
0x62523B,
0x685535,
0x52402C,
0x9F948E,
0xCECDC8,
0xE0DFCB,
0xF0E6CB,
0xF3E7D1,
0xEBE9DC,
0xF2F6E5,
0xE7E8C8,
0x8F8456,
0x8C8221,
0x989136,
0x838531,
0x727F2D,
0x77872F,
0x757F1C,
0x747A0E,
0x7B831C,
0x6C7D1F,
0x6A7D21,
0x76801D,
0x80831C,
0x737915,
0x828D2D,
0x768423,
0x6C7812,
0x858D2A,
0x787D1F,
0x7D8727,
0x768520,
0x88952B,
0x6E7516,
0x6A7634,
0x637D56,
0x5D8257,
0x718B42,
0xA2A844,
0xB0B95E,
0x7BA170,
0x438E8A,
0x2C879A,
0x2B879C,
0x28879D,
0x27869C,
0x26859B,
0x238399,
0x228298,
0x208096,
0x1F7F95,
0x1E7E94,
0x1D7D93,
0x1C7C92,
0x1C7B8F,
0x1B7A8E,
0x1A798D,
0x1C788D,
0x1B778C,
0x1A7789,
0x1B7688,
0x1A7587,
0x1C7587,
0x1C7386,
0x1B7285,
0x1B7084,
0x1B7182,
0x1D7082,
0x1C6F7F,
0x1B6E7E,
0x1C6E7C,
0x1D6C7B,
0x1E6C79,
0x1F6B79,
0x206978,
0x206978,
0x236975,
0x236774,
0x236774,
0x246672,
0x25656F,
0x26646F,
0x28646E,
0x2A646F,
0x2A656D,
0x28646E,
0x26656E,
0x25656F,
0x246672,
0x236774,
0x226776,
0x216977,
0x206A77,
0x206A77,
0x1F6B79,
0x1E6C79,
0x1D6C7B,
0x1C6D7E,
0x1B6E7E,
0x1C6F81,
0x1B7182,
0x1B7182,
0x1C7283,
0x1B7285,
0x1C7386,
0x1C7587,
0x1D7589,
0x1B7689,
0x1C778A,
0x1B778C,
0x1C788D,
0x1C788D,
0x1B7A8E,
0x1B7A8E,
0x1C7B8F,
0x1D7C90,
0x197C8F,
0x217F98,
0x237E9B,
0x207D9F,
0x2182A3,
0x27829F,
0x2A8097,
0x2C809A,
0x2885A7,
0x288BB2,
0x2F879B,
0x51856F,
0x727734,
0x947B1F,
0x977B27,
0x826E29,
0x9C9022,
0xBEAA2F,
0xC5AA27,
0xC1A52A,
0xB49D35,
0xA4902F,
0xAC972E,
0xBCA835,
0xB7AB35,
0xABA130,
0xAD9B2B,
0xBD9F27,
0xD4AE29,
0xE2C233,
0xD5C532,
0xBAB523,
0x849F52,
0xACBB7A,
0xCDD292,
0xA4A44C,
0xB8B939,
0xCDCB2B,
0xDACE2A,
0xD7C638,
0xAFA13E,
0xC6C38E,
0xD4D8C7,
0xC2C7C3,
0x9F9D91,
0x544C35,
0x483821,
0x594637,
0x5D442E,
0x513F35,
0x4A4138,
0x363614,
0x717431,
0x93903F,
0x88802F,
0x76691A,
0x918531,
0x7F6F22,
0x4E3A05,
0x44290C,
0x50321A,
0x62411E,
0x684815,
0x65470B,
0x755C19,
0x957E30,
0xC5B150,
0xE2D15B,
0xE4D44D,
0xEADB4E,
0xEADF51,
0xDBD644,
0xCECD35,
0xC5C42C,
0xBDB931,
0xAAA339,
0x8C8338,
0x6E6522,
0x908835,
0xB3AE48,
0x6F743E,
0x4B4427,
0x52433C,
0x554C45,
0x55503A,
0x5E5033,
0x573E2A,
0xA28F88,
0xCFCBC2,
0xDBDEC9,
0xE9E5C8,
0xE4DCC5,
0xDEE0D3,
0xE0EBDD,
0xF5FAE3,
0xDDD7B3,
0x898025,
0x8D8020,
0x898023,
0x7E7F2F,
0x797C35,
0x7B7829,
0x958C2F,
0x858724,
0x4F6409,
0x4E6918,
0x4F5E0D,
0x787A26,
0x555704,
0x6F7827,
0x565F0C,
0x72771D,
0x595F15,
0x686514,
0x65640A,
0x657714,
0x54720E,
0x607415,
0x757221,
0x8E843D,
0x7E7928,
0x968E21,
0xBDAA20,
0xC4B02B,
0x8F983B,
0x528962,
0x368A94,
0x2887A7,
0x29889E,
0x28879D,
0x26859B,
0x25849A,
0x228298,
0x218197,
0x208096,
0x1F7F95,
0x1E7E94,
0x1D7E91,
0x1D7C90,
0x1C7B8F,
0x1B7A8E,
0x1C788D,
0x1B788A,
0x1A7789,
0x1B7688,
0x1B7688,
0x1C7587,
0x1C7386,
0x1B7285,
0x1C7185,
0x1B7182,
0x1D7082,
0x1C6F81,
0x1C6F7F,
0x1C6D7E,
0x1D6C7B,
0x1E6B7B,
0x1F6B79,
0x206978,
0x206978,
0x236975,
0x226874,
0x236774,
0x236571,
0x256571,
0x25636E,
0x28646E,
0x2A646F,
0x2A656D,
0x28646E,
0x26656E,
0x25656F,
0x246672,
0x226874,
0x226776,
0x216977,
0x206A77,
0x206A77,
0x1F6B79,
0x1E6C79,
0x1D6C7B,
0x1C6D7E,
0x1C6F7F,
0x1D7082,
0x1B7182,
0x1C7283,
0x1C7283,
0x1C7386,
0x1D7487,
0x1C7587,
0x1D7589,
0x1B7689,
0x1C778A,
0x1B778C,
0x1C788D,
0x1D798E,
0x1B7A8E,
0x1C7B8F,
0x1D7C90,
0x1E7D91,
0x1F7C8B,
0x217F98,
0x207D9C,
0x237E9B,
0x26819C,
0x22809A,
0x288094,
0x3A848D,
0x387E7C,
0x348586,
0x2F8C94,
0x3D8989,
0x547556,
0x736A27,
0x7D6714,
0x968631,
0xC2AC3C,
0xCEB939,
0xC9B32C,
0xBEA932,
0xB09D37,
0x9F8D29,
0xB0A031,
0xB8A735,
0xA6932C,
0xA4912B,
0xB39E29,
0xC2AA26,
0xD0B42D,
0xCFB230,
0xCDB730,
0xC6B625,
0xAEA73F,
0x949336,
0x8E9337,
0xA4A339,
0xC0B839,
0xC4BA27,
0xC5BA24,
0xC9B72D,
0xB2A032,
0xA29954,
0xD8DDBF,
0xCED8CF,
0x9A9B8D,
0x4C3D28,
0x4B3925,
0x57473A,
0x59432E,
0x513F35,
0x52483E,
0x3E3E18,
0x787B38,
0x8A8B3B,
0x8D8837,
0x867C27,
0x8F8129,
0x9C8A42,
0x4B330D,
0x41241C,
0x72554F,
0x6F503B,
0x795933,
0x88653B,
0x805E39,
0x614110,
0x755717,
0x84661E,
0x95782A,
0xA1822F,
0xB2943A,
0xB69C3D,
0xBCAA46,
0xBEB649,
0xBAB744,
0xB0A93F,
0xA6983F,
0x81721F,
0x968E2C,
0xB4B13E,
0x575731,
0x3C3523,
0x4A403E,
0x37332A,
0x413F28,
0x5F553C,
0x493627,
0x9A887E,
0xD7CEBD,
0xD1CFB8,
0xE0DEC7,
0xE1DECD,
0xE1E2D2,
0xE2E5D2,
0xE0E2CC,
0xF4EFD9,
0xB6B984,
0x807732,
0x8E7E29,
0x8F8331,
0x807A2E,
0x675E11,
0x83781E,
0x7C7815,
0x7E8929,
0x7E8E36,
0x78812E,
0x858434,
0x817F34,
0x808336,
0x868A33,
0x8C8E2D,
0x7E8633,
0x94923B,
0x8B862A,
0x808529,
0x56660D,
0x65711B,
0x595704,
0x887926,
0x8F8025,
0x9F8E24,
0xB79F25,
0xBCA72A,
0xA19F3A,
0x60834D,
0x3D8786,
0x2789AC,
0x2A899F,
0x29889E,
0x27869C,
0x25849A,
0x238399,
0x228298,
0x208096,
0x208096,
0x1E7E94,
0x1D7E91,
0x1D7C90,
0x1C7B8F,
0x1B7A8E,
0x1C788D,
0x1B788A,
0x1A7789,
0x1B7688,
0x1B7688,
0x1C7587,
0x1D7487,
0x1C7386,
0x1C7185,
0x1B7182,
0x1D7082,
0x1D7082,
0x1C6F7F,
0x1C6D7E,
0x1E6D7C,
0x1E6B7B,
0x1F6B79,
0x216A79,
0x206978,
0x236975,
0x226874,
0x236774,
0x236571,
0x256571,
0x25636E,
0x28646E,
0x2A646F,
0x2A656D,
0x28646E,
0x26646F,
0x256571,
0x246674,
0x226874,
0x226776,
0x216977,
0x206A77,
0x206978,
0x1F6B79,
0x1E6C79,
0x1E6D7C,
0x1C6D7E,
0x1C6F81,
0x1D7082,
0x1B7182,
0x1C7283,
0x1B7285,
0x1C7386,
0x1D7487,
0x1C7587,
0x1D7589,
0x1C778A,
0x1C778A,
0x1C788D,
0x1D798E,
0x1B7A8E,
0x1C7B8F,
0x1D7C90,
0x1E7D91,
0x1F7E92,
0x257D87,
0x227F91,
0x1F7E94,
0x258093,
0x288594,
0x238497,
0x2B8698,
0x49918D,
0x69A080,
0x549167,
0x2F7D67,
0x3B898B,
0x48787A,
0x525937,
0x8C7D2A,
0xB29E2B,
0xC6AD37,
0xD3BC32,
0xCDB827,
0xC0B134,
0xA1952F,
0x938A23,
0xABA02D,
0xAD962C,
0xA1812A,
0xB99A3F,
0xCAB236,
0xC8B421,
0xD0BB2E,
0xC4AE37,
0xBBA639,
0xB4A331,
0xAF9325,
0x9E972F,
0x909634,
0x9D9936,
0xB6A138,
0xB19B2C,
0xB0A12C,
0xBCAB37,
0xA69128,
0xA79B53,
0xDEE8C5,
0xC8DACC,
0x9C9E91,
0x4A3524,
0x53392A,
0x58473F,
0x53412D,
0x513F35,
0x433528,
0x464119,
0x777832,
0x878936,
0x8D8A39,
0x8C8431,
0x907F2D,
0xA28D4C,
0x5D4426,
0x1E0701,
0x433125,
0x564323,
0x73582D,
0xAA845D,
0xB9936F,
0xB28C68,
0xD5B096,
0x88635A,
0x886261,
0x855A51,
0x6D3E22,
0x77461B,
0x714C15,
0x725E15,
0x746E10,
0x8B8620,
0xAB9F3B,
0x9F8F2D,
0xA99F2E,
0xB3B331,
0x4B4418,
0x3A3320,
0x413937,
0x4E453C,
0x463C23,
0x5A4F39,
0x52483C,
0xA3978B,
0xDBD1B8,
0xD7D0B6,
0xDBD9CA,
0xD1D1C7,
0xDCD7C3,
0xE6DDBE,
0xE4DCC7,
0xE0DBD5,
0xD9E9DF,
0xB4B490,
0x857633,
0x91812C,
0x988E37,
0x928C36,
0x837A1D,
0x8E8621,
0x7E7D13,
0x777A13,
0x767518,
0x7F7925,
0x7B7527,
0x73721F,
0x767817,
0x909425,
0x859125,
0x777C16,
0x8D882A,
0x9A8F35,
0x8C842F,
0x7B7C2C,
0x788132,
0x7C8131,
0x8B8531,
0x9F9139,
0xA49337,
0x9C8E2D,
0x96953B,
0x5D793F,
0x40867E,
0x288CAC,
0x2A899D,
0x29889E,
0x27869C,
0x26859B,
0x25849A,
0x228298,
0x218197,
0x208096,
0x1F7F95,
0x1E7F92,
0x1D7C90,
0x1C7B8F,
0x1B7A8E,
0x1C788D,
0x1B788A,
0x1B788A,
0x1B7689,
0x1B7689,
0x1D7688,
0x1D7487,
0x1C7386,
0x1C7185,
0x1C7185,
0x1D7082,
0x1D7082,
0x1C6F7F,
0x1C6D7E,
0x1E6D7C,
0x1F6C7C,
0x1F6B79,
0x216A79,
0x206978,
0x236975,
0x236975,
0x236774,
0x246672,
0x256571,
0x25636E,
0x28646E,
0x2A646F,
0x296570,
0x28646F,
0x25656F,
0x236571,
0x236772,
0x206874,
0x206A77,
0x1E6A77,
0x1E6A78,
0x1D6A7A,
0x1E6B7B,
0x1E6D7E,
0x1C6D7E,
0x1C6F81,
0x1A7081,
0x1A7081,
0x1B7182,
0x1B7283,
0x1C7386,
0x1C7386,
0x1C7587,
0x1D7688,
0x1B7689,
0x1C778A,
0x1B788A,
0x1B788A,
0x1B788A,
0x1B788A,
0x1C798B,
0x1E7B8D,
0x1F7C8E,
0x1F7C8E,
0x25779D,
0x277F97,
0x1F8091,
0x1B839E,
0x2285AC,
0x2A7A95,
0x618C79,
0xA9AE5A,
0xC1B02C,
0xC6B328,
0x9F9824,
0x7C7E2A,
0x747732,
0x736F26,
0x90852C,
0xA89934,
0xC6A531,
0xD0B532,
0xC3AF2A,
0xB7A632,
0x908323,
0x989232,
0xA4A034,
0x928A1D,
0xA7962E,
0xCFB346,
0xD1B02B,
0xD6B71F,
0xD4B829,
0xC1AA36,
0xA79839,
0x9F953E,
0x959527,
0x999732,
0x8E8D30,
0x969336,
0x9E9B34,
0x9C9928,
0x989524,
0x918C2E,
0x7C7433,
0xC7BD99,
0xECDFCE,
0xD9CDBF,
0xA59A88,
0x4B432E,
0x4D4834,
0x4C4938,
0x41361A,
0x434039,
0x302E31,
0x4B3F29,
0x837433,
0x89832B,
0x8A9038,
0x868737,
0x8D7C2E,
0xA38841,
0x776535,
0x080100,
0x0D070B,
0x22191C,
0x2F281E,
0x333422,
0x3C4029,
0x8C8664,
0xC1B28B,
0xA59571,
0xE0D1B2,
0xE5D2B1,
0xA78F69,
0xC8B192,
0x89796A,
0x47392E,
0x2F1D00,
0x957C2A,
0xBBA631,
0xAEA522,
0xADB234,
0xACB540,
0x39391D,
0x4A4531,
0x403424,
0x5E4D3B,
0x735E49,
0x564531,
0x5E5140,
0x9A9581,
0xD3D2BD,
0xCFD1BB,
0xD0CEBF,
0xD5D2C3,
0xD7D2BF,
0xDBD4C1,
0xD7D0BE,
0xDCD6C8,
0xD2D8CC,
0xEEEFD0,
0xBEB781,
0x8D7B33,
0xAC8F41,
0x9D7D2A,
0x9E7F25,
0x9E8427,
0x9B8426,
0xA18D34,
0x9C8A34,
0x9B8A32,
0xA19332,
0xA49930,
0x9B9326,
0xA6A030,
0x969342,
0x9A811C,
0xB8901F,
0xAD8F2D,
0x867A30,
0x6E6524,
0x796B22,
0x887B2C,
0x8C8431,
0x9C872A,
0xAA8419,
0x9D7B17,
0x737E3A,
0x4A8775,
0x348C9A,
0x2E8AA3,
0x2B8AA0,
0x2A899F,
0x28879D,
0x26859B,
0x25849A,
0x228298,
0x218197,
0x208096,
0x1F7F95,
0x1E7E94,
0x1D7D93,
0x1D7C92,
0x1C7B8F,
0x1B7A8E,
0x1A798D,
0x1B778C,
0x1B778C,
0x1B788A,
0x1B7689,
0x1C7587,
0x1C7587,
0x1C7384,
0x1B7283,
0x1C7283,
0x1B7182,
0x1B7182,
0x1C6F81,
0x1A6D7F,
0x1B6C7D,
0x1D6C7B,
0x1E6C79,
0x1E6A78,
0x206A77,
0x206A77,
0x226874,
0x236774,
0x256571,
0x266471,
0x28646F,
0x29656F,
0x296570,
0x26646F,
0x25656F,
0x236571,
0x236772,
0x206874,
0x206A77,
0x1E6A77,
0x1E6A78,
0x1E6B7B,
0x1E6B7B,
0x1E6D7E,
0x1C6D7E,
0x1C6F81,
0x1A7081,
0x1A7081,
0x1A7182,
0x1B7283,
0x1C7386,
0x1C7386,
0x1C7587,
0x1A7587,
0x1B7689,
0x1B788A,
0x1B788A,
0x1B788A,
0x1B788A,
0x1C788D,
0x1D798E,
0x1E7A8F,
0x1F7B90,
0x207D8F,
0x267D99,
0x297F8E,
0x24828E,
0x1985A1,
0x1D85A8,
0x368186,
0x78955F,
0xB1AC44,
0xCABC42,
0xB3AC3A,
0xBBB954,
0x9C9D3F,
0x94943C,
0x938F34,
0x9E902D,
0xAF9527,
0xBD9F27,
0xC2A82D,
0xBBA62F,
0x9D8F26,
0x94892D,
0xA69F44,
0x968E2B,
0x978A22,
0xB6A136,
0xCCAF37,
0xD3B428,
0xD6B920,
0xCFB425,
0xC6AE34,
0xB19E35,
0xA2952F,
0x9D9830,
0x989431,
0x8F8B30,
0x8A8531,
0x928D39,
0x898432,
0x908A40,
0x8F894F,
0xBCB48D,
0xDCD4BD,
0xE9E0CF,
0xDFD3C3,
0xB0A593,
0x4E4232,
0x4B4135,
0x3D3329,
0x392E12,
0x423B33,
0x322C2E,
0x362B19,
0x6D6127,
0x877F2C,
0x8E8E36,
0x868834,
0x867E2B,
0x968034,
0xA38853,
0x240F00,
0x1A0F17,
0x231B28,
0x190F0E,
0x180800,
0x0C0900,
0x181204,
0x2C2515,
0x2D2518,
0x585340,
0x676648,
0x2C2B0C,
0x2C291A,
0x291F20,
0x251309,
0x836936,
0xCBB34F,
0xAFA023,
0xAEA82C,
0xB6B946,
0x9FA636,
0x2C2C14,
0x423C2C,
0x514539,
0x342415,
0x5B4935,
0x5F4D39,
0x625340,
0xA69B87,
0xD5CFB5,
0xD5D1B6,
0xD5D0BA,
0xD2CBB8,
0xCFC7B0,
0xD2C8AD,
0xCFC5AC,
0xD3C8B6,
0xCAD5C5,
0xD2D7C3,
0xE5E2C3,
0xA99F6C,
0x96863B,
0xA89338,
0xAA9431,
0xAC9434,
0xAB9339,
0xA78F35,
0xA38E31,
0xA2902C,
0xA99830,
0xB1A336,
0xAC9F2F,
0xA69B28,
0xAD9E3F,
0x9E8A27,
0xA8922F,
0xAD983B,
0xA99137,
0x97791F,
0x8B7522,
0x7B7B3B,
0x688255,
0x68885F,
0x708759,
0x6B8254,
0x56846A,
0x3D8B8B,
0x2F90A1,
0x2A8EA5,
0x2C8BA1,
0x2B8AA0,
0x29889E,
0x27869C,
0x26859B,
0x238399,
0x228298,
0x218197,
0x208096,
0x1F7F95,
0x1D7D93,
0x1D7C92,
0x1C7B8F,
0x1B7A8E,
0x1A798D,
0x1B778C,
0x1B778C,
0x1B788A,
0x1B7689,
0x1C7587,
0x1C7587,
0x1C7384,
0x1B7283,
0x1C7283,
0x1B7182,
0x1B7182,
0x1C6F81,
0x1A6D7F,
0x1B6C7D,
0x1D6C7B,
0x1E6C79,
0x1E6A78,
0x206A77,
0x206A77,
0x226874,
0x236774,
0x256571,
0x266471,
0x28646F,
0x29656F,
0x296570,
0x26646F,
0x256571,
0x246672,
0x226872,
0x216975,
0x206A77,
0x1E6A77,
0x1E6A78,
0x1E6B7B,
0x1D6C7B,
0x1E6D7E,
0x1C6D7E,
0x1C6F81,
0x1A7081,
0x1B7182,
0x1A7184,
0x1B7285,
0x1C7386,
0x1B7486,
0x1C7587,
0x1A7588,
0x1B7689,
0x1B788A,
0x1B788A,
0x1B778C,
0x1C788D,
0x1D798E,
0x1E7A8F,
0x1F7B90,
0x207C91,
0x217D92,
0x207F93,
0x257C83,
0x288189,
0x1C85A2,
0x1A85A5,
0x3F897C,
0x849B4D,
0xAEA330,
0xB0A031,
0xA8A240,
0xB0AD4C,
0xA09B35,
0x969231,
0x959032,
0xA49329,
0xC4A82A,
0xC4A928,
0xC1A834,
0xAC9933,
0x8B7E20,
0xABA547,
0xACA648,
0x908627,
0xA69231,
0xC4AA3B,
0xC7AA28,
0xD0B123,
0xD4B526,
0xCDB129,
0xC7AE30,
0xB59E2C,
0xA89225,
0x9E9432,
0x938A2D,
0x8A8231,
0x79722C,
0x827B44,
0xB5AE82,
0xC9C3A1,
0xD8D4B9,
0xDED9C5,
0xDCD7C4,
0xD7D2BF,
0xD4CBBA,
0xAA9E8E,
0x493B2E,
0x4D3F36,
0x3B2925,
0x41351D,
0x3F3229,
0x3E3432,
0x383121,
0x5B5023,
0x8B7F37,
0x89812C,
0x86862E,
0x818129,
0x958530,
0xC09D5B,
0x6B4926,
0x100000,
0x393041,
0x44303B,
0x4C2424,
0x301C1E,
0x1A0B10,
0x150A12,
0x14080C,
0x0B0200,
0x060900,
0x071307,
0x000306,
0x080006,
0x5D422D,
0xCBAD65,
0xB6A12E,
0xB1A829,
0xB0AF39,
0xB7BA4F,
0x8D922C,
0x282512,
0x2C261A,
0x241A11,
0x4B3D32,
0x514131,
0x71604E,
0x6B5A48,
0x9B8C75,
0xCFC4A6,
0xD2C9A8,
0xD2CBAF,
0xCAC3A9,
0xC9C0A3,
0xC9C09F,
0xCBC0A2,
0xCFC3A9,
0xC4C9B5,
0xCCCEC1,
0xCFD0C2,
0xD3D2B3,
0x838144,
0x958F37,
0x9B922D,
0x9F8F2F,
0xA38E35,
0xA69138,
0xA99536,
0xA8952E,
0xB09E30,
0xB2A030,
0xB09C2C,
0xB09C2B,
0xBBA132,
0xA69835,
0x948E36,
0xA09134,
0xA7861D,
0xAB861E,
0x8A7E32,
0x5F7D5B,
0x509291,
0x4396A6,
0x4194A4,
0x408F9E,
0x398F9E,
0x3491A3,
0x2C91A5,
0x2790A3,
0x2D8CA2,
0x2C8BA1,
0x2A899F,
0x28879D,
0x27869C,
0x24849A,
0x238399,
0x228298,
0x208096,
0x1F7F95,
0x1E7E94,
0x1D7C92,
0x1D7C90,
0x1C7B8F,
0x1A798D,
0x1C788D,
0x1B778C,
0x1B788A,
0x1B7689,
0x1C7587,
0x1C7587,
0x1C7384,
0x1B7283,
0x1B7283,
0x1B7084,
0x1B7182,
0x1A7081,
0x1B6E80,
0x1B6C7D,
0x1D6C7B,
0x1E6C79,
0x1E6A78,
0x1E6A77,
0x206A77,
0x206876,
0x236774,
0x246672,
0x266471,
0x26646F,
0x296570,
0x296570,
0x26646F,
0x256571,
0x246672,
0x236973,
0x216975,
0x206A77,
0x1E6A77,
0x1F6B79,
0x1E6B7B,
0x1D6C7B,
0x1E6D7E,
0x1C6D7E,
0x1C6F81,
0x1B7182,
0x1B7182,
0x1B7285,
0x1C7386,
0x1C7386,
0x1C7587,
0x1C7488,
0x1B7689,
0x1B7689,
0x1B788A,
0x1B778C,
0x1C788D,
0x1C788D,
0x1D798E,
0x1E7A8F,
0x1F7B90,
0x207C91,
0x217D92,
0x1E8395,
0x268088,
0x2B838D,
0x22849F,
0x1D85A2,
0x358783,
0x689159,
0x8C8B31,
0x96821F,
0xAD9B35,
0x9F942B,
0x928827,
0x8C7F31,
0x8A7A2D,
0xA7962C,
0xD0C132,
0xCCB52B,
0xC6B13E,
0xA29135,
0x9B9235,
0xBAB54F,
0xA19A32,
0x988826,
0xB7A03A,
0xCBAE38,
0xC9AB29,
0xCBAD2B,
0xCDB032,
0xCEB235,
0xC4AA2F,
0xB69B2A,
0xAE912B,
0x998E32,
0x948937,
0x7D7230,
0x8E8453,
0xC4BB9A,
0xD5CFB7,
0xCCC7B3,
0xCBC9B2,
0xD2D2B8,
0xD2D1B5,
0xD0CCB1,
0xD1CBB5,
0x9C9080,
0x423429,
0x4B3A32,
0x3D2A24,
0x443724,
0x3F2F20,
0x3D3027,
0x393124,
0x3A3316,
0x776A33,
0x8A7D2F,
0x807E29,
0x888B34,
0x8E8027,
0xAB8733,
0xBF975A,
0x321A02,
0x26171C,
0x7A5A69,
0x884C58,
0x734744,
0x673F48,
0x573140,
0x5E3637,
0x623E32,
0x4C352F,
0x342A32,
0x090008,
0x493423,
0xD2B47E,
0xC1A84E,
0xB4A435,
0xAFA92F,
0xA5A42E,
0xB0B54F,
0x6A741D,
0x37301E,
0x423930,
0x322922,
0x433930,
0x403225,
0x433324,
0x5C4938,
0x928068,
0xCBBC9B,
0xCFC29F,
0xCDC4A5,
0xC0BB9E,
0xBDB89A,
0xBCB895,
0xC1BB9B,
0xC5BBA0,
0xCABDAD,
0xC8BCAC,
0xC4BFAC,
0xBABA9E,
0x818459,
0x525415,
0x878636,
0x918A32,
0x94862D,
0x96862E,
0x9C8B31,
0xA1912F,
0xB09F37,
0xB39E33,
0xB59E34,
0xBBA037,
0xBDA02B,
0xAA9D34,
0x9D9938,
0x9A8A28,
0xA78B2A,
0x897D31,
0x67835B,
0x559A97,
0x4199AF,
0x3D97B9,
0x3A93BB,
0x3593B9,
0x3192B2,
0x3292A8,
0x2F90A0,
0x2B8F9F,
0x2E8DA3,
0x2C8BA1,
0x2B8AA0,
0x29889E,
0x27869C,
0x25859B,
0x238399,
0x228298,
0x208096,
0x1F7F95,
0x1E7E94,
0x1E7D93,
0x1D7C90,
0x1C7B8F,
0x1B7A8E,
0x1A798D,
0x1B778C,
0x1B778C,
0x1B7689,
0x1C7587,
0x1C7587,
0x1C7386,
0x1B7283,
0x1B7285,
0x1B7084,
0x1B7084,
0x1A7081,
0x1B6E80,
0x1B6C7D,
0x1D6C7B,
0x1E6C79,
0x1D6B78,
0x1E6A77,
0x206A77,
0x206876,
0x236774,
0x246672,
0x256571,
0x26646F,
0x296570,
0x296570,
0x26646F,
0x256571,
0x236772,
0x236975,
0x216975,
0x206A77,
0x1E6A77,
0x1E6C79,
0x1E6B7B,
0x1D6C7B,
0x1E6D7E,
0x1C6D80,
0x1C6F81,
0x1B7182,
0x1B7182,
0x1B7285,
0x1C7386,
0x1D7487,
0x1C7587,
0x1C7488,
0x1B7689,
0x1C778A,
0x1B778C,
0x1B778C,
0x1C788D,
0x1B7A8E,
0x1B7A8E,
0x1C7B8F,
0x1F7B90,
0x207C91,
0x217D92,
0x1E829A,
0x238294,
0x288493,
0x2A8296,
0x29849F,
0x288499,
0x418884,
0x618057,
0x7F7B31,
0x8C7F1F,
0x817E21,
0x6F6D23,
0x7B6732,
0x896A29,
0xAD942F,
0xC4BC33,
0xC6AF2D,
0xB6A134,
0x9E8E2F,
0xB4AA48,
0xB3AB3C,
0xA39926,
0xA9992A,
0xB7A02E,
0xC4A82D,
0xC5A72B,
0xC4A433,
0xC2A63B,
0xC2A638,
0xBBA22E,
0xB3982D,
0xA78A2E,
0x8E8635,
0x786E29,
0x908452,
0xD3C6A6,
0xD4C9B3,
0xC1B9A4,
0xBCB69C,
0xC3C09D,
0xC9C7A1,
0xCFCDA7,
0xCCC7A7,
0xCAC4AA,
0x867E6B,
0x3D3123,
0x43352A,
0x423129,
0x35271C,
0x3E3120,
0x342917,
0x322A1D,
0x322918,
0x53471F,
0x8B823F,
0x7A7726,
0x858231,
0x90822A,
0xA08124,
0xC2A047,
0xA98F54,
0x200800,
0x491F20,
0x874851,
0x8F5547,
0x9C6060,
0xA46266,
0xB46E66,
0xBA7967,
0x9C655E,
0x613739,
0x593623,
0xCCAD74,
0xCDB55B,
0xA49834,
0xA29D35,
0xABA533,
0xAAA736,
0x969B41,
0x545F23,
0x4A3E2E,
0x372D23,
0x352C25,
0x3D342B,
0x463C30,
0x423223,
0x3F2C1D,
0x8D7961,
0xC9B695,
0xC7B994,
0xC1B897,
0xB3AF92,
0xB0AF91,
0xAFAF8D,
0xB7B495,
0xBAB49C,
0xCEB2A7,
0xC8B3A0,
0xC9BEA2,
0xB5B293,
0x808064,
0x323313,
0x5D5C2C,
0x86833C,
0x878129,
0x8C8324,
0x978D2E,
0x9D8E33,
0xA89537,
0xB69F3F,
0xC1A844,
0xB89C39,
0xB3992A,
0xB09E2E,
0xA5972A,
0x99892A,
0x7F793B,
0x64846D,
0x4F9CA4,
0x3C9FB2,
0x4099AB,
0x4A91A5,
0x4B8EAB,
0x4190B1,
0x3892AC,
0x3592A1,
0x368F9D,
0x358DA1,
0x2E8DA3,
0x2D8CA2,
0x2B8AA0,
0x29889E,
0x28879D,
0x25859B,
0x24849A,
0x238399,
0x218197,
0x208096,
0x1F7F95,
0x1E7D93,
0x1D7C90,
0x1C7B8F,
0x1B7A8E,
0x1A798D,
0x1C788D,
0x1B778C,
0x1B7689,
0x1A7587,
0x1C7587,
0x1C7386,
0x1B7285,
0x1B7285,
0x1A7184,
0x1B7084,
0x1A7081,
0x1B6E80,
0x1B6C7D,
0x1D6C7B,
0x1E6C79,
0x1E6C79,
0x1E6A77,
0x206A77,
0x216977,
0x236975,
0x246672,
0x256571,
0x26646F,
0x276570,
0x276570,
0x266471,
0x256571,
0x236772,
0x236975,
0x216975,
0x206A77,
0x1E6A78,
0x1E6C79,
0x1E6B7B,
0x1D6C7D,
0x1C6D7E,
0x1D6E81,
0x1C6F81,
0x1B7182,
0x1C7283,
0x1C7386,
0x1C7386,
0x1C7587,
0x1D7589,
0x1B7689,
0x1B7689,
0x1B778C,
0x1C788D,
0x1C788D,
0x1B7A8E,
0x1C7B8F,
0x1C7B8F,
0x1C7B8F,
0x1D7C90,
0x1F7E92,
0x207F93,
0x207E9A,
0x1F819A,
0x228394,
0x2C828F,
0x33859D,
0x2884A9,
0x2C89AB,
0x43868C,
0x608162,
0x637B49,
0x597A4D,
0x59754D,
0x766F3B,
0x9C7526,
0xB9922D,
0xBDA737,
0xAE972D,
0x9A841F,
0xAF9B38,
0xBAAC41,
0xADA02D,
0xB3A42F,
0xC1B03C,
0xB5A128,
0xB79F21,
0xB79E28,
0xB79B37,
0xB3993C,
0xAB932F,
0xAD982D,
0xAA9431,
0x97812B,
0x767233,
0x908955,
0xC8BE9A,
0xBFB399,
0xBFB39D,
0xBFB59A,
0xBFB695,
0xB7AF8B,
0xC3BB97,
0xC8BF9E,
0xC3BA9B,
0xBCB598,
0x70694F,
0x3B331E,
0x3A3122,
0x3E342A,
0x392A25,
0x342B1A,
0x37311B,
0x352B1F,
0x44372F,
0x342D11,
0x6D6C33,
0x818236,
0x8B8434,
0x8D7C2C,
0x968126,
0x9D8626,
0xC6AC57,
0xB08D55,
0x5B2E11,
0x562214,
0x642F1F,
0x72362B,
0x854138,
0x8D4435,
0x8E4630,
0x7D3B23,
0x8E5635,
0xD5AB6F,
0xD1B85E,
0xA19831,
0x9B9C36,
0x9FA038,
0xABA739,
0xA29D39,
0x666524,
0x555A3A,
0x554835,
0x413527,
0x453B31,
0x494134,
0x463D2E,
0x4F4232,
0x433022,
0x6E5841,
0xBDA988,
0xBCAD86,
0xB4A784,
0xA9A381,
0xA9A484,
0xADA986,
0xB6AF92,
0xBAB097,
0xC3ADA2,
0xBBAA96,
0xB6AD8E,
0xA49F81,
0x76745D,
0x555442,
0x3D3C20,
0x7B7843,
0x8D8B38,
0x8B8620,
0x90851C,
0x99892A,
0x9F8933,
0xA28831,
0xAD9334,
0xB49834,
0xAB9831,
0xA9972B,
0x9B8923,
0x898138,
0x8A9E85,
0x5997AC,
0x3B9DC2,
0x369EB7,
0x459CA3,
0x51949C,
0x4F90A4,
0x4592AC,
0x3B93A7,
0x36929F,
0x368FA1,
0x378DA8,
0x308CA3,
0x2D8CA2,
0x2C8BA1,
0x2A899F,
0x28879D,
0x26869C,
0x24849A,
0x238399,
0x218197,
0x208096,
0x1F7F95,
0x1F7E94,
0x1E7D93,
0x1D7C90,
0x1B7A8E,
0x1B7A8E,
0x1C788D,
0x1B778C,
0x1B7689,
0x1B7689,
0x1C7587,
0x1D7487,
0x1C7386,
0x1B7285,
0x1A7184,
0x1B7084,
0x1B7182,
0x1B6E80,
0x1C6D7E,
0x1E6D7C,
0x1E6B7B,
0x1E6C79,
0x1D6B77,
0x1E6A77,
0x206A77,
0x236975,
0x236774,
0x256571,
0x25656F,
0x276570,
0x276572,
0x256571,
0x236571,
0x236772,
0x236975,
0x206A75,
0x1E6A77,
0x1E6A78,
0x1E6C79,
0x1E6B7B,
0x1D6C7D,
0x1C6D7E,
0x1C6F81,
0x1D7082,
0x1B7182,
0x1C7185,
0x1C7386,
0x1D7488,
0x1C7488,
0x1D7589,
0x1B7689,
0x1C778C,
0x1C788D,
0x1C788D,
0x1A798D,
0x1B7A8E,
0x1D7C92,
0x1D7C92,
0x1E7D93,
0x1F7E94,
0x207F95,
0x228197,
0x27819C,
0x21839C,
0x228598,
0x2C8490,
0x328597,
0x2983A5,
0x288AB1,
0x3189A1,
0x3E878E,
0x468D93,
0x398791,
0x4F9088,
0x718756,
0x9C842C,
0xAF8425,
0xAE852D,
0x978229,
0x9E8729,
0xBDA741,
0xB7A132,
0xB29D2A,
0xB8A431,
0xC6B53F,
0xBAAA31,
0xA89723,
0xA6932C,
0xA59033,
0xA59035,
0x9E8B2C,
0x9F9130,
0x93882F,
0x8A7F30,
0x939065,
0xC7C29C,
0xCFC4A6,
0xB8AD91,
0xB7AC90,
0xB7AC8C,
0xB4A989,
0xB5A888,
0xB7A78D,
0xBDAD94,
0xBEB095,
0xADA485,
0x565133,
0x38321A,
0x332C1C,
0x372E25,
0x463636,
0x2A2418,
0x383725,
0x332A21,
0x40312C,
0x352E1B,
0x444B18,
0x80843A,
0x857C2D,
0x928034,
0x877729,
0x91822D,
0xA68E38,
0xC7A156,
0xCEA66B,
0xAD8758,
0x8A6048,
0x714329,
0x744325,
0x744126,
0x8E5B3C,
0xC09060,
0xE8C07A,
0xCAB25C,
0x938E30,
0x949A38,
0x989F37,
0x989A2C,
0x9E9B34,
0x7B772D,
0x5D5832,
0x655F53,
0x443520,
0x423524,
0x453C2B,
0x524B38,
0x514936,
0x584B3B,
0x4D3A2C,
0x5F4934,
0xAC9573,
0xB49F74,
0xAE9B71,
0xA79871,
0xA69871,
0xAD9F78,
0xB5A583,
0xB5A488,
0xAEA78D,
0xAEA78D,
0xAAA387,
0x888366,
0x69654A,
0x827D67,
0x3F3D24,
0x4D4B22,
0x807F3C,
0x999639,
0x999027,
0x91821D,
0x968128,
0xA38B35,
0xA89134,
0xA38D27,
0x9A933B,
0x897D27,
0x74681E,
0xC4C397,
0xBCD9D5,
0x5697B5,
0x44A0C5,
0x40A0B6,
0x429BA1,
0x4598A0,
0x3F96A9,
0x3796B2,
0x3595AE,
0x3594AA,
0x3591AA,
0x348EB0,
0x318DA4,
0x308CA3,
0x2C8BA1,
0x2B8AA0,
0x29889E,
0x26869C,
0x24849A,
0x238399,
0x228298,
0x218197,
0x1F7F95,
0x1E7E94,
0x1E7D93,
0x1D7C90,
0x1C7B8F,
0x1B7A8E,
0x1A798D,
0x1B778C,
0x1B788A,
0x1B7689,
0x1D7688,
0x1C7587,
0x1C7386,
0x1B7285,
0x1A7184,
0x1B7084,
0x1B7084,
0x1C6F81,
0x1C6D7E,
0x1E6D7C,
0x1F6C7C,
0x1E6C79,
0x1D6B78,
0x1E6A77,
0x206A77,
0x216975,
0x236774,
0x236571,
0x25656F,
0x276570,
0x276572,
0x266672,
0x246672,
0x246873,
0x216975,
0x206A77,
0x1E6A77,
0x1D6B78,
0x1E6B7B,
0x1D6C7B,
0x1D6C7D,
0x1C6D7E,
0x1C6F81,
0x1B7182,
0x1C7185,
0x1B7285,
0x1C7387,
0x1C7488,
0x1D7589,
0x1D7589,
0x1B7689,
0x1C778C,
0x1C788D,
0x1C788D,
0x1A798D,
0x1C7B91,
0x1D7C92,
0x1F7E94,
0x1F7E94,
0x207F95,
0x228197,
0x248399,
0x208396,
0x218197,
0x288499,
0x2A8597,
0x298697,
0x2A869B,
0x318DA6,
0x2F8A9F,
0x328EA3,
0x2D8FAA,
0x2F92B2,
0x3A91A4,
0x56917D,
0x768A4F,
0x928234,
0x9C7726,
0x998628,
0xB5A13E,
0xAE972F,
0xAF942B,
0xB99F31,
0xBDA833,
0xB9A92F,
0xB2A533,
0x998C2C,
0x958633,
0x91822D,
0x95882C,
0x968B31,
0x8F8838,
0x767131,
0x7D7A43,
0xC1BE9D,
0xC6C0A0,
0xBEB395,
0xC3B598,
0xB8AB8B,
0xADA07E,
0xAA9D7A,
0xAFA17C,
0xAC9C78,
0xB2A280,
0xB5A685,
0x94896D,
0x352F17,
0x2D2614,
0x352D20,
0x3F352B,
0x392A27,
0x28231D,
0x35362E,
0x322B23,
0x2D1E17,
0x4A4331,
0x373B16,
0x676B2E,
0x878135,
0x897B30,
0x837630,
0x867837,
0x927E37,
0x9B7E32,
0xAB8B3E,
0xBCA056,
0xCAA97C,
0xD0AF7C,
0xD3B47E,
0xCCB07E,
0xDABF8A,
0xCEB575,
0xA59347,
0x84822F,
0x838C39,
0x8B953C,
0x949631,
0x9F9B37,
0x7D7E2E,
0x4F511F,
0x7B775C,
0x6B5F4F,
0x584934,
0x332613,
0x4C412D,
0x463B25,
0x574C36,
0x4B3E2E,
0x3D2B21,
0x422D1C,
0x94805F,
0xA79066,
0xA79064,
0xA79064,
0xA68F63,
0xB0996F,
0xB59D77,
0xB2997A,
0xA7A27A,
0xA8A07C,
0xA29779,
0x786D4F,
0x716847,
0x777350,
0x80805E,
0x494B24,
0x4D4F1D,
0x767733,
0x979240,
0xA0973C,
0x91862A,
0x8F8128,
0x8F842A,
0x8B8227,
0x6C7232,
0x504F17,
0x7C7549,
0xD4D2B9,
0xBFD6D0,
0x68A0AF,
0x4B9CB7,
0x429EB5,
0x429EAB,
0x3E9FA8,
0x369DAC,
0x319BB3,
0x3496B3,
0x3892AD,
0x3590AB,
0x2E90AD,
0x328EA3,
0x318DA2,
0x2D8CA2,
0x2B8AA0,
0x2A899F,
0x28879D,
0x25859B,
0x24849A,
0x228298,
0x218197,
0x1F7F95,
0x1E7E94,
0x1E7D93,
0x1D7C92,
0x1C7B8F,
0x1B7A8E,
0x1A798D,
0x1C788D,
0x1B788A,
0x1B7689,
0x1D7688,
0x1C7587,
0x1C7386,
0x1C7386,
0x1A7184,
0x1A7184,
0x1B7084,
0x1C6F81,
0x1C6D7E,
0x1E6D7E,
0x1F6C7C,
0x1E6C79,
0x1D6B78,
0x1D6B78,
0x206A77,
0x216977,
0x236774,
0x236571,
0x256571,
0x266670,
0x266672,
0x256571,
0x246672,
0x226874,
0x216975,
0x1F6976,
0x1E6A77,
0x1E6C79,
0x1F6C7C,
0x1D6C7B,
0x1D6C7D,
0x1C6D80,
0x1C6F81,
0x1B7182,
0x1C7185,
0x1A7184,
0x1B7387,
0x1C7488,
0x1C7488,
0x1B7689,
0x1B768B,
0x1B778C,
0x1C788D,
0x1A798D,
0x1B7A90,
0x1C7B91,
0x1D7C92,
0x1E7D93,
0x207F95,
0x218096,
0x238298,
0x248399,
0x1E8898,
0x268599,
0x2B839B,
0x29859C,
0x258A9C,
0x278B9A,
0x318A9A,
0x328B99,
0x2F8D97,
0x33939F,
0x3791A9,
0x3992B2,
0x3C90AC,
0x519190,
0x6E8A5A,
0x898433,
0x9C8F27,
0x9E8D25,
0xA9942B,
0xAE9530,
0xB59D33,
0xBEAA37,
0xB8AC36,
0xA79B37,
0x908438,
0x837932,
0x817A29,
0x868126,
0x837D2F,
0x68632C,
0x42401A,
0x777657,
0xBCB799,
0xB9B08F,
0xB8A988,
0xBCAC8B,
0xB8A98A,
0xAB9E7E,
0xA39771,
0x9F9361,
0xA4955E,
0xAE9E6A,
0xAFA077,
0x7D7159,
0x31271D,
0x2F261F,
0x33291F,
0x382A1D,
0x35271E,
0x2C2421,
0x262521,
0x353128,
0x261A0E,
0x443629,
0x362F1C,
0x46471F,
0x7A7B39,
0x837C2E,
0x82762E,
0x847639,
0x847438,
0x8D7C36,
0x907A2E,
0x8B7426,
0x93792F,
0x9F8339,
0xA1853B,
0x9C863D,
0x907F39,
0x7E702F,
0x776F30,
0x727232,
0x788337,
0x818933,
0x939238,
0x88843A,
0x434715,
0x697255,
0x7F8169,
0x4B4427,
0x493A25,
0x40311E,
0x3E2F18,
0x4A3E24,
0x43371F,
0x3E3224,
0x382922,
0x302217,
0x77674D,
0x9C8A62,
0xA08B5E,
0xA48D61,
0xA48C60,
0xA99268,
0xB09872,
0xAE9777,
0xA89A6B,
0xA7996C,
0x9F8D65,
0x786642,
0x7E704D,
0x7F7752,
0x7E7C53,
0x8E9166,
0x61633C,
0x464620,
0x575626,
0x7D7C3B,
0x8E8D3B,
0x868834,
0x7D7F34,
0x6A6C2B,
0x4D5C33,
0x525635,
0xB2AB8F,
0xD7CCB6,
0xC5D1C3,
0x72A3A8,
0x4B9DB3,
0x41A1BA,
0x429EAD,
0x429DA2,
0x3D9BA3,
0x3A99AB,
0x3D95AD,
0x3F91A7,
0x3890A4,
0x2F92A5,
0x328EA3,
0x318DA2,
0x2D8CA2,
0x2B8AA0,
0x2A899F,
0x28879D,
0x26869C,
0x24849A,
0x238399,
0x218197,
0x1F7F95,
0x1E7E94,
0x1E7D93,
0x1D7C92,
0x1C7B8F,
0x1B7A8E,
0x1B7A8E,
0x1C788D,
0x1B778C,
0x1B7689,
0x1D7688,
0x1C7587,
0x1C7386,
0x1C7386,
0x1B7285,
0x1A7184,
0x1B7084,
0x1C6F81,
0x1C6D80,
0x1E6D7E,
0x1D6C7B,
0x1E6B7B,
0x1D6B78,
0x1D6B78,
0x1E6A77,
0x216977,
0x226874,
0x236571,
0x236571,
0x266670,
0x266672,
0x256571,
0x236772,
0x226874,
0x216975,
0x1F6976,
0x1E6A77,
0x1E6C79,
0x1E6D7C,
0x1D6C7B,
0x1B6C7D,
0x1C6D80,
0x1C6F81,
0x1B7084,
0x1C7185,
0x1B7285,
0x1B7387,
0x1C7488,
0x1C7488,
0x1B7689,
0x1B768B,
0x1B778C,
0x1C788D,
0x1B7A8E,
0x1B7A90,
0x1C7B91,
0x1D7D93,
0x1E7E94,
0x1F7F95,
0x228197,
0x238298,
0x248399,
0x22869D,
0x29839B,
0x2D839A,
0x28879D,
0x258A9E,
0x2A8B9C,
0x2F8A9C,
0x338C9A,
0x379199,
0x3C9194,
0x3F8D9A,
0x3D8DB2,
0x3591C4,
0x3E94B5,
0x5B9483,
0x7E965A,
0x90852B,
0x998C2C,
0xA49530,
0xA18F2B,
0xA5932F,
0xA1922B,
0x9F9130,
0x8B812C,
0x827A31,
0x736D21,
0x73731D,
0x787929,
0x6D6C33,
0x474425,
0x44422D,
0x78765F,
0xA8A282,
0xB7AD8A,
0xB4A480,
0xAE9B7A,
0xAE9D81,
0xA6987D,
0x9C8F6D,
0x9D8F62,
0x9B8B50,
0xA7975C,
0xA59469,
0x645842,
0x291E1A,
0x312726,
0x362923,
0x342619,
0x362918,
0x352B22,
0x2E2623,
0x292218,
0x392D1D,
0x342218,
0x523F3B,
0x322918,
0x5C5E2D,
0x848238,
0x817426,
0x867530,
0x877E3D,
0x75702C,
0x7F722C,
0x8F7633,
0x847826,
0x867120,
0x8B7023,
0x876D23,
0x7D6A28,
0x72662A,
0x696025,
0x6C6727,
0x79772D,
0x8B8C3E,
0x80813B,
0x404211,
0x4D4F37,
0x919586,
0x5A5C46,
0x50502E,
0x3F2F1F,
0x3A2A1A,
0x473622,
0x716148,
0x493A25,
0x35281F,
0x342828,
0x3F362F,
0x574F38,
0x918461,
0x94855C,
0x948157,
0x9D875E,
0xA18E64,
0xA18F69,
0xA39372,
0xA9966E,
0xA28F67,
0x9C8863,
0x735F3E,
0x7B6847,
0x887A53,
0x807548,
0x7E754A,
0x938B67,
0x877E61,
0x666040,
0x535023,
0x575B1E,
0x555E1D,
0x465119,
0x404C1E,
0x4E573A,
0x98977B,
0xC3B59A,
0xCDC1A7,
0xC0CDBC,
0x6AA1A4,
0x499FB6,
0x43A3BB,
0x469EAA,
0x4A9B9F,
0x4699A1,
0x4197A8,
0x4293A8,
0x4291A2,
0x3B91A0,
0x32929E,
0x338FA4,
0x318DA2,
0x2F8BA0,
0x2B8AA0,
0x2A899F,
0x29889E,
0x26869C,
0x24849A,
0x24849A,
0x228298,
0x208096,
0x1F7F95,
0x1F7E94,
0x1E7D93,
0x1D7C92,
0x1C7B91,
0x1B7A90,
0x1A798D,
0x1B778C,
0x1B7689,
0x1B7689,
0x1C7587,
0x1C7587,
0x1C7386,
0x1B7285,
0x1B7285,
0x1B7084,
0x1B7182,
0x1C6F81,
0x1C6D7E,
0x1E6D7C,
0x1F6C7C,
0x1C6B78,
0x1D6B78,
0x1E6A77,
0x216977,
0x226874,
0x226671,
0x236571,
0x266670,
0x266672,
0x236571,
0x236772,
0x226874,
0x216975,
0x1E6A77,
0x1D6B78,
0x1E6C79,
0x1D6C7B,
0x1D6C7D,
0x1C6D7E,
0x1C6D80,
0x1C6F81,
0x1B7084,
0x1B7285,
0x1B7285,
0x1B7387,
0x1C7488,
0x1B7689,
0x1B7689,
0x1B778C,
0x1B778C,
0x1A798D,
0x1B7A90,
0x1C7B91,
0x1C7C92,
0x1D7D93,
0x1E7E94,
0x208096,
0x218197,
0x238399,
0x24849A,
0x2B81A2,
0x2C849C,
0x2C879A,
0x29889C,
0x2A88A1,
0x2E88A0,
0x328A9E,
0x2F8EA0,
0x3091A4,
0x368FA1,
0x408EA4,
0x408FAE,
0x3A94B9,
0x3D91AD,
0x639A97,
0x91A88C,
0x8A7A47,
0x847A33,
0x90892E,
0x928A27,
0x90872A,
0x928634,
0x807229,
0x82762C,
0x6F6817,
0x737216,
0x75781F,
0x717431,
0x50502C,
0x353221,
0x6A6551,
0x6F6A4C,
0x999677,
0xB1A984,
0xB6A77E,
0xAD9B73,
0xA89575,
0xA09079,
0x9A8A73,
0x988567,
0xA28B62,
0xA38C60,
0x9E8A67,
0x554633,
0x2D241F,
0x312829,
0x2B201E,
0x33261E,
0x493D25,
0x514336,
0x3C2F27,
0x31271B,
0x2C2010,
0x342017,
0x361D20,
0x3A2824,
0x3C381B,
0x717037,
0x8D8439,
0x877A2B,
0x847D2D,
0x7B792E,
0x766D28,
0x796526,
0x71712B,
0x7D752C,
0x85742E,
0x7E662A,
0x745F2A,
0x695E20,
0x686216,
0x797221,
0x8D853C,
0x868245,
0x3C400E,
0x5B613D,
0xA5A28F,
0x877E75,
0x3E3727,
0x2E2C13,
0x32251C,
0x413328,
0x392919,
0x6D5D46,
0x635441,
0x44372F,
0x322829,
0x29221C,
0x4B452F,
0x847C57,
0x8C7F55,
0x917E56,
0x978158,
0x9A875C,
0x9E8D62,
0xA0906C,
0xA18E6D,
0xA69375,
0xA08F75,
0x79694F,
0x776746,
0x84724C,
0x8F7950,
0x8E754D,
0x846A45,
0x9B835D,
0x99865E,
0x8C8056,
0x847F55,
0x84855B,
0x7E855B,
0x858D64,
0xA9A381,
0xB2A481,
0xB9A685,
0xC1BDA4,
0xA8C6BE,
0x5F9FA9,
0x499EB2,
0x48A0B4,
0x479DAC,
0x469CAB,
0x439AAE,
0x4098B0,
0x4094AC,
0x3E93A7,
0x3A93A1,
0x33939F,
0x3490A5,
0x328EA3,
0x308CA1,
0x2C8BA1,
0x2B8AA0,
0x29889E,
0x26869C,
0x25859B,
0x24849A,
0x228298,
0x208096,
0x1F7F95,
0x1E7E94,
0x1E7D93,
0x1D7C92,
0x1C7B91,
0x1B7A90,
0x1A798D,
0x1B778C,
0x1B7689,
0x1B7689,
0x1D7688,
0x1C7587,
0x1B7486,
0x1B7285,
0x1B7285,
0x1A7184,
0x1A6F83,
0x1C6F81,
0x1C6D7E,
0x1E6D7C,
0x1E6B7B,
0x1D6C79,
0x1D6B78,
0x1E6A77,
0x206A77,
0x226874,
0x226673,
0x236571,
0x266670,
0x266672,
0x236571,
0x236774,
0x226874,
0x206A75,
0x1E6A77,
0x1D6B78,
0x1E6C79,
0x1D6C7B,
0x1D6C7D,
0x1C6D7E,
0x1D6E81,
0x1C6F81,
0x1B7084,
0x1B7285,
0x1B7285,
0x1C7488,
0x1C7488,
0x1B7689,
0x1C778A,
0x1B778C,
0x1C788D,
0x1A798D,
0x1B7A90,
0x1B7B91,
0x1C7C92,
0x1D7D93,
0x1F7F95,
0x208096,
0x228298,
0x238399,
0x24849A,
0x2D80A2,
0x2D859B,
0x2A8A98,
0x298A9D,
0x2C87A4,
0x3286A2,
0x34899D,
0x308FA3,
0x2A8FAB,
0x308EB0,
0x3C91AE,
0x3E93A8,
0x3F98A8,
0x449199,
0x759999,
0xAAACA1,
0x988775,
0x685C32,
0x7A762C,
0x848329,
0x8A8835,
0x89823C,
0x7F7436,
0x736921,
0x7C761E,
0x87862A,
0x7A7D30,
0x565827,
0x323019,
0x575141,
0x7B735C,
0x746B4C,
0x8B8C6C,
0xA29C78,
0xAA9D73,
0xA7946A,
0xA28E6B,
0x98876B,
0x907E66,
0x90785E,
0x9A7D5F,
0xA58866,
0x8F775B,
0x41321F,
0x2F281E,
0x2D2824,
0x2E2623,
0x2B201A,
0x493D23,
0x3C2D1A,
0x433127,
0x43352A,
0x372B1D,
0x433326,
0x3E271F,
0x321E17,
0x3B2F21,
0x484323,
0x79743D,
0x888037,
0x857D2A,
0x827A27,
0x7D7427,
0x7D732B,
0x737134,
0x71722E,
0x6E6B28,
0x73692C,
0x77662E,
0x796F27,
0x868229,
0x8A8833,
0x716C2C,
0x38350C,
0x595C3D,
0xAFB295,
0xA59A84,
0x5B4538,
0x382419,
0x4C4032,
0x312725,
0x332822,
0x514338,
0x5A4A3A,
0x5F4F40,
0x382622,
0x2B1D1D,
0x20160D,
0x473D22,
0x7F7349,
0x8B7A4E,
0x917A50,
0x8F774B,
0x937B4B,
0x9D8756,
0x9F8958,
0xA48D64,
0xA08969,
0x97866C,
0x7A6A51,
0x7A6B4C,
0x887650,
0x927B52,
0x997C54,
0x97764B,
0x977849,
0x957B48,
0x927D4E,
0x8B7C55,
0x928562,
0x988F68,
0x9A9264,
0x90845A,
0xA89366,
0xBFAB86,
0xBEC7B4,
0x85B5B7,
0x539CAB,
0x53A2B3,
0x4F9DB1,
0x479DB4,
0x3F9EBA,
0x3A9CB7,
0x3B99B3,
0x3F96B2,
0x3B95AF,
0x3895A7,
0x3494A0,
0x3490A5,
0x338FA4,
0x318DA2,
0x2D8CA0,
0x2B8AA0,
0x2A899F,
0x27879D,
0x25859B,
0x24849A,
0x228298,
0x208096,
0x1F7F95,
0x1E7E94,
0x1E7D93,
0x1D7C92,
0x1C7B91,
0x1B7A90,
0x1A798D,
0x1B778C,
0x1B7689,
0x1B7689,
0x1D7589,
0x1C7587,
0x1C7587,
0x1B7286,
0x1A7185,
0x1A7184,
0x1A6F83,
0x1B6E80,
0x1C6D7E,
0x1D6C7B,
0x1E6B7B,
0x1D6C79,
0x1D6B78,
0x1E6A78,
0x206A77,
0x226874,
0x226673,
0x236571,
0x246672,
0x246672,
0x236571,
0x236774,
0x206874,
0x206A77,
0x1E6A77,
0x1D6B78,
0x1D6C79,
0x1D6C7B,
0x1E6D7E,
0x1C6D7E,
0x1C6F81,
0x1D6F84,
0x1B7084,
0x1B7285,
0x1C7386,
0x1C7488,
0x1A7588,
0x1B7689,
0x1C778A,
0x1B778C,
0x1C788D,
0x1B7A8E,
0x1B7A8E,
0x1B7B91,
0x1C7C92,
0x1E7E94,
0x1F7F95,
0x208096,
0x228298,
0x238399,
0x24849A,
0x28839E,
0x2B8698,
0x2D8998,
0x2A8AA2,
0x2788A9,
0x2E87A5,
0x37899E,
0x358E9E,
0x3291A7,
0x358FA9,
0x3D8FA4,
0x3B93A1,
0x3B9BAB,
0x45939F,
0x7A9994,
0xB6A794,
0xA69687,
0x7E7353,
0x66612A,
0x787830,
0x828238,
0x83803B,
0x847D39,
0x877F36,
0x90893B,
0x746F2B,
0x4D4919,
0x363314,
0x565239,
0x726C54,
0x736854,
0x776A59,
0x858664,
0x9F9975,
0xA3956E,
0x9E8B61,
0xA18E66,
0x93845B,
0x85764D,
0x8A754A,
0x987D50,
0x9E8159,
0x8C7355,
0x352613,
0x312A20,
0x312E27,
0x2B271E,
0x352D22,
0x362A14,
0x382916,
0x322215,
0x392A23,
0x3C2F27,
0x322616,
0x33240F,
0x4C3B2B,
0x3D2F26,
0x322C20,
0x4A492B,
0x79733F,
0x897C36,
0x7F6F21,
0x807525,
0x73701F,
0x826F2B,
0x78712B,
0x727429,
0x7D7B2E,
0x8B8031,
0x968B3C,
0x8B873D,
0x646625,
0x323103,
0x5E5A3D,
0xB9B4A1,
0xAEA795,
0x5F5039,
0x523C25,
0x503927,
0x352116,
0x373332,
0x322D2A,
0x332A23,
0x3A2C1F,
0x3D2D20,
0x3E2B27,
0x2E1C1A,
0x332316,
0x3D2E0F,
0x7D6E43,
0x847348,
0x87714C,
0x8A724C,
0x8D7547,
0x947D4B,
0x97804C,
0xA38350,
0xA78B5C,
0x9A835A,
0x897655,
0x867756,
0x837352,
0x917F59,
0x978257,
0x927C4D,
0x97814F,
0x9B8754,
0x998553,
0x978253,
0x927C4D,
0x937B4D,
0x957A4B,
0x918E61,
0xAE9F74,
0xC0B393,
0xA8BCB0,
0x67A3AB,
0x51A2B3,
0x509EB2,
0x53A0BA,
0x479EBC,
0x3F9EBC,
0x3E9DB1,
0x4299AC,
0x4195AF,
0x3D94B2,
0x3894A9,
0x38959D,
0x3591A6,
0x338FA4,
0x318DA2,
0x2D8CA0,
0x2C8BA1,
0x2A899F,
0x27879D,
0x26869C,
0x25859B,
0x238399,
0x218197,
0x208096,
0x1F7F95,
0x1E7E94,
0x1D7C92,
0x1C7B91,
0x1A798F,
0x1A798F,
0x1B778C,
0x1A7789,
0x1B7689,
0x1D7589,
0x1C7488,
0x1B7387,
0x1A7286,
0x1B7286,
0x1A7184,
0x1B7084,
0x1C6F81,
0x1C6D7E,
0x1E6D7C,
0x1E6D7C,
0x1D6C79,
0x1D6B78,
0x1E6A78,
0x206A77,
0x226874,
0x226673,
0x236571,
0x246672,
0x246672,
0x236571,
0x236774,
0x206874,
0x206A77,
0x1E6A77,
0x1D6B78,
0x1D6C7B,
0x1D6C7B,
0x1C6D7E,
0x1D6E81,
0x1C6F81,
0x1D6F84,
0x1B7084,
0x1B7285,
0x1C7386,
0x1B7387,
0x1A7588,
0x1B7689,
0x1C778A,
0x1B778C,
0x1C788D,
0x1B7A8E,
0x1C7B8F,
0x1C7C92,
0x1D7D93,
0x1E7E94,
0x1F7F95,
0x208096,
0x228298,
0x238399,
0x25859B,
0x208A9A,
0x2B8899,
0x31889B,
0x2B89A3,
0x268BA9,
0x2B8AA6,
0x348B9F,
0x398C9E,
0x38909C,
0x3E9197,
0x429094,
0x3C92A1,
0x3798B8,
0x4096B1,
0x749B9A,
0xB3AA8B,
0xA79C80,
0xA39878,
0x7F784E,
0x5C5720,
0x6F6B2C,
0x7A7732,
0x807B37,
0x757030,
0x5C551F,
0x423912,
0x4E4429,
0x625A43,
0x766F52,
0x6E6848,
0x786D59,
0x6B5E55,
0x7A7756,
0x978F6B,
0xA18F69,
0x9A845D,
0x917F57,
0x86794D,
0x7E7441,
0x7F7339,
0x907C3F,
0x97804A,
0x87734E,
0x332615,
0x2F2A24,
0x2C2924,
0x2C2419,
0x443927,
0x382A1D,
0x524638,
0x2B1F13,
0x332620,
0x271C18,
0x33291D,
0x332B16,
0x291E0C,
0x372D24,
0x3C3934,
0x27271B,
0x3C371A,
0x756439,
0x86713A,
0x81722F,
0x747124,
0x876921,
0x7E6B27,
0x7A732B,
0x7B7926,
0x847C29,
0x786E29,
0x50491D,
0x343314,
0x77775B,
0xAAA58F,
0x8F8579,
0x4E4138,
0x483B2A,
0x504228,
0x403019,
0x352317,
0x1E1D19,
0x2F2C27,
0x221B13,
0x53493F,
0x403229,
0x513F3B,
0x362221,
0x2E1E11,
0x372C0E,
0x736B46,
0x787252,
0x7B755F,
0x7E7663,
0x7D765A,
0x837B57,
0x817851,
0x8F7749,
0x988251,
0x978253,
0x8B7A4F,
0x8C7C5A,
0x8B7B5A,
0x90815A,
0x928457,
0x8E804F,
0x918354,
0x97895C,
0x9A8A59,
0xA08C57,
0x9F864E,
0xA78955,
0xAD8C5F,
0x999B74,
0xA8A586,
0xB0B7A5,
0x7FA3A1,
0x5A9EA7,
0x55A8B8,
0x51A3B9,
0x4B9CB9,
0x489DBC,
0x479DB4,
0x489AA5,
0x4A97A1,
0x4795A9,
0x3F95B0,
0x3A95AA,
0x3A949D,
0x3591A6,
0x338FA4,
0x318DA2,
0x2F8BA0,
0x2C8BA1,
0x2B8AA0,
0x28889E,
0x26869C,
0x25859B,
0x238399,
0x218197,
0x208096,
0x1F7F95,
0x1E7E94,
0x1D7C92,
0x1C7B91,
0x1B7A90,
0x1A798F,
0x1B778C,
0x1A768B,
0x1B7689,
0x1D7589,
0x1C7488,
0x1C7488,
0x1A7286,
0x1B7286,
0x1A7184,
0x1B7084,
0x1C6F81,
0x1C6D7E,
0x1E6D7C,
0x1D6C7B,
0x1D6C79,
0x1D6B78,
0x1D6977,
0x206A77,
0x226776,
0x236774,
0x236571,
0x246672,
0x246672,
0x236573,
0x226874,
0x206874,
0x206A77,
0x1E6A77,
0x1E6C79,
0x1D6C7B,
0x1E6D7C,
0x1C6D7E,
0x1D6E81,
0x1C6F81,
0x1D6F84,
0x1B7084,
0x1B7285,
0x1C7386,
0x197487,
0x1A7588,
0x1B7689,
0x1C778A,
0x1C788D,
0x1A798D,
0x1B7A8E,
0x1B7C8F,
0x1C7C92,
0x1D7D93,
0x1E7E94,
0x1F7F95,
0x218197,
0x228298,
0x24849A,
0x25859B,
0x1F8B97,
0x2B889A,
0x3286A0,
0x2E8AA3,
0x288DA1,
0x2B8C9F,
0x358BA2,
0x378DA6,
0x358E9C,
0x3F9497,
0x439193,
0x4293A6,
0x3997B9,
0x3F96B4,
0x679A9D,
0xA0AC94,
0xA79D7A,
0xA09673,
0x9F926F,
0x877C4F,
0x67602C,
0x68612A,
0x6F6937,
0x6B663C,
0x6C6644,
0x756B50,
0x7F735B,
0x7E7258,
0x756A4C,
0x807756,
0x6F684B,
0x706B4E,
0x83795E,
0x948564,
0x9A845B,
0x8E794E,
0x7D6E47,
0x776E4D,
0x77734E,
0x766F41,
0x857841,
0x9B8C55,
0x70623B,
0x251D0A,
0x27221F,
0x271F1D,
0x3C2F26,
0x41301C,
0x30211A,
0x40382D,
0x201C11,
0x27221C,
0x332927,
0x302624,
0x483F38,
0x211D12,
0x121206,
0x181A0D,
0x2C2C20,
0x393126,
0x3F2D1F,
0x5B452D,
0x75623A,
0x7C6F3B,
0x856E38,
0x826C3A,
0x7A6836,
0x70652F,
0x676029,
0x595226,
0x645A41,
0x91877B,
0x938B80,
0x736B60,
0x51473D,
0x33291F,
0x241E10,
0x25220F,
0x363022,
0x564941,
0x2A281C,
0x2C281F,
0x201C13,
0x372F24,
0x372D24,
0x392929,
0x261616,
0x392F25,
0x28260F,
0x858A73,
0xA5B3A6,
0xB0C2C6,
0xB0C4CB,
0xABBFC0,
0xA5B9B0,
0x99AB9F,
0x8C9582,
0x7A8066,
0x868664,
0x787049,
0x867750,
0xA18E66,
0x927D52,
0x907E4E,
0x917F4D,
0x8E7C4C,
0x968456,
0xA18C5D,
0xA28E5C,
0xA18B5A,
0xA58F5E,
0xAA9264,
0x9C9674,
0x9CA796,
0x84ABAA,
0x62A0AB,
0x62AEBC,
0x4FA1AF,
0x53A6B8,
0x4EA0B6,
0x4C9EB4,
0x4D9BAF,
0x4C9AA7,
0x4A97A7,
0x4496AB,
0x3E96AC,
0x3A95A8,
0x3995A4,
0x3591A6,
0x3490A5,
0x318DA2,
0x308CA1,
0x2D8CA2,
0x2B8AA0,
0x28889E,
0x27879D,
0x25859B,
0x238399,
0x218197,
0x208096,
0x1F7F95,
0x1E7E94,
0x1D7C92,
0x1C7B91,
0x1B7A90,
0x1B7A90,
0x1C788D,
0x1B778C,
0x1B7689,
0x1D7589,
0x1C7488,
0x1C7488,
0x1B7387,
0x1B7286,
0x1A7185,
0x1A6F83,
0x1C6F81,
0x1C6D7E,
0x1D6C7D,
0x1D6C7B,
0x1D6C79,
0x1D6B78,
0x1D6977,
0x206A77,
0x236877,
0x236774,
0x246672,
0x246672,
0x246672,
0x226673,
0x226874,
0x216975,
0x206A77,
0x1E6A77,
0x1E6C79,
0x1D6C7B,
0x1E6D7C,
0x1C6D7E,
0x1D6E81,
0x1C6F81,
0x1C6E83,
0x1B7084,
0x1B7285,
0x1C7386,
0x197487,
0x1A7588,
0x1B7689,
0x1B788A,
0x1C798B,
0x1A798D,
0x1B7A8E,
0x1A7B8E,
0x1C7C92,
0x1D7D93,
0x1E7E94,
0x208096,
0x218197,
0x228298,
0x24849A,
0x25859B,
0x238B96,
0x2B879E,
0x3285A3,
0x2F8A9F,
0x2B8D96,
0x2F8D99,
0x348CA4,
0x348DAD,
0x3191A9,
0x3896A2,
0x3D8F9B,
0x4495AA,
0x4198B4,
0x4196AB,
0x5A9599,
0x88A5A0,
0xABA080,
0xA49777,
0x9E906D,
0x998A61,
0x96895D,
0x938A61,
0x979073,
0x857F69,
0x807A64,
0x7E7459,
0x827555,
0x817255,
0x7C6E53,
0x7B7052,
0x747148,
0x747442,
0x8C7B69,
0x907D5D,
0x887145,
0x856F40,
0x8F7F5D,
0x98917E,
0xA29F90,
0xACA791,
0xA49A76,
0x9D9064,
0x6C5F3C,
0x1A1301,
0x25201D,
0x372D2E,
0x3C2B23,
0x47301E,
0x30211E,
0x312A22,
0x26261C,
0x222218,
0x2B2623,
0x2F242A,
0x190F17,
0x1A1A1A,
0x21271B,
0x161E09,
0x282715,
0x3A2F29,
0x392628,
0x422D2C,
0x533E2D,
0x6D5839,
0x736D4B,
0x816E4E,
0x836950,
0x7D6B55,
0x746F5B,
0x787762,
0x857D6A,
0x807267,
0x584A49,
0x413736,
0x3C352B,
0x2E2B1A,
0x28291B,
0x2B2E27,
0x1C1C1A,
0x25201D,
0x302A1C,
0x2C281C,
0x272115,
0x332D21,
0x31281F,
0x342828,
0x302426,
0x28231D,
0x202717,
0xA3B8A9,
0xC2DFE3,
0xABD1E8,
0xA4CDEB,
0xA3D0E7,
0xA4D0DD,
0xA2CDD4,
0x9CC7CE,
0x9BBDBE,
0x96A89C,
0x8B8E71,
0x86794D,
0x907A49,
0xA68A5B,
0xA58658,
0x9D8151,
0x977E46,
0x917840,
0x977F4F,
0x9F885F,
0xA6926D,
0xA4956A,
0xA39765,
0xAB9878,
0x8B9F94,
0x67A6B5,
0x55A9C1,
0x51A3B8,
0x5DA9B9,
0x54A2AF,
0x50A2AE,
0x509FAE,
0x4F9CB0,
0x4999B2,
0x4399B4,
0x3E9AB1,
0x3C99AB,
0x3A96AB,
0x3894AB,
0x3591A6,
0x3490A5,
0x328EA3,
0x308CA1,
0x2D8CA2,
0x2B8AA0,
0x29899F,
0x27879D,
0x26869C,
0x24849A,
0x228298,
0x218197,
0x208096,
0x1F7F95,
0x1E7D93,
0x1D7C92,
0x1C7B91,
0x1B7A90,
0x1C788D,
0x1B778C,
0x1C778A,
0x1D7589,
0x1C7488,
0x1C7488,
0x1B7387,
0x1C7387,
0x1B7286,
0x1B7084,
0x1C6F81,
0x1D6E7F,
0x1E6D7E,
0x1E6D7C,
0x1D6C7B,
0x1D6B78,
0x1D6977,
0x206A77,
0x236877,
0x246875,
0x246672,
0x246672,
0x256773,
0x246875,
0x236975,
0x216975,
0x206A77,
0x1E6C79,
0x1E6C79,
0x1D6C7B,
0x1C6D7E,
0x1C6D7E,
0x1C6F81,
0x1D7082,
0x1B7182,
0x1A7184,
0x1B7285,
0x1B7486,
0x1C7488,
0x1D7589,
0x1B7689,
0x1C778A,
0x1B778C,
0x1C788D,
0x1B7A8E,
0x1C7B8F,
0x1D7C92,
0x1E7D93,
0x1F7E94,
0x207F95,
0x228197,
0x248399,
0x25849A,
0x26859B,
0x27869C,
0x29889E,
0x2B8AA0,
0x2D8CA2,
0x2F8BA0,
0x308CA1,
0x328DA2,
0x358DA3,
0x368EA2,
0x3A91A5,
0x3D92A6,
0x4193A8,
0x4596A9,
0x4899AC,
0x4A9BAE,
0x509FB0,
0xBBA375,
0xB49E79,
0xA69675,
0x9C8A66,
0x957F4E,
0x927941,
0x8C7543,
0x837149,
0x847754,
0x8D8159,
0x8D8053,
0x84754A,
0x81714F,
0x7D6E51,
0x796A4B,
0x92845F,
0x7E6E4C,
0x747357,
0x718375,
0x83A5A4,
0x92BEC9,
0x91C4D5,
0x93C8DA,
0x97C6D0,
0xAEC8C7,
0x979588,
0x5C4A34,
0x250F00,
0x2E210E,
0x3A3122,
0x443727,
0x45301F,
0x362920,
0x40332A,
0x271B0F,
0x2E2216,
0x23190D,
0x30261A,
0x3C3226,
0x40382B,
0x332B20,
0x362D24,
0x4C4239,
0x281E15,
0x362C23,
0x2C2218,
0x30261C,
0x4C3F36,
0x655B51,
0x6D6359,
0x6B6155,
0x6C6256,
0x6C6354,
0x6C6354,
0x5C5246,
0x443A2E,
0x30261C,
0x382E25,
0x241A11,
0x443933,
0x3F342E,
0x3D342B,
0x453C33,
0x30271E,
0x23261F,
0x1F1E19,
0x1C1411,
0x3C312D,
0x32251D,
0x3A2C21,
0x35271E,
0x2E2520,
0x1E2223,
0x9DB0B6,
0xBAD9DE,
0xADD0D6,
0xABCCD3,
0xAACCD5,
0xA5CDD5,
0x9DCCD6,
0x94CED2,
0x8FC9CD,
0x93C8CC,
0x95C5C5,
0x85ADA5,
0x738C79,
0x7C8364,
0x998C60,
0xA3854F,
0xAA8143,
0xA87A3C,
0xA17840,
0xA18254,
0xA08D6C,
0x98927A,
0x969785,
0x6AADBD,
0x66ABBA,
0x62AAB9,
0x5FA8B7,
0x5AA6B6,
0x57A4B4,
0x55A4B5,
0x50A1B2,
0x4E9FB2,
0x4B9EB0,
0x499BB0,
0x459AAE,
0x4299AC,
0x3F97AB,
0x3A95A8,
0x3994A7,
0x3792A5,
0x3590A5,
0x338EA3,
0x308CA1,
0x2F8BA0,
0x2E8A9F,
0x2A899F,
0x28879D,
0x25859B,
0x24849A,
0x238399,
0x218197,
0x208096,
0x1E7E94,
0x1D7D93,
0x1D7C92,
0x1C7B91,
0x1B7A90,
0x1C788D,
0x1B778C,
0x1C778C,
0x1D7589,
0x1C7488,
0x1C7488,
0x1B7486,
0x1A7385,
0x1B7285,
0x1B7182,
0x1B7182,
0x1C6F81,
0x1C6D7E,
0x1E6D7E,
0x1D6C7B,
0x1D6C7B,
0x1D6B78,
0x1E6A78,
0x206A77,
0x216977,
0x226874,
0x236774,
0x246672,
0x236774,
0x236975,
0x216975,
0x206A77,
0x1E6C79,
0x1E6C79,
0x1D6C7B,
0x1C6D7E,
0x1C6D7E,
0x1C6F81,
0x1D7082,
0x1B7182,
0x1B7285,
0x1B7285,
0x1B7486,
0x1C7488,
0x1B7689,
0x1B7689,
0x1C778A,
0x1B778C,
0x1C788D,
0x1B7A8E,
0x1C7B8F,
0x1D7C92,
0x1E7D93,
0x1E7E94,
0x208096,
0x228197,
0x248399,
0x25849A,
0x26859B,
0x28879D,
0x2A899F,
0x2B8AA0,
0x2D8CA2,
0x308CA1,
0x318DA2,
0x338EA3,
0x368EA4,
0x3890A4,
0x3C93A7,
0x3E93A7,
0x4294A9,
0x4697AA,
0x4798AB,
0x499AAD,
0x4E9DAE,
0x7B998D,
0x9BA58D,
0xAE9F78,
0xAE8E5B,
0xA7884F,
0x9C834B,
0x957E4C,
0x987F56,
0x96825D,
0x948A67,
0x827E5B,
0x7A704D,
0x7E6A47,
0x7E6745,
0x8E7F5E,
0x7E775A,
0x6F7567,
0x8A9C90,
0x9ABAB5,
0x9AC3C5,
0x92BEC7,
0x8FBFCD,
0x91C5D3,
0x92C0CD,
0xA9C6CA,
0x575E57,
0x271F14,
0x302418,
0x332C22,
0x2A2319,
0x4A3C2F,
0x664F3F,
0x352820,
0x362921,
0x392C23,
0x32281E,
0x22180E,
0x362C22,
0x3F352B,
0x2F261D,
0x281F18,
0x382F28,
0x2E251E,
0x221711,
0x261B15,
0x30261D,
0x372D24,
0x2F251C,
0x3D332A,
0x4E443A,
0x463C32,
0x3E3428,
0x3D3327,
0x42382C,
0x473D31,
0x463C30,
0x2D2319,
0x2A2017,
0x2E241B,
0x2D231A,
0x2E251C,
0x332A21,
0x383025,
0x393126,
0x21241B,
0x21201B,
0x2A221F,
0x2B201C,
0x382B23,
0x584A3F,
0x33251C,
0x2A211C,
0x1D2122,
0xA2B5BB,
0xBBDADF,
0xAFD2D8,
0xADCED5,
0xACCDD6,
0xA5CDD5,
0x9ECDD5,
0x9BCAD4,
0x9ACAD4,
0x96C6D0,
0x93C2CA,
0x96C2C5,
0x9ABCBB,
0x95ADA0,
0x8E9A84,
0x959677,
0x96906C,
0x918C66,
0x8B8867,
0x878F77,
0x899B8B,
0x84A29A,
0x84A6A5,
0x6AADBD,
0x66ABBA,
0x62AAB9,
0x60A9B8,
0x5AA6B6,
0x58A5B5,
0x55A4B5,
0x50A1B2,
0x4E9FB2,
0x4C9FB1,
0x499BB0,
0x459AAE,
0x4299AC,
0x3F97AB,
0x3B96A9,
0x3994A7,
0x3792A5,
0x3691A6,
0x338EA3,
0x318DA2,
0x2F8BA0,
0x2E8A9F,
0x2A899F,
0x28879D,
0x26869C,
0x25859B,
0x238399,
0x218197,
0x208096,
0x1F7F95,
0x1D7D93,
0x1D7C92,
0x1C7B91,
0x1B7A90,
0x1C788D,
0x1B778C,
0x1C778C,
0x1D7589,
0x1D7589,
0x1C7488,
0x1B7486,
0x1B7486,
0x1B7285,
0x1B7182,
0x1B7182,
0x1C6F81,
0x1C6D7E,
0x1E6D7E,
0x1D6C7B,
0x1D6C7B,
0x1D6B78,
0x1E6A78,
0x206A77,
0x216977,
0x226874,
0x236774,
0x246672,
0x236774,
0x226874,
0x216975,
0x206A77,
0x1E6C79,
0x1D6C79,
0x1D6C7B,
0x1C6D7E,
0x1C6D7E,
0x1C6F81,
0x1D7082,
0x1B7182,
0x1B7285,
0x1C7386,
0x1B7486,
0x1C7488,
0x1B7689,
0x1B7689,
0x1C778A,
0x1B778C,
0x1C788D,
0x1B7A8E,
0x1C7B8F,
0x1D7C92,
0x1D7D93,
0x1F7F95,
0x208096,
0x218197,
0x248399,
0x25849A,
0x27869C,
0x29889E,
0x2A899F,
0x2C8BA1,
0x2D8CA2,
0x318DA2,
0x328EA3,
0x348FA4,
0x378FA5,
0x3991A5,
0x3C93A7,
0x3E93A7,
0x4294A9,
0x4598AA,
0x4798AB,
0x4899AC,
0x4D9CAD,
0x489FB3,
0x71A29E,
0x9E9E7A,
0xAF9258,
0xAB8C4C,
0xA78A50,
0xA78955,
0xA78758,
0x9F835B,
0x8D7D59,
0x7D7453,
0x796B48,
0x816943,
0x8F7856,
0x7D7960,
0x7C8A7B,
0x99BBC4,
0x92BAC2,
0x88B9C0,
0x8BBCC3,
0x90BFC7,
0x92C1CB,
0x94C4D0,
0x96C2CD,
0xA8C9D0,
0x465455,
0x161513,
0x201C19,
0x282826,
0x2E2D28,
0x3B2F23,
0x462F1F,
0x3B2E26,
0x382B23,
0x352820,
0x271D14,
0x2A2017,
0x2A2017,
0x2E251C,
0x393029,
0x352C27,
0x3D342F,
0x29201B,
0x261D18,
0x29201B,
0x3E352E,
0x372D24,
0x2D231A,
0x2E241B,
0x2F251C,
0x33291F,
0x281E12,
0x2A2014,
0x2D2317,
0x2B2115,
0x392F23,
0x2F251B,
0x30261C,
0x3A3027,
0x281F16,
0x393027,
0x3B3328,
0x30281D,
0x2E261B,
0x1E1F17,
0x211E17,
0x251E18,
0x392E28,
0x3B2E25,
0x3A2C21,
0x382A21,
0x2C251F,
0x202425,
0xAABFC4,
0xBAD9DE,
0xB1D4DA,
0xAECFD6,
0xADCED7,
0xA7CDD6,
0xA1CDD8,
0xA6CAD6,
0xA0C6D1,
0x9AC4D2,
0x96C4D3,
0x95C5D3,
0x93C3CF,
0x92C1C9,
0x95C1C4,
0x92B9B6,
0x8FB4AD,
0x8BB3AB,
0x84AEAA,
0x7DAFB0,
0x7AB3BA,
0x74B4BE,
0x70B3C3,
0x6BAEBE,
0x67ACBB,
0x63ABBA,
0x60A9B8,
0x5BA7B7,
0x58A5B5,
0x56A5B6,
0x51A2B3,
0x4FA0B3,
0x4C9FB1,
0x4A9CB1,
0x469BAF,
0x439AAD,
0x4098AC,
0x3B96A9,
0x3A95A8,
0x3893A6,
0x3691A6,
0x348FA4,
0x318DA2,
0x308CA1,
0x2E8A9F,
0x2A899F,
0x29889E,
0x26869C,
0x25859B,
0x238399,
0x228298,
0x208096,
0x1F7F95,
0x1E7E94,
0x1E7D93,
0x1C7B91,
0x1B7A90,
0x1C788D,
0x1B778C,
0x1C778C,
0x1B7689,
0x1D7589,
0x1C7488,
0x1B7486,
0x1B7486,
0x1B7285,
0x1A7182,
0x1B7182,
0x1C6F81,
0x1C6D7E,
0x1D6C7D,
0x1D6C7B,
0x1D6C7B,
0x1D6B78,
0x1E6A78,
0x206A77,
0x216977,
0x226874,
0x236774,
0x246672,
0x236774,
0x226874,
0x216975,
0x206A77,
0x1E6C79,
0x1D6C79,
0x1D6C7B,
0x1C6D7E,
0x1C6D7E,
0x1C6F81,
0x1D7082,
0x1B7182,
0x1B7285,
0x1C7386,
0x1B7486,
0x1A7588,
0x1B7689,
0x1B7689,
0x1C778A,
0x1C788D,
0x1C788D,
0x1B7A8E,
0x1C7B8F,
0x1C7C92,
0x1D7D93,
0x1F7F95,
0x218197,
0x218197,
0x238399,
0x26859B,
0x27869C,
0x29889E,
0x2A899F,
0x2C8BA1,
0x2E8DA3,
0x328EA3,
0x338FA4,
0x3590A5,
0x3890A6,
0x3991A5,
0x3C93A7,
0x3E93A7,
0x4193A8,
0x4598AA,
0x4899AC,
0x499AAD,
0x4E9DAE,
0x42A4BD,
0x57A0A9,
0x7A9C8E,
0x949873,
0xA1945F,
0xA89258,
0xAD8F59,
0xAB8B5A,
0x9E7F51,
0x896E41,
0x836B3B,
0x8B7345,
0x8F7B56,
0x7C7761,
0x799088,
0x95BEC2,
0x8ABACE,
0x86BBCD,
0x85BFCD,
0x8AC3CE,
0x91C1CB,
0x93BFC8,
0x95C1CA,
0x9CC5CB,
0xA9C2C6,
0x5A6362,
0x241F1C,
0x1B1311,
0x252120,
0x2D2A25,
0x33261D,
0x422B1D,
0x3B2E25,
0x35291D,
0x3D3125,
0x3A3024,
0x392F23,
0x241C11,
0x292116,
0x2D241B,
0x362D26,
0x332A23,
0x2A211A,
0x292019,
0x30271E,
0x393126,
0x2F271C,
0x372F22,
0x31271D,
0x41372D,
0x30261A,
0x3A3024,
0x6B6253,
0x786F60,
0x5A5142,
0x42382C,
0x40362C,
0x30261C,
0x281F16,
0x31281F,
0x3D342B,
0x352D22,
0x322A1F,
0x2B2519,
0x26261C,
0x3A352F,
0x382F2A,
0x382D27,
0x3B2E25,
0x35291D,
0x382B22,
0x201913,
0x2C3232,
0xB5CACF,
0xB6D7DC,
0xB1D4DA,
0xB1CFD7,
0xAFD0D9,
0xA7CDD6,
0xA2CED9,
0xA6C7D0,
0xA2C7D0,
0x9FC7D3,
0x97C5D4,
0x91C2D1,
0x8BC0D2,
0x88C1D2,
0x89C2D3,
0x83BECC,
0x80BCC7,
0x80BEC9,
0x7CBCC8,
0x74B8C5,
0x72B7C7,
0x6BB4C5,
0x64AFC2,
0x6BAEBE,
0x67ACBB,
0x63ABBA,
0x61AAB9,
0x5BA7B7,
0x59A6B6,
0x56A5B6,
0x51A2B3,
0x50A1B4,
0x4DA0B2,
0x4B9DB2,
0x469BAF,
0x449BAE,
0x4098AC,
0x3C97AA,
0x3A95A8,
0x3893A6,
0x3691A6,
0x348FA4,
0x318DA2,
0x308CA1,
0x2E8A9F,
0x2A899F,
0x29889E,
0x26869C,
0x25859B,
0x24849A,
0x228298,
0x218197,
0x1F7F95,
0x1E7E94,
0x1E7D93,
0x1C7B91,
0x1B7A90,
0x1C788D,
0x1B778C,
0x1C778C,
0x1B7689,
0x1D7589,
0x1C7488,
0x1B7486,
0x1B7486,
0x1B7285,
0x1A7182,
0x1B7182,
0x1C6F81,
0x1C6D7E,
0x1D6C7D,
0x1D6C7B,
0x1D6C7B,
0x1D6B78,
0x1E6A78,
0x206A77,
0x216977,
0x226874,
0x236774,
0x246674,
0x236774,
0x226874,
0x216977,
0x1E6A77,
0x1E6C79,
0x1D6C79,
0x1E6D7C,
0x1C6D7E,
0x1C6D7E,
0x1C6F81,
0x1D7082,
0x1B7182,
0x1B7285,
0x1C7386,
0x1B7486,
0x1A7588,
0x1B7689,
0x1C778A,
0x1B788A,
0x1C788D,
0x1A798D,
0x1B7A8E,
0x1C7B8F,
0x1C7C92,
0x1D7D93,
0x1F7F95,
0x218197,
0x218197,
0x238399,
0x25859B,
0x26869C,
0x29889E,
0x2A899F,
0x2C8BA1,
0x2E8DA3,
0x328EA3,
0x3490A5,
0x3691A6,
0x3792A7,
0x3A92A6,
0x3D94A8,
0x3E93A7,
0x4193A8,
0x4497A9,
0x4899AC,
0x4A9BAE,
0x4C9DB0,
0x4E9BAB,
0x549FB2,
0x5B9DAB,
0x6C9A98,
0x839B83,
0x90926B,
0x90895F,
0x948B64,
0x887A53,
0x8A7546,
0x907945,
0x958458,
0x82846C,
0x7C9693,
0x8EBECA,
0x83C0D2,
0x8CBACA,
0x85BACA,
0x84BECC,
0x87BFCC,
0x90C2CD,
0x95C4CC,
0x99C4CB,
0xA0C3C7,
0xB0C0BF,
0x504C43,
0x1F0B00,
0x29150A,
0x352820,
0x322922,
0x3B2D24,
0x3F291C,
0x332717,
0x332717,
0x403424,
0x3A3120,
0x362D1E,
0x372E1F,
0x3B3326,
0x2B2318,
0x3A3128,
0x2E271D,
0x2C251B,
0x3B3529,
0x393124,
0x231C0C,
0x322B19,
0x403927,
0x4B4233,
0x756C5D,
0x6C6354,
0x453C2B,
0x857C6B,
0xB3AA99,
0x92897A,
0x504738,
0x2F2519,
0x362E23,
0x2F261D,
0x31281F,
0x382F26,
0x322920,
0x332C22,
0x383127,
0x2C281D,
0x332C24,
0x3F342E,
0x3C3229,
0x392F23,
0x35291B,
0x372D21,
0x26211B,
0x444D4C,
0xBED6DA,
0xB5D6DB,
0xB1D4DA,
0xB2D0D8,
0xB2D0DA,
0xA9CED7,
0xA4CEDA,
0xA4C9D2,
0xA4CAD3,
0xA0C8D2,
0x99C5D0,
0x95C5D1,
0x92C6D3,
0x8CC1CF,
0x83BDCB,
0x84BFCF,
0x7DBBCA,
0x7DBCCB,
0x7BBAC9,
0x76B5C4,
0x76B5C4,
0x73B3BF,
0x70B0BC,
0x6CAFBF,
0x68ADBC,
0x64ACBB,
0x61AAB9,
0x5CA8B8,
0x59A6B6,
0x57A6B7,
0x52A3B4,
0x50A1B4,
0x4DA0B2,
0x4B9DB2,
0x479CB0,
0x449BAE,
0x4199AD,
0x3E96AA,
0x3A95A8,
0x3893A6,
0x3691A6,
0x348FA4,
0x318DA2,
0x308CA1,
0x2E8A9F,
0x2A899F,
0x29889E,
0x26869C,
0x25859B,
0x24849A,
0x228298,
0x218197,
0x1F7F95,
0x1E7E94,
0x1E7D93,
0x1C7B91,
0x1B7A90,
0x1C788D,
0x1B778C,
0x1C778C,
0x1B7689,
0x1D7589,
0x1C7488,
0x1C7587,
0x1B7486,
0x1B7285,
0x1A7182,
0x1B7182,
0x1C6F81,
0x1C6D7E,
0x1D6C7D,
0x1E6D7C,
0x1D6C7B,
0x1D6B78,
0x1E6A78,
0x206A77,
0x216977,
0x226874,
0x236774,
0x246674,
0x236774,
0x236975,
0x216977,
0x1E6A77,
0x1E6C79,
0x1D6C79,
0x1E6D7C,
0x1B6C7D,
0x1C6D7E,
0x1C6F81,
0x1D7082,
0x1B7182,
0x1B7285,
0x1B7486,
0x1B7486,
0x1A7588,
0x1B7689,
0x1C778A,
0x1B788A,
0x1C788D,
0x1A798D,
0x1C7B8F,
0x1C7B8F,
0x1C7C92,
0x1D7D93,
0x1F7F95,
0x208096,
0x228298,
0x238399,
0x25859B,
0x26869C,
0x29889E,
0x2A899F,
0x2C8BA1,
0x2E8DA1,
0x328EA3,
0x3490A5,
0x3792A7,
0x3893A8,
0x3C94A8,
0x3F96AA,
0x4095A9,
0x4196AA,
0x4598AA,
0x4899AC,
0x499AAD,
0x4C9DB0,
0x5C9BAA,
0x5BA2B6,
0x58A4BE,
0x5AA2B8,
0x66A5AE,
0x73A4A1,
0x799F94,
0x7E9A8B,
0x839680,
0x8D9A7E,
0x8D9C7F,
0x8DA490,
0x8BB2AF,
0x8BBFCC,
0x7BB8CA,
0x84C3D5,
0x97BFC7,
0x8BBAC4,
0x87BDC7,
0x8CC2CE,
0x92C4CF,
0x93C2CC,
0x96C1CA,
0xA3C5C7,
0xAEBBB4,
0x4D4133,
0x402512,
0x462B18,
0x382618,
0x3F3229,
0x44362D,
0x3C281D,
0x3C2F1E,
0x3D3220,
0x423626,
0x372B1B,
0x3A3122,
0x41372B,
0x30281B,
0x261E13,
0x30271E,
0x362F25,
0x221B11,
0x231D11,
0x2F271A,
0x2B2414,
0x534C3A,
0x79725F,
0x8E8671,
0x968E79,
0x928A75,
0x49412C,
0x433B26,
0xB6AE99,
0xBCB4A1,
0x726958,
0x3C3226,
0x30261C,
0x2D241B,
0x241B14,
0x342B24,
0x322922,
0x2B241C,
0x3A332B,
0x383226,
0x30271E,
0x362C23,
0x3B2E26,
0x2F2519,
0x2D2415,
0x393124,
0x1D1A13,
0x616B6A,
0xC7DFE3,
0xB5D6DB,
0xB2D5DB,
0xB3D1D9,
0xB2D0DA,
0xAACFD8,
0xA4CFD8,
0xA4CEDC,
0xA0CAD8,
0x9DC7D3,
0x9BC5D1,
0x98C3CC,
0x94C0CB,
0x91C1CB,
0x8FC3CE,
0x8AC0CC,
0x83BDC9,
0x81BDC8,
0x7BB9C4,
0x78B6C1,
0x77B6BF,
0x73B2BB,
0x74B0B8,
0x6CAFBF,
0x68ADBC,
0x64ACBB,
0x61AAB9,
0x5CA8B8,
0x5AA7B7,
0x57A6B7,
0x52A3B4,
0x50A1B4,
0x4E9FB2,
0x4B9DB2,
0x479CB0,
0x449BAE,
0x4199AD,
0x3E96AA,
0x3B96A9,
0x3893A6,
0x3792A7,
0x348FA4,
0x328EA3,
0x308CA1,
0x2E8A9F,
0x2A899F,
0x29889E,
0x26869C,
0x25859B,
0x24849A,
0x228298,
0x218197,
0x1F7F95,
0x1E7E94,
0x1E7D93,
0x1B7B91,
0x1B7A90,
0x1A798D,
0x1B778C,
0x1C778C,
0x1B7689,
0x1D7589,
0x1C7488,
0x1C7587,
0x1B7486,
0x1B7285,
0x1A7182,
0x1B7182,
0x1C6F81,
0x1C6D7E,
0x1D6C7D,
0x1E6D7C,
0x1D6C7B,
0x1D6B78,
0x1E6A78,
0x206A77,
0x216977,
0x226874,
0x236774,
0x256775,
0x246875,
0x236975,
0x216977,
0x1E6A77,
0x1E6C79,
0x1D6C7B,
0x1E6D7C,
0x1C6D7E,
0x1C6D7E,
0x1C6F81,
0x1D7082,
0x1B7084,
0x1B7285,
0x1B7486,
0x1C7587,
0x1A7588,
0x1B7689,
0x1C778A,
0x1B788A,
0x1C788D,
0x1B7A8E,
0x1C7B8F,
0x1B7C8F,
0x1D7D93,
0x1D7D93,
0x1F7F95,
0x218197,
0x228298,
0x24849A,
0x25859B,
0x26869C,
0x29889E,
0x2B8AA0,
0x2D8CA0,
0x2E8DA1,
0x328EA3,
0x3490A5,
0x3792A7,
0x3893A8,
0x3D95A9,
0x4198AC,
0x4398AC,
0x4398AC,
0x479AAC,
0x499AAD,
0x4B9CAF,
0x4E9FB2,
0x569DB3,
0x589AB0,
0x5CA0B5,
0x59A5BD,
0x55ABC6,
0x5CB2CD,
0x68B5C7,
0x70AFB6,
0x7AB0B0,
0x79B1B2,
0x7EBAC2,
0x77B9C5,
0x79BBCB,
0x7AB8C7,
0x86C0CE,
0x85BFCB,
0x90B8C0,
0x8BBAC2,
0x8CC1C9,
0x8EC0C9,
0x92C1CB,
0x98C4D1,
0x99C5D0,
0xA2C7CF,
0xA9BDBC,
0x626054,
0x483524,
0x362110,
0x291B0E,
0x382E24,
0x3E2D23,
0x483225,
0x47392C,
0x3A2C1F,
0x3B2D20,
0x3A2E22,
0x3C2F26,
0x3E342A,
0x372C26,
0x332A23,
0x3D342F,
0x211813,
0x271E19,
0x393029,
0x322920,
0x393124,
0x777060,
0xA39C8A,
0xB1A78E,
0xB1A78C,
0xA3997E,
0x8F856A,
0x554B32,
0xB4A993,
0xC9BEAA,
0x8F8472,
0x494031,
0x372D23,
0x473D34,
0x362D26,
0x2C231E,
0x322924,
0x372E29,
0x312823,
0x342C21,
0x362C22,
0x3C2F27,
0x392C23,
0x30261A,
0x2D2616,
0x302A1C,
0x2E2E26,
0x798384,
0xC9E2E6,
0xB4D5DC,
0xB3D6DC,
0xB4D2DA,
0xB2D0DA,
0xADCFD9,
0xA6CED8,
0x9ECBDE,
0x9FCADB,
0x9CC7D7,
0x9BC5D1,
0x9BC3CD,
0x99C4CD,
0x94C0C9,
0x8CBCC6,
0x87BDC7,
0x82BCC8,
0x7EBCC9,
0x79BBC7,
0x77BBC8,
0x75B9C6,
0x6FB3BE,
0x6FB3BE,
0x6CAFBF,
0x68ADBC,
0x65ADBC,
0x62ABBA,
0x5CA8B8,
0x5AA7B7,
0x57A6B7,
0x54A3B4,
0x50A1B4,
0x4E9FB2,
0x4B9DB2,
0x479CB0,
0x449BAE,
0x4199AD,
0x3F97AB,
0x3B96A9,
0x3994A7,
0x3792A7,
0x3590A5,
0x328EA3,
0x308CA1,
0x2F8BA0,
0x2B8AA0,
0x29889E,
0x27879D,
0x25859B,
0x24849A,
0x228298,
0x218197,
0x1F7F95,
0x1E7E94,
0x1E7D93,
0x1C7C92,
0x1C7B91,
0x1B7A8E,
0x1C788D,
0x1C778C,
0x1C778A,
0x1D7589,
0x1D7589,
0x1C7587,
0x1B7486,
0x1B7285,
0x1A7182,
0x1B7182,
0x1C6F81,
0x1C6D7E,
0x1B6C7D,
0x1E6D7C,
0x1D6C7B,
0x1D6B78,
0x1E6A78,
0x206A77,
0x216977,
0x226874,
0x236774,
0x246875,
0x246875,
0x236975,
0x216977,
0x1F6B78,
0x1E6C79,
0x1D6C7B,
0x1E6D7C,
0x1C6D7E,
0x1C6D7E,
0x1C6F81,
0x1D7082,
0x1C7185,
0x1B7285,
0x1B7486,
0x1C7587,
0x1A7588,
0x1B7689,
0x1C778A,
0x1B788A,
0x1C788D,
0x1B7A8E,
0x1C7B8F,
0x1B7C8F,
0x1D7D93,
0x1E7E94,
0x1F7F95,
0x218197,
0x238399,
0x24849A,
0x26869C,
0x26869C,
0x2A899D,
0x2B8A9E,
0x2D8CA0,
0x2F8EA2,
0x328EA3,
0x3490A5,
0x3792A7,
0x3994A9,
0x3C94A8,
0x4097AB,
0x4398AC,
0x459AAE,
0x4A9CB1,
0x4B9EB0,
0x4D9EB1,
0x50A1B4,
0x55A1B8,
0x5DA4B6,
0x62A4B2,
0x60A9BA,
0x5CB0CA,
0x54AFCC,
0x59ADC7,
0x68B3C6,
0x6CB4C3,
0x6CB7CC,
0x67B7D0,
0x6EBCD3,
0x78BBCC,
0x82BCC8,
0x85BBC5,
0x86BFCA,
0x90C0CC,
0x88BCC7,
0x8ABEC9,
0x8FBFC9,
0x96C1CA,
0x9AC2CE,
0x96C1D1,
0x9AC6D1,
0xA9CACF,
0x8F9B97,
0x373529,
0x130D01,
0x231F14,
0x282216,
0x2E1E11,
0x492F20,
0x402F25,
0x3C2B21,
0x35271C,
0x382A21,
0x382B23,
0x3B2E28,
0x3F3430,
0x382D2B,
0x372B2B,
0x2C2221,
0x302624,
0x2F2621,
0x312821,
0x494037,
0x8E8679,
0xB3AB9E,
0xBEB598,
0xB5AC8F,
0xAEA588,
0xA9A083,
0xBEB598,
0xC5BBA0,
0xBEB39D,
0x998E7A,
0x605444,
0x4A4034,
0x372D24,
0x2E231D,
0x322723,
0x332A25,
0x372E29,
0x362D28,
0x342A1E,
0x382B22,
0x362920,
0x372D23,
0x2F2818,
0x2F2919,
0x2E2B1C,
0x1D1E16,
0x849090,
0xC9E2E6,
0xB2D3DA,
0xB4D5DC,
0xB5D2DA,
0xB2CED9,
0xAED0DA,
0xA7CFD9,
0xA0CEDD,
0x9DCBD8,
0x9DC9D4,
0x9DC9D2,
0x9BC7D0,
0x96C2CB,
0x91C0C8,
0x8FC1CA,
0x89BFCB,
0x84BECA,
0x7EBCCB,
0x77B9C9,
0x76B9CA,
0x72B6C9,
0x6CB0C3,
0x6EB2C5,
0x6CAFBF,
0x68ADBC,
0x65ADBC,
0x62ABBA,
0x5DA9B9,
0x5AA7B7,
0x57A6B7,
0x55A4B5,
0x50A1B4,
0x4E9FB2,
0x4B9DB2,
0x479CB0,
0x449BAE,
0x4199AD,
0x3F97AB,
0x3B96A9,
0x3994A7,
0x3792A7,
0x3590A5,
0x328EA3,
0x318DA2,
0x2F8BA0,
0x2B8AA0,
0x29889E,
0x27879D,
0x26869C,
0x24849A,
0x228298,
0x218197,
0x1F7F95,
0x1E7E94,
0x1E7D93,
0x1C7C92,
0x1C7B91,
0x1B7A8E,
0x1C788D,
0x1C778C,
0x1C778A,
0x1C778A,
0x1D7589,
0x1C7587,
0x1B7486,
0x1A7385,
0x1A7182,
0x1B7182,
0x1C6F81,
0x1C6D7E,
0x1B6C7D,
0x1E6D7C,
0x1D6C7B,
0x1D6B78,
0x1E6A78,
0x206A77,
0x216977,
0x226874,
0x236774,
0x236774,
0x236774,
0x226874,
0x216977,
0x1E6A77,
0x1E6C79,
0x1D6C7B,
0x1D6C7B,
0x1C6D7E,
0x1D6E7F,
0x1C6F81,
0x1B7182,
0x1B7084,
0x1B7285,
0x1B7486,
0x1C7587,
0x1A7588,
0x1B7689,
0x1B7689,
0x1B788A,
0x1C788D,
0x1A798D,
0x1C7B8F,
0x1C7D90,
0x1D7D93,
0x1E7E94,
0x1F7F95,
0x218197,
0x228298,
0x24849A,
0x25859B,
0x27879D,
0x29889C,
0x2B8A9E,
0x2D8CA0,
0x2F8EA2,
0x328EA3,
0x3490A5,
0x3792A7,
0x3994A9,
0x3D95A9,
0x3F96AA,
0x4299AC,
0x459AAE,
0x499BB0,
0x4B9EB0,
0x4FA0B3,
0x50A1B4,
0x59A3AE,
0x59A5B2,
0x5AA8B5,
0x5CA9B9,
0x61AABB,
0x67ACBC,
0x6AADBD,
0x6CAFC0,
0x6EB2C7,
0x70B4C9,
0x74B5C9,
0x7BB6C6,
0x80B9C4,
0x85B9C4,
0x86BCC6,
0x87BFCC,
0x8BC0CE,
0x8BC4CF,
0x8AC0CA,
0x90BFC9,
0x99BFCA,
0x9BC0D0,
0x9AC5D6,
0x98C8D6,
0xA4CBD2,
0xA5BBB9,
0x495146,
0x2D2F22,
0x2E3023,
0x333021,
0x483828,
0x523724,
0x4B3B2E,
0x413124,
0x403023,
0x392B20,
0x35271E,
0x2E2119,
0x2F221C,
0x2C211D,
0x2B201C,
0x352A26,
0x302521,
0x30251F,
0x30261D,
0x554B3F,
0x9E9586,
0xB9B09F,
0xC3B99E,
0xBCB096,
0xB4A98D,
0xB8AD91,
0xC3B79D,
0xBCB297,
0xB4A993,
0x9C917D,
0x605444,
0x44382A,
0x2B2115,
0x2C2218,
0x382E25,
0x2B2118,
0x2D241B,
0x2A2118,
0x312819,
0x2C2014,
0x35281F,
0x31271B,
0x322C1C,
0x322F1C,
0x262415,
0x595C53,
0xC1CFCF,
0xBED7DB,
0xB5D6DD,
0xB4D5DC,
0xB7D4DC,
0xB3CFDA,
0xAED0DA,
0xA9D1DB,
0xA0D0D4,
0x9ECED2,
0x9CCCD0,
0x99C8CE,
0x96C7CC,
0x94C5CC,
0x90C2CB,
0x8CC0CB,
0x8AC0CC,
0x86BECB,
0x82BBCC,
0x7EB9CB,
0x7CB7C9,
0x78B5C8,
0x75B2C5,
0x73B0C5,
0x6CAFBF,
0x6AAFBE,
0x65ADBC,
0x62ABBA,
0x5EAABA,
0x5AA7B7,
0x57A6B7,
0x55A4B5,
0x50A1B4,
0x4E9FB2,
0x4B9DB2,
0x479CB0,
0x449BAE,
0x4299AD,
0x3F97AB,
0x3B96A9,
0x3994A7,
0x3792A7,
0x3590A5,
0x328EA3,
0x318DA2,
0x2E8A9F,
0x2A899F,
0x2A899F,
0x27879D,
0x25859B,
0x24849A,
0x228298,
0x218197,
0x208096,
0x1E7E94,
0x1E7D93,
0x1C7C92,
0x1C7B91,
0x1B7A8E,
0x1C788D,
0x1C778C,
0x1C778A,
0x1B7689,
0x1C7488,
0x1C7587,
0x1B7486,
0x1A7385,
0x1B7283,
0x1B7182,
0x1C6F81,
0x1C6D7E,
0x1B6C7D,
0x1E6D7C,
0x1D6C7B,
0x1E6C79,
0x1E6A78,
0x206A77,
0x216977,
0x236975,
0x246875,
0x236774,
0x236774,
0x236975,
0x206A77,
0x1E6A77,
0x1E6C79,
0x1D6C7B,
0x1C6E7C,
0x1C6D7E,
0x1D6E81,
0x1C6F81,
0x1B7182,
0x1B7084,
0x1B7285,
0x1B7486,
0x1C7587,
0x1A7588,
0x1B7689,
0x1C778A,
0x1B788A,
0x1C788D,
0x1B7A8E,
0x1C7B8F,
0x1C7D90,
0x1D7D93,
0x1E7E94,
0x1F7F95,
0x218197,
0x228298,
0x24849A,
0x26869C,
0x27879D,
0x29889C,
0x2B8A9E,
0x2D8CA0,
0x2F8EA2,
0x328EA3,
0x3490A5,
0x3792A7,
0x3994A9,
0x3D95A9,
0x3F96AA,
0x4299AC,
0x459AAE,
0x499BB0,
0x4B9EB0,
0x4EA1B3,
0x50A1B4,
0x5AA4A7,
0x57A7B2,
0x53ABB9,
0x59AABB,
0x65A9B4,
0x70A9B0,
0x72ABB4,
0x70AEBD,
0x72B1C2,
0x7AB2C1,
0x82B2BE,
0x85B4BE,
0x86B8C3,
0x84BAC6,
0x86BCC8,
0x8CBEC7,
0x8DC1CC,
0x8DC3CD,
0x8BBFCA,
0x92BEC9,
0x99BFCC,
0x9ABFCF,
0x99C4D5,
0x98C6D3,
0xA3C9CC,
0x9DAEA6,
0x828572,
0x7D7B66,
0x64624D,
0x55503C,
0x503F2B,
0x482D18,
0x3D2C18,
0x3C2D18,
0x3E2F1A,
0x3D2E1B,
0x413423,
0x2B1E0E,
0x322618,
0x32261A,
0x32281C,
0x2C2216,
0x33291D,
0x342B1C,
0x3D3423,
0x726A55,
0xABA38C,
0xBFB7A0,
0xC4B9A3,
0xBDB29C,
0xB9AE98,
0xB5AB92,
0xAFA58C,
0xB3A990,
0xB5AA94,
0xA09581,
0x756A58,
0x3A2E1E,
0x322618,
0x32281C,
0x251B0F,
0x271D11,
0x2D2317,
0x2C2314,
0x332A1B,
0x413529,
0x4A3E32,
0x483E32,
0x413B2B,
0x474433,
0x676556,
0xB0B3AC,
0xCCDADA,
0xC1DADF,
0xB7D8DF,
0xB7D8DF,
0xB6D3DB,
0xB2CFD7,
0xADCFD9,
0xA5CDD7,
0xA3CFD0,
0xA1CDCE,
0x9CCCD0,
0x99CACF,
0x96C8D1,
0x91C5D0,
0x8EC4D0,
0x8CC1CF,
0x89BECC,
0x87BCCA,
0x84B9C7,
0x82B7C5,
0x7EB6C5,
0x7CB4C3,
0x78B1C2,
0x76AFC0,
0x6DB0C0,
0x6AAFBE,
0x65ADBC,
0x62ABBA,
0x5EAABA,
0x5AA7B7,
0x57A6B7,
0x55A4B5,
0x50A1B4,
0x4FA0B3,
0x4B9DB2,
0x479CB0,
0x459AAE,
0x4299AD,
0x3F97AB,
0x3B96A9,
0x3994A7,
0x3792A7,
0x3590A5,
0x338FA4,
0x318DA2,
0x2E8A9F,
0x2A899F,
0x2A899F,
0x27879D,
0x25859B,
0x238399,
0x228298,
0x218197,
0x208096,
0x1E7E94,
0x1E7D93,
0x1C7C92,
0x1C7B91,
0x1B7A8E,
0x1C788D,
0x1C778C,
0x1C778A,
0x1B7689,
0x1C7488,
0x1C7587,
0x1B7486,
0x1A7385,
0x1A7182,
0x1B7182,
0x1C6F81,
0x1C6D7E,
0x1C6D7E,
0x1D6C7B,
0x1D6C7B,
0x1E6C79,
0x1E6A78,
0x206A77,
0x216977,
0x236975,
0x246875,
0x246875,
0x246875,
0x236975,
0x206A77,
0x1E6A77,
0x1E6C79,
0x1D6C7B,
0x1C6E7C,
0x1C6D7E,
0x1D6E81,
0x1C6F81,
0x1B7182,
0x1B7084,
0x1B7285,
0x1B7486,
0x1C7587,
0x1A7588,
0x1B7689,
0x1C778A,
0x1B788A,
0x1C788D,
0x1B7A8E,
0x1C7B8F,
0x1D7C90,
0x1D7D93,
0x1E7E94,
0x208096,
0x218197,
0x228298,
0x24849A,
0x26869C,
0x27879D,
0x29889C,
0x2B8A9E,
0x2D8CA0,
0x2F8EA2,
0x328EA3,
0x3490A5,
0x3792A7,
0x3994A7,
0x3D95A9,
0x4097AB,
0x4299AC,
0x459AAE,
0x479CB0,
0x4B9EB0,
0x4EA1B3,
0x50A3B5,
0x55A4B3,
0x54A7B7,
0x56A9B9,
0x5CABBA,
0x65AAB9,
0x6BAAB9,
0x6CAEBE,
0x6DB0C0,
0x72B1C2,
0x7AB2C1,
0x7FB3C1,
0x7EB6C5,
0x7FBACA,
0x81BCCA,
0x87BBC6,
0x90BDC3,
0x91C0C8,
0x91C3CC,
0x8EC0C9,
0x93BFCA,
0x99BFCC,
0x9ABFD1,
0x99C4D5,
0x9BC6CF,
0xAAC8C6,
0xAAB19F,
0xAAA184,
0xA19271,
0x8D8264,
0x90866D,
0x908069,
0x8E755F,
0x706247,
0x76684D,
0x64563B,
0x615339,
0x71654B,
0x685C44,
0x60553F,
0x5E533F,
0x504833,
0x48402D,
0x524A35,
0x5B533E,
0x6E674D,
0x968F73,
0xB3AC8F,
0xBEB598,
0xCABEB0,
0xC9BDAD,
0xCBBFAF,
0xC7BCAA,
0xBCB19F,
0xC0B5A1,
0xBEB39F,
0xACA18F,
0x938876,
0x635747,
0x504736,
0x4F4635,
0x574E3D,
0x5A523F,
0x4A422F,
0x574F3C,
0x655C4B,
0x797061,
0x7F7569,
0x827A6D,
0x8A8776,
0x989785,
0x989989,
0xACAFA8,
0xCDDBDB,
0xBED7DC,
0xB5D6DD,
0xB7D8DF,
0xB6D3DB,
0xB4D1D9,
0xB2D4DD,
0xA8D0D8,
0xA7CCD2,
0xA5CCD3,
0x9FCAD3,
0x9AC8D5,
0x96C7D5,
0x90C5D5,
0x8CC4D3,
0x8AC2D1,
0x86BECB,
0x84BDC8,
0x81BAC5,
0x80B9C4,
0x7DB7C2,
0x78B6C1,
0x74B4C0,
0x72B2BE,
0x6DB0C0,
0x6AAFBE,
0x67ACBC,
0x62ABBA,
0x5EAABA,
0x5AA7B7,
0x57A6B7,
0x55A4B5,
0x51A2B5,
0x4FA0B3,
0x4C9EB3,
0x479CB0,
0x459AAE,
0x4299AD,
0x3F97AB,
0x3B96A9,
0x3994A7,
0x3792A7,
0x3590A5,
0x338FA4,
0x318DA2,
0x2F8BA0,
0x2B8AA0,
0x2A899F,
0x27879D,
0x25859B,
0x238399,
0x228298,
0x218197,
0x208096,
0x1E7E94,
0x1E7D93,
0x1C7C92,
0x1C7B91,
0x1B7A8E,
0x1C788D,
0x1C778C,
0x1C778A,
0x1B7689,
0x1A7588,
0x1C7587,
0x1B7486,
0x1A7385,
0x1A7182,
0x1B7182,
0x1C6F81,
0x1C6D7E,
0x1C6D7E,
0x1D6C7B,
0x1D6C7B,
0x1E6C79,
0x1E6A78,
0x206A77,
0x216977,
0x236975,
0x246875,
0x246875,
0x246875,
0x236975,
0x206A77,
0x1E6A78,
0x1E6C79,
0x1D6C7B,
0x1B6C7D,
0x1C6D7E,
0x1C6F81,
0x1C6F81,
0x1B7084,
0x1B7084,
0x1B7285,
0x1B7486,
0x1C7587,
0x1B7689,
0x1B7689,
0x1C778A,
0x1B788A,
0x1C788D,
0x1B7A8E,
0x1C7B8F,
0x1D7C90,
0x1D7D93,
0x1E7E94,
0x208096,
0x218197,
0x228298,
0x24849A,
0x26869C,
0x27879D,
0x2A899D,
0x2B8A9E,
0x2D8CA0,
0x2F8EA2,
0x328EA3,
0x3490A5,
0x3792A7,
0x3994A7,
0x3D95A9,
0x3F97AB,
0x4299AD,
0x459AAE,
0x479CB0,
0x4B9DB2,
0x4EA1B3,
0x50A3B5,
0x4FA4C3,
0x54A6BE,
0x5AA7B9,
0x60A9BA,
0x62ABBE,
0x62ADC4,
0x64B0C7,
0x67B3C3,
0x6CB4C2,
0x73B5C5,
0x75B6C8,
0x76B9CC,
0x79BCCD,
0x7FBDCA,
0x86BCC6,
0x8DBDC7,
0x90C0CA,
0x90C2CD,
0x8DBFCA,
0x91C0CA,
0x99C1CD,
0x9AC1D0,
0x9BC5D3,
0x9EC6CE,
0xADC6C0,
0xB4B6A1,
0xB5A282,
0xAC9573,
0xA49473,
0xA1957B,
0x9F9079,
0x9E8974,
0x9F9176,
0x9D8F74,
0x8C8165,
0x80745A,
0x8A7E64,
0x958B72,
0x90866D,
0x928A73,
0x8E8671,
0x928A75,
0x8A826D,
0x918972,
0x9E977D,
0xA59E82,
0xB7B094,
0xC3BC9F,
0xCDC4BB,
0xD4CAC0,
0xD5CBBF,
0xD2C8BC,
0xCBC2B3,
0xC9C0AF,
0xBCB3A2,
0xB5AC9B,
0xA79E8D,
0x968D7C,
0x8A8170,
0x8B8370,
0x918A77,
0x8E8872,
0x837D67,
0x8C866E,
0x8F8876,
0x908778,
0x988E82,
0xA19B8D,
0xA09F8D,
0x9C9D8B,
0x959688,
0xAFB4AD,
0xCAD8D9,
0xC0D9DE,
0xB6D7DE,
0xB8D6DE,
0xB6D3DB,
0xB4D1D9,
0xB0D2DB,
0xA7CFD7,
0xAACBD4,
0xA8CAD4,
0xA2C8D5,
0x9CC7D7,
0x97C6D6,
0x93C4D5,
0x8FC2D3,
0x8CC1CF,
0x88BECA,
0x86BCC6,
0x82BBC4,
0x7EBAC2,
0x79B9C2,
0x74B8C3,
0x6FB7C3,
0x6BB5C2,
0x6DB0C0,
0x6BAEBE,
0x67ACBC,
0x62ABBA,
0x5EAABA,
0x5AA7B7,
0x57A6B7,
0x55A4B5,
0x51A2B5,
0x4FA0B3,
0x4C9EB3,
0x499BB0,
0x459AAE,
0x4299AD,
0x3F97AB,
0x3D95A9,
0x3994A7,
0x3792A7,
0x3590A5,
0x338FA4,
0x318DA2,
0x2F8BA0,
0x2B8AA0,
0x2A899F,
0x27879D,
0x26869C,
0x24849A,
0x228298,
0x218197,
0x208096,
0x1E7E94,
0x1E7D93,
0x1C7C92,
0x1B7B91,
0x1B7A8E,
0x1C788D,
0x1B778C,
0x1C778A,
0x1B7689,
0x1A7588,
0x197486,
0x1B7486,
0x1A7385,
0x1A7182,
0x1B7182,
0x1C6F81,
0x1C6D7E,
0x1C6D7E,
0x1D6C7B,
0x1D6C7B,
0x1E6C79,
0x1E6A78,
0x206A77,
0x216977,
0x236975,
0x246875,
0x236774,
0x246875,
0x236877,
0x206A77,
0x1E6A78,
0x1E6C79,
0x1D6C7B,
0x1B6C7D,
0x1C6D7E,
0x1B6E80,
0x1C6F81,
0x1B7084,
0x1A7184,
0x1B7285,
0x1B7486,
0x1C7587,
0x1B7689,
0x1B7689,
0x1C778A,
0x1C778A,
0x1C788D,
0x1D798E,
0x1C7B8F,
0x1D7C90,
0x1D7D93,
0x1E7E94,
0x208096,
0x218197,
0x228298,
0x24849A,
0x27869C,
0x28879D,
0x2A899D,
0x2B8A9E,
0x2D8CA0,
0x2F8EA2,
0x328EA3,
0x3490A5,
0x3792A7,
0x3994A7,
0x3D95A9,
0x3F97AB,
0x4299AD,
0x459AAE,
0x479CB0,
0x4B9DB2,
0x4EA1B3,
0x50A3B5,
0x52A2C5,
0x55A4C2,
0x5BA7BE,
0x5EA9BC,
0x62AAC0,
0x64ACC2,
0x65B0C3,
0x66B4C1,
0x69B5C2,
0x6FB7C5,
0x75B7C7,
0x79B9C5,
0x7DBBC6,
0x7FBDC8,
0x80BECD,
0x83C0D3,
0x8DC0D1,
0x8EC3D3,
0x8AC0CC,
0x90C0CA,
0x99C1CB,
0x9AC2CC,
0x9AC6D1,
0x9EC6CE,
0xACC6C3,
0xA2A594,
0xA39278,
0xA99477,
0xA29278,
0x968C73,
0x9C907A,
0xA09079,
0x9C8F7C,
0x8E826C,
0x978C76,
0x968B75,
0x9D927C,
0xA09883,
0xA39B88,
0x9A927F,
0x9A9381,
0xAEA795,
0xABA492,
0xB5AC9B,
0xBCB4A1,
0xB6AE99,
0xC6BEA9,
0xD5CDB6,
0xD4CDC5,
0xD9D2CA,
0xD7D0C6,
0xCFC9BD,
0xC6BEB1,
0xC3BCAC,
0xB8B1A1,
0xB5AE9E,
0xA9A292,
0xA59E8E,
0xA39C8C,
0xA39C8A,
0x9F9886,
0x9A9380,
0x9B9680,
0x99947E,
0x8D8674,
0x918A7A,
0x989083,
0x9E9B8C,
0xA09F8D,
0xA2A391,
0x989A8C,
0xA0A59E,
0xC2D0D1,
0xC3DCE1,
0xBBD9E1,
0xB6D4DC,
0xB8D3DC,
0xB4D1D9,
0xADCFD8,
0xA5D0D7,
0xA9CCD2,
0xA5CAD2,
0xA2C8D1,
0x9DC8D1,
0x9AC6D1,
0x95C3D0,
0x92C2CE,
0x90C0CA,
0x8CBEC7,
0x8ABCC5,
0x84BBC2,
0x80B9C2,
0x7AB8C3,
0x75B7C3,
0x70B5C4,
0x6BB4C3,
0x6DB0C0,
0x6BAEBE,
0x67ACBC,
0x63ABBA,
0x5EAABA,
0x5AA7B7,
0x57A6B7,
0x55A4B5,
0x51A2B5,
0x4FA0B3,
0x4C9EB3,
0x499BB0,
0x459AAE,
0x4299AD,
0x3F97AB,
0x3D95A9,
0x3994A7,
0x3792A7,
0x3590A5,
0x338FA4,
0x318DA2,
0x2F8BA0,
0x2B8AA0,
0x2B8AA0,
0x27879D,
0x26869C,
0x24849A,
0x228298,
0x218197,
0x208096,
0x1E7E94,
0x1E7D93,
0x1C7C92,
0x1B7B91,
0x1B7A8E,
0x1C788D,
0x1B778C,
0x1C778A,
0x1B7689,
0x1A7588,
0x197486,
0x1B7486,
0x1A7385,
0x1A7182,
0x1B7182,
0x1C6F81,
0x1C6D7E,
0x1C6D7E,
0x1D6C7B,
0x1D6C7B,
0x1E6C79,
0x1E6A78,
0x206A77,
0x216977,
0x236975,
0x246875,
0x236774,
0x246875,
0x236877,
0x206A77,
0x1E6A78,
0x1D6B78,
0x1D6C7B,
0x1B6C7D,
0x1C6D7E,
0x1B6E80,
0x1C6F81,
0x1B7084,
0x1A7184,
0x1B7285,
0x1B7486,
0x1C7587,
0x1C7488,
0x1B7689,
0x1C778A,
0x1C778A,
0x1C788D,
0x1D798E,
0x1C7B8F,
0x1D7C90,
0x1E7D93,
0x1E7E94,
0x208096,
0x218197,
0x228298,
0x25849A,
0x27869C,
0x28879D,
0x29889C,
0x2B8A9E,
0x2D8CA0,
0x2E8DA1,
0x328EA3,
0x3490A5,
0x3692A7,
0x3994A7,
0x3D95A9,
0x3F97AB,
0x4299AD,
0x459AAE,
0x479CB0,
0x4B9DB2,
0x4EA1B3,
0x50A3B5,
0x59A1BA,
0x57A5BC,
0x57A9BF,
0x5CAABE,
0x65AABA,
0x6BAAB9,
0x6CAEBA,
0x6AB3BC,
0x6AB4BF,
0x72B4C0,
0x7BB4BD,
0x80B5BB,
0x83BABD,
0x81BDC7,
0x7DBED0,
0x7EC1DB,
0x8BC2D7,
0x8CC3D7,
0x8BC1CD,
0x90C1C6,
0x99C2C6,
0x99C2C8,
0x97C6D0,
0x9BC7D0,
0xAAC8CA,
0x9FA49E,
0xA19484,
0xA2907C,
0x978C78,
0x958E7B,
0x9B937E,
0xA2937E,
0x9E9284,
0x94887A,
0x8E8272,
0x948B7A,
0xA9A08F,
0xB8B19F,
0xC0B9A9,
0xBBB4A4,
0xBBB3A6,
0xC5BDB2,
0xC4BCB1,
0xC6BEB3,
0xC7BFB2,
0xCAC3B3,
0xD5CEBE,
0xDBD2C3,
0xD7D3CA,
0xD8D4C9,
0xD7D3C8,
0xD1CDC1,
0xC4C1B2,
0xC2BFB0,
0xBDB7A9,
0xADA799,
0xABA597,
0xAAA498,
0xAAA498,
0xA6A092,
0xA59F91,
0xA29C8E,
0x9F9989,
0x9F9989,
0x9A9582,
0x979181,
0x8F897B,
0x9B9889,
0x9D9E8C,
0xA2A592,
0xA1A395,
0x9EA39D,
0xB9C7C8,
0xC1D8DE,
0xBAD8E0,
0xB4D2DA,
0xB9D4DB,
0xB5D2DA,
0xADD0D6,
0xA8D3D9,
0xA5CED0,
0xA3CCD0,
0x9FCAD0,
0x9CC9CF,
0x9AC7CD,
0x95C4CC,
0x92C1C9,
0x91C0C8,
0x8EBFC6,
0x8BBBC5,
0x87B9C2,
0x82B8C4,
0x7DB7C3,
0x78B6C5,
0x72B4C4,
0x70B1C3,
0x6DB0C0,
0x6BAEBE,
0x67ACBC,
0x63ABBA,
0x5EAABA,
0x5AA7B7,
0x58A5B7,
0x55A4B5,
0x51A2B5,
0x4FA0B3,
0x4B9DB2,
0x499BB0,
0x459AAE,
0x4299AD,
0x3E96AA,
0x3D95A9,
0x3994A7,
0x3792A7,
0x3590A5,
0x338FA4,
0x318DA2,
0x2F8BA0,
0x2B8AA0,
0x2A899F,
0x27879D,
0x26869C,
0x24849A,
0x228298,
0x218197,
0x208096,
0x1E7E94,
0x1E7D93,
0x1C7C92,
0x1B7B91,
0x1B7A8E,
0x1C788D,
0x1B778C,
0x1C778A,
0x1B7689,
0x1A7588,
0x1A7587,
0x1B7486,
0x1A7385,
0x1B7283,
0x1B7182,
0x1C6F81,
0x1D6E7F,
0x1C6D7E,
0x1D6C7B,
0x1D6C7B,
0x1E6C79,
0x1E6A78,
0x206A77,
0x216977,
0x236975,
0x236774,
0x246875,
0x246875,
0x236877,
0x206A77,
0x1E6A78,
0x1E6C79,
0x1D6C7B,
0x1B6C7D,
0x1C6D7E,
0x1B6E80,
0x1C6F81,
0x1B7084,
0x1A7184,
0x1B7285,
0x1B7486,
0x1C7587,
0x1C7488,
0x1B7689,
0x1C778A,
0x1C778A,
0x1C788D,
0x1D798E,
0x1C7B8F,
0x1D7C90,
0x1E7D93,
0x1F7E94,
0x1F7F95,
0x218197,
0x238298,
0x25849A,
0x27869C,
0x28879D,
0x29889C,
0x2B8A9E,
0x2D8CA0,
0x2E8DA1,
0x328EA3,
0x3490A5,
0x3692A7,
0x3994A7,
0x3D95A9,
0x3F97AB,
0x4299AD,
0x459AAE,
0x479CB0,
0x4B9DB2,
0x4EA1B3,
0x50A3B5,
0x59A2B1,
0x57A6B5,
0x56A9B9,
0x5CABBA,
0x66A9B9,
0x6CAAB7,
0x6CAEBA,
0x6AB2BE,
0x6DB3BF,
0x75B3C0,
0x7EB2BF,
0x80B5BD,
0x81BAC3,
0x7FBEC7,
0x80BECB,
0x86BFD2,
0x8DBFD6,
0x8EC3D5,
0x8CC0CB,
0x91C1C3,
0x99C3C2,
0x98C4C7,
0x97C8CF,
0x99C8D2,
0xA7C5CD,
0xA8B1B0,
0xA69991,
0x99897C,
0x908778,
0x969382,
0x9A9581,
0x9C917B,
0xA1978B,
0xA69C90,
0x9C9286,
0xA29B8B,
0xAEA797,
0xBFB8A8,
0xC1BBAB,
0xC8C2B4,
0xCEC8BC,
0xD4CEC2,
0xD2CCC0,
0xD0CABE,
0xD5CDC2,
0xD5CDC2,
0xD2CABD,
0xCEC6B9,
0xD3D3C7,
0xD5D6C8,
0xD7D5C8,
0xD4D2C3,
0xCCCABB,
0xC3C1B2,
0xBDBAAB,
0xAAA69A,
0xAAA69B,
0xA9A59A,
0xABA79E,
0xA9A59C,
0x9D9990,
0xA09C93,
0xA29E93,
0x9E9A8F,
0xA3A08D,
0x9A9484,
0x8E887A,
0x979586,
0x9A9B8B,
0xA6A996,
0xADAFA1,
0xAFB4AE,
0xBEC9CB,
0xC1D8DE,
0xBAD8E0,
0xB5D3DB,
0xB7D2D9,
0xB3D0D6,
0xAACFD5,
0xA5D0D6,
0xA2CFD4,
0xA0CDD3,
0x9ECBD1,
0x9AC9D1,
0x97C7D1,
0x93C5D0,
0x90C2CD,
0x8DC1CE,
0x8ABECB,
0x87BCCA,
0x82BAC7,
0x7FB9C7,
0x7CB7C5,
0x78B6C5,
0x74B3C4,
0x70B2C2,
0x6CAFBF,
0x6AADBD,
0x67ACBC,
0x62AAB9,
0x5DA9B9,
0x5AA7B7,
0x58A5B7,
0x55A4B5,
0x53A1B5,
0x4FA0B3,
0x4B9DB2,
0x499BB0,
0x459AAE,
0x4299AD,
0x3F97AB,
0x3D95A9,
0x3994A7,
0x3893A8,
0x3590A5,
0x338FA4,
0x318DA2,
0x2F8BA0,
0x2B8AA0,
0x2A899F,
0x27879D,
0x25859B,
0x24849A,
0x228298,
0x218197,
0x208096,
0x1E7E94,
0x1E7D93,
0x1C7C92,
0x1B7B91,
0x1B7A8E,
0x1C788D,
0x1B778C,
0x1C778A,
0x1B7689,
0x1A7588,
0x1A7587,
0x1B7486,
0x1A7385,
0x1B7283,
0x1B7182,
0x1C6F81,
0x1D6E7F,
0x1C6D7E,
0x1D6C7B,
0x1D6C7B,
0x1E6C79,
0x1E6A78,
0x206A77,
0x216977,
0x226874,
0x236774,
0x246875,
0x246875,
0x236877,
0x206A77,
0x1E6A78,
0x1E6C79,
0x1D6C7B,
0x1B6C7D,
0x1B6C7D,
0x1B6E80,
0x1C6F81,
0x1B7084,
0x1A7184,
0x1B7285,
0x1B7486,
0x1C7587,
0x1C7488,
0x1D7589,
0x1B7689,
0x1C778A,
0x1C788D,
0x1C788D,
0x1C7B8F,
0x1D7C90,
0x1E7D93,
0x1F7E94,
0x207F95,
0x228197,
0x238298,
0x25849A,
0x26859B,
0x28879D,
0x29889C,
0x2B8A9E,
0x2C8B9F,
0x2E8DA1,
0x318DA2,
0x3490A5,
0x3692A7,
0x3994A7,
0x3D95A9,
0x3F97AB,
0x4299AD,
0x459AAE,
0x479CB0,
0x4B9DB2,
0x4EA1B3,
0x50A3B5,
0x55A5B0,
0x55A8B0,
0x59A9B0,
0x5EABB3,
0x65ABB7,
0x68ABBC,
0x68AFC1,
0x69B2C3,
0x6DB2C1,
0x77B2C2,
0x7BB2C6,
0x7BB6C8,
0x7ABCCC,
0x7EBECA,
0x86BDC4,
0x91BCC2,
0x90BDD2,
0x91C0D0,
0x8EBFC6,
0x92C0C0,
0x99C3C1,
0x98C4C5,
0x97C8CF,
0x98C8D4,
0xA8C9D2,
0xA9B1B4,
0xA49693,
0xA6948A,
0x9B9185,
0x999687,
0xA29D8A,
0xA19984,
0x9C9487,
0x948C7F,
0xA9A292,
0xB2AB9B,
0xB5AE9E,
0xBDB7A7,
0xC5BFAF,
0xCCC6B8,
0xD6D0C2,
0xDFD9CD,
0xD4CEC2,
0xCFC9BD,
0xD6D0C4,
0xD2CABD,
0xD0C8BB,
0xD6CEC1,
0xD5D6C8,
0xDBDCCC,
0xD6D7C7,
0xD0D1C1,
0xCDCEBE,
0xBCBAAB,
0xB8B6A7,
0xB4B2A5,
0xB4B2A6,
0xB8B4AB,
0xA6A19B,
0xA49F99,
0xA09B97,
0xA8A39D,
0xA49F99,
0x9A958F,
0x969380,
0x8F8979,
0x949182,
0x9F9D8E,
0x9FA291,
0xAAAD9C,
0xA7A99C,
0xABB0AA,
0xC3CED0,
0xBFD6DC,
0xB9D7DF,
0xB8D5DD,
0xB6D1D8,
0xB3D0D6,
0xACD1D7,
0xA5D0D6,
0xA2CED9,
0x9ECCD9,
0x9CCAD7,
0x99C9D7,
0x96C7D6,
0x90C5D5,
0x8CC4D5,
0x89C2D3,
0x85C0D2,
0x81BED0,
0x7DBCCD,
0x79BBCB,
0x77B9C9,
0x74B7C7,
0x71B4C4,
0x6FB3C0,
0x6CAFBF,
0x6AADBD,
0x67ACBC,
0x62AAB9,
0x5DA9B9,
0x5AA7B7,
0x58A5B7,
0x55A4B5,
0x53A1B5,
0x4FA0B3,
0x4B9DB2,
0x499BB0,
0x459AAE,
0x4299AD,
0x3F97AB,
0x3D95A9,
0x3994A7,
0x3893A8,
0x3590A5,
0x338FA4,
0x318DA2,
0x2E8A9F,
0x2B8AA0,
0x2A899F,
0x27879D,
0x25859B,
0x238399,
0x228298,
0x218197,
0x208096,
0x1E7E94,
0x1E7D93,
0x1C7C92,
0x1B7B91,
0x1B7A8E,
0x1C788D,
0x1B778C,
0x1C778A,
0x1B7689,
0x1A7588,
0x1A7587,
0x1C7587,
0x1B7486,
0x1B7283,
0x1B7182,
0x1D7082,
0x1D6E7F,
0x1C6D7E,
0x1D6C7B,
0x1D6C7B,
0x1E6C79,
0x1F6B79,
0x206A77,
0x216977,
0x226874,
0x236774,
0x236772,
0x236774,
0x226874,
0x1F6976,
0x1E6A77,
0x1E6C79,
0x1D6C7B,
0x1B6D7B,
0x1C6D7E,
0x1B6E80,
0x1C6F81,
0x1B7084,
0x1B7084,
0x1B7285,
0x1B7486,
0x1B7387,
0x1A7588,
0x1B7689,
0x1B7689,
0x1B788A,
0x1C788D,
0x1B7A8E,
0x1C7B8F,
0x1D7C90,
0x1D7D93,
0x1E7E94,
0x1F7F95,
0x218197,
0x228298,
0x238399,
0x26859B,
0x27869C,
0x29889E,
0x2B8AA0,
0x2D8CA0,
0x2F8EA2,
0x328EA3,
0x3490A5,
0x3792A7,
0x3994A7,
0x3D95A9,
0x3F96AA,
0x4299AC,
0x459AAE,
0x499BB0,
0x4B9DB2,
0x4D9FB4,
0x50A1B6,
0x53A4B5,
0x56A5B6,
0x5AA7B9,
0x5DA8BB,
0x61AABB,
0x65ACBE,
0x69AEBE,
0x6CAFC0,
0x70B3C3,
0x72B4C4,
0x76B5C4,
0x79B7C6,
0x7DB8C6,
0x81BBC9,
0x85BDCC,
0x87BFCE,
0x90BECB,
0x8FC4CC,
0x8AC4C8,
0x8DBFC6,
0x98BFCE,
0x9BC0D3,
0x96C4D4,
0x95C9D6,
0x9FCBCE,
0xADC0BC,
0x9C988C,
0x9F937D,
0x9F987C,
0x9A967B,
0xA59D8A,
0xAE9C92,
0xA79C8A,
0xA49C89,
0xACA491,
0xB6AD9C,
0xB9B2A2,
0xBEB7A7,
0xC7C1B3,
0xCDC7B9,
0xDBD5C9,
0xD9D3C7,
0xC8C2B6,
0xCBC5B9,
0xD5CFC3,
0xCFC9BD,
0xD3CDC1,
0xDDD7CB,
0xE7DECD,
0xE1D8C7,
0xDBD4C4,
0xD3CCBC,
0xCCC5B5,
0xBCB5A5,
0xB8B0A3,
0xBFB7AA,
0xC4BEB0,
0xC2BCAE,
0xB7B1A3,
0xADA797,
0xACA696,
0xA8A292,
0x9F9989,
0x9B9585,
0x989187,
0x9B9482,
0x9E9882,
0xA5A08D,
0xA3A194,
0xA9A798,
0xA6A591,
0xB2B6A8,
0xC7D5D8,
0xBAD5E6,
0xB4D8E8,
0xB6D7E0,
0xB6D1D8,
0xB6CED8,
0xAECFD8,
0xA7CFD7,
0xA5CDD7,
0xA2CAD4,
0x9EC9D2,
0x9BC7D0,
0x95C4CE,
0x92C2CE,
0x91C3CE,
0x8CC0CD,
0x89BECC,
0x84BCC9,
0x81BBC9,
0x7EB9C9,
0x79B7C6,
0x76B5C6,
0x72B4C4,
0x6FB1C1,
0x6CB1C0,
0x6AAFBE,
0x65ADBB,
0x62AAB9,
0x5FA8B7,
0x5BA7B7,
0x57A4B6,
0x55A4B5,
0x50A1B4,
0x4EA1B3,
0x4B9DB2,
0x479CB0,
0x449BAF,
0x4299AD,
0x4097AB,
0x3E95A9,
0x3994A9,
0x3792A7,
0x3590A5,
0x328EA3,
0x308CA3,
0x2D8CA2,
0x2B8AA0,
0x28889E,
0x28889E,
0x26869C,
0x24849A,
0x228298,
0x218197,
0x208096,
0x1E7E94,
0x1D7D93,
0x1C7D90,
0x1A7B8E,
0x1A798D,
0x1C788D,
0x1C788D,
0x1C778A,
0x1B7689,
0x1A7587,
0x1C7587,
0x1B7486,
0x1A7383,
0x1B7283,
0x1B7182,
0x1D7080,
0x1D6E7F,
0x1C6D7E,
0x1D6C7B,
0x1D6C7B,
0x1D6B78,
0x1E6A78,
0x206A77,
0x216977,
0x236975,
0x246875,
0x236772,
0x236774,
0x226874,
0x1F6976,
0x1E6A77,
0x1D6B78,
0x1D6C7B,
0x1B6D7B,
0x1C6D7E,
0x1B6E80,
0x1C6F81,
0x1B7084,
0x1B7084,
0x1B7285,
0x1B7486,
0x1B7387,
0x1A7588,
0x1B7689,
0x1B7689,
0x1B788A,
0x1C788D,
0x1B7A8E,
0x1C7B8F,
0x1D7C90,
0x1D7D93,
0x1E7E94,
0x1F7F95,
0x218197,
0x228298,
0x238399,
0x26859B,
0x27869C,
0x29889E,
0x2B8AA0,
0x2D8CA0,
0x2F8EA2,
0x328EA3,
0x3490A5,
0x3792A7,
0x3994A7,
0x3D95A9,
0x4097AB,
0x4299AC,
0x459AAE,
0x499BB0,
0x4B9DB2,
0x4D9FB4,
0x50A1B6,
0x53A4B5,
0x56A5B6,
0x5AA7B9,
0x5DA8BB,
0x61AABB,
0x64ABBD,
0x69AEBE,
0x6CAFC0,
0x6FB2C2,
0x72B4C4,
0x76B5C4,
0x79B7C6,
0x7DB8C6,
0x80BAC8,
0x84BCCB,
0x86BECD,
0x8EBECA,
0x8FC4CC,
0x8EC5CC,
0x8FC1CA,
0x98C0CC,
0x9ABFD1,
0x99C3D3,
0x97C9D4,
0x9DCACF,
0xABC4C1,
0x9F9D91,
0xA49784,
0x9D9276,
0x9D967A,
0xAA9F8D,
0xAB998D,
0xA89D8B,
0xAFA492,
0xAEA594,
0xB2A998,
0xBBB4A4,
0xC0B9A9,
0xC6BEB1,
0xD0CABC,
0xD3CDBF,
0xD6D0C2,
0xCAC4B6,
0xCBC5B7,
0xD4CEC0,
0xD4CEC0,
0xD9D3C5,
0xDED8CA,
0xDFD8C8,
0xDCD5C5,
0xD7CFC2,
0xD0C8BB,
0xCDC5B8,
0xC3BBAE,
0xC4BCAF,
0xC9C1B4,
0xCCC4B7,
0xCAC4B6,
0xC3BDAD,
0xBBB5A5,
0xB6B19E,
0xAFAA97,
0xA8A390,
0xA5A08D,
0xA39B8E,
0xA69F8D,
0xA09986,
0xA49F8C,
0xA29F90,
0xA4A391,
0xA8A999,
0xB8C0B5,
0xC5D7D9,
0xBBD7E3,
0xB5D6E5,
0xB7D5DF,
0xB6D1D8,
0xB4CFD6,
0xADCFD8,
0xA4CFD8,
0xA4CCD6,
0xA2CAD4,
0x9EC9D2,
0x9BC7D0,
0x96C5CF,
0x93C3CF,
0x90C2CD,
0x8CC0CD,
0x89BECC,
0x84BCC9,
0x81BBC9,
0x7EB9C9,
0x79B7C6,
0x76B5C6,
0x72B4C4,
0x6FB1C1,
0x6BB0BF,
0x69AEBD,
0x65ADBC,
0x62AAB9,
0x5FA8B9,
0x5BA7B7,
0x57A4B6,
0x55A4B5,
0x50A1B4,
0x4DA0B2,
0x4B9DB2,
0x479CB0,
0x449BAF,
0x4299AD,
0x4097AB,
0x3E95A9,
0x3994A9,
0x3792A7,
0x3590A5,
0x328EA3,
0x318DA4,
0x2D8CA2,
0x2B8AA0,
0x28889E,
0x27879D,
0x26869C,
0x24849A,
0x228298,
0x218197,
0x208096,
0x1E7E94,
0x1D7D93,
0x1C7D90,
0x1B7C8F,
0x1A798D,
0x1C788D,
0x1B778C,
0x1C778A,
0x1B7689,
0x1A7587,
0x1C7587,
0x1B7486,
0x1A7383,
0x1A7182,
0x1B7182,
0x1C6F7F,
0x1C6D7E,
0x1E6D7E,
0x1D6C7B,
0x1D6C7B,
0x1D6B78,
0x1E6A78,
0x206A77,
0x216977,
0x236975,
0x246875,
0x236772,
0x236774,
0x226874,
0x1F6976,
0x1E6A77,
0x1D6B78,
0x1D6C7B,
0x1B6D7B,
0x1B6C7D,
0x1C6D80,
0x1C6F81,
0x1B7084,
0x1B7084,
0x1B7285,
0x1B7486,
0x1B7486,
0x1A7588,
0x1B7689,
0x1B7689,
0x1B788A,
0x1C788D,
0x1B7A8E,
0x1C7B8F,
0x1D7C90,
0x1D7D93,
0x1E7E94,
0x1F7F95,
0x218197,
0x228298,
0x238399,
0x26859B,
0x27869C,
0x29889E,
0x2A899F,
0x2C8B9F,
0x2E8DA1,
0x328EA3,
0x338FA4,
0x3792A7,
0x3893A6,
0x3D95A9,
0x4097AB,
0x4299AD,
0x459AAE,
0x479CB0,
0x4B9DB2,
0x4D9FB4,
0x4FA1B6,
0x53A4B5,
0x56A5B6,
0x5AA7B9,
0x5DA9B9,
0x61AABB,
0x64ABBD,
0x69AEBE,
0x6BAEBF,
0x6EB1C1,
0x71B3C1,
0x76B5C4,
0x79B7C6,
0x7DB8C6,
0x80BAC8,
0x84BCCB,
0x85BDCA,
0x8AC0CA,
0x8FC3CE,
0x92C4CF,
0x92C2CE,
0x94C0CB,
0x99BFCC,
0x9BC1CE,
0x99C8D2,
0x9BCAD0,
0xA9C7C7,
0xA5AAA3,
0x9C8F7E,
0xA6967C,
0x9E9578,
0xA1957F,
0xAC9A8C,
0xA69A8A,
0xAB9F8F,
0xACA392,
0xB3AA99,
0xB7AE9F,
0xB3AC9C,
0xBBB4A4,
0xCFC8B8,
0xD7D1C1,
0xD7D1C1,
0xCAC4B4,
0xCAC4B4,
0xD6D0C0,
0xDFDAC7,
0xE1DCC9,
0xDBD6C3,
0xD6D0C2,
0xD3CDBF,
0xCFC7BA,
0xC7BFB2,
0xC9C1B4,
0xC9C1B4,
0xCFC7BA,
0xD5CDC0,
0xD1C9BC,
0xC9C2B2,
0xBFB8A8,
0xB7B09E,
0xB2AB99,
0xAEA794,
0xA9A28F,
0xA49D8A,
0xA29988,
0xA59E8E,
0x9F9989,
0x9E9986,
0xA09E89,
0x9A9985,
0xA3A69B,
0xB8C4C2,
0xC2DADE,
0xBAD7DF,
0xB7D5E0,
0xB7D2DD,
0xB5D1D5,
0xB2D1D4,
0xAACFD7,
0xA3CDDB,
0xA4CCD6,
0xA1C9D3,
0x9EC9D2,
0x9BC7D0,
0x96C5CF,
0x92C4CF,
0x90C2CD,
0x8CC0CD,
0x88BDCB,
0x84BCC9,
0x81BBC9,
0x7EB9C9,
0x79B7C6,
0x75B4C3,
0x72B4C4,
0x6EB0C0,
0x6BB0BF,
0x69AEBD,
0x64ACBB,
0x62AAB9,
0x5FA8B9,
0x5BA7B7,
0x57A4B6,
0x54A3B4,
0x50A1B4,
0x4D9FB4,
0x4B9DB2,
0x469BAF,
0x449BAF,
0x4299AD,
0x3F96AA,
0x3E95A9,
0x3994A9,
0x3792A7,
0x3590A5,
0x328EA3,
0x318DA4,
0x2D8CA2,
0x2B8AA0,
0x28889E,
0x27879D,
0x25859B,
0x238399,
0x228298,
0x218197,
0x1F7F95,
0x1E7E94,
0x1D7D93,
0x1D7E91,
0x1B7C8F,
0x1A798D,
0x1B778C,
0x1B778C,
0x1B7689,
0x1B7689,
0x1A7588,
0x1C7587,
0x1B7486,
0x1A7383,
0x1A7182,
0x1B7182,
0x1C6F81,
0x1C6D7E,
0x1D6C7D,
0x1D6C7B,
0x1D6C7B,
0x1D6B78,
0x1E6A78,
0x206A77,
0x206876,
0x226874,
0x236774,
0x236772,
0x236774,
0x226874,
0x206874,
0x1E6A77,
0x1D6B78,
0x1D6C7B,
0x1D6C7B,
0x1C6D7E,
0x1C6D80,
0x1C6F81,
0x1B7084,
0x1B7084,
0x1B7285,
0x1B7486,
0x1B7486,
0x1A7588,
0x1B7689,
0x1B7689,
0x1B788A,
0x1C788D,
0x1A798D,
0x1C7B8F,
0x1D7C90,
0x1D7D93,
0x1E7E94,
0x1F7F95,
0x208096,
0x228298,
0x238399,
0x25859B,
0x27869C,
0x29889E,
0x2A899F,
0x2C8B9F,
0x2E8DA1,
0x318DA2,
0x338FA4,
0x3792A7,
0x3893A6,
0x3C94A8,
0x3E96AA,
0x4198AC,
0x4499AD,
0x469BAF,
0x4A9CB1,
0x4D9FB4,
0x4EA0B5,
0x52A3B4,
0x56A5B6,
0x59A6B8,
0x5DA9B9,
0x61AABB,
0x64ABBD,
0x68ADBD,
0x6AAFBF,
0x6EB1C1,
0x71B3C1,
0x76B5C4,
0x79B7C4,
0x7DB8C6,
0x80BAC8,
0x83BBCA,
0x85BDCA,
0x86C1C9,
0x8EC2CF,
0x93C3D1,
0x93C3CF,
0x93C2CA,
0x98BEC7,
0x9EC0CA,
0x9BC6CF,
0x97C8CF,
0xA5CBCC,
0xADB8B2,
0x9B8E7E,
0xA38F76,
0x9C8D70,
0x9F9079,
0xA29181,
0x9F9482,
0x9D9280,
0xA79C8A,
0xB0A796,
0xAEA594,
0xABA291,
0xB5AE9C,
0xC4BDAB,
0xC9C2B0,
0xCBC4B2,
0xC4BFAB,
0xC1BCA8,
0xC4BFAB,
0xCBC6B2,
0xD4CFBB,
0xD5D0BC,
0xD6D0C2,
0xD3CDBF,
0xCFC9BB,
0xC6C0B2,
0xC4BEB0,
0xC4BCAF,
0xC7BFB2,
0xC9C1B4,
0xCBC4B4,
0xBFB8A8,
0xB5AC9D,
0xAEA594,
0xABA291,
0xAFA794,
0xAFA794,
0xA79F8C,
0xACA18B,
0xA59E8E,
0xA09A8E,
0x9B9683,
0x959176,
0x959480,
0xA4ABA4,
0xC5D8DC,
0xBDDADE,
0xB8D7DC,
0xB8D3DC,
0xB9D1DB,
0xB4D0D4,
0xAFD1D3,
0xA8D1D7,
0xA2CDDD,
0xA3CBD5,
0xA1C9D3,
0x9DC8D1,
0x9BC7D0,
0x95C5CF,
0x92C4CF,
0x8EC2CD,
0x8BBFCC,
0x88BDCB,
0x83BBC8,
0x80BAC8,
0x7EB9C9,
0x78B6C5,
0x75B4C3,
0x71B3C3,
0x6DB0C0,
0x6AAFBE,
0x68ADBC,
0x64ACBB,
0x60A9B8,
0x5FA8B9,
0x5AA6B6,
0x57A4B6,
0x54A2B6,
0x50A1B4,
0x4D9FB4,
0x489DB1,
0x469BAF,
0x439AAE,
0x4299AD,
0x3F96AA,
0x3E95A9,
0x3994A9,
0x3792A7,
0x3590A5,
0x328EA3,
0x308CA3,
0x2D8CA2,
0x2B8AA0,
0x28889E,
0x27879D,
0x25859B,
0x238399,
0x228298,
0x218197,
0x1F7F95,
0x1E7E94,
0x1C7C92,
0x1C7C92,
0x1B7C8F,
0x1A798D,
0x1B778C,
0x1B778C,
0x1B7689,
0x1A7588,
0x1C7488,
0x1C7587,
0x1B7486,
0x1B7283,
0x1A7182,
0x1B7182,
0x1C6F81,
0x1C6D7E,
0x1D6C7D,
0x1D6C7B,
0x1D6C7B,
0x1E6C79,
0x1E6A78,
0x206A77,
0x206876,
0x236774,
0x236774,
0x256773,
0x236774,
0x226874,
0x216975,
0x1E6A77,
0x1D6B78,
0x1D6C79,
0x1D6C7B,
0x1C6D7E,
0x1C6D80,
0x1C6F81,
0x1D6F84,
0x1B7084,
0x1B7285,
0x1B7486,
0x1B7486,
0x1A7588,
0x1B7689,
0x1B7689,
0x1C778A,
0x1C788D,
0x1C788D,
0x1B7A8E,
0x1C7B8F,
0x1D7D93,
0x1E7E94,
0x1F7F95,
0x208096,
0x228298,
0x238399,
0x25859B,
0x27869C,
0x29889E,
0x2A899F,
0x2C8BA1,
0x2E8DA1,
0x318DA2,
0x338FA4,
0x3692A7,
0x3893A6,
0x3C94A8,
0x3E96AA,
0x4198AC,
0x4499AD,
0x469BAF,
0x4A9CB1,
0x4C9EB3,
0x4EA0B5,
0x52A3B4,
0x56A5B6,
0x59A6B8,
0x5CA8B8,
0x60A9BA,
0x63AABC,
0x68ADBD,
0x69AEBD,
0x6DB0C0,
0x70B2C0,
0x73B5C3,
0x79B7C4,
0x7DB8C6,
0x80BAC8,
0x83BBCA,
0x84BCC9,
0x82C2CB,
0x8CC0CE,
0x95C0D3,
0x94C2D1,
0x92C3CA,
0x98C1C7,
0x9FC0C9,
0x9DC3CC,
0x96C7CE,
0xA3CFD2,
0xA1B0AB,
0x837568,
0xA99179,
0x968365,
0x998970,
0x9B8A7A,
0x988B78,
0x988B78,
0x9E937F,
0xA69B87,
0xABA08C,
0xB2A793,
0xB7AF9A,
0xB9B19C,
0xBEB6A1,
0xBEB8A2,
0xBCB6A0,
0xBCB6A0,
0xBBB59F,
0xBFB9A3,
0xCAC4AE,
0xD1CBB5,
0xCAC5B2,
0xCCC7B4,
0xD0CBB8,
0xCDC7B7,
0xC9C3B3,
0xC3BCAC,
0xC1BAAA,
0xBEB7A7,
0xBAB1A2,
0xB3AA9B,
0xADA495,
0xA99D8F,
0xA69A8A,
0xAEA292,
0xB2A696,
0xAA9E8E,
0xA89D87,
0xA29B8B,
0x949084,
0x999481,
0x958F75,
0x999884,
0xB8C3BF,
0xC1DAE1,
0xB7D8DF,
0xB7D6DB,
0xBAD3D8,
0xBAD1D7,
0xB2CFD3,
0xABD1D2,
0xA5D0D7,
0xA2CEDB,
0xA2CAD4,
0x9EC9D2,
0x9DC8D1,
0x98C7D1,
0x94C4CE,
0x91C3CE,
0x8DC1CC,
0x89BFCB,
0x87BCCA,
0x83BBC8,
0x80BAC8,
0x7DB8C6,
0x78B6C5,
0x75B4C3,
0x71B3C3,
0x6DB0C0,
0x6AAFBE,
0x66AEBD,
0x63ABBA,
0x60A9B8,
0x5EA7B8,
0x5AA6B6,
0x57A4B6,
0x54A2B6,
0x4FA0B3,
0x4D9FB4,
0x489DB1,
0x469BAF,
0x439AAE,
0x4198AC,
0x3F96AA,
0x3D94A8,
0x3893A8,
0x3691A6,
0x348FA4,
0x318DA2,
0x308CA3,
0x2C8BA1,
0x2A899F,
0x28889E,
0x26869C,
0x25859B,
0x238399,
0x228298,
0x218197,
0x1F7F95,
0x1E7E94,
0x1C7C92,
0x1C7C92,
0x1A7A90,
0x1A798D,
0x1C788D,
0x1C778C,
0x1B768B,
0x1A7588,
0x1C7488,
0x1B7486,
0x1B7486,
0x1B7285,
0x1A7182,
0x1B7182,
0x1C6F81,
0x1C6D80,
0x1D6C7D,
0x1D6C7B,
0x1E6C79,
0x1E6C79,
0x1E6A78,
0x206A77,
0x226776,
0x236774,
0x236774,
0x256773,
0x246873,
0x226874,
0x216975,
0x1E6A77,
0x1D6B78,
0x1D6C79,
0x1D6C7B,
0x1B6C7D,
0x1C6D7E,
0x1C6F81,
0x1D7082,
0x1B7084,
0x1B7285,
0x1A7385,
0x1B7486,
0x1C7488,
0x1B7689,
0x1B7689,
0x1C778A,
0x1C788D,
0x1C788D,
0x1B7A8E,
0x1C7B8F,
0x1C7C92,
0x1D7D93,
0x1F7F95,
0x208096,
0x218197,
0x238399,
0x24849A,
0x26869C,
0x29889E,
0x2A899F,
0x2C8BA1,
0x2E8DA1,
0x318DA2,
0x338FA4,
0x3591A6,
0x3893A6,
0x3C94A8,
0x3E96AA,
0x4198AC,
0x439AAD,
0x469BAF,
0x489DB1,
0x4C9EB3,
0x4EA0B5,
0x51A2B3,
0x55A4B5,
0x59A6B8,
0x5CA8B8,
0x60A9BA,
0x63AABC,
0x67ACBC,
0x69AEBD,
0x6DB0C0,
0x70B2C0,
0x73B5C3,
0x77B7C3,
0x7AB8C5,
0x7FB9C7,
0x81BBC9,
0x84BCC9,
0x82C2CB,
0x8BBFCD,
0x94BFD2,
0x94C2D2,
0x92C2CC,
0x96C1C7,
0x9EC1C7,
0x9CC2CB,
0x97C8CF,
0xA0CCCD,
0x879794,
0x56483B,
0x937B63,
0xA28D70,
0x928269,
0x938270,
0x93876D,
0x978B71,
0x988C72,
0x9D9179,
0xA99D85,
0xADA189,
0xADA38A,
0xB3A892,
0xBAAF99,
0xB1A992,
0xADA590,
0xB3AB96,
0xBCB49F,
0xC0B8A3,
0xC2BAA5,
0xC2BCA6,
0xB5B09C,
0xBBB6A2,
0xC7C2AE,
0xCBC6B2,
0xCDC6B3,
0xCAC3B1,
0xCAC3B1,
0xC7BEAD,
0xBEB5A4,
0xB8AFA0,
0xB5A99B,
0xAFA395,
0xAB9F91,
0xB1A597,
0xB2A497,
0xA99B8E,
0x938874,
0x968F7F,
0x939081,
0x918C79,
0xA49E88,
0x7D7C6A,
0xA3AFAB,
0xC4DFE6,
0xB4D6E0,
0xB4D5DE,
0xB7D3D7,
0xB8D1D6,
0xB1D0D5,
0xA9D0D5,
0xA4CFD6,
0xA2CDD6,
0xA2CAD4,
0x9DC8D1,
0x9BC7D0,
0x97C6D0,
0x93C3CD,
0x91C3CE,
0x8CC0CB,
0x88BECA,
0x87BCCA,
0x82BAC7,
0x7FB9C7,
0x7CB7C5,
0x77B5C4,
0x74B3C2,
0x70B2C2,
0x6CAFBF,
0x67AFBE,
0x65ADBC,
0x63ABBA,
0x60A9BA,
0x5CA8B8,
0x59A6B8,
0x56A3B5,
0x53A1B5,
0x4FA0B3,
0x4C9EB3,
0x489DB1,
0x469BAF,
0x439AAE,
0x4198AC,
0x3F96AA,
0x3C94A8,
0x3792A7,
0x3691A6,
0x338EA3,
0x318DA2,
0x2F8BA2,
0x2C8BA1,
0x2A899F,
0x27879D,
0x26869C,
0x25859B,
0x238399,
0x218197,
0x208096,
0x1F7F95,
0x1D7D93,
0x1C7C92,
0x1B7B91,
0x1B7A90,
0x1A798D,
0x1C788D,
0x1C778C,
0x1B768B,
0x1C7488,
0x1C7488,
0x1B7486,
0x1B7486,
0x1B7285,
0x1B7182,
0x1D7082,
0x1D6E81,
0x1C6D80,
0x1E6C80,
0x1D6C79,
0x1E6C79,
0x1E6C79,
0x1E6A78,
0x206A77,
0x226776,
0x236774,
0x246674,
0x256773,
0x246873,
0x226874,
0x216975,
0x206A77,
0x1D6B78,
0x1D6C79,
0x1D6C7B,
0x1B6C7D,
0x1C6D7E,
0x1C6F81,
0x1C6F81,
0x1B7084,
0x1A7184,
0x1B7285,
0x1B7486,
0x1C7488,
0x1D7589,
0x1B7689,
0x1C778A,
0x1C788D,
0x1C788D,
0x1B7A8E,
0x1C7B8F,
0x1D7C92,
0x1D7D93,
0x1E7E94,
0x208096,
0x218197,
0x238399,
0x24849A,
0x25859B,
0x28879D,
0x2A899F,
0x2B8AA0,
0x2D8CA0,
0x318DA2,
0x338FA4,
0x3591A6,
0x3893A8,
0x3A95A8,
0x3D95A9,
0x4198AC,
0x439AAE,
0x469BAF,
0x489DB1,
0x4C9EB3,
0x4EA0B5,
0x51A2B3,
0x55A4B5,
0x58A5B7,
0x5BA7B7,
0x5FA8B9,
0x62A9BB,
0x67ACBC,
0x68ADBD,
0x6DB0C0,
0x6EB2BF,
0x72B4C2,
0x76B6C2,
0x79B7C4,
0x7DB8C6,
0x81BBC9,
0x84BCC9,
0x82C2CB,
0x8ABECC,
0x94BFD2,
0x94C2D2,
0x90C0CA,
0x93C0C6,
0x9AC1C8,
0x9AC2CA,
0x9BC8CD,
0x99C0BF,
0x7E8B84,
0x5B4F41,
0x5E452F,
0x937E63,
0x95836B,
0x897864,
0x8F8164,
0x938568,
0x948669,
0x9B8D72,
0xA19378,
0x9E9075,
0x9F9177,
0xA79B81,
0xACA088,
0xACA088,
0xAEA28C,
0xB2A791,
0xBAAF9B,
0xBFB4A0,
0xBDB5A0,
0xBFB7A4,
0xB5AF99,
0xBBB59F,
0xC6C0AA,
0xC7C1AB,
0xC9C2AF,
0xCBC3B0,
0xCDC5B2,
0xC6BEAB,
0xBEB6A3,
0xBBB09E,
0xB7AB9B,
0xB4A898,
0xAFA393,
0xACA090,
0xA79A8A,
0x9D9080,
0x928676,
0x8F8876,
0x928C7C,
0x989282,
0x7D7666,
0x514F42,
0xAEB9B1,
0xC2DBDF,
0xB5D7E3,
0xB2D4E0,
0xB4D3D8,
0xB3D2D5,
0xADD0D6,
0xA8CED9,
0xA6CED8,
0xA3CCD2,
0xA0CBD4,
0x9DC8D1,
0x9AC6CF,
0x96C5CF,
0x92C2CC,
0x90C2CD,
0x8CC0CB,
0x87BDC9,
0x84BCC9,
0x81B9C6,
0x7EB8C6,
0x7CB7C7,
0x77B5C4,
0x72B4C2,
0x70B2C2,
0x6CAFBF,
0x66AEBD,
0x65ADBC,
0x61AAB9,
0x5FA8B9,
0x5BA7B7,
0x58A5B7,
0x54A3B4,
0x51A2B5,
0x4FA0B3,
0x4C9EB3,
0x479CB0,
0x449BAF,
0x4299AD,
0x4097AB,
0x3D95A9,
0x3C94A8,
0x3792A7,
0x3590A5,
0x328EA3,
0x308CA1,
0x2F8BA2,
0x2C8BA1,
0x2A899F,
0x27879D,
0x26869C,
0x24849A,
0x238399,
0x218197,
0x208096,
0x1F7F95,
0x1D7D93,
0x1D7C92,
0x1B7B91,
0x1B7A90,
0x1A798F,
0x1C788D,
0x1C778C,
0x1B768B,
0x1C7488,
0x1D7589,
0x1B7486,
0x1A7385,
0x1B7285,
0x1B7182,
0x1D7082,
0x1D6E81,
0x1E6C80,
0x1E6C80,
0x1D6C79,
0x1E6C79,
0x1E6C79,
0x1E6A77,
0x216977,
0x226874,
0x236774,
0x246674,
0x256773,
0x246873,
0x236975,
0x216975,
0x206A77,
0x1D6B78,
0x1E6C79,
0x1D6C7B,
0x1D6C7D,
0x1B6C7D,
0x1C6D80,
0x1C6F81,
0x1A6F83,
0x1A7184,
0x1B7285,
0x1A7385,
0x1C7488,
0x1D7589,
0x1B7689,
0x1C778A,
0x1C788D,
0x1C788D,
0x1B7A8E,
0x1C7B8F,
0x1D7C92,
0x1D7D93,
0x1E7E94,
0x1F7F95,
0x218197,
0x228298,
0x24849A,
0x25859B,
0x28879D,
0x29889E,
0x2B8AA0,
0x2D8CA2,
0x2E8DA1,
0x328EA3,
0x3591A6,
0x3893A8,
0x3994A7,
0x3D95A9,
0x3F97AB,
0x4299AD,
0x459AAE,
0x479CB0,
0x499EB2,
0x4D9FB4,
0x51A2B3,
0x55A4B5,
0x57A6B7,
0x5BA6B9,
0x5FA8B9,
0x62A9BB,
0x64ACBB,
0x68ADBD,
0x6CAFBF,
0x6EB1C1,
0x71B3C1,
0x75B4C3,
0x78B6C3,
0x7DB8C6,
0x80BAC8,
0x82BCCA,
0x85C1CC,
0x89BECC,
0x91C0D0,
0x94C2D1,
0x90BECB,
0x8FBEC8,
0x93C2CA,
0x96C3C9,
0x9FC7C9,
0x93B0AC,
0x888F87,
0x817567,
0x58432E,
0x4C3820,
0x897960,
0x93846D,
0x8A7B5C,
0x8B7C5D,
0x928364,
0x98896C,
0x998A6D,
0x9A8B6E,
0x9C8D70,
0x9B8B71,
0x9D8D73,
0xA6987E,
0xAD9E87,
0xAFA089,
0xB4A892,
0xBCAF9C,
0xBFB29F,
0xC3B6A3,
0xBFB7A4,
0xC5BDAA,
0xCFC4B2,
0xCABFAB,
0xC9BEAA,
0xCABFAB,
0xC8BDA9,
0xBCB19D,
0xAEA38F,
0xB0A591,
0xB6AB97,
0xB8AD99,
0xB0A58F,
0xA59A84,
0x9B907A,
0x948973,
0x898176,
0x8D8674,
0x9B9481,
0x6F6858,
0x413A30,
0x85857B,
0xB3BDB4,
0xBACECD,
0xBAD6E1,
0xB2D3E2,
0xAED3DC,
0xABD2D7,
0xA9CFD8,
0xA7CFDB,
0xA6CCD5,
0xA4CACD,
0xA0CBD4,
0x9BC7D0,
0x97C6D0,
0x94C4CE,
0x91C3CE,
0x8DC1CC,
0x8AC0CC,
0x87BDC9,
0x83BBC8,
0x80BAC6,
0x7EB8C6,
0x79B7C6,
0x75B4C3,
0x71B3C1,
0x6FB2C2,
0x6CAFBF,
0x66AEBD,
0x63ACBB,
0x61AABB,
0x5DA9B9,
0x5BA7B7,
0x58A5B7,
0x54A2B6,
0x50A1B4,
0x4D9FB4,
0x4B9DB2,
0x479CB0,
0x449BAF,
0x4299AD,
0x4097AB,
0x3D95A9,
0x3B93A7,
0x3792A7,
0x3590A5,
0x328EA3,
0x308CA1,
0x2F8BA2,
0x2C8BA1,
0x2A899F,
0x27879D,
0x25859B,
0x24849A,
0x228298,
0x218197,
0x208096,
0x1F7F95,
0x1E7D93,
0x1D7C92,
0x1B7B91,
0x1B7A90,
0x1A798D,
0x1B778C,
0x1B768B,
0x1B768B,
0x1D7589,
0x1D7589,
0x1B7486,
0x1A7385,
0x1B7285,
0x1B7182,
0x1D7082,
0x1D6E81,
0x1F6D81,
0x1E6C80,
0x1E6C79,
0x1E6C79,
0x1F6B79,
0x206A77,
0x216977,
0x226874,
0x236774,
0x236571,
0x246672,
0x226671,
0x226874,
0x206874,
0x206A77,
0x1E6A77,
0x1E6C79,
0x1E6D7C,
0x1E6D7E,
0x1C6D7E,
0x1C6D80,
0x1C6F81,
0x1B7084,
0x1C7185,
0x1C7386,
0x1A7385,
0x1B7387,
0x1C7488,
0x1D7589,
0x1B7689,
0x1C778C,
0x1C788D,
0x1B7A8E,
0x1C7B8F,
0x1D7C92,
0x1D7D93,
0x1E7E94,
0x208096,
0x218197,
0x228298,
0x24849A,
0x25859B,
0x28879D,
0x29889E,
0x2B8AA0,
0x2D8CA2,
0x2E8DA1,
0x328EA3,
0x3490A5,
0x3792A7,
0x3994A7,
0x3C94A8,
0x3F97AB,
0x4299AD,
0x439AAD,
0x469BAF,
0x489DB1,
0x4A9FB3,
0x50A1B2,
0x52A3B4,
0x56A5B6,
0x59A6B8,
0x5CA8B8,
0x60A9BA,
0x64ACBB,
0x68ADBD,
0x6AAFBE,
0x6DB0C0,
0x70B2C0,
0x74B3C2,
0x77B5C2,
0x7CB7C5,
0x7FB9C7,
0x81BBC9,
0x87BFCC,
0x87BDC9,
0x8EC2CD,
0x8EBCC9,
0x91BFCE,
0x8FC0CF,
0x8EC2CF,
0x95C4CA,
0xA4C4C3,
0x899891,
0x95978A,
0x9B9281,
0x877865,
0x594733,
0x42321B,
0x766750,
0x938266,
0x857458,
0x89785C,
0x907F63,
0x89785C,
0x907F63,
0x958468,
0x948367,
0x958468,
0x98876D,
0xA4947A,
0xA99980,
0xB2A28B,
0xB8A992,
0xB6A790,
0xBBAC97,
0xB9AC9B,
0xB5A897,
0xC1B4A3,
0xBFB29F,
0xC5B8A5,
0xC3B6A3,
0xC1B59F,
0xBAAF99,
0xB7AC96,
0xB1A78E,
0xB0A68D,
0xA69C83,
0x9B947A,
0x979074,
0x8E876B,
0x8A8367,
0x878078,
0x8E8777,
0x5C5441,
0x2F271A,
0x635E58,
0xA5A69E,
0xABB3A8,
0xAAB7B0,
0xC1D5DC,
0xB0D0DD,
0xAAD2DE,
0xA6D2DD,
0xA6D0DE,
0xA4C9D9,
0xA9CED6,
0xA4C7C9,
0x9DC9D4,
0x9BC7D2,
0x97C6D0,
0x94C4CE,
0x91C3CE,
0x8CC0CB,
0x88BECA,
0x84BDC8,
0x82BAC7,
0x7FB9C7,
0x7CB7C5,
0x78B6C5,
0x75B4C3,
0x71B3C3,
0x6DB0C0,
0x6AAFBE,
0x65AEBD,
0x63ACBB,
0x60A9BA,
0x5CA8B8,
0x5AA5B8,
0x57A4B6,
0x54A2B6,
0x50A1B4,
0x4D9FB4,
0x499EB2,
0x479CB0,
0x449BAF,
0x4299AD,
0x3F97AB,
0x3D95A9,
0x3B93A7,
0x3691A6,
0x3590A5,
0x328EA3,
0x308CA1,
0x2E8AA1,
0x2B8AA0,
0x29889E,
0x27879D,
0x25859B,
0x24849A,
0x238399,
0x218197,
0x208096,
0x1E7E94,
0x1E7D93,
0x1E7D93,
0x1C7B91,
0x1B7A90,
0x1C788D,
0x1B778C,
0x1C778C,
0x1D758B,
0x1D7589,
0x1C7488,
0x1B7486,
0x1B7285,
0x1B7285,
0x1B7182,
0x1C6F81,
0x1C6D80,
0x1E6C80,
0x1E6D7E,
0x1E6C79,
0x1E6C79,
0x1F6B79,
0x206A77,
0x216977,
0x226874,
0x236774,
0x246672,
0x246672,
0x246672,
0x236774,
0x206874,
0x206A77,
0x1E6A77,
0x1E6C79,
0x1E6D7C,
0x1E6D7E,
0x1C6D7E,
0x1C6D80,
0x1C6F81,
0x1B7084,
0x1C7185,
0x1B7285,
0x1C7386,
0x1C7488,
0x1C7488,
0x1D7589,
0x1C778A,
0x1C778C,
0x1C788D,
0x1D798E,
0x1C7B8F,
0x1D7C92,
0x1D7D93,
0x1E7E94,
0x208096,
0x218197,
0x228298,
0x24849A,
0x25859B,
0x27869C,
0x29889E,
0x2B8AA0,
0x2D8CA2,
0x2E8DA1,
0x328EA3,
0x338FA4,
0x3591A6,
0x3893A6,
0x3C94A8,
0x3E96AA,
0x4198AC,
0x439AAE,
0x469BAF,
0x489DB1,
0x4A9FB3,
0x50A1B2,
0x51A2B3,
0x56A5B6,
0x59A6B8,
0x5CA8B8,
0x60A9BA,
0x63ABBA,
0x65ADBC,
0x6AAFBE,
0x6BB0BF,
0x6FB3C0,
0x72B4C2,
0x76B6C2,
0x79B7C4,
0x7EB9C7,
0x81BBC9,
0x84B7C8,
0x86BFCA,
0x8CC5CC,
0x8CBCC8,
0x8EB9CA,
0x8BBECF,
0x88C2CE,
0x97C6CC,
0x95AEAB,
0x8C8F84,
0xA39D8D,
0xA49F8C,
0x9A9180,
0x958575,
0x665744,
0x3C3018,
0x594731,
0x816F59,
0x88765E,
0x827058,
0x816D54,
0x87735A,
0x8E7A5F,
0x8C785D,
0x927E63,
0x89755A,
0x917D62,
0x98876B,
0xA19074,
0xAC9C82,
0xB0A086,
0xB3A389,
0xB6A591,
0xB1A08C,
0xAE9F8A,
0xB3A48F,
0xBAAB96,
0xAC9D86,
0xAFA38B,
0xB3A78F,
0xAFA38B,
0xADA38A,
0x9B9178,
0x8C856B,
0x898268,
0x898268,
0x8B846A,
0x867F65,
0x726E65,
0x4A4235,
0x3B3221,
0x595144,
0x86837C,
0x9B9C96,
0x979A91,
0x9BA099,
0xB3C1C1,
0xB9D6DE,
0xA9D3E1,
0xA3D2E2,
0xA5D3E2,
0xA7CDDA,
0xA3C5CE,
0xA7C9CB,
0x9BC7D2,
0x98C7D1,
0x95C5CF,
0x92C4CF,
0x8EC2CD,
0x8BC1CD,
0x86BFCA,
0x84BCC9,
0x80BAC6,
0x7EB8C6,
0x7BB6C4,
0x77B5C4,
0x74B3C2,
0x6FB2C2,
0x6CAFBF,
0x69AEBD,
0x64ADBE,
0x62ABBC,
0x60A9BA,
0x5CA8B8,
0x59A6B8,
0x56A5B6,
0x53A1B5,
0x50A1B4,
0x4C9EB3,
0x499EB2,
0x469BB0,
0x439AAE,
0x4299AD,
0x3F97AB,
0x3C94A8,
0x3B93A7,
0x3691A6,
0x3490A5,
0x328EA3,
0x308CA1,
0x2C8BA1,
0x2A899F,
0x29889E,
0x27879D,
0x24849A,
0x24849A,
0x238399,
0x218197,
0x1F7F95,
0x1F7E94,
0x1E7D93,
0x1E7D93,
0x1B7A90,
0x1B7A8E,
0x1C788D,
0x1B778C,
0x1C778C,
0x1D7589,
0x1C7488,
0x1C7488,
0x1B7486,
0x1B7285,
0x1B7283,
0x1B7182,
0x1C6F81,
0x1C6D80,
0x1E6D7E,
0x1E6D7E,
0x1E6C79,
0x1E6C79,
0x1F6B78,
0x206A77,
0x216977,
0x226874,
0x236774,
0x246672,
0x246672,
0x246672,
0x236774,
0x216975,
0x206A77,
0x1E6A77,
0x1E6C79,
0x1E6D7C,
0x1D6C7D,
0x1E6D7E,
0x1C6D80,
0x1C6F81,
0x1D6F84,
0x1B7084,
0x1B7285,
0x1C7386,
0x1C7488,
0x1C7488,
0x1D7589,
0x1C778A,
0x1C778C,
0x1C788D,
0x1D798E,
0x1B7A8E,
0x1D7C92,
0x1D7D93,
0x1E7E94,
0x1F7F95,
0x218197,
0x228298,
0x238399,
0x24849A,
0x26859B,
0x28879D,
0x2A899F,
0x2C8BA1,
0x2E8DA3,
0x2F8EA2,
0x338FA4,
0x3490A5,
0x3893A8,
0x3994A7,
0x3E96AA,
0x4098AC,
0x439AAE,
0x459CAF,
0x489DB1,
0x499EB2,
0x4EA1B3,
0x51A2B5,
0x55A4B5,
0x58A5B7,
0x5BA6B9,
0x5FA8B9,
0x61AABB,
0x64ACBB,
0x69AEBE,
0x6BB0BF,
0x6EB1C1,
0x71B3C1,
0x75B4C3,
0x79B7C4,
0x7DB8C6,
0x7FBAC8,
0x87B8C9,
0x85BFCA,
0x87C2C8,
0x89BBC4,
0x8DB8C9,
0x8BBED1,
0x89C3CF,
0x96C2C5,
0x899893,
0x8D877B,
0x9C9481,
0x9A9784,
0x979181,
0x9F9385,
0x928272,
0x7C7259,
0x4A3929,
0x412E1D,
0x5D4A39,
0x7F6D59,
0x86715C,
0x89755D,
0x856F57,
0x887358,
0x8B7659,
0x846F52,
0x887555,
0x857252,
0x817150,
0x918062,
0x9C8D6E,
0x9E8F70,
0xA5947A,
0xA6957B,
0x9C8C72,
0x97876D,
0xA29278,
0x9D8F75,
0xA19379,
0xA1957B,
0xA49880,
0x968C73,
0x8D826C,
0x897E68,
0x7F7762,
0x817964,
0x78705B,
0x544C39,
0x363226,
0x4A4235,
0x6F6357,
0x857D72,
0x8A8A80,
0x8A8D86,
0x8D8C87,
0x97978D,
0xA8AFA7,
0xB5CDCD,
0xABD3DF,
0x9FCEE2,
0x9DCBDB,
0xA5CDD5,
0xA5C8CE,
0xA5C6CD,
0x98C7D1,
0x97C6D0,
0x94C4D0,
0x91C3CE,
0x8DC1CC,
0x8AC0CC,
0x85BEC9,
0x82BCC8,
0x80BAC8,
0x7DB8C6,
0x79B7C6,
0x76B5C4,
0x72B4C4,
0x6EB1C1,
0x6BB0BF,
0x69AEBD,
0x63ACBD,
0x62ABBC,
0x5DA9B9,
0x5BA6B9,
0x58A5B7,
0x55A4B5,
0x51A2B5,
0x4FA0B3,
0x4C9EB3,
0x489DB1,
0x459CB0,
0x439AAE,
0x4098AC,
0x3E96AA,
0x3C94A8,
0x3A92A6,
0x3691A6,
0x338FA4,
0x318DA2,
0x2F8BA0,
0x2C8BA1,
0x2A899F,
0x29889E,
0x26869C,
0x24849A,
0x238399,
0x228298,
0x208096,
0x1F7F95,
0x1F7E94,
0x1E7D93,
0x1D7C92,
0x1B7A8E,
0x1A798D,
0x1B778C,
0x1C778C,
0x1B768B,
0x1D7589,
0x1C7488,
0x1C7587,
0x1C7386,
0x1B7285,
0x1B7283,
0x1B7182,
0x1C6F81,
0x1C6D7E,
0x1E6D7E,
0x1F6C7E,
0x1E6C79,
0x1E6C79,
0x1F6B78,
0x206A77,
0x216975,
0x226874,
0x236774,
0x246672,
0x246672,
0x246672,
0x236772,
0x216975,
0x206A77,
0x1E6A77,
0x1D6B78,
0x1D6C7B,
0x1D6C7B,
0x1E6D7E,
0x1C6D80,
0x1C6F81,
0x1C6F81,
0x1B7084,
0x1B7285,
0x1C7386,
0x1B7387,
0x1C7488,
0x1D7589,
0x1B7689,
0x1C778C,
0x1B778C,
0x1C788D,
0x1B7A8E,
0x1C7B91,
0x1D7C92,
0x1E7E94,
0x1F7F95,
0x208096,
0x228298,
0x238399,
0x24849A,
0x26859B,
0x28879D,
0x2A899F,
0x2C8BA1,
0x2D8CA2,
0x2F8EA2,
0x338FA4,
0x3490A5,
0x3792A7,
0x3994A7,
0x3D95A9,
0x3F97AB,
0x4299AD,
0x449BAE,
0x479CB0,
0x499EB2,
0x4DA0B2,
0x50A1B4,
0x52A3B6,
0x57A6B7,
0x5AA5B8,
0x5CA8B8,
0x61AABB,
0x63ABBA,
0x67AFBE,
0x6AAFBE,
0x6DB2C1,
0x70B4C1,
0x73B5C3,
0x77B6C5,
0x7AB8C5,
0x7EB9C7,
0x8ABACE,
0x83BDC9,
0x81BEC3,
0x87BCC4,
0x8CBAC9,
0x89BCCD,
0x8EC4D0,
0x90B3B7,
0x898F8B,
0x968A7E,
0x958D7A,
0x8E907B,
0x949283,
0x968C80,
0x908372,
0x8E866F,
0x8A7A6D,
0x5D4B3D,
0x372616,
0x443120,
0x604B38,
0x7A664E,
0x89735B,
0x8B765B,
0x8A7558,
0x826D4E,
0x846F50,
0x837050,
0x7E6B4B,
0x817150,
0x867655,
0x867756,
0x897958,
0x867655,
0x897958,
0x877658,
0x928364,
0x948568,
0x8D7F64,
0x8D7F65,
0x8B7F67,
0x877B65,
0x8D826E,
0x867B69,
0x695D4D,
0x473E2F,
0x3A3024,
0x463C30,
0x6D6A59,
0x80786B,
0x897F75,
0x878173,
0x848478,
0x7E7F79,
0x7D7C77,
0x938F84,
0x9B9E8D,
0xA1B2A8,
0xAECFD6,
0xA5D0E1,
0xA1CCDC,
0xA2CBD1,
0xA4C9D1,
0xA1C5D1,
0x98C6D3,
0x95C5D1,
0x92C4CF,
0x8EC2CD,
0x8BC1CD,
0x86BFCA,
0x83BDC9,
0x81BBC7,
0x7EB9C7,
0x7CB7C5,
0x78B6C5,
0x75B4C3,
0x71B3C3,
0x6EB1C1,
0x6AAFBE,
0x66AEBC,
0x62ABBC,
0x5FABBB,
0x5CA8B8,
0x59A6B8,
0x58A5B7,
0x55A3B7,
0x50A1B4,
0x4EA0B5,
0x4B9DB2,
0x479CB0,
0x449BAF,
0x4299AD,
0x4098AC,
0x3E96AA,
0x3B93A7,
0x3A92A6,
0x3490A5,
0x338FA4,
0x318DA2,
0x2F8BA0,
0x2B8AA0,
0x2A899F,
0x28879D,
0x26869C,
0x24849A,
0x238399,
0x218197,
0x208096,
0x207F95,
0x1F7E94,
0x1E7D93,
0x1D7C92,
0x1B7A8E,
0x1A798D,
0x1B778C,
0x1C778C,
0x1C778A,
0x1D7589,
0x1C7488,
0x1D7487,
0x1C7386,
0x1B7283,
0x1B7182,
0x1B7182,
0x1C6F7F,
0x1C6D7E,
0x1E6D7E,
0x1E6B7D,
0x1E6C79,
0x1E6C78,
0x1E6A77,
0x206A77,
0x216975,
0x226874,
0x236774,
0x246672,
0x246672,
0x246672,
0x236772,
0x236975,
0x206A77,
0x1E6A77,
0x1D6B78,
0x1E6B7B,
0x1D6C7B,
0x1E6D7E,
0x1C6D80,
0x1D6E81,
0x1C6F81,
0x1B7084,
0x1B7285,
0x1C7386,
0x1C7387,
0x1C7488,
0x1C7488,
0x1D7589,
0x1B768B,
0x1B778C,
0x1C788D,
0x1A798D,
0x1C7B91,
0x1D7C92,
0x1D7D93,
0x1F7F95,
0x208096,
0x218197,
0x238399,
0x24849A,
0x26859B,
0x28879D,
0x29889E,
0x2B8AA0,
0x2C8BA1,
0x2E8DA1,
0x328EA3,
0x3490A5,
0x3792A7,
0x3893A6,
0x3D95A9,
0x3F97AB,
0x4299AD,
0x449BAF,
0x469DB0,
0x479EB1,
0x4DA0B2,
0x4FA2B4,
0x52A3B6,
0x56A4B8,
0x59A6B8,
0x5CA7BA,
0x60A9BA,
0x62ABBC,
0x66AEBD,
0x67AFBE,
0x6CB1C0,
0x6FB2C2,
0x72B4C2,
0x76B5C4,
0x7AB8C7,
0x7BB9C8,
0x83B7CC,
0x82BCC8,
0x81BDC5,
0x85BCC3,
0x88BAC5,
0x88BAC5,
0x95C1CA,
0x869EA0,
0x84837F,
0x95897D,
0x8F8A77,
0x848774,
0x868475,
0x8B8175,
0x897E6C,
0x7E765F,
0x897B6E,
0x918172,
0x776757,
0x544331,
0x3B2915,
0x3D2911,
0x4F3B22,
0x644F34,
0x7B6649,
0x826D50,
0x887354,
0x857252,
0x827153,
0x837254,
0x7F7051,
0x7C6D4E,
0x80704E,
0x817150,
0x8B7B5A,
0x877857,
0x867758,
0x857659,
0x7E7055,
0x7C6E54,
0x7E6F58,
0x726552,
0x615443,
0x493C2C,
0x3E3226,
0x44382C,
0x5C4F46,
0x83766D,
0x86816D,
0x888171,
0x887E72,
0x878173,
0x868779,
0x84857D,
0x868580,
0x918D81,
0x9D9C88,
0x9AA594,
0xADC3C1,
0xA9CBD5,
0xA4CEDA,
0x9FCCD2,
0x9AC5CE,
0x9DC4D3,
0x96C6D2,
0x94C4D0,
0x91C3CE,
0x8DC1CE,
0x8AC0CC,
0x86BECB,
0x82BCC8,
0x7FBAC8,
0x7DB8C6,
0x79B7C6,
0x76B5C4,
0x71B3C3,
0x6EB1C1,
0x6BB0BF,
0x67AFBE,
0x65ADBC,
0x60ACBC,
0x5EAABA,
0x5CA7BA,
0x59A6B8,
0x57A4B6,
0x54A2B6,
0x50A1B4,
0x4D9FB4,
0x489DB1,
0x479CB0,
0x449BAF,
0x4299AD,
0x3F97AB,
0x3D95A9,
0x3B93A7,
0x3991A5,
0x3490A5,
0x328EA3,
0x308CA1,
0x2E8A9F,
0x2B8AA0,
0x29889E,
0x27869C,
0x25859B,
0x24849A,
0x238399,
0x218197,
0x208096,
0x207F95,
0x1E7D93,
0x1D7C92,
0x1C7B91,
0x1B7A8E,
0x1A798D,
0x1B778C,
0x1C778C,
0x1C778A,
0x1D7589,
0x1C7488,
0x1D7487,
0x1B7285,
0x1B7283,
0x1B7182,
0x1A7081,
0x1C6F7F,
0x1C6D7E,
0x1D6C7D,
0x1E6B7D,
0x1E6C78,
0x1F6B78,
0x1E6A77,
0x206A77,
0x216975,
0x236774,
0x246672,
0x246672,
0x266672,
0x246672,
0x236772,
0x236975,
0x206A77,
0x1E6A77,
0x1D6B78,
0x1E6B7B,
0x1D6C7B,
0x1D6C7D,
0x1C6D80,
0x1D6E81,
0x1C6F81,
0x1B7084,
0x1B7285,
0x1B7285,
0x1C7387,
0x1D7488,
0x1D7589,
0x1D7589,
0x1C778C,
0x1B778C,
0x1C788D,
0x1B7A8E,
0x1C7B91,
0x1D7C92,
0x1D7D93,
0x1E7E94,
0x208096,
0x218197,
0x228298,
0x238399,
0x26859B,
0x27869C,
0x29889E,
0x2A899F,
0x2B8AA0,
0x2D8CA0,
0x328EA3,
0x3490A5,
0x3691A6,
0x3893A6,
0x3C94A8,
0x3E96AA,
0x4198AC,
0x439AAE,
0x459CAF,
0x469DB0,
0x4C9FB1,
0x4EA1B3,
0x51A2B5,
0x55A3B7,
0x58A5B7,
0x5BA6B9,
0x5FA8B9,
0x61AABB,
0x65ADBC,
0x66AEBD,
0x6BB0BF,
0x6EB1C1,
0x71B3C1,
0x75B4C3,
0x78B6C5,
0x7AB8C7,
0x7EB7CA,
0x81BCCA,
0x81BBC6,
0x80B9C2,
0x84BBC2,
0x8CBFC3,
0x92B5B9,
0x7C8789,
0x8E8683,
0x92887E,
0x868775,
0x828977,
0x868777,
0x898071,
0x847967,
0x736D57,
0x7C715F,
0x7F7462,
0x8F826F,
0x8D7E69,
0x76664F,
0x635139,
0x48341B,
0x3D290E,
0x3C270C,
0x4A351A,
0x574328,
0x5F4B30,
0x67563C,
0x726147,
0x76664D,
0x78684F,
0x77674D,
0x79694F,
0x76664C,
0x75654B,
0x716148,
0x6A5A41,
0x5F5137,
0x4B3C25,
0x3A2B16,
0x3B2C17,
0x40311E,
0x584838,
0x6E6151,
0x7A6D5D,
0x847669,
0x87796C,
0x817B65,
0x7E7765,
0x817A6A,
0x888274,
0x8A887B,
0x88887E,
0x8A8A80,
0x949285,
0x999782,
0xA7A994,
0xB8C4B8,
0xB3CDCC,
0x9EC5CC,
0x9CCBD5,
0x9BC9D8,
0x9BC6D6,
0x94C4D0,
0x92C3D1,
0x8DC1CE,
0x8AC0CC,
0x87C0CB,
0x84BECA,
0x82BCC8,
0x7FBAC8,
0x7AB8C5,
0x78B6C5,
0x75B4C3,
0x70B2C2,
0x6EB1C1,
0x6BB0C0,
0x66AEBD,
0x65ADBC,
0x5FABBB,
0x5DA9B9,
0x5BA6B9,
0x58A5B7,
0x55A4B5,
0x53A1B5,
0x4FA0B3,
0x4C9EB3,
0x489DB1,
0x469BB0,
0x439AAE,
0x4198AC,
0x3E96AA,
0x3C94A8,
0x3A92A6,
0x3991A5,
0x3490A5,
0x328EA3,
0x2F8BA0,
0x2D899E,
0x2A899F,
0x29889E,
0x27869C,
0x24849A,
0x24849A,
0x238399,
0x218197,
0x1F7F95,
0x1F7E94,
0x1E7D93,
0x1C7B91,
0x1B7A90,
0x1B7A8E,
0x1A798D,
0x1B778C,
0x1C778A,
0x1E768A,
0x1D7589,
0x1C7587,
0x1C7386,
0x1B7285,
0x1B7283,
0x1B7182,
0x1C6F7F,
0x1C6D7E,
0x1E6D7E,
0x1D6C7D,
0x1E6B7D,
0x1E6C78,
0x1E6A77,
0x1E6A77,
0x206A77,
0x206874,
0x236774,
0x246672,
0x266672,
0x266672,
0x236571,
0x236772,
0x236975,
0x206A75,
0x1E6A77,
0x1D6B78,
0x1E6B7B,
0x1D6C7B,
0x1D6C7D,
0x1C6D80,
0x1C6D80,
0x1C6F81,
0x1B7084,
0x1B7285,
0x1B7285,
0x1C7387,
0x1D7488,
0x1D7589,
0x1D7589,
0x1C778C,
0x1C778C,
0x1C788D,
0x1B7A8E,
0x1C7B91,
0x1D7C92,
0x1D7D93,
0x1E7E94,
0x1F7F95,
0x208096,
0x228298,
0x238399,
0x26859B,
0x27869C,
0x29889E,
0x2A899F,
0x2B8AA0,
0x2D8CA2,
0x318DA2,
0x338FA4,
0x3591A6,
0x3792A5,
0x3A95A8,
0x3E96AA,
0x3F97AB,
0x4299AD,
0x449BAE,
0x459CAF,
0x499EB2,
0x4DA0B2,
0x50A1B4,
0x54A2B6,
0x57A4B8,
0x5AA5B8,
0x5CA7BA,
0x60A9BA,
0x63ACBB,
0x65ADBC,
0x67AFBD,
0x6BB0BF,
0x6EB1C1,
0x72B4C2,
0x76B5C4,
0x79B7C6,
0x7BBACC,
0x7FBACA,
0x7EB6C3,
0x7DB8C0,
0x81BCC0,
0x8DBFC0,
0x809C9D,
0x767779,
0x8D8280,
0x938C82,
0x878E7E,
0x838C7B,
0x888677,
0x897D6F,
0x867B69,
0x807A64,
0x827A63,
0x8A8369,
0x857B62,
0x8C8066,
0x94866B,
0x96866C,
0x8E7D61,
0x7E6A4F,
0x6D593E,
0x57432A,
0x422E15,
0x39270F,
0x382610,
0x362511,
0x362513,
0x322310,
0x362416,
0x3E2C1E,
0x413020,
0x443323,
0x3C2B19,
0x322310,
0x362712,
0x3B2C17,
0x51422D,
0x73644D,
0x82735C,
0x877861,
0x8C7D66,
0x857661,
0x7D6E59,
0x7A6B56,
0x7B705A,
0x857D68,
0x87826E,
0x868374,
0x8C887D,
0x8D8D81,
0x8E9083,
0x919282,
0xA19E8B,
0xBAB59F,
0xBCC0A9,
0xB8CCC0,
0xA4CACD,
0x97C6D6,
0x97C8D9,
0x93C4D3,
0x92C3D1,
0x90C1CF,
0x8CC0CD,
0x89BFCB,
0x85BDCA,
0x82BCC8,
0x7FBAC8,
0x7EB9C7,
0x79B7C6,
0x78B6C5,
0x74B3C4,
0x70B2C2,
0x6DB0C0,
0x6AAFBF,
0x66AEBD,
0x63ACBB,
0x5EAABA,
0x5DA9B9,
0x5AA5B8,
0x57A4B6,
0x54A3B4,
0x52A0B4,
0x4E9FB2,
0x4B9DB2,
0x479CB0,
0x459AAF,
0x4299AD,
0x4097AB,
0x3E96AA,
0x3C94A8,
0x3A92A6,
0x3890A4,
0x3490A5,
0x318DA2,
0x2F8BA0,
0x2D899E,
0x2A899F,
0x29889E,
0x26859B,
0x24849A,
0x24849A,
0x228298,
0x208096,
0x1F7F95,
0x1F7E94,
0x1E7D93,
0x1C7B91,
0x1B7A90,
0x1B7A8E,
0x1C788D,
0x1B788A,
0x1C778A,
0x1E768A,
0x1D7589,
0x1D7487,
0x1C7386,
0x1B7283,
0x1B7283,
0x1B7182,
0x1C6F7F,
0x1C6D7E,
0x1E6D7E,
0x1D6C7D,
0x1E6B7D,
0x1D6B77,
0x1E6A77,
0x1E6A77,
0x206A77,
0x226874,
0x236774,
0x246672,
0x266672,
0x266672,
0x236571,
0x236772,
0x236975,
0x206A75,
0x1E6A77,
0x1D6B78,
0x1E6B7B,
0x1D6C7B,
0x1D6C7D,
0x1B6C7F,
0x1C6D80,
0x1C6F81,
0x1B7084,
0x1C7185,
0x1B7285,
0x1B7286,
0x1C7387,
0x1C7488,
0x1D7589,
0x1B768B,
0x1C778C,
0x1B778C,
0x1A798D,
0x1C7B91,
0x1D7C92,
0x1D7D93,
0x1E7E94,
0x1F7F95,
0x208096,
0x228298,
0x228298,
0x25849A,
0x27869C,
0x28879D,
0x2A899F,
0x2B8AA0,
0x2C8BA1,
0x308CA1,
0x328EA3,
0x3490A5,
0x3792A5,
0x3994A7,
0x3D95A9,
0x3F97AB,
0x4198AC,
0x439AAD,
0x459CAF,
0x499EB2,
0x4C9EB3,
0x4FA0B3,
0x53A1B5,
0x56A3B7,
0x59A4B7,
0x5BA6B9,
0x5FA8B9,
0x62ABBC,
0x64ACBB,
0x67AFBD,
0x6BB0BF,
0x6DB0C0,
0x71B3C1,
0x75B4C3,
0x78B6C5,
0x77B8CA,
0x7BB6C8,
0x7BB3C2,
0x7EB9C1,
0x7EBBBC,
0x86B8B5,
0x6D8381,
0x787475,
0x8E807F,
0x87847B,
0x7F8A7A,
0x828D7C,
0x858374,
0x857969,
0x877C6A,
0x85806A,
0x888366,
0x888165,
0x887F62,
0x897E62,
0x8C7E61,
0x918265,
0x8E7D61,
0x938266,
0x968267,
0x948067,
0x8E7A62,
0x7E6C58,
0x72614F,
0x6F5E4E,
0x584839,
0x302011,
0x36251E,
0x3D2C25,
0x3A2921,
0x301F15,
0x3D2D20,
0x5D4D3E,
0x746454,
0x897A67,
0x90816C,
0x897962,
0x86765D,
0x7D6D53,
0x76664C,
0x7A6B4E,
0x7F7053,
0x847558,
0x837862,
0x827B61,
0x827D67,
0x888576,
0x8E8A7F,
0x918F83,
0x959887,
0x9A9D8C,
0x918E7B,
0x918972,
0x6B6A4C,
0x7E8D78,
0xA8CCCC,
0x96C7D8,
0x90C4D9,
0x91C5D3,
0x92C3D1,
0x90C1CF,
0x8BBFCC,
0x88BECA,
0x84BCC9,
0x81BBC7,
0x7EB9C7,
0x7CB7C5,
0x78B6C5,
0x75B4C3,
0x73B2C3,
0x6FB1C1,
0x6CAFBF,
0x69AEBE,
0x65ADBC,
0x62ABBA,
0x5EAABA,
0x5CA7BA,
0x59A6B8,
0x56A3B5,
0x53A1B5,
0x519FB3,
0x4D9EB1,
0x4A9CB1,
0x469BAF,
0x459AAF,
0x4198AC,
0x3F97AB,
0x3D95A9,
0x3B93A7,
0x3991A5,
0x378FA3,
0x3490A5,
0x318DA2,
0x2F8BA0,
0x2D899E,
0x2A899F,
0x28879D,
0x26859B,
0x238399,
0x238399,
0x218197,
0x1F7F95,
0x1E7E94,
0x1F7E94,
0x1E7D93,
0x1C7B91,
0x1B7A90,
0x1A798D,
0x1C788D,
0x1B788A,
0x1C778A,
0x1D7589,
0x1C7488,
0x1C7386,
0x1C7386,
0x1B7283,
0x1B7283,
0x1B7182,
0x1C6F7F,
0x1C6D7E,
0x1E6D7E,
0x1D6C7D,
0x1E6B7B,
0x1D6B77,
0x1E6A77,
0x1E6A77,
0x206A77,
0x226874,
0x236774,
0x246672,
0x266672,
0x25656F,
0x25656F,
0x236571,
0x236774,
0x236877,
0x206978,
0x206978,
0x1E6A7A,
0x1E6B7B,
0x1E6B7B,
0x1E6D7C,
0x1E6D7E,
0x1D6E81,
0x1D7082,
0x1C7185,
0x1B7285,
0x1B7486,
0x1B7486,
0x1C7587,
0x1D7589,
0x1B7689,
0x1C778A,
0x1C798B,
0x1D798E,
0x1B7A8E,
0x1C7B8F,
0x1C7C92,
0x1D7D93,
0x1E7E94,
0x208098,
0x22829A,
0x23839B,
0x25859B,
0x26869C,
0x27879D,
0x29889E,
0x2B8AA0,
0x2D8CA2,
0x2F8EA4,
0x328EA5,
0x3490A5,
0x3691A6,
0x3994A9,
0x3C94A8,
0x3E96AA,
0x4098AC,
0x439AAE,
0x459CB0,
0x479CB0,
0x499EB2,
0x4EA0B5,
0x51A2B5,
0x55A3B7,
0x58A5B7,
0x5AA7B9,
0x5CA8B8,
0x5EAAB8,
0x62ABBA,
0x65AFBC,
0x68B0BE,
0x6DB0C0,
0x70B2C2,
0x74B3C4,
0x76B5C6,
0x7BB6BC,
0x80B5BB,
0x7BB4BB,
0x76B8C6,
0x80BBCB,
0x8CA6B3,
0x7A7879,
0x7C7A6B,
0x7A8871,
0x7D8A76,
0x8E8077,
0x927273,
0x897375,
0x7F7D71,
0x7E7F5F,
0x8A7F51,
0x80795D,
0x847A5F,
0x877861,
0x83735C,
0x897960,
0x8B7D63,
0x7F7458,
0x7B7253,
0x827757,
0x8A7B5C,
0x8B7858,
0x867152,
0x8B7657,
0x8C7C5B,
0x786B49,
0x5C522F,
0x72643D,
0x776942,
0x746641,
0x675934,
0x756744,
0x7D704E,
0x796C4C,
0x796B4E,
0x72674B,
0x73674D,
0x746850,
0x726751,
0x736852,
0x79715C,
0x7F7762,
0x7F7762,
0x717B72,
0x838071,
0x8B806E,
0x928B81,
0x8C8B87,
0x97948D,
0xA49B8C,
0x807A64,
0x46462E,
0x3F3A1C,
0x463813,
0x594E2E,
0xA5B3A4,
0x9DC8CF,
0x8FC9D7,
0x88C2CE,
0x8EC2CF,
0x8CC0CD,
0x89BFCB,
0x85BDCA,
0x82BCC8,
0x7FBAC8,
0x7BB9C6,
0x79B7C6,
0x76B5C4,
0x73B2C3,
0x70B2C2,
0x6DB0C1,
0x69AEBE,
0x66ADBF,
0x63ACBD,
0x61AABB,
0x5BA8B8,
0x5AA7B9,
0x58A5B7,
0x55A4B5,
0x53A2B3,
0x4FA0B3,
0x4C9FB1,
0x489EAF,
0x469BAF,
0x439AAD,
0x4098AC,
0x3E96AA,
0x3A95A8,
0x3893A8,
0x3691A6,
0x3590A5,
0x328EA3,
0x2E8DA1,
0x2C8BA1,
0x2A899F,
0x29889E,
0x27879D,
0x25859B,
0x25859B,
0x228298,
0x218197,
0x208096,
0x207F95,
0x1E7D93,
0x1D7C92,
0x1C7B91,
0x1B7A90,
0x1C788D,
0x1C788D,
0x1B788A,
0x1A7789,
0x1B7689,
0x1A7588,
0x1B7486,
0x1B7486,
0x1B7285,
0x1C7283,
0x1D7080,
0x1C6F7F,
0x1D6F7D,
0x1C6E7C,
0x1D6C7B,
0x1D6C7B,
0x1E6C79,
0x1F6B79,
0x216B78,
0x236877,
0x236774,
0x246672,
0x256571,
0x26646F,
0x25656F,
0x25656F,
0x236571,
0x236774,
0x226776,
0x206978,
0x206978,
0x1E6A7A,
0x1E6B7B,
0x1E6B7B,
0x1E6D7C,
0x1E6D7E,
0x1D6E7F,
0x1D7082,
0x1C7185,
0x1B7285,
0x1B7285,
0x1B7486,
0x1C7587,
0x1C7488,
0x1A7588,
0x1B7689,
0x1B788A,
0x1C788D,
0x1B7A8E,
0x1C7B8F,
0x1C7C92,
0x1D7D93,
0x1E7E94,
0x1F7F95,
0x218199,
0x22829A,
0x24849A,
0x25859B,
0x26869C,
0x29889E,
0x2A899F,
0x2C8BA1,
0x2E8DA3,
0x318DA4,
0x338FA4,
0x3590A5,
0x3792A7,
0x3B93A7,
0x3D95A9,
0x3F97AB,
0x4299AD,
0x439AAE,
0x469BAF,
0x489DB1,
0x4D9FB4,
0x50A1B6,
0x54A2B6,
0x57A4B6,
0x59A6B8,
0x5BA7B7,
0x5EAABA,
0x62ABBA,
0x64ADBC,
0x67AFBD,
0x6BB0BF,
0x6FB1C1,
0x73B2C3,
0x75B4C5,
0x7DB6C7,
0x74AFBD,
0x76B8C4,
0x73B7C2,
0x83BAC1,
0x8DA8A3,
0x727562,
0x848067,
0x80826D,
0x808076,
0x847B76,
0x80716A,
0x7E746A,
0x7E7D69,
0x7E7E62,
0x837F5A,
0x827B5F,
0x81755B,
0x837359,
0x877358,
0x8F7A5D,
0x917C5F,
0x877454,
0x837351,
0x867352,
0x8A7655,
0x8B7454,
0x876E4F,
0x826B4B,
0x836F4E,
0x827250,
0x827451,
0x7F714E,
0x796B48,
0x7D6F4C,
0x7B6C4B,
0x7C6F4F,
0x75674A,
0x71664A,
0x7A6E54,
0x7E745B,
0x7D755E,
0x736B58,
0x6C6553,
0x726D5A,
0x736D5D,
0x716E5F,
0x7B7869,
0x75827B,
0x818376,
0x8E8976,
0x939081,
0x969485,
0x98907B,
0x7D6C50,
0x564728,
0x40361B,
0x43391E,
0x4E3B1B,
0x564529,
0x797B6D,
0xA6C5C8,
0x91C5D2,
0x87C5D2,
0x8CC0CD,
0x8AC0CC,
0x88BECA,
0x84BCC9,
0x81BBC7,
0x7EB9C7,
0x7AB8C5,
0x78B6C5,
0x75B4C3,
0x73B2C3,
0x6FB1C1,
0x6CAFC0,
0x68ADBD,
0x65ACBE,
0x63ACBD,
0x60A9BA,
0x5BA8BA,
0x59A6B8,
0x57A4B6,
0x54A3B4,
0x50A1B2,
0x4E9FB2,
0x4B9EB0,
0x479CB0,
0x449BAE,
0x4299AC,
0x3F97AB,
0x3D95A9,
0x3994A7,
0x3893A8,
0x3691A6,
0x348FA4,
0x318DA2,
0x2E8DA1,
0x2C8BA1,
0x2A899F,
0x29889E,
0x26869C,
0x25859B,
0x24849A,
0x228298,
0x218197,
0x208096,
0x1E7E94,
0x1E7D93,
0x1D7C92,
0x1C7B91,
0x1B7A90,
0x1C788D,
0x1C788D,
0x1B788A,
0x1A7789,
0x1B7689,
0x1A7588,
0x1B7486,
0x1A7385,
0x1B7285,
0x1B7182,
0x1C6F7F,
0x1C6F7F,
0x1C6E7C,
0x1C6E7C,
0x1D6C7B,
0x1D6C7B,
0x1D6B78,
0x1F6B79,
0x206A77,
0x236877,
0x236774,
0x236571,
0x256571,
0x276570,
0x27666F,
0x25656F,
0x236571,
0x236774,
0x226776,
0x216978,
0x206978,
0x1E6A78,
0x1F6B7B,
0x1E6B7B,
0x1E6D7C,
0x1E6D7E,
0x1C6D7E,
0x1D7082,
0x1B7084,
0x1A7184,
0x1B7285,
0x1B7486,
0x1B7486,
0x1C7488,
0x1A7588,
0x1B7689,
0x1B788A,
0x1C788D,
0x1B7A8E,
0x1B7A8E,
0x1B7B91,
0x1C7C92,
0x1D7D93,
0x1F7F95,
0x208098,
0x22829A,
0x238399,
0x24849A,
0x26869C,
0x27879D,
0x29889E,
0x2B8AA0,
0x2D8CA2,
0x308CA3,
0x328EA3,
0x3490A5,
0x3792A7,
0x3994A9,
0x3C94A8,
0x3E96AA,
0x4198AC,
0x439AAE,
0x469BAF,
0x489DB1,
0x4C9EB3,
0x4FA0B5,
0x53A1B5,
0x55A4B5,
0x58A5B7,
0x5AA7B7,
0x5DA9B9,
0x5FABB9,
0x63ACBB,
0x66AEBC,
0x6AAFBE,
0x6DB0C0,
0x70B2C2,
0x73B2C3,
0x76B0C6,
0x72B3C7,
0x73BCCB,
0x70B4BF,
0x87B9B6,
0x67806A,
0x4F512C,
0x867956,
0x8C7D6A,
0x887D77,
0x827D79,
0x7A7869,
0x787A62,
0x7D8065,
0x7D7D65,
0x7C7B66,
0x7F7760,
0x81775E,
0x87775E,
0x8A765B,
0x90795A,
0x937C5C,
0x8D7654,
0x85714E,
0x816E4D,
0x806D4D,
0x826F4F,
0x867152,
0x877253,
0x837050,
0x7F6F4E,
0x80704F,
0x83755A,
0x7A6C51,
0x827459,
0x82745A,
0x7C7058,
0x786C54,
0x776C56,
0x78705B,
0x736C5A,
0x756E5E,
0x706A5C,
0x686556,
0x676558,
0x69695D,
0x707064,
0x79796D,
0x78827A,
0x848679,
0x948F7B,
0x95907A,
0x868066,
0x675837,
0x4C350C,
0x594219,
0x503D1D,
0x4A3A21,
0x58422A,
0x5E442B,
0x625542,
0x9BACA6,
0x95C1CA,
0x85C4D5,
0x8BBFCC,
0x88BECA,
0x84BCC9,
0x81BBC7,
0x7FBBC6,
0x7BB9C6,
0x78B6C3,
0x76B5C4,
0x74B3C2,
0x6FB1C1,
0x6EAFC1,
0x6BAEBF,
0x67ACBC,
0x64ABBD,
0x61AABB,
0x5FA8B9,
0x59A6B8,
0x58A5B7,
0x55A4B5,
0x53A2B3,
0x4FA0B1,
0x4D9EB1,
0x4A9DAF,
0x469BAF,
0x439AAD,
0x4198AB,
0x3E96AA,
0x3C94A8,
0x3994A9,
0x3792A7,
0x3590A5,
0x338EA3,
0x2E8DA1,
0x2D8CA0,
0x2B8AA0,
0x2A899F,
0x28879D,
0x26869C,
0x24849A,
0x238399,
0x218197,
0x208096,
0x1F7F95,
0x1E7E94,
0x1E7D93,
0x1C7B91,
0x1B7A90,
0x1A798F,
0x1C798B,
0x1B788A,
0x1A7789,
0x1B7689,
0x1A7588,
0x1A7588,
0x1B7486,
0x1B7285,
0x1A7182,
0x1B7182,
0x1C6F7F,
0x1B6E7E,
0x1C6E7C,
0x1E6D7C,
0x1D6C7B,
0x1D6C7B,
0x1D6B78,
0x1E6A78,
0x206A77,
0x236877,
0x236774,
0x236571,
0x256571,
0x276570,
0x27666F,
0x25656F,
0x236571,
0x226673,
0x226776,
0x216977,
0x206978,
0x1E6A78,
0x1E6A78,
0x1E6B7B,
0x1E6D7C,
0x1E6D7E,
0x1C6D7E,
0x1D7082,
0x1B7084,
0x1A7184,
0x1B7285,
0x1C7386,
0x1C7587,
0x1C7587,
0x1B7689,
0x1B7689,
0x1B788A,
0x1C788D,
0x1A798D,
0x1B7A8E,
0x1C7B91,
0x1C7C92,
0x1D7D93,
0x1E7E94,
0x208096,
0x218199,
0x238399,
0x24849A,
0x25859B,
0x26869C,
0x29889E,
0x2B8AA0,
0x2C8BA1,
0x2E8DA3,
0x328EA3,
0x3490A5,
0x3792A7,
0x3994A9,
0x3C94A8,
0x3E96AA,
0x4098AC,
0x4299AD,
0x459AAE,
0x479CB0,
0x4C9EB3,
0x4FA0B5,
0x50A1B4,
0x54A2B6,
0x58A5B7,
0x59A6B6,
0x5CA8B8,
0x5EAAB8,
0x62ABBA,
0x65ADBC,
0x69AEBD,
0x6BAEBE,
0x6EB0C0,
0x70B2C2,
0x7BB6C8,
0x6CABBD,
0x6AADC0,
0x79B7C4,
0x8AB4B2,
0x505C46,
0x493D17,
0x5E471E,
0x736040,
0x796E5A,
0x837B6E,
0x817C69,
0x7E7C63,
0x7D7D63,
0x7B7865,
0x7E766B,
0x7C7668,
0x7C7565,
0x7F7462,
0x83745F,
0x88785F,
0x8B7C5F,
0x847558,
0x796B4E,
0x7C7155,
0x7A7055,
0x786E55,
0x766C53,
0x766A52,
0x7A6B54,
0x7D6E57,
0x807258,
0x7B705E,
0x776C5A,
0x7C715F,
0x756959,
0x6D6453,
0x716857,
0x706758,
0x6C6555,
0x736B5E,
0x6C6658,
0x6A6758,
0x666256,
0x605E51,
0x686659,
0x777569,
0x7B7B6F,
0x848170,
0x8D8570,
0x908268,
0x6F5F45,
0x4C3E23,
0x463615,
0x4E3813,
0x513913,
0x564221,
0x4D3920,
0x553F27,
0x64492C,
0x624E33,
0x888C7D,
0x9CC4C6,
0x82C1D2,
0x88BDCB,
0x87BCCA,
0x83BBC8,
0x80BAC6,
0x7DB8C6,
0x7AB8C5,
0x76B5C4,
0x75B4C3,
0x73B2C3,
0x6EB0C0,
0x6CAFC0,
0x6AADBE,
0x66AABD,
0x63AABC,
0x60A9BA,
0x5CA8B8,
0x59A6B8,
0x57A4B6,
0x54A3B4,
0x52A1B2,
0x4E9FB2,
0x4B9EB0,
0x499CAE,
0x459AAE,
0x4299AC,
0x4097AB,
0x3D95A9,
0x3B93A7,
0x3893A8,
0x3691A6,
0x348FA4,
0x338EA3,
0x2E8DA1,
0x2C8B9F,
0x2B8AA0,
0x29889E,
0x27879D,
0x25859B,
0x24849A,
0x238399,
0x218197,
0x208096,
0x1F7F95,
0x1D7D93,
0x1D7C92,
0x1C7B91,
0x1B7A90,
0x1A798D,
0x1C798B,
0x1B788A,
0x1A7789,
0x1B7689,
0x1A7588,
0x1C7488,
0x1B7486,
0x1B7285,
0x1A7182,
0x1B7182,
0x1C6F7F,
0x1C6D7E,
0x1C6E7C,
0x1E6D7C,
0x1D6C7B,
0x1D6C79,
0x1D6B78,
0x1E6A77,
0x206A77,
0x226874,
0x236774,
0x256571,
0x266670,
0x276570,
0x27666F,
0x266670,
0x236571,
0x226673,
0x226776,
0x216977,
0x206978,
0x1E6A78,
0x1E6A78,
0x1E6B7B,
0x1D6C7B,
0x1D6C7B,
0x1C6D7E,
0x1C6F81,
0x1B7084,
0x1A7184,
0x1B7285,
0x1C7386,
0x1C7587,
0x1C7587,
0x1D7589,
0x1B7689,
0x1B788A,
0x1C788D,
0x1A798D,
0x1B7A8E,
0x1C7B91,
0x1C7C92,
0x1D7D93,
0x1E7E94,
0x208096,
0x218197,
0x228298,
0x238399,
0x25859B,
0x26869C,
0x29889E,
0x2A899F,
0x2C8BA1,
0x2D8CA2,
0x318DA4,
0x338FA4,
0x3691A6,
0x3792A7,
0x3B93A7,
0x3D95A9,
0x3F97AB,
0x4098AC,
0x439AAD,
0x469BAF,
0x489DB1,
0x4C9EB3,
0x4FA0B3,
0x53A1B5,
0x55A4B5,
0x58A5B7,
0x5AA7B7,
0x5DA9B9,
0x5FABB9,
0x63ACBB,
0x66AEBC,
0x6AADBD,
0x6DAFBF,
0x6FB1C1,
0x73ADBB,
0x73AEC0,
0x73AEC0,
0x85B5C1,
0x748C8C,
0x443D2B,
0x573D1C,
0x593E11,
0x514012,
0x554C25,
0x64553E,
0x6F5D4F,
0x7A6C5F,
0x827968,
0x847967,
0x867667,
0x787266,
0x767062,
0x756F5F,
0x766F5D,
0x766F5C,
0x78715E,
0x78705B,
0x726B58,
0x69624F,
0x6B6652,
0x6E6956,
0x6E6956,
0x6D6855,
0x6E6755,
0x716956,
0x736B58,
0x766E63,
0x766E63,
0x776F64,
0x70685D,
0x6D6558,
0x6E6659,
0x6A6255,
0x6C6457,
0x6E685A,
0x6C6658,
0x6F6959,
0x716B5B,
0x6B6857,
0x6E6B5A,
0x7A7766,
0x827F6E,
0x8C7B5F,
0x806749,
0x755637,
0x56391B,
0x443018,
0x3F331D,
0x453921,
0x453418,
0x4E3B1B,
0x554224,
0x564122,
0x69512F,
0x5B4724,
0x7C7C64,
0xA1C4C0,
0x81BFCE,
0x87BCCA,
0x83BBC8,
0x80BAC6,
0x7EBAC5,
0x7AB8C5,
0x78B6C3,
0x75B4C3,
0x71B3C1,
0x70B2C2,
0x6DAFBF,
0x6AADBE,
0x68ADBD,
0x63AABC,
0x61AABB,
0x5DA9B9,
0x5BA7B7,
0x57A6B7,
0x55A4B5,
0x53A2B3,
0x4FA0B1,
0x4D9EB1,
0x4A9DAF,
0x469CAD,
0x4499AD,
0x4198AB,
0x3F97AB,
0x3D95A9,
0x3994A7,
0x3893A8,
0x3691A6,
0x348FA4,
0x318DA2,
0x2D8CA0,
0x2C8B9F,
0x2A899F,
0x29889E,
0x26869C,
0x25859B,
0x238399,
0x228298,
0x218197,
0x208096,
0x1E7E94,
0x1D7D93,
0x1D7C92,
0x1C7B8F,
0x1B7A8E,
0x1A798D,
0x1B788A,
0x1C778A,
0x1B7689,
0x1A7588,
0x1A7588,
0x1B7387,
0x1A7385,
0x1B7285,
0x1B7182,
0x1B7182,
0x1C6F7F,
0x1C6D7E,
0x1C6E7C,
0x1E6D7C,
0x1D6C7B,
0x1D6C79,
0x1E6A77,
0x1E6A77,
0x206A77,
0x226874,
0x236774,
0x256571,
0x276570,
0x276570,
0x27666F,
0x26646F,
0x25656F,
0x226671,
0x226874,
0x206876,
0x206978,
0x1E6A78,
0x1E6A78,
0x1E6C79,
0x1E6B7B,
0x1D6C7B,
0x1C6D7E,
0x1C6F81,
0x1B7182,
0x1A6F83,
0x1A7184,
0x1B7285,
0x1C7386,
0x1C7587,
0x1C7587,
0x1B7689,
0x1B7689,
0x1B788A,
0x1C788D,
0x1B7A8E,
0x1C7B8F,
0x1C7B91,
0x1C7C92,
0x1E7E94,
0x1F7F95,
0x208096,
0x228298,
0x238399,
0x24849A,
0x26869C,
0x28879D,
0x2A899F,
0x2B8AA0,
0x2D8CA2,
0x308CA3,
0x328EA3,
0x3590A5,
0x3792A7,
0x3893A8,
0x3C94A8,
0x3E96AA,
0x3F97AB,
0x439AAE,
0x449BAE,
0x479CB0,
0x4B9DB2,
0x4E9FB4,
0x50A1B4,
0x54A2B6,
0x56A5B6,
0x59A6B8,
0x5AA7B7,
0x5EAABA,
0x62ABBA,
0x64ACBB,
0x68ADBC,
0x6BAEBE,
0x6EB0C0,
0x73AABF,
0x72B3C7,
0x71AFBE,
0x85A8AE,
0x625F58,
0x4E331E,
0x593C1A,
0x5F4A1B,
0x564612,
0x4E3D0F,
0x463315,
0x432D20,
0x523934,
0x685046,
0x73624E,
0x7B7054,
0x80745E,
0x817660,
0x78715E,
0x726D5A,
0x716E5D,
0x777163,
0x756D60,
0x6C6354,
0x6F6353,
0x6C6050,
0x68604D,
0x69634D,
0x6A654F,
0x6A6551,
0x6B6652,
0x6E6754,
0x6F675A,
0x726A5D,
0x726A5D,
0x6E6659,
0x706A5C,
0x736D5F,
0x696355,
0x6C6658,
0x706D5E,
0x6C695A,
0x6E6C5D,
0x6E6C5D,
0x6B6C5C,
0x757664,
0x777866,
0x666956,
0x5D4C32,
0x4F3416,
0x5B3511,
0x563212,
0x533A24,
0x433A2B,
0x3C3626,
0x433822,
0x4C391B,
0x5D4626,
0x614829,
0x634B27,
0x513F1B,
0x72745C,
0x9CBFBB,
0x80BECB,
0x84BCC9,
0x81BBC7,
0x7FB9C7,
0x7CB7C5,
0x79B7C4,
0x76B5C4,
0x71B3C1,
0x70B2C2,
0x6DB0C0,
0x6AADBE,
0x68ADBD,
0x67ABBE,
0x62A9BB,
0x60A9BA,
0x5CA8B8,
0x58A5B7,
0x56A5B6,
0x54A3B4,
0x52A1B2,
0x4E9FB2,
0x4B9EB0,
0x499CAE,
0x459AAE,
0x439AAD,
0x4097AA,
0x3E96AA,
0x3C94A8,
0x3893A8,
0x3792A7,
0x3590A5,
0x328EA3,
0x318DA2,
0x2C8B9F,
0x2B8A9E,
0x2A899F,
0x27879D,
0x26869C,
0x24849A,
0x238399,
0x228298,
0x208096,
0x1F7F95,
0x1E7E94,
0x1D7D93,
0x1D7C90,
0x1C7B8F,
0x1A798D,
0x1A798D,
0x1C778A,
0x1C778A,
0x1B7689,
0x1A7588,
0x1C7488,
0x1B7486,
0x1B7285,
0x1A7184,
0x1B7182,
0x1D7082,
0x1C6F7F,
0x1C6D7E,
0x1E6D7C,
0x1D6C7B,
0x1E6C79,
0x1D6B78,
0x1E6A77,
0x1E6A77,
0x216977,
0x226874,
0x236571,
0x256571,
0x276570,
0x276570,
0x27666F,
0x26656E,
0x25656F,
0x236571,
0x226874,
0x206876,
0x206978,
0x206978,
0x1E6A78,
0x1E6C79,
0x1E6B7B,
0x1D6C7B,
0x1C6D7E,
0x1C6F81,
0x1A7081,
0x1A6F83,
0x1A7184,
0x1B7285,
0x1C7386,
0x1C7587,
0x1C7587,
0x1A7588,
0x1B7689,
0x1B788A,
0x1C788D,
0x1A798D,
0x1B7A8E,
0x1C7B91,
0x1D7C92,
0x1D7D93,
0x1E7E94,
0x1F7F95,
0x218197,
0x228298,
0x238399,
0x25859B,
0x26869C,
0x29889E,
0x2B8AA0,
0x2C8BA1,
0x308CA3,
0x318DA2,
0x338FA4,
0x3691A6,
0x3792A7,
0x3B93A7,
0x3D95A9,
0x3F97AB,
0x4299AD,
0x449BAE,
0x479CB0,
0x4A9CB1,
0x4C9EB3,
0x4FA0B3,
0x53A1B5,
0x55A4B5,
0x56A5B6,
0x59A6B6,
0x5BA8B8,
0x5EAAB8,
0x63ABBA,
0x67ACBC,
0x6AADBD,
0x6BAEBE,
0x6DAAC7,
0x5FABC2,
0x6EB4BE,
0x86A19A,
0x58442C,
0x5B3713,
0x554015,
0x504517,
0x4F3C11,
0x553911,
0x51381A,
0x44321E,
0x432B21,
0x482A1F,
0x46311C,
0x443F1F,
0x5F4826,
0x6F5C3C,
0x73694E,
0x74725D,
0x737464,
0x707064,
0x706C61,
0x70665C,
0x7B6D60,
0x7A6B58,
0x766A52,
0x746A4F,
0x716A4E,
0x6E684E,
0x716B53,
0x756F59,
0x766D5C,
0x776E5D,
0x736C5A,
0x706959,
0x726C5C,
0x777465,
0x6E6C5D,
0x6E6F61,
0x6C7062,
0x676D5F,
0x6E7669,
0x778377,
0x889689,
0xA3B0A6,
0xA5B5AA,
0x8D9D92,
0x606358,
0x483D29,
0x4C2F11,
0x583714,
0x4E3718,
0x453B22,
0x463E27,
0x48381E,
0x4E3718,
0x57391D,
0x5E3F23,
0x54391C,
0x48381E,
0x6D7665,
0x97BFBE,
0x80BECB,
0x81BBC9,
0x7FB9C7,
0x7CB7C5,
0x78B6C3,
0x76B5C4,
0x72B4C2,
0x6FB1BF,
0x6DB0C0,
0x6CAFC0,
0x68ADBD,
0x67ACBC,
0x63AABC,
0x60A9BA,
0x5CA7BA,
0x5AA7B9,
0x57A4B6,
0x55A3B7,
0x51A2B5,
0x4FA0B3,
0x4D9EB1,
0x4A9DAF,
0x469BAF,
0x4499AD,
0x4198AB,
0x3E96AA,
0x3D95A9,
0x3994A7,
0x3893A8,
0x3691A6,
0x338FA4,
0x318DA2,
0x2F8BA0,
0x2C8B9F,
0x2B8AA0,
0x29889E,
0x27879D,
0x25859B,
0x24849A,
0x228298,
0x218197,
0x208096,
0x1F7F95,
0x1D7D93,
0x1C7C92,
0x1C7B8F,
0x1B7A8E,
0x1A798D,
0x19788C,
0x1C778A,
0x1B7689,
0x1A7588,
0x1A7588,
0x1B7387,
0x1B7486,
0x1B7285,
0x1A7184,
0x1B7182,
0x1C6F81,
0x1C6D7E,
0x1C6E7C,
0x1E6D7C,
0x1D6C7B,
0x1D6B78,
0x1D6B78,
0x1E6A77,
0x206A77,
0x216975,
0x226874,
0x236571,
0x256571,
0x276570,
0x29656F,
0x29656F,
0x26656E,
0x25656F,
0x236571,
0x236774,
0x226776,
0x216977,
0x206978,
0x1E6A78,
0x1F6B79,
0x1E6C79,
0x1D6C7B,
0x1C6D7E,
0x1C6F7F,
0x1A7081,
0x1A7081,
0x1A7184,
0x1B7285,
0x1C7386,
0x1D7487,
0x1C7587,
0x1D7688,
0x1C778A,
0x1C778A,
0x1B788A,
0x1C788D,
0x1B7A8E,
0x1C7B8F,
0x1C7B91,
0x1E7D93,
0x1F7E94,
0x207F95,
0x208096,
0x218197,
0x238399,
0x24849A,
0x26869C,
0x28879D,
0x2A899F,
0x2B8AA0,
0x2D8CA2,
0x308CA3,
0x328EA3,
0x3590A5,
0x3691A6,
0x3893A8,
0x3C94A8,
0x3D95A9,
0x4098AC,
0x439AAE,
0x459CAF,
0x479CB0,
0x4A9CB1,
0x4D9EB3,
0x50A1B4,
0x51A2B5,
0x55A4B5,
0x57A6B7,
0x5AA7B7,
0x5DA9B9,
0x61AAB9,
0x64ACBB,
0x68ADBD,
0x69AEBE,
0x61ABC6,
0x61B2C7,
0x71B2B8,
0x748676,
0x563C19,
0x624114,
0x504317,
0x4A4621,
0x513D1C,
0x543312,
0x4C320F,
0x453615,
0x49391F,
0x4A341C,
0x3E2914,
0x332812,
0x412404,
0x3F2A0D,
0x49412A,
0x737766,
0x95A298,
0x90A19B,
0x81908D,
0x7B847F,
0x787B72,
0x767767,
0x73715C,
0x6F6D56,
0x706E57,
0x76715D,
0x797461,
0x7B7464,
0x75705C,
0x716C58,
0x6E6B58,
0x6D6C5A,
0x696A5A,
0x6D7162,
0x6A7265,
0x717D73,
0x75857B,
0x879A94,
0x99B0AA,
0xA3BDBA,
0xAAC8C6,
0xA8C8C7,
0xA3C5C4,
0xA7C9CA,
0xA9C9D4,
0x9EAAA6,
0x5E533D,
0x4B3412,
0x4F3912,
0x513E16,
0x523C13,
0x533B15,
0x573A1C,
0x593620,
0x56331D,
0x4C301A,
0x403523,
0x708179,
0x93C2C8,
0x7BBACB,
0x7FBAC8,
0x7DB8C6,
0x79B7C4,
0x76B5C4,
0x72B4C2,
0x6FB3C0,
0x6DB0C0,
0x6BAEBE,
0x69AEBE,
0x67ACBC,
0x64ABBD,
0x62ABBC,
0x5DA8BB,
0x5AA7B9,
0x59A6B8,
0x55A4B5,
0x52A3B6,
0x50A1B4,
0x4E9FB2,
0x4B9EB0,
0x499BB0,
0x459AAE,
0x4299AC,
0x4198AB,
0x3E96AA,
0x3A95A8,
0x3893A8,
0x3792A7,
0x3490A5,
0x338FA4,
0x308CA1,
0x2F8BA0,
0x2B8A9E,
0x2A899F,
0x28889E,
0x26869C,
0x25859B,
0x238399,
0x218197,
0x208096,
0x1F7F95,
0x1E7E94,
0x1D7D93,
0x1C7D90,
0x1C7B8F,
0x1B7A8E,
0x1A798D,
0x19788C,
0x1C778A,
0x1B7689,
0x1A7588,
0x1C7488,
0x1B7486,
0x1C7386,
0x1B7285,
0x1B7182,
0x1B7182,
0x1C6F7F,
0x1C6D7E,
0x1E6D7C,
0x1D6C7B,
0x1E6C79,
0x1D6B78,
0x1D6B78,
0x1E6A77,
0x206A77,
0x216975,
0x236774,
0x236571,
0x26646F,
0x276570,
0x29656F,
0x29656D,
0x28646E,
0x26646F,
0x246672,
0x236774,
0x226776,
0x216977,
0x216B78,
0x206978,
0x1E6A78,
0x1E6C79,
0x1D6C7B,
0x1C6E7C,
0x1B6E7E,
0x1C6F81,
0x1A7081,
0x1B7182,
0x1B7182,
0x1B7285,
0x1B7285,
0x1B7486,
0x1C7587,
0x1B7689,
0x1B7689,
0x1B788A,
0x1B778C,
0x1A798D,
0x1B7A8E,
0x1C7B91,
0x1D7C92,
0x1F7E94,
0x1F7E94,
0x208096,
0x218197,
0x228298,
0x24849A,
0x25859B,
0x27879D,
0x2A899F,
0x2B8AA0,
0x2D8CA2,
0x308CA3,
0x318DA2,
0x328EA3,
0x3590A5,
0x3792A7,
0x3994A9,
0x3A95AA,
0x3F97AB,
0x4098AC,
0x439AAE,
0x469BAF,
0x4A9CB1,
0x4B9DB2,
0x4E9FB4,
0x4FA0B3,
0x52A3B6,
0x56A5B6,
0x57A6B7,
0x5AA7B7,
0x5EAABA,
0x62ABBA,
0x64ACBB,
0x67ACBC,
0x5BAEBC,
0x63ADB8,
0x7AABA8,
0x616952,
0x5E4823,
0x674D28,
0x53482A,
0x453F27,
0x4A391F,
0x533415,
0x4E320D,
0x55431D,
0x463A12,
0x3D2D0B,
0x3F2912,
0x42271C,
0x3A2917,
0x4B4434,
0x7F877C,
0xA5BCB6,
0x9EC0C1,
0x99C0C5,
0x9CC3CA,
0x94B7BB,
0x95B1B2,
0x94ADA9,
0x8DA299,
0x85978B,
0x809085,
0x838E86,
0x858B87,
0x828685,
0x808879,
0x808B7D,
0x848F81,
0x879387,
0x89998E,
0x8FA098,
0x97ACA7,
0x9DB6B2,
0xA4C1BF,
0xA8C7C9,
0xA9CCCE,
0xA7CED3,
0xA4CDD3,
0x9FCCD2,
0x9DC9D2,
0x93C2CA,
0x8FC2D7,
0x9EB9C0,
0x343527,
0x3E2D11,
0x553C14,
0x593A0B,
0x603F0C,
0x5C3B10,
0x56381C,
0x533722,
0x4E2D1A,
0x482D18,
0x3B3829,
0x7E9B99,
0x88BECA,
0x78BBCC,
0x7DB8C8,
0x79B7C6,
0x76B5C4,
0x73B5C3,
0x71B3C1,
0x6EB1C1,
0x6BB0BF,
0x6AAFBF,
0x68ADBD,
0x64ABBD,
0x63AABC,
0x60A9BA,
0x5DA8BB,
0x59A6B8,
0x56A5B6,
0x54A3B4,
0x50A1B4,
0x4FA0B3,
0x4C9FB1,
0x4A9CB1,
0x469BAF,
0x4499AD,
0x4198AB,
0x3F97AB,
0x3D95A9,
0x3A95A8,
0x3893A8,
0x3591A6,
0x338FA4,
0x318DA2,
0x308CA1,
0x2F8BA2,
0x2A899F,
0x29889E,
0x27879D,
0x25859B,
0x24849A,
0x238399,
0x218197,
0x208096,
0x1F7F95,
0x1E7E94,
0x1C7D90,
0x1C7B8F,
0x1B7A8E,
0x1A798D,
0x19788C,
0x1B778C,
0x1B7689,
0x1D7589,
0x1C7488,
0x1C7488,
0x1B7486,
0x1B7285,
0x1B7285,
0x1B7182,
0x1C6F81,
0x1C6F7F,
0x1C6D7E,
0x1E6D7C,
0x1D6C7B,
0x1E6C79,
0x1D6B78,
0x1D6B78,
0x206A77,
0x206874,
0x226874,
0x236772,
0x256571,
0x26646F,
0x28646F,
0x29656F,
0x29646C,
0x28646E,
0x26646F,
0x256571,
0x236573,
0x216773,
0x206876,
0x216977,
0x206A77,
0x1E6A78,
0x1F6B79,
0x1E6C79,
0x1E6D7C,
0x1C6D7E,
0x1C6F81,
0x1A7081,
0x1B7182,
0x1B7182,
0x1B7283,
0x1B7285,
0x1C7386,
0x1C7587,
0x1B7689,
0x1B7689,
0x1A7789,
0x1B778C,
0x1C788D,
0x1B7A8E,
0x1C7B8F,
0x1D7C92,
0x1E7D93,
0x1F7E94,
0x208096,
0x208096,
0x228298,
0x238399,
0x25859B,
0x26869C,
0x29889E,
0x2A899F,
0x2C8BA1,
0x2D8CA2,
0x308CA3,
0x328EA3,
0x348FA4,
0x3691A6,
0x3893A8,
0x3A95AA,
0x3C97AA,
0x3F97AB,
0x4299AD,
0x449BAF,
0x479CB1,
0x4B9DB2,
0x4C9EB3,
0x4FA0B5,
0x51A2B5,
0x53A4B7,
0x56A5B6,
0x59A6B8,
0x5CA8B8,
0x60A9BA,
0x62AAB9,
0x62AAB9,
0x5AACB7,
0x6BABAB,
0x7F9D91,
0x5E583E,
0x6A5132,
0x6A5238,
0x564936,
0x4A3F2D,
0x4A361D,
0x513317,
0x5E4325,
0x4D3918,
0x43310D,
0x422E0B,
0x432B13,
0x442A1D,
0x505C58,
0x8FA3A1,
0xA9C7C9,
0x95BEC4,
0x8DBDC7,
0x8DC1CF,
0x8EBFCE,
0x95C3D2,
0x99C3CF,
0x9DC3CC,
0x9EC4C7,
0x9CC2C3,
0x9EC1C3,
0xA1C2C7,
0xA3C0C8,
0xA1BEC6,
0xA0BDBB,
0xA2BFBD,
0xA5C2C0,
0xA7C5C5,
0xA8C6C8,
0xA8C7C9,
0xA9CBCD,
0xAACDD1,
0xA3C8CE,
0xA5CCD3,
0xA0CBD2,
0x9DC9D2,
0x99C8D2,
0x9AC8D5,
0x9ACAD6,
0x94C4D0,
0x96C9DA,
0x738C90,
0x1F1B10,
0x3A240D,
0x573A1A,
0x634016,
0x674112,
0x5B3B14,
0x4E361C,
0x49331E,
0x52361E,
0x3B270C,
0x404737,
0x8BB3B5,
0x7FBAC8,
0x76B7C9,
0x79B7C6,
0x77B6C5,
0x75B4C3,
0x71B3C1,
0x6FB2C2,
0x6CB1C0,
0x68B0BF,
0x67AFBE,
0x64ACBB,
0x62ABBC,
0x60A9BA,
0x5CA7BA,
0x59A6B8,
0x56A5B6,
0x52A3B4,
0x51A2B5,
0x4EA1B3,
0x4DA0B2,
0x4B9DB2,
0x479CB0,
0x459AAE,
0x4299AC,
0x4097AB,
0x3E96AA,
0x3A95A8,
0x3994A9,
0x3692A7,
0x3490A5,
0x328EA3,
0x318DA4,
0x2D8CA2,
0x2B8AA0,
0x2A899F,
0x29889E,
0x27869C,
0x25859B,
0x24849A,
0x228298,
0x218197,
0x208096,
0x1E7E94,
0x1D7D93,
0x1C7D90,
0x1C7B8F,
0x1B7A8E,
0x1A798D,
0x1B778C,
0x1B778C,
0x1D7589,
0x1D7589,
0x1C7488,
0x1C7587,
0x1C7386,
0x1B7285,
0x1C7185,
0x1C7283,
0x1C6F81,
0x1D6E7F,
0x1C6D7E,
0x1E6D7C,
0x1E6B7B,
0x1E6C79,
0x1E6A78,
0x1E6A78,
0x206A75,
0x206874,
0x226874,
0x246672,
0x256571,
0x26646F,
0x28646E,
0x2A646F,
0x2A656D,
0x29636E,
0x28646E,
0x25656F,
0x246672,
0x236774,
0x226776,
0x216977,
0x206A77,
0x206A77,
0x1F6B79,
0x1E6C79,
0x1E6D7C,
0x1C6D7E,
0x1B6E7E,
0x1C6F81,
0x1B7182,
0x1B7182,
0x1C7283,
0x1B7285,
0x1C7386,
0x1C7587,
0x1D7688,
0x1B7689,
0x1B7689,
0x1B788A,
0x1C788D,
0x1D798E,
0x1B7A8E,
0x1C7B8F,
0x1E7D93,
0x1E7D93,
0x1F7F95,
0x208096,
0x218197,
0x238399,
0x24849A,
0x26869C,
0x27879D,
0x29889E,
0x2B8AA0,
0x2C8BA1,
0x308CA3,
0x318DA2,
0x338FA4,
0x3691A6,
0x3893A8,
0x3994A9,
0x3B96A9,
0x3C97AA,
0x4098AC,
0x439AAE,
0x469BB0,
0x479CB1,
0x4B9DB2,
0x4C9EB3,
0x4FA1B6,
0x52A3B6,
0x54A5B6,
0x56A5B6,
0x5AA7B7,
0x5EAABA,
0x60A9B8,
0x5FA7B6,
0x5BA6B9,
0x72ACAD,
0x758E79,
0x615434,
0x79573B,
0x6D4F37,
0x5E4E37,
0x504026,
0x4C3217,
0x563820,
0x4A311D,
0x422C17,
0x472D12,
0x472E10,
0x43341D,
0x6A6B59,
0x91BEC3,
0x92C1C7,
0x90C1C8,
0x8FC1CA,
0x93C4D2,
0x95C4D4,
0x94C2D2,
0x97C0D2,
0x98BFD0,
0x97C1CF,
0x98C4D1,
0x96C5CF,
0x96C6D0,
0x97C7D1,
0x97C7D1,
0x96C6D0,
0x9FCAD3,
0x9FCAD3,
0x9FCAD3,
0xA0CBD4,
0xA0CBD4,
0x9FCAD3,
0x9EC9D2,
0x9EC8D4,
0x9EC8D4,
0xA0CAD6,
0x9DC9D4,
0x9CC8D3,
0x9AC6D3,
0x9AC6D3,
0x98C6D3,
0x90BECB,
0x99C5CE,
0x657572,
0x342414,
0x412510,
0x4E3018,
0x5B3C1F,
0x5B3D19,
0x4E3516,
0x46341C,
0x4A361D,
0x4D3415,
0x3A2D0D,
0x566858,
0x8CBFC2,
0x78B7C6,
0x78B6C3,
0x76B5C4,
0x75B4C3,
0x71B3C3,
0x6EB1C1,
0x6CB1C0,
0x68B0BF,
0x66AEBD,
0x64ADBC,
0x62ABBC,
0x61AABB,
0x5CA7BA,
0x5AA5B8,
0x58A5B7,
0x55A3B7,
0x51A2B5,
0x4FA2B4,
0x4D9FB4,
0x4B9DB2,
0x489DB1,
0x469BAF,
0x439AAD,
0x4198AC,
0x3E96AA,
0x3D95A9,
0x3893A6,
0x3792A7,
0x3591A6,
0x3490A5,
0x308FA3,
0x2E8DA3,
0x2C8BA1,
0x2A899F,
0x29889E,
0x28879D,
0x26859B,
0x24849A,
0x238399,
0x228298,
0x208096,
0x1F7F95,
0x1E7E94,
0x1D7E91,
0x1C7B8F,
0x1B7A8E,
0x1A798D,
0x1C788D,
0x1B778C,
0x1A7789,
0x1D7589,
0x1D7688,
0x1C7587,
0x1B7486,
0x1C7386,
0x1B7285,
0x1C7283,
0x1B7182,
0x1C6F81,
0x1C6D7E,
0x1E6D7C,
0x1E6D7C,
0x1E6C79,
0x1E6C79,
0x1E6A78,
0x1E6A78,
0x216975,
0x206874,
0x236774,
0x246672,
0x26646F,
0x28646F,
0x29636E,
0x2A646F,
0x2B646D,
0x29646C,
0x28646E,
0x276570,
0x246672,
0x236774,
0x226776,
0x236877,
0x216977,
0x206A77,
0x1E6A78,
0x1E6C79,
0x1D6C7B,
0x1C6D7E,
0x1C6D7E,
0x1B6E80,
0x1B7182,
0x1B7182,
0x1C7283,
0x1B7283,
0x1C7386,
0x1C7587,
0x1C7587,
0x1B7689,
0x1B7689,
0x1B788A,
0x1C788D,
0x1C788D,
0x1D798E,
0x1C7B8F,
0x1D7C92,
0x1E7D93,
0x1E7E94,
0x1F7F95,
0x218197,
0x228298,
0x238399,
0x25859B,
0x26869C,
0x29889E,
0x2A899F,
0x2B8AA0,
0x2F8BA2,
0x308CA3,
0x328EA3,
0x3590A5,
0x3792A7,
0x3893A8,
0x3A95A8,
0x3B96A9,
0x3F97AB,
0x4299AD,
0x439AAE,
0x469BB0,
0x489DB2,
0x4B9DB2,
0x4D9FB4,
0x50A3B5,
0x53A4B7,
0x55A4B5,
0x59A6B8,
0x5DA9B9,
0x5FA8B9,
0x5CA5B4,
0x56A6C1,
0x6FACB1,
0x6F8A77,
0x594A2B,
0x77502F,
0x6D4927,
0x644A25,
0x513913,
0x543717,
0x4F301B,
0x432C1C,
0x452F1A,
0x412911,
0x45361F,
0x707E6F,
0x99C2BC,
0x87C1CC,
0x86C0CB,
0x8AC0CA,
0x8DC2CA,
0x90C0CA,
0x94C0CB,
0x99C1CD,
0x9AC2CE,
0x9DC4D3,
0x9BC5D3,
0x98C6D5,
0x97C7D3,
0x96C8D3,
0x94C8D3,
0x95CAD2,
0x97CCD4,
0x99C9D5,
0x98C8D4,
0x97C7D3,
0x98C6D3,
0x98C6D3,
0x99C7D4,
0x9AC8D5,
0x9CC8D5,
0x99C5D2,
0x9BC7D4,
0x9AC6D3,
0x9BC7D4,
0x9AC6D3,
0x99C5D2,
0x97C3D0,
0x8EBAC7,
0x99C6C9,
0x68756B,
0x402A12,
0x4E2C11,
0x51351F,
0x4E3C24,
0x47371D,
0x47361C,
0x49351C,
0x54391C,
0x4C2F0D,
0x3B3417,
0x749288,
0x7FBAC2,
0x77B6C5,
0x7BB4BF,
0x74B3C4,
0x71B3C3,
0x6EB1C1,
0x6CB1C0,
0x68B0BF,
0x67AFBE,
0x64ADBC,
0x63ACBD,
0x61AABB,
0x5DA8BB,
0x5BA6B9,
0x58A5B7,
0x56A4B8,
0x52A3B6,
0x4FA2B4,
0x4EA1B3,
0x4B9DB2,
0x489DB1,
0x469BAF,
0x4499AD,
0x4299AD,
0x4097AB,
0x3D95A9,
0x3A95A8,
0x3893A8,
0x3692A7,
0x3490A5,
0x308FA3,
0x2F8EA4,
0x2D8CA2,
0x2B8AA0,
0x2A899F,
0x28879D,
0x27869C,
0x26859B,
0x248399,
0x228298,
0x218197,
0x1F7F95,
0x1E7E94,
0x1D7D93,
0x1D7C90,
0x1C7B8F,
0x1B7A8E,
0x1C788D,
0x1C788D,
0x1B788A,
0x1A7789,
0x1D7688,
0x1C7587,
0x1C7587,
0x1C7386,
0x1B7285,
0x1C7185,
0x1B7182,
0x1D7082,
0x1B6E7E,
0x1C6D7E,
0x1E6D7C,
0x1E6B7B,
0x1E6C79,
0x1E6A78,
0x1E6A78,
0x1E6A78,
0x216975,
0x226874,
0x236772,
0x266672,
0x26646F,
0x28646F,
0x29636E,
0x2B646D,
0x2A636C,
0x2A636C,
0x29636E,
0x26646F,
0x256571,
0x236573,
0x236774,
0x226776,
0x216977,
0x216977,
0x206A77,
0x1E6A78,
0x1E6B7B,
0x1D6C7B,
0x1C6D7E,
0x1B6E7E,
0x1B7182,
0x1B7182,
0x1B7182,
0x1C7283,
0x1C7386,
0x1D7487,
0x1C7587,
0x1B7689,
0x1B7689,
0x1C778A,
0x1C788D,
0x1C788D,
0x1D798E,
0x1E7A8F,
0x1D7C90,
0x1E7D91,
0x1E7E94,
0x1F7F95,
0x208096,
0x218197,
0x238399,
0x24849A,
0x26869C,
0x27879D,
0x29889E,
0x2A899F,
0x2C8BA1,
0x2F8BA2,
0x318DA2,
0x338FA4,
0x3691A6,
0x3792A7,
0x3994A7,
0x3A95A8,
0x3E96AA,
0x4098AC,
0x4299AD,
0x459AAF,
0x479CB1,
0x489DB2,
0x4B9DB2,
0x4EA0B5,
0x51A2B5,
0x52A3B4,
0x56A5B6,
0x5BA7B7,
0x5BA7B7,
0x5BA4B5,
0x4FA6C1,
0x67A6AF,
0x778F82,
0x54492D,
0x694825,
0x673D15,
0x6A4115,
0x684619,
0x553914,
0x472F13,
0x432910,
0x3F270B,
0x473E21,
0x768976,
0x8FBFC3,
0x7ABDD0,
0x81BBC9,
0x85BDCC,
0x89BDCA,
0x8EC0CB,
0x91C2C9,
0x8EBFC6,
0x8DBEC5,
0x91C3CA,
0x8EC3CB,
0x8FC3CE,
0x90C4D1,
0x95C6D4,
0x96C6D4,
0x95C5D1,
0x96C5CF,
0x99C5D0,
0x97C6D0,
0x98C7D1,
0x98C7D1,
0x98C6D3,
0x98C6D3,
0x98C6D3,
0x98C6D3,
0x97C5D2,
0x97C5D2,
0x97C5D2,
0x95C3D2,
0x95C3D2,
0x94C4D2,
0x94C4D2,
0x93C3D1,
0x8BBBC9,
0x8FC6CD,
0x6F8378,
0x422D10,
0x54300E,
0x51381A,
0x473D24,
0x423B21,
0x46361C,
0x503518,
0x5B381A,
0x4B3013,
0x454432,
0x88AEAF,
0x78B6C5,
0x77B6C7,
0x7AB0BA,
0x71B3C3,
0x70B2C2,
0x6DB0C0,
0x6AAFBF,
0x67AFBE,
0x64ADBC,
0x63ACBD,
0x5FABBB,
0x5DA9B9,
0x5BA6B9,
0x59A6B8,
0x57A4B6,
0x55A3B7,
0x51A2B5,
0x4DA0B2,
0x4C9FB1,
0x489DB1,
0x479CB0,
0x459AAE,
0x4299AD,
0x4198AC,
0x3E96AA,
0x3C94A8,
0x3994A7,
0x3792A7,
0x3591A6,
0x338FA4,
0x2F8EA2,
0x2E8DA3,
0x2C8BA1,
0x2B8AA0,
0x2A899F,
0x27869C,
0x26859B,
0x25849A,
0x248399,
0x228298,
0x208096,
0x1F7F95,
0x1E7E94,
0x1D7E91,
0x1D7C90,
0x1B7A8E,
0x1B7A8E,
0x1C788D,
0x1C798B,
0x1B788A,
0x1B7689,
0x1C7587,
0x1D7487,
0x1C7386,
0x1B7285,
0x1B7285,
0x1B7084,
0x1B7182,
0x1C6F81,
0x1C6D7E,
0x1C6D7E,
0x1E6D7C,
0x1E6B7B,
0x1E6C79,
0x1E6A78,
0x1E6A78,
0x1E6A77,
0x236975,
0x226874,
0x236571,
0x256571,
0x28646F,
0x29636E,
0x2A636E,
0x2A636C,
0x2D636D,
0x2A636C,
0x29636E,
0x28646F,
0x256571,
0x236573,
0x236774,
0x226776,
0x236877,
0x216977,
0x206A77,
0x1E6A78,
0x1E6B7B,
0x1D6C7B,
0x1C6D7E,
0x1C6D7E,
0x1A7081,
0x1B7182,
0x1B7182,
0x1C7283,
0x1B7285,
0x1C7386,
0x1C7587,
0x1C7587,
0x1B7689,
0x1B7689,
0x1B788A,
0x1C788D,
0x1C788D,
0x1D798E,
0x1E7A8F,
0x1F7B90,
0x1D7D93,
0x1E7E94,
0x208096,
0x218197,
0x228298,
0x238399,
0x25859B,
0x26869C,
0x28879D,
0x29889E,
0x2B8AA0,
0x2F8BA2,
0x308CA3,
0x328EA3,
0x338FA4,
0x3590A5,
0x3794A6,
0x3994A7,
0x3B96A9,
0x3F97AB,
0x4198AC,
0x439AAE,
0x469BB0,
0x479CB1,
0x489DB2,
0x4C9EB3,
0x4EA1B3,
0x51A2B5,
0x55A4B5,
0x58A5B5,
0x5AA6B6,
0x5CA5B6,
0x56ACBD,
0x78AAB1,
0x717E75,
0x5A4E36,
0x573D1A,
0x61370D,
0x65360A,
0x5C370D,
0x564019,
0x4C3813,
0x51330F,
0x533F1C,
0x67755B,
0x8CC1B9,
0x74BCCB,
0x75BADB,
0x87BCCE,
0x86BBCD,
0x88BBCC,
0x8ABBC9,
0x86BAC5,
0x89C2CB,
0x8AC7CC,
0x83C2C9,
0x86C7CD,
0x87C6CF,
0x8BC5D0,
0x91C5D2,
0x93C3D1,
0x95C1CE,
0x96C0CE,
0x97C1CD,
0x95C4CC,
0x96C5CF,
0x96C5CF,
0x95C4CE,
0x95C4CE,
0x95C4CE,
0x94C2CF,
0x94C2CF,
0x94C4D0,
0x94C4D0,
0x91C2D0,
0x91C2D0,
0x91C2D0,
0x8FC3D0,
0x8EC2D0,
0x87BBC9,
0x80C2CE,
0x7B968F,
0x4C391B,
0x5F3811,
0x533513,
0x47391E,
0x433C20,
0x453212,
0x563514,
0x5B381A,
0x3B2A16,
0x64736E,
0x85B5BF,
0x73B6C7,
0x72B3C5,
0x76B0BE,
0x6FB1C1,
0x6DB0C0,
0x6AAFBF,
0x67AFBE,
0x65ADBC,
0x63ACBB,
0x5FABBB,
0x5EAABA,
0x5BA6B9,
0x59A6B8,
0x58A5B7,
0x55A3B7,
0x52A3B6,
0x4FA2B4,
0x4C9FB1,
0x489EAF,
0x479CB0,
0x469BAF,
0x4499AE,
0x4198AC,
0x3F96AA,
0x3D95A9,
0x3994A7,
0x3893A8,
0x3591A6,
0x3490A5,
0x308FA3,
0x2E8DA3,
0x2D8CA2,
0x2B8AA0,
0x2A899F,
0x29889E,
0x27869C,
0x26859B,
0x248399,
0x238298,
0x228197,
0x208096,
0x1E7E94,
0x1D7D93,
0x1D7C90,
0x1C7B8F,
0x1B7A8E,
0x1C788D,
0x1C788D,
0x1B788A,
0x1C778A,
0x1B7689,
0x1D7487,
0x1D7487,
0x1C7386,
0x1B7285,
0x1A7184,
0x1B7182,
0x1A7081,
0x1C6F81,
0x1C6D7E,
0x1C6D7E,
0x1D6C7B,
0x1E6B7B,
0x1E6A78,
0x1E6A78,
0x1E6A77,
0x206A77,
0x236975,
0x236774,
0x236571,
0x256571,
0x28646F,
0x29636E,
0x2A636E,
0x2A636C,
0x2D636D,
0x2A636C,
0x29636E,
0x28646F,
0x276572,
0x236573,
0x236774,
0x226776,
0x236877,
0x216977,
0x206A77,
0x1E6A78,
0x1D6B78,
0x1D6C7B,
0x1C6D7E,
0x1C6D7E,
0x1C6F81,
0x1A7081,
0x1B7182,
0x1B7182,
0x1B7283,
0x1C7386,
0x1C7587,
0x1C7587,
0x1B7689,
0x1B7689,
0x1B788A,
0x1B778C,
0x1C788D,
0x1D798E,
0x1E7A8F,
0x1F7B90,
0x1D7D93,
0x1E7E94,
0x1F7F95,
0x208096,
0x218197,
0x238399,
0x24849A,
0x25859B,
0x28879D,
0x29889E,
0x2A899F,
0x2E8AA1,
0x2F8BA2,
0x318DA2,
0x328EA3,
0x338FA4,
0x3693A5,
0x3893A6,
0x3A95A8,
0x3D95A9,
0x4097AB,
0x4299AD,
0x4499AE,
0x459AAF,
0x489DB2,
0x499EB2,
0x4DA0B2,
0x51A2B5,
0x54A3B4,
0x55A2B4,
0x58A4B4,
0x5AA6B6,
0x3D7B86,
0x27484D,
0x27291E,
0x58462E,
0x5C401B,
0x5E3811,
0x5E3710,
0x593B17,
0x523B19,
0x553914,
0x5F3D1A,
0x54482E,
0x70948A,
0x78BFC5,
0x72BBCA,
0x7EB5C9,
0x7CB7C5,
0x7FB9C7,
0x80B8C7,
0x85BACA,
0x8ABFCF,
0x85BFCB,
0x83BFCA,
0x83C1CC,
0x84C2CD,
0x87C1CD,
0x8AC0CC,
0x8DBECC,
0x90BECD,
0x90C0CC,
0x92C2CE,
0x91C3CE,
0x92C4CF,
0x92C4CF,
0x91C3CE,
0x90C2CD,
0x90C2CD,
0x91C3CE,
0x92C4CF,
0x92C4CF,
0x90C2CD,
0x90C2CD,
0x8DC1CC,
0x8EC2CD,
0x8DC1CC,
0x8CC0CB,
0x8BBFCA,
0x84BAC4,
0x7CC4D2,
0x809F9A,
0x4F3E24,
0x673915,
0x613916,
0x4E361A,
0x463519,
0x4C3412,
0x583714,
0x4C3418,
0x383A2D,
0x81A4A6,
0x77B5C0,
0x6DB5C4,
0x6BB2C4,
0x70B1C5,
0x6CAFBF,
0x6BAEBF,
0x69AEBE,
0x65ADBC,
0x62ABBA,
0x61AABB,
0x5DA9B9,
0x5CA8B8,
0x5AA5B8,
0x58A5B7,
0x56A3B5,
0x53A1B5,
0x50A1B4,
0x4DA0B2,
0x499FB0,
0x479DAE,
0x459AAE,
0x4499AE,
0x4198AC,
0x4097AB,
0x3D95A9,
0x3B93A7,
0x3893A6,
0x3792A7,
0x3490A5,
0x338FA4,
0x2F8EA2,
0x2D8CA2,
0x2C8BA1,
0x2A899F,
0x29889E,
0x27879D,
0x26859B,
0x25849A,
0x248399,
0x228197,
0x218096,
0x207F95,
0x1F7E94,
0x1E7D93,
0x1C7B8F,
0x1C7B8F,
0x1A798D,
0x1C788D,
0x1B788A,
0x1C778A,
0x1C778A,
0x1B7689,
0x1D7487,
0x1D7487,
0x1C7386,
0x1B7285,
0x1B7084,
0x1A7081,
0x1C6F81,
0x1C6F81,
0x1C6D7E,
0x1D6C7D,
0x1D6C7B,
0x1E6B7B,
0x1E6A78,
0x1E6A78,
0x206A77,
0x206A77,
0x236975,
0x236774,
0x236571,
0x266471,
0x28646F,
0x29636E,
0x2A636E,
0x2C626C,
0x2C636A,
0x29626B,
0x28626D,
0x28646F,
0x266471,
0x236573,
0x226673,
0x236774,
0x236877,
0x216977,
0x206A77,
0x1E6A78,
0x1D6B78,
0x1D6C7B,
0x1B6C7D,
0x1C6D7E,
0x1C6F81,
0x1A7081,
0x1B7182,
0x1B7182,
0x1B7283,
0x1C7386,
0x1B7486,
0x1C7587,
0x1A7588,
0x1B7689,
0x1B788A,
0x1B778C,
0x1C788D,
0x1C788D,
0x1D798E,
0x1E7A8F,
0x1C7C92,
0x1D7D93,
0x1E7E94,
0x208096,
0x218197,
0x228298,
0x238399,
0x24849A,
0x27869C,
0x28879D,
0x2A899F,
0x2D89A0,
0x2F8BA2,
0x308CA1,
0x318DA2,
0x328EA3,
0x3592A4,
0x3693A5,
0x3994A7,
0x3D95A9,
0x3F96AA,
0x4198AC,
0x4398AD,
0x4499AE,
0x479CB1,
0x479CB0,
0x4B9DB2,
0x4FA0B3,
0x52A1B2,
0x529FB1,
0x55A1B1,
0x58A4B4,
0x345A67,
0x001011,
0x2D2719,
0x5E442B,
0x5D3915,
0x5F3C16,
0x5B3F1A,
0x4C3919,
0x4B3415,
0x573313,
0x4D2A0E,
0x615D51,
0x81B0B6,
0x6BB7C7,
0x75B8C0,
0x8BB5B1,
0x76BCC4,
0x77BAC3,
0x7FBBC6,
0x82BAC9,
0x82B7C9,
0x84B9CB,
0x87BCCE,
0x86BBCB,
0x88BDCD,
0x8CBDCC,
0x8FBDCC,
0x91BDCA,
0x8FBDCA,
0x8CC0CB,
0x88C3CB,
0x85C4CB,
0x89C1D0,
0x8AC2D1,
0x89C1CE,
0x89C1CE,
0x8CC1CF,
0x8EC3D1,
0x8DC3CF,
0x8BC1CD,
0x8CC0CB,
0x8CC0CB,
0x8ABEC9,
0x8ABFC7,
0x89BEC6,
0x89BEC6,
0x8ABFC7,
0x86BBC3,
0x74BCC8,
0x8EAEAB,
0x604C34,
0x612F0E,
0x612E0F,
0x533116,
0x483014,
0x513714,
0x583C17,
0x34280E,
0x4D6156,
0x85B8BB,
0x6DB3BD,
0x69B5C2,
0x69B2C5,
0x68B0C9,
0x6BAEBF,
0x6AADBE,
0x67ACBC,
0x64ACBB,
0x61AAB9,
0x5DA9B9,
0x5CA8B8,
0x5AA6B6,
0x57A4B6,
0x56A3B5,
0x53A0B2,
0x519FB3,
0x4D9EB1,
0x4B9EB0,
0x479DAE,
0x469CAD,
0x4499AE,
0x4398AD,
0x4097AB,
0x3F96AA,
0x3C94A8,
0x3A92A6,
0x3792A5,
0x3691A6,
0x328EA3,
0x328EA3,
0x2E8DA1,
0x2D8CA2,
0x2C8BA1,
0x2A899F,
0x27879D,
0x26869C,
0x25849A,
0x248399,
0x238298,
0x228197,
0x218096,
0x207F95,
0x1E7D93,
0x1D7C92,
0x1C7B8F,
0x1B7A8E,
0x1A798D,
0x1B778C,
0x1B788A,
0x1C778A,
0x1B7689,
0x1B7689,
0x1D7487,
0x1D7487,
0x1C7386,
0x1B7285,
0x1B7084,
0x1A7081,
0x1C6F81,
0x1C6F81,
0x1C6D7E,
0x1D6C7D,
0x1D6C7B,
0x1D6A7A,
0x1E6A78,
0x1E6A78,
0x206A77,
0x206A77,
0x236975,
0x236774,
0x236571,
0x266471,
0x28646F,
0x2A636E,
0x2A636E,
0x2C626C,
0x2E626D,
0x2C626C,
0x2A636C,
0x29636E,
0x28646E,
0x25656F,
0x236571,
0x236772,
0x246875,
0x236877,
0x216977,
0x206A77,
0x1E6A78,
0x1E6C79,
0x1D6C79,
0x1D6C79,
0x1B6E80,
0x1C6F81,
0x1C6F81,
0x1B7182,
0x1B7084,
0x1B7285,
0x1C7386,
0x1C7488,
0x1C7488,
0x1A7588,
0x1B768B,
0x1A768B,
0x1B778C,
0x1C788D,
0x1D798E,
0x1E7A8F,
0x1C7B8F,
0x1D7C90,
0x1F7E92,
0x207F95,
0x208096,
0x218197,
0x228298,
0x238399,
0x25859B,
0x26869C,
0x29889E,
0x2A899D,
0x2B8A9E,
0x2D8CA0,
0x318DA2,
0x328EA3,
0x348EA9,
0x3490A5,
0x3394A4,
0x3596A7,
0x3897AD,
0x3B98A9,
0x3D9CA2,
0x429BA3,
0x4997AE,
0x5394BA,
0x5595B9,
0x559BB4,
0x519FAC,
0x509FAC,
0x519EAE,
0x569FB2,
0x589BA1,
0x02272D,
0x111111,
0x4A2F24,
0x5F3C20,
0x5B3810,
0x603B0E,
0x5B3409,
0x573416,
0x433324,
0x576568,
0x7FA8BC,
0x75B1CD,
0x70B2CC,
0x71B2C4,
0x79B8C1,
0x7AB5C5,
0x7BB6C6,
0x7CB7C7,
0x7DB8C6,
0x7EB9C7,
0x80BAC8,
0x81BBC9,
0x81BBC7,
0x82BAC7,
0x84BCC9,
0x85BDCA,
0x87BCCA,
0x87BCCA,
0x87BCCA,
0x88BDCB,
0x89BECC,
0x83C2C9,
0x83C2C9,
0x86C2CA,
0x88C1CC,
0x8AC0CC,
0x8ABECB,
0x8ABECB,
0x8ABECC,
0x8ABECC,
0x88BDCB,
0x88BDCB,
0x87BFCC,
0x87BFCC,
0x85BDCA,
0x83BBC8,
0x83BBC8,
0x72B7C6,
0x90B8B7,
0x595B43,
0x553C14,
0x54370F,
0x492F14,
0x4D301E,
0x562B1A,
0x5B2D1D,
0x3E3128,
0x789FA4,
0x71BACB,
0x65B2C6,
0x6DAFBF,
0x71ACBE,
0x6EACC3,
0x69AEBD,
0x61AFBB,
0x59B0BA,
0x59ACBA,
0x5DA8BB,
0x5EA5B9,
0x5AA5B8,
0x55A6B7,
0x53A5B3,
0x58A1B2,
0x599DB2,
0x549CB2,
0x4A9FB3,
0x469EB4,
0x469AB4,
0x4A97B5,
0x4299AD,
0x4198AC,
0x3E96AA,
0x3C94A8,
0x3B93A9,
0x3792A7,
0x3691A6,
0x338FA4,
0x318DA4,
0x2E8DA3,
0x2D8CA2,
0x2B8AA0,
0x2A899F,
0x28879D,
0x27869C,
0x26859B,
0x23839B,
0x22829A,
0x218199,
0x208098,
0x1F7F97,
0x1F7E94,
0x1E7D93,
0x1D7C92,
0x1E7A91,
0x1D798E,
0x1D798E,
0x1D788D,
0x1C778A,
0x1E768A,
0x1D7589,
0x1C7488,
0x1C7488,
0x1C7387,
0x1C7387,
0x1C7185,
0x1D6F84,
0x1D6E81,
0x1D6E81,
0x1E6D7E,
0x1E6D7C,
0x1D6C7B,
0x1E6C79,
0x1E6A78,
0x1F6976,
0x216977,
0x236877,
0x246777,
0x246672,
0x236571,
0x25656F,
0x26646F,
0x28646E,
0x28636B,
0x29626B,
0x2C636A,
0x2D626A,
0x2C626C,
0x2A636C,
0x29646C,
0x28646E,
0x25656F,
0x23666F,
0x246672,
0x246875,
0x236975,
0x216977,
0x206A77,
0x1E6A78,
0x1E6C79,
0x1D6C79,
0x1D6C79,
0x1B6E80,
0x1B6E80,
0x1C6F81,
0x1B7182,
0x1B7084,
0x1B7285,
0x1C7386,
0x1B7387,
0x1C7488,
0x1A7588,
0x1B768B,
0x1A768B,
0x1B778C,
0x1B778C,
0x1C788D,
0x1D798E,
0x1C7B8F,
0x1D7C90,
0x1E7D91,
0x207F95,
0x208096,
0x218197,
0x228298,
0x238399,
0x24849A,
0x25859B,
0x27879D,
0x29889E,
0x2A899D,
0x2C8B9F,
0x308CA1,
0x318DA2,
0x358DA3,
0x338FA6,
0x3392A8,
0x3793A8,
0x3994A6,
0x3B96A8,
0x3B98A9,
0x3F98A8,
0x4698AD,
0x4898B1,
0x4697B5,
0x489BB9,
0x4C9EB4,
0x4E9FB0,
0x489FB0,
0x48A5B7,
0x618E94,
0x0B272A,
0x161614,
0x2A1710,
0x3F2A19,
0x493B1E,
0x4C3E19,
0x4F3715,
0x533823,
0x1F1A17,
0x5A7F88,
0x77B9C7,
0x68B0BE,
0x74B3C2,
0x70B2BE,
0x6CB6BF,
0x76B4C3,
0x77B5C4,
0x78B6C5,
0x79B7C4,
0x7CB7C5,
0x7DB8C6,
0x7EB9C7,
0x80BAC8,
0x7FB9C5,
0x80BAC6,
0x83BBC8,
0x83BBC8,
0x83BBC8,
0x85BAC8,
0x86BBC9,
0x87BCCA,
0x81C0C9,
0x83BFC9,
0x84BEC9,
0x85BEC9,
0x87BDC9,
0x87BCCA,
0x87BBC9,
0x87BBC9,
0x87BCCA,
0x87BCCA,
0x83BBC8,
0x82BAC7,
0x82BCC8,
0x82BCC8,
0x81BBC7,
0x7EB8C4,
0x7BB8CA,
0x82A1A3,
0x48412E,
0x543A15,
0x523C15,
0x493C1C,
0x47331A,
0x59371C,
0x512D13,
0x4B4231,
0x345453,
0x62A5AE,
0x66B6C1,
0x61AEB8,
0x6CB4C0,
0x62A9BB,
0x61ADBA,
0x5FADB9,
0x5EACB9,
0x5DA9B9,
0x5DA6B9,
0x5CA5B8,
0x58A3B6,
0x56A3B5,
0x51A2B3,
0x4FA2B2,
0x4EA1B3,
0x4D9EB1,
0x4B9CB1,
0x499BB1,
0x4599B1,
0x4498B0,
0x4198AC,
0x3F97AB,
0x3D95A9,
0x3B93A7,
0x3893A8,
0x3691A6,
0x3590A5,
0x338FA4,
0x308CA3,
0x2D8CA2,
0x2C8BA1,
0x2A899F,
0x29889E,
0x28879D,
0x26859B,
0x25849A,
0x22829A,
0x22829A,
0x208098,
0x1F7F97,
0x1E7E94,
0x1E7D93,
0x1D7C92,
0x1C7B91,
0x1D7990,
0x1D798E,
0x1C788D,
0x1C778C,
0x1C778A,
0x1E768A,
0x1D7589,
0x1C7488,
0x1B7387,
0x1C7387,
0x1B7285,
0x1C7185,
0x1D6F84,
0x1D6E81,
0x1C6D80,
0x1E6D7E,
0x1E6D7C,
0x1E6B7B,
0x1D6B78,
0x1E6A78,
0x1F6976,
0x206876,
0x236877,
0x246777,
0x246672,
0x236571,
0x25656F,
0x26646F,
0x28646E,
0x28636B,
0x29626B,
0x2C636A,
0x2D626A,
0x2D626A,
0x2B616B,
0x29646C,
0x28646E,
0x26646F,
0x23666F,
0x246672,
0x236774,
0x226874,
0x206876,
0x206A77,
0x1E6A78,
0x1E6C79,
0x1D6C79,
0x1D6C79,
0x1B6E7E,
0x1B6E80,
0x1C6F81,
0x1C6F81,
0x1B7084,
0x1A7184,
0x1B7285,
0x1B7387,
0x1C7488,
0x1A7588,
0x1B7689,
0x1B768B,
0x1A768B,
0x1B778C,
0x1C788D,
0x1C788D,
0x1C7B8F,
0x1C7B8F,
0x1E7D93,
0x1F7E94,
0x1F7F95,
0x208096,
0x218197,
0x228298,
0x238399,
0x24849A,
0x26869C,
0x28879D,
0x29889C,
0x2B8A9E,
0x2C8B9F,
0x2E8DA1,
0x318E9F,
0x328EA5,
0x358FAA,
0x3890A8,
0x3C92A3,
0x3C93A6,
0x3B95AD,
0x3998AC,
0x3999A5,
0x40A0AC,
0x429DB2,
0x449BB9,
0x499EBB,
0x4AA1B5,
0x429EAD,
0x4BA9B5,
0x455A5F,
0x142020,
0x1D1C1A,
0x1D1111,
0x201816,
0x252B1F,
0x272C15,
0x2F2712,
0x2D1E17,
0x0D1721,
0x558D9C,
0x63B8BF,
0x65B4B8,
0x6FB0B4,
0x70B3BC,
0x69B7C4,
0x73B2C1,
0x74B3C2,
0x75B4C3,
0x77B5C4,
0x77B5C2,
0x78B6C3,
0x79B7C4,
0x7CB7C5,
0x7DB8C6,
0x7EB8C6,
0x80BAC6,
0x80BAC6,
0x80BAC8,
0x81B9C8,
0x82BAC9,
0x83BBCA,
0x7EBCC7,
0x80BCC7,
0x81BBC7,
0x82BAC7,
0x82BAC7,
0x84B9C7,
0x84B9C9,
0x82BAC9,
0x82BAC9,
0x82BCCA,
0x81BBC9,
0x7DB8C6,
0x7EB9C7,
0x81BCCA,
0x7FBAC8,
0x7BB6C4,
0x7EBBCD,
0x455D61,
0x362617,
0x533A1C,
0x443715,
0x3B3618,
0x453A1C,
0x4C3515,
0x4F3819,
0x3B3421,
0x000D08,
0x5B8E92,
0x6CB7BC,
0x58B0B2,
0x62B2BB,
0x65AABA,
0x5EACB8,
0x5EAAB8,
0x60A7B9,
0x61A5BA,
0x5DA4BA,
0x5AA2B8,
0x56A1B6,
0x54A1B5,
0x4FA2B4,
0x47A3B2,
0x45A2B3,
0x469EB2,
0x499BB1,
0x4899AE,
0x4398AC,
0x4099AB,
0x3E96AA,
0x3D95A9,
0x3B93A7,
0x3A92A8,
0x3691A6,
0x3590A5,
0x338FA4,
0x328EA3,
0x2F8BA2,
0x2C8BA1,
0x2A899F,
0x29889E,
0x28879D,
0x27869C,
0x25859B,
0x24849A,
0x228298,
0x218197,
0x208096,
0x1F7F95,
0x1E7E94,
0x1E7D93,
0x1C7B91,
0x1C7B91,
0x1D7990,
0x1C788D,
0x1B778C,
0x1C778C,
0x1C778A,
0x1D7589,
0x1C7488,
0x1C7488,
0x1B7387,
0x1B7286,
0x1B7285,
0x1B7084,
0x1C6E83,
0x1D6E81,
0x1C6D7E,
0x1E6D7E,
0x1D6C7B,
0x1E6B7B,
0x1D6B78,
0x1E6A78,
0x206A77,
0x226776,
0x226776,
0x236774,
0x246672,
0x23666F,
0x25656F,
0x26656E,
0x28646E,
0x29626B,
0x2B6269,
0x2D626A,
0x2D626A,
0x2C6169,
0x2B616B,
0x29646C,
0x28646E,
0x26656E,
0x25656F,
0x246672,
0x236774,
0x226874,
0x226776,
0x206876,
0x1E6A78,
0x1D6B78,
0x1D6C79,
0x1D6C79,
0x1B6C7D,
0x1C6D7E,
0x1C6F81,
0x1C6F81,
0x1A7081,
0x1B7084,
0x1B7285,
0x1C7387,
0x1C7488,
0x1C7488,
0x1B7689,
0x1B768B,
0x1B768B,
0x1A768B,
0x1B778C,
0x1C788D,
0x1B7A8E,
0x1C7B8F,
0x1D7C92,
0x1E7D93,
0x1E7E94,
0x1F7F95,
0x208096,
0x218197,
0x228298,
0x238399,
0x25859B,
0x26869C,
0x28879D,
0x2A899D,
0x2B8A9E,
0x2D8CA0,
0x2B8F9F,
0x318DA2,
0x388CA8,
0x3C8BA9,
0x408DA9,
0x3F8FA8,
0x3A95A8,
0x319BA9,
0x2FA0A8,
0x379DA2,
0x479AA8,
0x4B9BB2,
0x419BB5,
0x3F9EB2,
0x53A1AD,
0x488387,
0x12211E,
0x151716,
0x21161C,
0x271C24,
0x16151B,
0x121B1A,
0x161C12,
0x201C13,
0x181415,
0x304351,
0x72AAB9,
0x63B2B9,
0x61B0B4,
0x68ADB2,
0x70B2BE,
0x6CAFC2,
0x6FB1BF,
0x70B2C0,
0x71B3C1,
0x72B4C2,
0x74B3C2,
0x75B5C1,
0x76B6C2,
0x78B6C3,
0x79B7C4,
0x7AB8C5,
0x7CB7C5,
0x7DB8C6,
0x7DB8C6,
0x7DB8C6,
0x7EB9C7,
0x7FB9C7,
0x7EB9C7,
0x7EB9C7,
0x7FB9C7,
0x7FB9C7,
0x80B8C7,
0x80B8C7,
0x80B8C7,
0x7FB9C7,
0x7DB8C6,
0x7EB9C7,
0x7CBAC7,
0x7BB9C6,
0x7BB9C6,
0x7BB9C6,
0x79B7C4,
0x76B4C1,
0x65A9B4,
0x021C1D,
0x27170A,
0x452C18,
0x362B17,
0x37321F,
0x3B2F19,
0x3E2D13,
0x352911,
0x2A261A,
0x151E1D,
0x628186,
0x72B3B7,
0x5DB2B7,
0x5EAAB8,
0x6CA6BC,
0x60A8B7,
0x5FA7B6,
0x5DA6B9,
0x5DA4BA,
0x5CA3B9,
0x57A2B7,
0x53A1B5,
0x4EA1B1,
0x4AA1B2,
0x48A1B3,
0x469EB2,
0x459DB3,
0x419CAF,
0x409BAD,
0x4099A9,
0x3F98A6,
0x3D95A9,
0x3C94A8,
0x3A92A6,
0x3792A7,
0x3590A5,
0x348FA4,
0x328EA3,
0x318DA2,
0x2C8BA1,
0x2B8AA0,
0x2A899F,
0x28879D,
0x27869C,
0x25859B,
0x24849A,
0x238399,
0x218197,
0x208096,
0x1F7F95,
0x1E7E94,
0x1D7D93,
0x1D7C92,
0x1C7B91,
0x1B7A90,
0x1D7990,
0x1C788D,
0x1C778C,
0x1C778A,
0x1B7689,
0x1C7488,
0x1C7488,
0x1B7387,
0x1A7286,
0x1B7285,
0x1B7084,
0x1B7084,
0x1C6F81,
0x1D6E81,
0x1C6D7E,
0x1E6D7E,
0x1D6C7B,
0x1E6C79,
0x1E6C79,
0x1E6A77,
0x216977,
0x226776,
0x236774,
0x226673,
0x246770,
0x25656F,
0x25656F,
0x26656E,
0x29636E,
0x29626B,
0x2B6269,
0x2D626A,
0x2F616A,
0x2C6169,
0x2B6269,
0x2A636C,
0x28646C,
0x25646D,
0x25656F,
0x246672,
0x226671,
0x226673,
0x226776,
0x206876,
0x206A77,
0x1E6A78,
0x1E6C79,
0x1D6C79,
0x1B6C7D,
0x1C6D7E,
0x1D6E7F,
0x1C6F81,
0x1C6F81,
0x1B7182,
0x1B7285,
0x1B7285,
0x1B7387,
0x1B7387,
0x1A7588,
0x1B7689,
0x1B7689,
0x1C778C,
0x1C788D,
0x1C788D,
0x1B7A8E,
0x1C7B8F,
0x1C7B91,
0x1D7C92,
0x1D7D93,
0x1E7E94,
0x208096,
0x208096,
0x218197,
0x238399,
0x25859B,
0x26869C,
0x28879D,
0x29889E,
0x2A899F,
0x2C8B9F,
0x26909E,
0x2D8E9F,
0x368CA3,
0x3D89A9,
0x408AAD,
0x3E8EA9,
0x3895A6,
0x2F9BA5,
0x309CA8,
0x3C98A5,
0x4C98A6,
0x4395A3,
0x3D9EAF,
0x409DAE,
0x5F95A1,
0x213335,
0x081911,
0x1B1D1C,
0x20151D,
0x2A1D27,
0x1F1A21,
0x1A1A1A,
0x221E1B,
0x26211E,
0x262729,
0x1F3238,
0x65919A,
0x6AACB8,
0x62B0BC,
0x65B1BF,
0x68ABBE,
0x70AAC0,
0x6CAFBF,
0x6DB0C0,
0x6EB1C1,
0x6FB3C0,
0x70B4C1,
0x71B3C1,
0x72B4C2,
0x73B5C1,
0x75B5C1,
0x76B6C2,
0x77B7C3,
0x78B6C3,
0x78B6C3,
0x79B7C4,
0x79B7C4,
0x79B7C4,
0x7CB7C5,
0x7CB7C7,
0x7CB7C7,
0x7DB6C7,
0x7DB6C7,
0x7DB6C7,
0x7CB7C7,
0x7AB8C7,
0x7AB8C7,
0x79B8C7,
0x79B8C7,
0x76B8C6,
0x77B7C3,
0x75B5C1,
0x74B2BF,
0x74B2BF,
0x6FAEB7,
0x1F3435,
0x221109,
0x311A12,
0x271E19,
0x201B17,
0x2A2016,
0x2B2213,
0x222014,
0x161614,
0x16161E,
0x2A3C46,
0x6296A1,
0x62B0BC,
0x5FA7BD,
0x6AA1BF,
0x60A4B7,
0x5DA4B8,
0x59A4B9,
0x56A2B9,
0x55A1B8,
0x51A2B5,
0x4BA3B1,
0x49A3AE,
0x49A1AF,
0x4B9CAF,
0x4A9AB1,
0x449AB1,
0x3D9CB0,
0x389CAC,
0x3A9AA8,
0x3E98A3,
0x3C94A8,
0x3B93A7,
0x3792A7,
0x3590A5,
0x348FA4,
0x328EA3,
0x308CA1,
0x2E8DA3,
0x2C8BA1,
0x2B8AA0,
0x29889E,
0x27879D,
0x26869C,
0x24849A,
0x238399,
0x228298,
0x218197,
0x208096,
0x1E7E94,
0x1D7D93,
0x1D7D93,
0x1C7B91,
0x1B7A90,
0x1A798F,
0x1D798E,
0x1C788D,
0x1C778C,
0x1B7689,
0x1D7589,
0x1C7488,
0x1B7387,
0x1B7486,
0x1B7285,
0x1B7285,
0x1B7084,
0x1D6F84,
0x1C6F81,
0x1D6E81,
0x1E6D7E,
0x1E6D7C,
0x1D6C7B,
0x1E6C79,
0x1F6B79,
0x206A77,
0x216977,
0x226874,
0x236774,
0x226673,
0x266670,
0x25656F,
0x26646F,
0x28646E,
0x29646C,
0x29626B,
0x2B6269,
0x2D626A,
0x2F6168,
0x2E6069,
0x2B6269,
0x296269,
0x28636B,
0x25646D,
0x25656F,
0x25656F,
0x236571,
0x226673,
0x226874,
0x206876,
0x206A77,
0x1E6A77,
0x1D6B78,
0x1D6B78,
0x1D6C7D,
0x1C6D7E,
0x1C6D7E,
0x1D6E7F,
0x1C6F81,
0x1A7081,
0x1B7084,
0x1B7285,
0x1B7285,
0x1B7387,
0x1C7488,
0x1A7588,
0x1B7689,
0x1C778A,
0x1D788B,
0x1D788B,
0x1A798D,
0x1B7A8E,
0x1C7B91,
0x1D7C92,
0x1C7C92,
0x1D7D93,
0x1F7F95,
0x208096,
0x218197,
0x228298,
0x24849A,
0x25859B,
0x26869C,
0x28879D,
0x29889E,
0x2B8AA0,
0x25909A,
0x28909D,
0x2F8EA2,
0x378BA7,
0x3C8BAA,
0x398FA8,
0x3594A8,
0x3695A7,
0x3C91A6,
0x4894AC,
0x4395AB,
0x3E9FB0,
0x40A2AF,
0x4E96A4,
0x23414C,
0x17161E,
0x17261F,
0x1D2623,
0x232325,
0x231D21,
0x2A2124,
0x282222,
0x2A2521,
0x2B2620,
0x2F2B28,
0x182626,
0x5D888E,
0x69ACBC,
0x57A8BD,
0x5FADC4,
0x66ADC1,
0x6AACBC,
0x69AEBD,
0x6AAFBE,
0x6BB0BF,
0x6CB1C0,
0x6CB1C0,
0x6EB2BF,
0x6FB3C0,
0x70B4C1,
0x71B3C1,
0x72B4C2,
0x72B4C2,
0x73B5C3,
0x75B4C3,
0x76B5C4,
0x76B5C4,
0x76B5C4,
0x78B6C5,
0x7AB5C5,
0x7AB5C5,
0x7AB5C5,
0x7AB5C5,
0x78B5C7,
0x78B5C7,
0x77B6C5,
0x75B7C5,
0x74B6C4,
0x72B6C3,
0x74B6C4,
0x73B5C3,
0x73B3BF,
0x74B4C0,
0x76B4C1,
0x6E909C,
0x1D2528,
0x2F1D19,
0x2D1E1B,
0x242225,
0x1C2021,
0x191A15,
0x1C1F16,
0x141A16,
0x14171E,
0x23212E,
0x2F3945,
0x3D6773,
0x68B2BF,
0x54A5BA,
0x5AA4BD,
0x56A7B8,
0x58A5B9,
0x59A1BA,
0x56A0BB,
0x50A0B7,
0x4AA3B5,
0x47A3B0,
0x48A2AB,
0x4A9DAB,
0x4C9BAC,
0x4898AF,
0x4399B2,
0x3D99B0,
0x3A99AD,
0x3898A8,
0x3996A5,
0x3893A6,
0x3792A7,
0x3691A6,
0x348FA4,
0x328EA3,
0x318DA2,
0x2F8BA0,
0x2C8BA1,
0x2B8AA0,
0x2A899F,
0x27879D,
0x26869C,
0x25859B,
0x238399,
0x228298,
0x218197,
0x208096,
0x1F7F95,
0x1E7E94,
0x1D7D93,
0x1C7C92,
0x1C7B91,
0x1B7A90,
0x1A798D,
0x1C788D,
0x1B778C,
0x1B768B,
0x1B7689,
0x1D7589,
0x1C7488,
0x1B7486,
0x1B7486,
0x1B7285,
0x1A7184,
0x1B7084,
0x1C6F81,
0x1D6E81,
0x1C6D7E,
0x1E6D7E,
0x1E6D7C,
0x1E6B7B,
0x1E6C79,
0x1E6A77,
0x206A77,
0x216975,
0x226874,
0x236774,
0x236573,
0x266670,
0x26646F,
0x26656E,
0x28646E,
0x2A636C,
0x2B6269,
0x2C6169,
0x2F6168,
0x306168,
0x2E6067,
0x2C6167,
0x2B6269,
0x28636B,
0x27636B,
0x25646D,
0x25656F,
0x236571,
0x236571,
0x236774,
0x226874,
0x216977,
0x206A77,
0x1E6A77,
0x1D6B77,
0x1D6C7B,
0x1D6C7B,
0x1E6D7E,
0x1C6D7E,
0x1C6F7F,
0x1C6F81,
0x1B7182,
0x1C7185,
0x1B7285,
0x1B7387,
0x1B7387,
0x1C7488,
0x1B7689,
0x1B7689,
0x1C778A,
0x1D788B,
0x1A798D,
0x1A798D,
0x1B7A90,
0x1C7B91,
0x1C7C92,
0x1D7D93,
0x1E7E94,
0x1F7F95,
0x208096,
0x218197,
0x238399,
0x24849A,
0x24849A,
0x26869C,
0x28879D,
0x29889E,
0x288D95,
0x258F9D,
0x288EA4,
0x318DA4,
0x368CA3,
0x358FA7,
0x3292AA,
0x3A90A7,
0x438DA6,
0x4693AD,
0x3391AB,
0x399CB1,
0x48909C,
0x23444B,
0x111A23,
0x22212F,
0x222327,
0x1D2623,
0x242D28,
0x242422,
0x2E282A,
0x2F2B2C,
0x2B2A25,
0x322B21,
0x2F2822,
0x1A2A2A,
0x538993,
0x5AADBF,
0x58ACC4,
0x5CA7BE,
0x62AEBC,
0x5DB0B6,
0x65ADBC,
0x65ADBC,
0x67AFBE,
0x67AFBD,
0x6AAFBE,
0x6BB0BF,
0x6BB0BF,
0x6CB2BE,
0x6EB2BF,
0x6FB3C0,
0x6FB3C0,
0x71B3C1,
0x71B3C1,
0x71B3C1,
0x72B4C2,
0x72B4C2,
0x75B2C4,
0x75B2C4,
0x76B3C5,
0x76B3C5,
0x76B3C5,
0x74B3C4,
0x72B4C4,
0x72B4C4,
0x71B4C4,
0x70B4C1,
0x70B4C1,
0x70B4C1,
0x71B3C1,
0x73B3BF,
0x73B1BE,
0x72B0BD,
0x4E5462,
0x140D14,
0x291B1A,
0x271F1D,
0x1A1E1F,
0x111B1A,
0x172119,
0x121C13,
0x141A18,
0x1A1B20,
0x241E2A,
0x151924,
0x42606A,
0x6BABB5,
0x58AEBB,
0x4BA8B7,
0x49ABB6,
0x50A6B7,
0x58A0B9,
0x589DBC,
0x4F9FB8,
0x47A2B4,
0x45A1AE,
0x479FA9,
0x489CA7,
0x449CA8,
0x419AAC,
0x4197AE,
0x4394AF,
0x3F93AD,
0x3A95AA,
0x3397A7,
0x3792A7,
0x3691A6,
0x348FA4,
0x328EA3,
0x318DA2,
0x2F8BA0,
0x2C8BA1,
0x2B8AA0,
0x29889E,
0x27879D,
0x26869C,
0x25859B,
0x24849A,
0x228298,
0x218197,
0x208096,
0x1F7F95,
0x1E7E94,
0x1D7D93,
0x1C7C92,
0x1B7B91,
0x1B7A90,
0x1A798F,
0x1A798D,
0x1B778C,
0x1B778C,
0x1B7689,
0x1A7588,
0x1C7488,
0x1C7587,
0x1C7386,
0x1B7285,
0x1A7184,
0x1A7184,
0x1B7182,
0x1C6F81,
0x1C6D80,
0x1E6D7E,
0x1D6C7B,
0x1E6B7B,
0x1D6B78,
0x1E6A78,
0x1E6A77,
0x1F6974,
0x226874,
0x236774,
0x246672,
0x246672,
0x26646F,
0x26656E,
0x28646E,
0x29646C,
0x2A636C,
0x2B6269,
0x2E6067,
0x2E6067,
0x2F6067,
0x2F6067,
0x2C6167,
0x2B6269,
0x296269,
0x28636B,
0x25646D,
0x26646F,
0x25656F,
0x236571,
0x236774,
0x226874,
0x216977,
0x206A77,
0x1E6A77,
0x1E6A77,
0x1E6B7B,
0x1D6C7B,
0x1E6D7C,
0x1E6D7E,
0x1C6D7E,
0x1C6F81,
0x1C6F81,
0x1B7084,
0x1B7285,
0x1C7386,
0x1B7387,
0x1C7488,
0x1C7488,
0x1B7689,
0x1B7689,
0x1C778A,
0x1C788D,
0x1C788D,
0x1B7A8E,
0x1B7A90,
0x1C7B91,
0x1C7C92,
0x1D7D93,
0x1E7E94,
0x1F7F95,
0x208096,
0x228298,
0x238399,
0x24849A,
0x25859B,
0x26869C,
0x28889E,
0x2C8998,
0x288C9C,
0x278EA1,
0x2D8DA3,
0x328CA4,
0x348EA6,
0x3490A5,
0x3592A3,
0x3893A5,
0x348FA2,
0x479CB1,
0x458294,
0x19343B,
0x1A1819,
0x2B2525,
0x27262C,
0x2F2432,
0x2A282B,
0x292E28,
0x292E2A,
0x2A2B2F,
0x2E2C31,
0x2E2925,
0x362C23,
0x262521,
0x203E46,
0x61A5B8,
0x53ABC1,
0x5AABC0,
0x60A5B5,
0x60AAB5,
0x57AEB5,
0x62ABBA,
0x64ACBB,
0x65ADBC,
0x65ADBC,
0x66AEBD,
0x67AFBD,
0x67AFBD,
0x6AAFBE,
0x6BB0BF,
0x6BB0BF,
0x6CAFBF,
0x6DB0C0,
0x6EB1C1,
0x6EB1C1,
0x6EB1C1,
0x6EB1C1,
0x70B2C2,
0x72B1C2,
0x72B1C2,
0x70B2C2,
0x70B2C2,
0x70B2C2,
0x6EB1C1,
0x6EB1C1,
0x6EB3C2,
0x6FB5C1,
0x6FB3C0,
0x6DB1BE,
0x6FB1BD,
0x71AFBC,
0x6FAAB8,
0x69A4B2,
0x28252E,
0x221C1E,
0x231B18,
0x211C18,
0x191917,
0x191D1C,
0x151A14,
0x171812,
0x1F1B1A,
0x1F191D,
0x221B23,
0x212026,
0x1B292C,
0x386062,
0x5EA2A5,
0x58ADB2,
0x4AA7AE,
0x50A3B1,
0x539FB6,
0x529EB8,
0x4E9EB9,
0x499DB5,
0x479DAE,
0x459BA8,
0x439DA6,
0x3F9DA7,
0x3D9AA9,
0x4195AD,
0x4491AD,
0x4390AE,
0x3B93AB,
0x3295A8,
0x3591A6,
0x3490A5,
0x338FA4,
0x318DA2,
0x308CA1,
0x2C8B9F,
0x2B8AA0,
0x2A899F,
0x28879D,
0x26869C,
0x25859B,
0x24849A,
0x238399,
0x228298,
0x218197,
0x208096,
0x1E7E94,
0x1D7D93,
0x1C7C92,
0x1C7C92,
0x1C7B91,
0x1B7A8E,
0x1A798D,
0x1B778C,
0x1B778C,
0x1C778A,
0x1B7689,
0x1C7488,
0x1C7587,
0x1C7386,
0x1B7285,
0x1A7184,
0x1A7182,
0x1B7182,
0x1A7081,
0x1C6F81,
0x1C6D7E,
0x1E6D7E,
0x1E6B7B,
0x1E6C79,
0x1E6A78,
0x1E6A77,
0x1F6976,
0x206874,
0x226874,
0x226671,
0x246672,
0x266672,
0x26656E,
0x28646E,
0x28626D,
0x2A636C,
0x2B6269,
0x2C6169,
0x2E6067,
0x2F6067,
0x305F65,
0x2E5F64,
0x2D5F66,
0x2C6167,
0x2A636A,
0x29646C,
0x28646C,
0x26656E,
0x26646F,
0x256571,
0x236571,
0x236774,
0x236975,
0x216977,
0x206A77,
0x1E6A77,
0x1E6B7B,
0x1E6B7B,
0x1E6B7B,
0x1E6D7C,
0x1C6D7E,
0x1D6E7F,
0x1D7082,
0x1B7182,
0x1C7185,
0x1B7285,
0x1B7285,
0x1B7387,
0x1C7488,
0x1D7589,
0x1D7589,
0x1D7589,
0x1B778C,
0x1B778C,
0x1C788D,
0x1B7A8E,
0x1B7A90,
0x1B7B91,
0x1C7C92,
0x1D7D93,
0x1F7F95,
0x208096,
0x218197,
0x228298,
0x238399,
0x25859B,
0x26869C,
0x27879D,
0x2B87A0,
0x2C899B,
0x2E8B9C,
0x2C8CA2,
0x2F8DA9,
0x358CA7,
0x368F9F,
0x2C959B,
0x27989E,
0x3A9AA6,
0x427684,
0x242D36,
0x271B1D,
0x332824,
0x2C2220,
0x302826,
0x312938,
0x302931,
0x312D2E,
0x2E2E30,
0x2F2C33,
0x332931,
0x322628,
0x24201D,
0x22373A,
0x528A97,
0x59A7BE,
0x52A4BC,
0x60A7B9,
0x61A3AF,
0x60A6B2,
0x5BA9B6,
0x5FA8B9,
0x60A9BA,
0x61AABB,
0x62ABBA,
0x64ACBB,
0x64ACBB,
0x65ADBC,
0x66AEBD,
0x6AAFBE,
0x6AAFBE,
0x6AAFBE,
0x6AAFBE,
0x6AAFBE,
0x6CAFBF,
0x6CAFBF,
0x6CAFBF,
0x6DB0C0,
0x6DB0C0,
0x6DB0C0,
0x6DB0C0,
0x6DB0C0,
0x6DB0C0,
0x6CB1C0,
0x6CB1C0,
0x6BB0BF,
0x6DB3BF,
0x6CB2BE,
0x6AAEBB,
0x6EAEBA,
0x6CAAB7,
0x76B1BF,
0x447E8C,
0x171715,
0x232220,
0x221F1A,
0x241C19,
0x28201D,
0x221E1D,
0x211D1E,
0x231A1D,
0x24191D,
0x211B1F,
0x1F1D22,
0x252326,
0x2A2625,
0x1C2220,
0x23403E,
0x4D7B7B,
0x65A1AB,
0x5CA2AC,
0x4EA0AE,
0x49A0B3,
0x469AB2,
0x4C9CB5,
0x4797AE,
0x459CAD,
0x3F9DA9,
0x3D9BA7,
0x3E97A9,
0x4193AB,
0x4191AC,
0x3D91AD,
0x3991A9,
0x3591A8,
0x3490A5,
0x338FA4,
0x318DA2,
0x308CA1,
0x2C8B9F,
0x2B8AA0,
0x2A899F,
0x29889E,
0x26869C,
0x25859B,
0x24849A,
0x228298,
0x228298,
0x218197,
0x208096,
0x1F8093,
0x1E7F92,
0x1D7E91,
0x1D7C90,
0x1C7B8F,
0x1B7A8E,
0x1A798D,
0x1B778C,
0x1B778C,
0x19758A,
0x1B7689,
0x1D7589,
0x1C7488,
0x1C7386,
0x1B7285,
0x1A7184,
0x1A7184,
0x1B7182,
0x1B7182,
0x1D7082,
0x1D6E7F,
0x1E6D7E,
0x1D6C7B,
0x1E6B7B,
0x1F6B79,
0x1E6A78,
0x206A77,
0x206A75,
0x206874,
0x236772,
0x236571,
0x256571,
0x256571,
0x28646E,
0x28646E,
0x2A656D,
0x2A636C,
0x2B6269,
0x2E6067,
0x2F6067,
0x305F65,
0x305F65,
0x2F5E64,
0x2E5F64,
0x2C6167,
0x2C636A,
0x2A636A,
0x29646C,
0x28646E,
0x26646F,
0x25656F,
0x256571,
0x246674,
0x226874,
0x216975,
0x216977,
0x206A77,
0x1F6B79,
0x1E6C79,
0x1E6B7B,
0x1F6C7C,
0x1E6D7C,
0x1D6E7F,
0x1D6E7F,
0x1D7082,
0x1C7283,
0x1C7185,
0x1B7285,
0x1C7386,
0x1C7488,
0x1D7589,
0x1D7589,
0x1D7589,
0x1B778C,
0x1B778C,
0x1C788D,
0x1A798D,
0x1B7A8E,
0x1C7B91,
0x1C7C92,
0x1D7D93,
0x1E7E94,
0x1F7F95,
0x208096,
0x218197,
0x228298,
0x238399,
0x25859B,
0x26869C,
0x2787A0,
0x2C879C,
0x2F889A,
0x2B8AA0,
0x2B8AA6,
0x308CA5,
0x338EA0,
0x2E939D,
0x3597A0,
0x377A83,
0x1F3038,
0x2D1D20,
0x372727,
0x282725,
0x2D2D2F,
0x362D32,
0x2F2E34,
0x2F2C33,
0x353036,
0x342931,
0x37282F,
0x33262D,
0x242328,
0x2F484C,
0x558E95,
0x59A6B6,
0x4FA1B6,
0x539EB3,
0x61A4B5,
0x5FA1B1,
0x5FA3B6,
0x5EA6BC,
0x5EA7B8,
0x5EA7B8,
0x5FA8B9,
0x60A9BA,
0x61AABB,
0x61AAB9,
0x62ABBA,
0x64ACBB,
0x65ADBC,
0x65ADBC,
0x65ADBC,
0x68ADBD,
0x68ADBD,
0x69AEBE,
0x69AEBE,
0x69AEBE,
0x68B0BE,
0x6AAFBE,
0x6AAFBE,
0x6AAFBE,
0x6AAFBE,
0x6AAFBE,
0x68B0BE,
0x68B0BE,
0x67AFBD,
0x64ACBA,
0x69AFBB,
0x69ADBA,
0x6CABBA,
0x6CAAB7,
0x72ADBB,
0x205A68,
0x1A170E,
0x2C2421,
0x2F2422,
0x2C1E1B,
0x2B221D,
0x252120,
0x211F24,
0x242128,
0x1E1B22,
0x1F2227,
0x20272D,
0x222327,
0x2D2523,
0x342923,
0x2B2724,
0x1E2629,
0x374955,
0x5A828A,
0x5AA0A8,
0x48A0AA,
0x439EB0,
0x4498B0,
0x4599B3,
0x4298AF,
0x4098AC,
0x4097AB,
0x4193A9,
0x4092A8,
0x3C92A9,
0x3791A9,
0x3690A8,
0x358FA7,
0x328EA3,
0x318DA2,
0x308CA1,
0x2C8B9F,
0x2B8AA0,
0x2A899F,
0x28879D,
0x26869C,
0x26869C,
0x25859B,
0x238399,
0x228298,
0x218197,
0x208096,
0x1F7F95,
0x1E7F92,
0x1E7D91,
0x1D7C90,
0x1C7B8F,
0x1B7A8E,
0x1A798D,
0x19788C,
0x1B778C,
0x1B778C,
0x1A7588,
0x1B7689,
0x1D7589,
0x1C7587,
0x1C7386,
0x1B7285,
0x1B7084,
0x1B7182,
0x1B7182,
0x1D7082,
0x1C6F7F,
0x1C6D7E,
0x1E6D7E,
0x1E6B7B,
0x1F6B79,
0x1E6A78,
0x206A77,
0x206A77,
0x206874,
0x226872,
0x236772,
0x236571,
0x25656F,
0x26646F,
0x29636E,
0x29646C,
0x2A636C,
0x2C636A,
0x2C6169,
0x2E6067,
0x2F6065,
0x305F65,
0x336065,
0x325F64,
0x2F6065,
0x2E6165,
0x2D6268,
0x2A636A,
0x29646C,
0x29636E,
0x28646E,
0x26646F,
0x256571,
0x246672,
0x236774,
0x226874,
0x216975,
0x216975,
0x1E6A78,
0x1F6B79,
0x1E6C79,
0x1E6B7B,
0x1E6D7C,
0x1E6D7E,
0x1D6E7F,
0x1C6F81,
0x1D7082,
0x1B7084,
0x1A7184,
0x1B7285,
0x1C7386,
0x1C7587,
0x1C7488,
0x1C7488,
0x1B768B,
0x1B778C,
0x1C788D,
0x1C788D,
0x1A798D,
0x1B7A8E,
0x1B7B91,
0x1C7C92,
0x1D7D93,
0x1E7E94,
0x1F7F95,
0x208096,
0x218197,
0x228298,
0x24849A,
0x25859B,
0x24879E,
0x28869F,
0x2C86A0,
0x2B879E,
0x278A9F,
0x278DA3,
0x2B8DA6,
0x3B8BA4,
0x427588,
0x1C2D37,
0x251F23,
0x34282A,
0x282627,
0x282D31,
0x2E323D,
0x343142,
0x3A3633,
0x353533,
0x353130,
0x332326,
0x362126,
0x1E1D22,
0x2F5558,
0x589DA4,
0x50A3B1,
0x4EA5B6,
0x4DA3B2,
0x52A4B2,
0x58A1B2,
0x5A9EB3,
0x5BA3BB,
0x54A4BD,
0x59A4B7,
0x5AA5B8,
0x5BA6B9,
0x5CA8B8,
0x5CA8B8,
0x5FA8B9,
0x60A9BA,
0x60A9BA,
0x61AAB9,
0x62AAB9,
0x63ABBA,
0x63ABBA,
0x64ACBB,
0x65ADBC,
0x65ADBC,
0x66AEBD,
0x64ADBC,
0x64ADBC,
0x64ADBC,
0x66AEBD,
0x66AEBD,
0x65ADBC,
0x65ADBC,
0x65ADBB,
0x67AFBD,
0x65ADBB,
0x69AEBD,
0x67ABB8,
0x68AAB8,
0x67A6B5,
0x6BA6B6,
0x124D5D,
0x1F160F,
0x271515,
0x341B1E,
0x34221E,
0x292519,
0x23261F,
0x212528,
0x202126,
0x24282B,
0x1B2526,
0x1B292C,
0x21292C,
0x2B2726,
0x322924,
0x342828,
0x332932,
0x2A232B,
0x1E2B31,
0x3B646A,
0x51969D,
0x49A0AA,
0x419CAE,
0x3F99B1,
0x4197B2,
0x4392B0,
0x4491AD,
0x4090A9,
0x3C90A8,
0x3792A5,
0x3392A4,
0x3190A2,
0x308FA3,
0x318DA2,
0x308CA1,
0x2D8CA0,
0x2C8B9F,
0x2B8AA0,
0x29889E,
0x27869C,
0x25859B,
0x25859B,
0x24849A,
0x228298,
0x218197,
0x208096,
0x1F7F95,
0x1E7F92,
0x1D7E91,
0x1D7C90,
0x1D7C90,
0x1C7B8F,
0x1B7A8E,
0x1A798D,
0x1B778C,
0x1B778C,
0x1C778A,
0x1A7588,
0x1D7589,
0x1C7488,
0x1D7487,
0x1C7386,
0x1C7185,
0x1B7182,
0x1B7182,
0x1C6F7F,
0x1C6F7F,
0x1C6D7E,
0x1E6D7E,
0x1E6B7B,
0x1E6B7B,
0x1E6A78,
0x206A77,
0x206A77,
0x216975,
0x226874,
0x236772,
0x236571,
0x25656F,
0x26646F,
0x28646F,
0x29646C,
0x2A636C,
0x2A636C,
0x2C636A,
0x2E6067,
0x2F6067,
0x305F65,
0x325F65,
0x345F65,
0x325F64,
0x306064,
0x2F6065,
0x2D6268,
0x2C636A,
0x296269,
0x29646C,
0x28646E,
0x28646F,
0x266471,
0x256571,
0x246674,
0x236774,
0x226874,
0x206874,
0x1E6A78,
0x1E6A78,
0x1E6A78,
0x1E6C79,
0x1E6B7B,
0x1E6D7C,
0x1C6D7E,
0x1D6E7F,
0x1D7082,
0x1B7182,
0x1B7084,
0x1B7285,
0x1C7386,
0x1D7487,
0x1D7487,
0x1C7587,
0x1B7689,
0x1C778A,
0x1B778C,
0x1C788D,
0x1C788D,
0x1B7A8E,
0x1B7A8E,
0x1B7B91,
0x1D7D93,
0x1D7D93,
0x1E7E94,
0x1F7F95,
0x208096,
0x218197,
0x238399,
0x24849A,
0x25859B,
0x2486A1,
0x2786A4,
0x2C869E,
0x29889A,
0x248DA2,
0x278FAA,
0x4087A5,
0x2D4255,
0x2A171D,
0x3E2521,
0x32251F,
0x332F2C,
0x343031,
0x362A34,
0x3A2C3D,
0x3D2A24,
0x3D3027,
0x3B2F23,
0x3C2B24,
0x282224,
0x2A494E,
0x509DA3,
0x42A4AF,
0x419EB0,
0x469BB0,
0x4BA2B3,
0x4AA3B1,
0x4DA0AE,
0x56A3B5,
0x55A6BB,
0x49A4B9,
0x56A3B5,
0x57A4B6,
0x58A5B7,
0x59A6B8,
0x5AA5B8,
0x5BA7B7,
0x5CA8B8,
0x5CA8B8,
0x5DA9B9,
0x5FA8B9,
0x60A9BA,
0x60A9BA,
0x61AABB,
0x62ABBC,
0x62ABBC,
0x62ABBC,
0x60ACBA,
0x60ACBA,
0x60ACBA,
0x62ABBA,
0x62ABBA,
0x62ABBA,
0x62ABBA,
0x62ABBA,
0x61AAB9,
0x63ACBB,
0x63ABBA,
0x60A8B6,
0x66A9B9,
0x63A5B5,
0x67A6B7,
0x387788,
0x151A16,
0x24191D,
0x321B21,
0x2E1D16,
0x282516,
0x28281C,
0x27221F,
0x2D2324,
0x27221F,
0x252621,
0x262729,
0x26272B,
0x2B2B2B,
0x2D2D2B,
0x2D282E,
0x342939,
0x322829,
0x2B2728,
0x1C252A,
0x203F44,
0x417781,
0x4E9CA9,
0x3994A6,
0x3A96AF,
0x3E91AF,
0x428EAE,
0x418EAA,
0x3B8FA7,
0x3392A4,
0x2F93A2,
0x2D91A0,
0x2E8D9F,
0x2F8BA0,
0x2D8CA0,
0x2C8B9F,
0x2B8AA0,
0x2A899F,
0x28879D,
0x27869C,
0x25859B,
0x24849A,
0x238399,
0x228298,
0x208096,
0x1F7F95,
0x1F7E94,
0x1E7D91,
0x1D7C90,
0x1C7B8F,
0x1C7B8F,
0x1B7A8E,
0x1C788D,
0x1C788D,
0x1B778C,
0x1B778C,
0x1C778A,
0x1C7488,
0x1C7488,
0x1D7487,
0x1C7386,
0x1C7185,
0x1B7182,
0x1D7082,
0x1D7082,
0x1B6E7E,
0x1B6E7E,
0x1C6D7E,
0x1D6C7B,
0x1E6B7B,
0x1F6B79,
0x206978,
0x206A77,
0x216977,
0x206874,
0x226872,
0x236772,
0x23666F,
0x26646F,
0x28646F,
0x28646F,
0x29626B,
0x29626B,
0x2C636A,
0x2C6169,
0x2E6067,
0x2F6065,
0x305F65,
0x325F64,
0x345F65,
0x335E64,
0x325F64,
0x2F6065,
0x2F6266,
0x2C6167,
0x2B6269,
0x2A636C,
0x29636E,
0x28646F,
0x26646F,
0x256571,
0x246672,
0x236774,
0x226874,
0x226874,
0x206A77,
0x206A77,
0x1E6A78,
0x1F6B79,
0x1E6B7B,
0x1D6C7B,
0x1E6D7E,
0x1C6D7E,
0x1C6F81,
0x1D7082,
0x1B7084,
0x1C7185,
0x1B7285,
0x1C7386,
0x1D7487,
0x1D7487,
0x1B7689,
0x1B7689,
0x1C778A,
0x1B778C,
0x1C788D,
0x1A798D,
0x1B7A8E,
0x1C7B8F,
0x1C7C92,
0x1C7C92,
0x1D7D93,
0x1E7E94,
0x1F7F95,
0x208096,
0x218197,
0x238399,
0x26819C,
0x2385A0,
0x2786A2,
0x2E85A0,
0x2F859E,
0x278A9F,
0x258EA1,
0x378A9A,
0x2E505A,
0x281C1C,
0x3E1E13,
0x422618,
0x3E2A1F,
0x38231E,
0x482A2A,
0x4A2626,
0x4A2C2E,
0x42271E,
0x3B2819,
0x262418,
0x102C2F,
0x4E909E,
0x3E9EAE,
0x3B9EB1,
0x479FB5,
0x4999B4,
0x499DB7,
0x48A0B4,
0x49A1AD,
0x4CA3AD,
0x4CA2B1,
0x4CA4B8,
0x54A2B6,
0x54A2B6,
0x55A3B7,
0x57A4B6,
0x58A5B7,
0x58A5B7,
0x59A6B8,
0x5AA7B7,
0x5AA7B7,
0x5BA7B7,
0x5BA7B7,
0x5CA8B8,
0x5CA8B8,
0x5DA9B9,
0x5DA9B9,
0x5DA9B9,
0x5DAABA,
0x5DAABA,
0x5EAABA,
0x5EAABA,
0x60A9BA,
0x60A9BA,
0x60A9BA,
0x5EAABA,
0x5CA8B8,
0x5EAABA,
0x5CA8B8,
0x5EA7B6,
0x5FA7B6,
0x5DA5B4,
0x62A7B7,
0x5C9FB0,
0x3C6664,
0x1B3337,
0x16191E,
0x1D1C18,
0x1E1C10,
0x251B12,
0x311917,
0x3A1C1C,
0x371F1B,
0x33201A,
0x33211F,
0x302222,
0x2C2829,
0x272B2E,
0x2A2730,
0x35283A,
0x332E2A,
0x36312E,
0x393031,
0x2D282C,
0x1F2831,
0x335966,
0x4F9BA8,
0x3599A9,
0x3396AD,
0x3C8FAD,
0x408BAA,
0x3C8EA6,
0x3192A2,
0x2A94A1,
0x2C90A0,
0x308B9E,
0x2B8A9E,
0x2B8AA0,
0x2A899F,
0x29889E,
0x28879D,
0x27869C,
0x25849A,
0x238399,
0x238399,
0x228298,
0x218197,
0x207F95,
0x1F7E94,
0x1E7D93,
0x1D7C90,
0x1C7B8F,
0x1B7A8E,
0x1B7A8E,
0x1C788D,
0x1B778C,
0x1B778C,
0x1A768B,
0x1B7689,
0x1B7689,
0x1C7488,
0x1C7488,
0x1C7386,
0x1D7286,
0x1C7185,
0x1D7082,
0x1D7082,
0x1C6F81,
0x1B6E7E,
0x1C6D7E,
0x1E6D7C,
0x1E6B7B,
0x1F6B7B,
0x1E6A78,
0x206978,
0x216977,
0x206874,
0x226874,
0x236772,
0x236571,
0x25656F,
0x26646F,
0x28646F,
0x29636E,
0x29626B,
0x2B6269,
0x2B6269,
0x2C6167,
0x2F6067,
0x305F65,
0x325F65,
0x335E64,
0x365F63,
0x335F62,
0x315E63,
0x305F65,
0x2E6165,
0x2C6167,
0x2B6269,
0x2A636C,
0x2A636E,
0x29636E,
0x28646F,
0x266471,
0x256571,
0x246674,
0x236774,
0x226874,
0x206A77,
0x206A77,
0x206978,
0x1E6A78,
0x1E6C79,
0x1E6B7B,
0x1E6D7E,
0x1C6D7E,
0x1C6F81,
0x1C6F81,
0x1B7182,
0x1B7084,
0x1B7285,
0x1B7285,
0x1C7386,
0x1D7487,
0x1C7488,
0x1B7689,
0x1B7689,
0x1C778A,
0x1B778C,
0x1C788D,
0x1B7A8E,
0x1B7A8E,
0x1B7C8F,
0x1C7C92,
0x1D7D93,
0x1D7D93,
0x1E7E94,
0x1F7F95,
0x218295,
0x228396,
0x25809D,
0x248399,
0x268599,
0x2A849E,
0x2E83A2,
0x2B87A0,
0x268C98,
0x2A8D93,
0x327B81,
0x213F41,
0x2B221B,
0x392316,
0x3D2B1F,
0x3A2D25,
0x3A261F,
0x44261C,
0x2D212B,
0x2F2526,
0x2B2E27,
0x203F3A,
0x3F7E87,
0x449BAF,
0x3F9BB2,
0x4098AC,
0x4595AC,
0x4A97B3,
0x4C9BBA,
0x499DB7,
0x48A0AE,
0x48A1A9,
0x49A0AA,
0x50A1B4,
0x50A1B4,
0x52A0B4,
0x53A1B5,
0x54A2B6,
0x55A4B5,
0x55A4B5,
0x56A5B6,
0x57A6B7,
0x58A5B7,
0x58A5B7,
0x59A6B6,
0x59A6B6,
0x59A6B8,
0x5AA7B9,
0x5AA7B9,
0x5AA7B9,
0x5AA7B9,
0x5BA8BA,
0x5BA8BA,
0x5CA7BA,
0x5CA7BA,
0x5CA7BA,
0x5CA7BA,
0x5BA6B9,
0x5DAABC,
0x59A6B8,
0x5AA7B9,
0x5CA9B9,
0x57A4B4,
0x5AA6B6,
0x59A5B5,
0x5CA5B6,
0x5FAAB0,
0x589399,
0x497276,
0x375557,
0x2E4040,
0x2E2F33,
0x291C23,
0x27141A,
0x241415,
0x261714,
0x341F1C,
0x35201F,
0x2B2122,
0x2C2A2D,
0x322B33,
0x33212D,
0x372F2D,
0x332E2A,
0x3F342E,
0x48302E,
0x482D34,
0x302E39,
0x376771,
0x3D9AA2,
0x2D98A8,
0x3591AA,
0x3F8AA7,
0x3B8BA4,
0x2D91A1,
0x2593A0,
0x2A8DA0,
0x33879F,
0x2B8AA0,
0x2A899F,
0x29889E,
0x28879D,
0x27869C,
0x26859B,
0x248399,
0x238399,
0x228298,
0x218197,
0x218096,
0x207F95,
0x1F7E94,
0x1E7D93,
0x1C7B8F,
0x1C7B8F,
0x1C788D,
0x1C788D,
0x1C788D,
0x1B778C,
0x1A768B,
0x1B7689,
0x1B7689,
0x1C7488,
0x1C7488,
0x1C7386,
0x1B7285,
0x1C7185,
0x1D7082,
0x1D7082,
0x1C6F81,
0x1C6F81,
0x1C6D7E,
0x1C6E7C,
0x1D6C7B,
0x1E6B7B,
0x1F6B79,
0x206978,
0x206A77,
0x216977,
0x226874,
0x226874,
0x236772,
0x23666F,
0x25656F,
0x28646F,
0x29636E,
0x29636E,
0x2B6269,
0x2B6269,
0x2C6169,
0x2E6067,
0x2E5F66,
0x2F5E64,
0x325F64,
0x335E64,
0x365F63,
0x355E62,
0x315E63,
0x306064,
0x2F6065,
0x2E6067,
0x2B6269,
0x2C626C,
0x2A636E,
0x29636E,
0x28646F,
0x266471,
0x256571,
0x236571,
0x236774,
0x236774,
0x206A77,
0x206A77,
0x206A77,
0x1E6A78,
0x1F6B79,
0x1E6B7B,
0x1D6C7B,
0x1C6D7E,
0x1C6D80,
0x1C6F81,
0x1A7081,
0x1B7084,
0x1B7084,
0x1B7285,
0x1C7386,
0x1D7487,
0x1C7488,
0x1C7488,
0x1B7689,
0x1B7689,
0x1B788A,
0x1B778C,
0x1A798D,
0x1B7A8E,
0x1C7B8F,
0x1B7C8F,
0x1C7D90,
0x1D7E91,
0x1E7F92,
0x1F8093,
0x208194,
0x218295,
0x228193,
0x238393,
0x258595,
0x25859B,
0x2785A1,
0x2785A1,
0x28889E,
0x268A9A,
0x2D8A9B,
0x3B8495,
0x3B6978,
0x335058,
0x273F41,
0x233D3C,
0x243C3C,
0x293D3E,
0x2B4C5B,
0x3A6671,
0x437F89,
0x4B94A3,
0x4597AD,
0x3B91AA,
0x4299AD,
0x4598A8,
0x4A98AC,
0x4B99B0,
0x4697B4,
0x4399B2,
0x459EB0,
0x4AA0AD,
0x4D9EAF,
0x4C9AAE,
0x4D9EB1,
0x4E9FB2,
0x4FA0B3,
0x50A1B4,
0x50A1B4,
0x51A2B3,
0x51A2B3,
0x54A3B4,
0x55A4B5,
0x55A4B5,
0x56A5B6,
0x56A5B6,
0x56A5B6,
0x57A6B7,
0x58A5B7,
0x58A5B7,
0x57A6B7,
0x58A5B7,
0x58A5B7,
0x59A4B7,
0x59A4B9,
0x59A4B9,
0x59A4B9,
0x58A5B9,
0x57A5B9,
0x54A2B6,
0x54A5B6,
0x56A7B8,
0x53A4B5,
0x58A7B8,
0x56A5B6,
0x54A3B4,
0x539FB6,
0x59A2B1,
0x60A6B0,
0x62A6AF,
0x5F9AA8,
0x598898,
0x4E7481,
0x3B5D69,
0x223F47,
0x162528,
0x1D1916,
0x291C16,
0x291E1C,
0x272121,
0x322725,
0x432E29,
0x3C2B33,
0x342A29,
0x3D352A,
0x473126,
0x573430,
0x422C2F,
0x2B3F46,
0x41878F,
0x3294A1,
0x3192A5,
0x368CA3,
0x368AA2,
0x2D8E9F,
0x288FA0,
0x2B8BA1,
0x3187A2,
0x2A899F,
0x29889E,
0x28879D,
0x26859B,
0x26859B,
0x25849A,
0x248399,
0x238298,
0x218096,
0x218096,
0x207F95,
0x1F7E94,
0x1E7D93,
0x1D7C92,
0x1C7B8F,
0x1B7A8E,
0x1C788D,
0x1C788D,
0x1C788D,
0x1B778C,
0x1B788A,
0x1B7689,
0x1A7588,
0x1C7488,
0x1C7488,
0x1C7386,
0x1C7185,
0x1B7084,
0x1D7082,
0x1D7082,
0x1D6E81,
0x1C6D80,
0x1C6E7C,
0x1B6D7B,
0x1D6C7B,
0x1E6B7B,
0x1E6A78,
0x206978,
0x216977,
0x216977,
0x226874,
0x236772,
0x226671,
0x25656F,
0x26646F,
0x28646F,
0x29636E,
0x2A636E,
0x2B6269,
0x2B6269,
0x2C6169,
0x2E6067,
0x2E5F64,
0x2F5E64,
0x325F64,
0x335E64,
0x365F63,
0x345D61,
0x325D63,
0x2F5F63,
0x2F6065,
0x2E6067,
0x2C6169,
0x2C626C,
0x2A636E,
0x29636E,
0x28646F,
0x266471,
0x256571,
0x236571,
0x236774,
0x236774,
0x206A77,
0x206A77,
0x206A77,
0x1E6A78,
0x1F6B79,
0x1E6B7B,
0x1D6C7B,
0x1B6C7D,
0x1C6D7E,
0x1C6F81,
0x1A7081,
0x1A6F83,
0x1B7084,
0x1A7184,
0x1C7386,
0x1D7487,
0x1B7387,
0x1C7488,
0x1A7588,
0x1B7689,
0x1A7789,
0x1B778C,
0x1A798D,
0x1B7A8E,
0x1B7A8E,
0x1A7B8E,
0x1B7C8F,
0x1C7D90,
0x1C7D90,
0x1E7F92,
0x1F8093,
0x208194,
0x218489,
0x22848D,
0x218594,
0x208798,
0x1F889D,
0x2386A3,
0x2885A7,
0x2B85A7,
0x2B8AAA,
0x2588A8,
0x2F8DB1,
0x3586A4,
0x3E8393,
0x3F7E85,
0x3B7F88,
0x3E8798,
0x458E9D,
0x3897A9,
0x2EA1B6,
0x2995B1,
0x3B94B2,
0x4494AD,
0x3F95A2,
0x4097A1,
0x4496A4,
0x4795AB,
0x4299AD,
0x3F9EB2,
0x439BAF,
0x4697AC,
0x4D99B0,
0x4B9CAF,
0x4A9DAF,
0x4B9EB0,
0x4C9FB1,
0x4E9FB2,
0x4E9FB2,
0x4FA0B1,
0x50A1B2,
0x50A1B2,
0x51A2B3,
0x51A2B3,
0x53A2B3,
0x54A3B4,
0x54A3B4,
0x55A4B5,
0x55A4B5,
0x55A4B5,
0x55A3B7,
0x56A3B7,
0x56A3B7,
0x57A2B7,
0x57A2B7,
0x57A2B7,
0x56A3B7,
0x56A3B7,
0x54A2B6,
0x55A6B9,
0x53A4B7,
0x50A3B5,
0x51A4B6,
0x4FA0B1,
0x50A1B2,
0x4FA0B1,
0x5B97BB,
0x589EB7,
0x4B9EAE,
0x43A3B3,
0x45A5BB,
0x49A5BA,
0x4EA7B9,
0x4BA1B0,
0x529CA9,
0x4E7F84,
0x40504F,
0x2A2720,
0x211A14,
0x1E1913,
0x251B0F,
0x352410,
0x3A222F,
0x382A2A,
0x31291E,
0x41301E,
0x4E3025,
0x4D312E,
0x32353A,
0x487A83,
0x3A919B,
0x2E92A2,
0x2E8FA2,
0x308CA1,
0x2D8C9E,
0x2B8C9F,
0x2B89A2,
0x2D87A2,
0x29889E,
0x28879D,
0x26859B,
0x25849A,
0x248399,
0x238298,
0x228197,
0x228197,
0x218096,
0x207F95,
0x1F7E94,
0x1E7D93,
0x1D7C92,
0x1C7B91,
0x1B7A8E,
0x1A798D,
0x1C798B,
0x1B788A,
0x1B788A,
0x1A7789,
0x1B7689,
0x1A7588,
0x1A7588,
0x1B7387,
0x1C7488,
0x1C7386,
0x1C7185,
0x1B7084,
0x1D7082,
0x1C6F81,
0x1D6E81,
0x1C6D80,
0x1B6D7B,
0x1D6C7B,
0x1D6C7B,
0x1E6B7B,
0x1E6A78,
0x206978,
0x216977,
0x206876,
0x226874,
0x236772,
0x236571,
0x25656F,
0x26646F,
0x28646E,
0x29636E,
0x2A636E,
0x2C636A,
0x2C636A,
0x2C6169,
0x2E6067,
0x2E5F64,
0x2F5E64,
0x335E64,
0x355E64,
0x365E60,
0x365D62,
0x355E62,
0x325F64,
0x305F65,
0x2F6067,
0x2E6069,
0x2C6169,
0x2B6269,
0x296269,
0x28636B,
0x28646E,
0x26646F,
0x256571,
0x236571,
0x236774,
0x236774,
0x236975,
0x236975,
0x206A77,
0x1E6A77,
0x1D6B78,
0x1D6C79,
0x1D6C7B,
0x1E6D7C,
0x1C6D7E,
0x1D6E7F,
0x1D7082,
0x1D7082,
0x1C7283,
0x1C7185,
0x1B7285,
0x1B7486,
0x1B7486,
0x1C7587,
0x1C7488,
0x1B7689,
0x1B7689,
0x1B788A,
0x1C788D,
0x1C788D,
0x1B7A8E,
0x1C7B8F,
0x1B7B91,
0x1C7C92,
0x1D7D93,
0x1C7F94,
0x1D8095,
0x208096,
0x218197,
0x228298,
0x228298,
0x238399,
0x24849A,
0x26869C,
0x27879D,
0x27879D,
0x28889E,
0x2A899F,
0x2B8AA0,
0x2E8DA3,
0x3190A6,
0x3291A7,
0x3190A6,
0x328EA3,
0x338FA4,
0x3591A6,
0x3692A7,
0x3894A9,
0x3994A9,
0x3A95A8,
0x3B96A9,
0x3F97AB,
0x3E96AA,
0x4199AD,
0x3F97AB,
0x449BAF,
0x4299AD,
0x459CB0,
0x449BAE,
0x479CB0,
0x469BAF,
0x499EB2,
0x499EB2,
0x499EB2,
0x4BA0B4,
0x4A9FB3,
0x4BA0B4,
0x4BA0B4,
0x4D9FB4,
0x4EA0B5,
0x4EA1B3,
0x4FA2B4,
0x50A1B4,
0x51A2B5,
0x51A2B5,
0x50A3B5,
0x4FA2B4,
0x52A3B6,
0x50A1B4,
0x52A3B6,
0x50A1B4,
0x51A2B5,
0x50A1B4,
0x52A3B6,
0x51A2B7,
0x4FA1B6,
0x4EA0B5,
0x4EA0B5,
0x4EA0B5,
0x4EA0B5,
0x4EA0B5,
0x4E9DBB,
0x4F9CB8,
0x4F9AB7,
0x4E9AB4,
0x4D9AB4,
0x499BB1,
0x449DAF,
0x3D9DA9,
0x3D9FA8,
0x44A3A7,
0x4CA1A4,
0x478B8C,
0x396568,
0x2D4247,
0x2C2C34,
0x2C222B,
0x292410,
0x2F1D0F,
0x3C1613,
0x4B1C22,
0x49232E,
0x332C34,
0x315257,
0x438D8E,
0x2F9395,
0x29949C,
0x298F9E,
0x318BA5,
0x3287A4,
0x2F86A4,
0x2988A4,
0x248AA3,
0x2F7FA0,
0x2F8199,
0x2C8492,
0x288692,
0x228193,
0x257F9A,
0x2A7D9D,
0x277D98,
0x227E93,
0x1F7E92,
0x217C91,
0x207B90,
0x1A7A8A,
0x187C8B,
0x1B7C8D,
0x1D788D,
0x1D788B,
0x1C778A,
0x1D788A,
0x1B7688,
0x1C7587,
0x1D7688,
0x1C7386,
0x1D7485,
0x1A7081,
0x1E7183,
0x207182,
0x1F7081,
0x217081,
0x1F6E7F,
0x1E6B7D,
0x1F6C7E,
0x1E6A7A,
0x1F6B7B,
0x1F6B7B,
0x206978,
0x216978,
0x236877,
0x236676,
0x236573,
0x266672,
0x266471,
0x27636E,
0x29636E,
0x2A636E,
0x2B616D,
0x2A606A,
0x2B616B,
0x2F616A,
0x2E6069,
0x31616B,
0x2E5D65,
0x356268,
0x305B61,
0x355E64,
0x375E63,
0x375D60,
0x365D62,
0x355E62,
0x335E64,
0x305F65,
0x2F6067,
0x2E6067,
0x2C6169,
0x2B6269,
0x296269,
0x29646C,
0x28646E,
0x26656E,
0x25656F,
0x236571,
0x236573,
0x236774,
0x236774,
0x236975,
0x206A77,
0x1E6A77,
0x1D6B77,
0x1D6C79,
0x1D6C7B,
0x1D6C7B,
0x1C6D7E,
0x1C6D7E,
0x1D6E81,
0x1D7082,
0x1B7182,
0x1C7185,
0x1B7285,
0x1A7385,
0x1B7486,
0x1C7587,
0x1C7488,
0x1B7689,
0x1B7689,
0x1C778A,
0x1B788A,
0x1C788D,
0x1A798D,
0x1B7A8E,
0x1B7B91,
0x1B7B91,
0x1C7C92,
0x1D7D93,
0x1C7F94,
0x1F7F95,
0x208096,
0x218197,
0x218197,
0x228298,
0x238399,
0x25859B,
0x25859B,
0x27879D,
0x27879D,
0x29889E,
0x2B8AA0,
0x2C8BA1,
0x2C8BA1,
0x2C8BA1,
0x2D8CA2,
0x2F8BA0,
0x328EA3,
0x338FA4,
0x3490A5,
0x338FA4,
0x3994A9,
0x3994A7,
0x3994A7,
0x3A95A8,
0x3D95A9,
0x3E96AA,
0x3F97AB,
0x4097AB,
0x4198AC,
0x4299AD,
0x439AAE,
0x4499AD,
0x4499AD,
0x469BAF,
0x459AAE,
0x459AAE,
0x479CB0,
0x479CB0,
0x489DB1,
0x4A9FB3,
0x499EB2,
0x4C9EB3,
0x4C9EB3,
0x4D9FB4,
0x4EA1B3,
0x4FA0B3,
0x4E9FB2,
0x4EA1B3,
0x4EA1B3,
0x4EA1B3,
0x4FA2B4,
0x4FA0B3,
0x4FA0B3,
0x4FA0B3,
0x4FA0B3,
0x4E9FB4,
0x4EA0B5,
0x4EA0B5,
0x4EA0B5,
0x4D9FB4,
0x4C9EB3,
0x4B9DB2,
0x4B9DB2,
0x45A0B3,
0x459DB1,
0x479CB1,
0x4B9BB2,
0x4E9AB2,
0x4A98AF,
0x4698AE,
0x449BAF,
0x409DAF,
0x3999A9,
0x3B99A5,
0x459CA6,
0x4F9CA6,
0x55939E,
0x518590,
0x497882,
0x3F706D,
0x3B6164,
0x415560,
0x444F61,
0x405165,
0x406577,
0x408290,
0x36929D,
0x25919D,
0x2391A0,
0x268DA0,
0x2E8AA3,
0x3086A1,
0x2C839E,
0x2A869F,
0x25859B,
0x2885A7,
0x26819C,
0x258496,
0x1F8593,
0x1E8397,
0x2382A0,
0x1D7897,
0x1F7D99,
0x1D8097,
0x1A7E95,
0x1D7D95,
0x1D7B94,
0x187B8E,
0x167D8E,
0x167B8F,
0x18768F,
0x19788E,
0x1A798F,
0x167589,
0x1A768B,
0x19758A,
0x177388,
0x1A758A,
0x1A7286,
0x1D7589,
0x197084,
0x196E82,
0x196E82,
0x1A6C81,
0x1C6E83,
0x1E6F84,
0x1A6B80,
0x1F6E7D,
0x1C6B7A,
0x1C6B7A,
0x1D6A7A,
0x1D6977,
0x1E6776,
0x206876,
0x236975,
0x216572,
0x236571,
0x256571,
0x26646F,
0x28646E,
0x28646E,
0x2A646F,
0x2B6570,
0x2D5F68,
0x2C5E65,
0x2E5F66,
0x2E5D63,
0x336065,
0x315C62,
0x335C60,
0x355C61,
0x375D60,
0x365C5F,
0x345D61,
0x335E64,
0x325F65,
0x2F6067,
0x2E6067,
0x2C6167,
0x2C6169,
0x2C636A,
0x2A636C,
0x29646C,
0x28646E,
0x25656F,
0x256571,
0x236571,
0x236573,
0x236774,
0x236975,
0x216975,
0x206A77,
0x1E6A77,
0x1D6B78,
0x1D6C79,
0x1D6C7B,
0x1D6C7B,
0x1C6D7E,
0x1D6E7F,
0x1C6F81,
0x1D7082,
0x1B7182,
0x1B7182,
0x1B7285,
0x1C7386,
0x1B7486,
0x1C7587,
0x1C7488,
0x1B7689,
0x1B7689,
0x1B788A,
0x1B778C,
0x1A798D,
0x1A798D,
0x1B7A8E,
0x1A7A90,
0x1B7B91,
0x1C7C92,
0x1D7D93,
0x1E7E94,
0x1E7E94,
0x1F7F95,
0x208096,
0x218197,
0x228298,
0x238399,
0x24849A,
0x26869C,
0x25859B,
0x27869C,
0x2A899F,
0x2B8AA0,
0x2A899F,
0x2B8AA0,
0x2E8DA3,
0x308CA1,
0x318DA2,
0x328EA3,
0x328EA3,
0x338FA4,
0x3490A5,
0x3792A7,
0x3792A7,
0x3893A6,
0x3C94A8,
0x3C94A8,
0x3F97AB,
0x3C94A8,
0x3F97AB,
0x3F96AA,
0x4299AD,
0x4499AD,
0x4398AC,
0x469BAF,
0x459AAE,
0x459AAE,
0x479CB0,
0x469BAF,
0x479CB0,
0x479CB0,
0x479CB0,
0x479CB0,
0x4A9CB1,
0x4A9CB1,
0x4B9DB2,
0x4B9DB2,
0x4A9CB1,
0x4B9EB0,
0x4C9FB1,
0x4A9DAF,
0x4DA0B2,
0x4A9DAF,
0x4D9FB4,
0x4A9CB1,
0x4B9DB2,
0x4B9DB2,
0x4B9DB2,
0x4C9EB3,
0x4B9DB2,
0x4A9CB1,
0x479CB0,
0x479CB0,
0x489DB1,
0x3BA2AB,
0x3D9FAA,
0x419DAA,
0x469CAD,
0x4A98AE,
0x4A96AE,
0x4895B1,
0x4596B3,
0x3E95B1,
0x3A98B2,
0x389AB3,
0x349AB1,
0x3096AC,
0x3196AA,
0x3598AB,
0x389CAC,
0x369AAA,
0x3C99AB,
0x3D92A7,
0x3F8CA6,
0x428FA9,
0x3C92AB,
0x2F8FA7,
0x258FA5,
0x2392A6,
0x2491A8,
0x278BA3,
0x2B89A2,
0x2F879F,
0x2C849A,
0x2B8699,
0x288596,
0x1981A4,
0x2187A0,
0x22899A,
0x1B8593,
0x1A8398,
0x2284A1,
0x1E7B9C,
0x21809C,
0x187E94,
0x178095,
0x1B7E95,
0x1C7C94,
0x177C90,
0x137C8F,
0x157B91,
0x197992,
0x1A798F,
0x19788E,
0x1C788F,
0x1C788F,
0x1C788F,
0x1B778E,
0x197489,
0x197489,
0x1A7288,
0x1E7589,
0x1C7387,
0x1B7085,
0x1D7287,
0x1D6F85,
0x1B6D82,
0x1F7186,
0x1C6E7C,
0x1F6E7D,
0x1C6B7A,
0x1E6C79,
0x206E7B,
0x1E6A77,
0x1E6875,
0x236B77,
0x236975,
0x256974,
0x246770,
0x24646E,
0x25636E,
0x27666F,
0x28646E,
0x25616B,
0x316267,
0x316267,
0x316066,
0x346166,
0x335F62,
0x365F63,
0x375F61,
0x3A6061,
0x395C60,
0x365C5F,
0x355C61,
0x355E64,
0x325F65,
0x305F65,
0x2F6067,
0x2E6067,
0x2C6167,
0x2C636A,
0x2A636A,
0x29646C,
0x28646E,
0x26646F,
0x256571,
0x256571,
0x236573,
0x236774,
0x226874,
0x216975,
0x206A77,
0x1E6A77,
0x1D6B78,
0x1E6C79,
0x1D6C7B,
0x1D6C7B,
0x1E6D7E,
0x1C6D7E,
0x1D6E81,
0x1C6F81,
0x1B7182,
0x1B7182,
0x1B7285,
0x1B7285,
0x1C7386,
0x1B7486,
0x1C7587,
0x1C7488,
0x1B7689,
0x1C778A,
0x1B788A,
0x1B778C,
0x1A798D,
0x1A798D,
0x1B7A8E,
0x1A7A90,
0x1B7B91,
0x1C7C92,
0x1D7D93,
0x1E7E94,
0x1E7E94,
0x1F7F95,
0x208096,
0x218197,
0x228298,
0x238399,
0x25859B,
0x25859B,
0x26869C,
0x28879D,
0x29889E,
0x2A899F,
0x2C8BA1,
0x2E8DA3,
0x2F8EA2,
0x2D8CA0,
0x318DA2,
0x318DA2,
0x3490A5,
0x308CA1,
0x338FA4,
0x3691A6,
0x3792A7,
0x3893A6,
0x3A92A6,
0x3C94A8,
0x3B93A7,
0x3D95A9,
0x3D95A9,
0x3F97AB,
0x3F96A9,
0x3F96A9,
0x4299AC,
0x4299AC,
0x4299AC,
0x439AAD,
0x4299AC,
0x439AAD,
0x449BAE,
0x459AAE,
0x469BAF,
0x469BAF,
0x469BAF,
0x489AAF,
0x499BB0,
0x499BB0,
0x499BB0,
0x499BB0,
0x489AAF,
0x4A9CB1,
0x489AAF,
0x4A9CB1,
0x499BB0,
0x4A9CB1,
0x499BB0,
0x499BB0,
0x499BB0,
0x469BAF,
0x469BAF,
0x469BAF,
0x469BAF,
0x479CB0,
0x3B9DA6,
0x3E9FA8,
0x419DA8,
0x4199A7,
0x4197A8,
0x4496AB,
0x4595AE,
0x4394B1,
0x4095B2,
0x3A93B1,
0x3693B2,
0x3395B2,
0x3295B2,
0x3197B0,
0x2E97AD,
0x2B94AA,
0x2895AA,
0x2690A6,
0x2893A7,
0x2A93A8,
0x268FA4,
0x258EA4,
0x2A90A7,
0x2A90A7,
0x278BA3,
0x2A8AA3,
0x2886A0,
0x29859E,
0x28849B,
0x258496,
0x268797,
0x248595,
0x1B87A4,
0x20869C,
0x1D7E8E,
0x1D7D89,
0x187789,
0x1E7590,
0x207691,
0x1D758B,
0x177787,
0x177787,
0x1B7387,
0x1C7186,
0x1A7184,
0x167183,
0x167184,
0x1B6F87,
0x1C6E83,
0x1B6D82,
0x1A6C81,
0x1A6C81,
0x1A6D7F,
0x1A6D7F,
0x196A7D,
0x18697C,
0x196879,
0x156475,
0x176476,
0x196678,
0x196575,
0x196575,
0x1B6475,
0x186172,
0x1B6371,
0x19616F,
0x1A626E,
0x1E6470,
0x1C626E,
0x1C606B,
0x1D5F6B,
0x1E5E68,
0x215F6A,
0x215D67,
0x215B66,
0x245D66,
0x245D66,
0x245A64,
0x245B62,
0x265D64,
0x2A575C,
0x2A555B,
0x2B575A,
0x2E575B,
0x2F5759,
0x305657,
0x325656,
0x325455,
0x3A5C5E,
0x385B5D,
0x365C5F,
0x355E62,
0x335E64,
0x315E64,
0x305F65,
0x2F6067,
0x2E6067,
0x2C6167,
0x2B6269,
0x2A636A,
0x29646C,
0x28646E,
0x26646F,
0x25656F,
0x256571,
0x246672,
0x236774,
0x226874,
0x206874,
0x1E6A77,
0x1E6A77,
0x1E6C79,
0x1E6C79,
0x1E6B7B,
0x1D6C7B,
0x1E6D7E,
0x1C6D7E,
0x1C6F81,
0x1C6F81,
0x1A7081,
0x1A7184,
0x1B7285,
0x1B7285,
0x1C7386,
0x1C7386,
0x1C7587,
0x1C7488,
0x1B7689,
0x1B7689,
0x1B778C,
0x1C788D,
0x1A798D,
0x1A798D,
0x1B7A8E,
0x1C7B91,
0x1D7C92,
0x1D7C92,
0x1D7D93,
0x1E7E94,
0x1F7F95,
0x1F7F95,
0x208096,
0x228298,
0x228298,
0x208096,
0x238399,
0x25859B,
0x26859B,
0x26859B,
0x27869C,
0x28879D,
0x27869C,
0x2A899F,
0x2C8BA1,
0x2B8A9E,
0x2F8BA0,
0x2E8A9F,
0x318DA2,
0x328EA3,
0x338FA4,
0x348FA4,
0x338EA3,
0x3590A5,
0x3590A3,
0x3A92A6,
0x3A92A6,
0x3C94A8,
0x3B93A7,
0x3D94A8,
0x3C93A7,
0x3F96AA,
0x3E95A9,
0x3F96AA,
0x4198AC,
0x4097AA,
0x439AAD,
0x4198AB,
0x439AAD,
0x449BAE,
0x459AAE,
0x4398AC,
0x4398AC,
0x4499AD,
0x459AAE,
0x459AAE,
0x4499AD,
0x459AAE,
0x4398AC,
0x459AAE,
0x4499AD,
0x469BAF,
0x469BAF,
0x459AAE,
0x4499AD,
0x4398AC,
0x4398AC,
0x4499AD,
0x439AAD,
0x4299AC,
0x4198AB,
0x4399A8,
0x4199A7,
0x4098A6,
0x3E97A5,
0x3D96A4,
0x3B97A6,
0x3D96A8,
0x3F97AB,
0x3D93AA,
0x3B91AA,
0x3C90AA,
0x3C90AA,
0x3B8DA5,
0x3B8DA5,
0x388DA2,
0x378CA1,
0x358B9C,
0x368D9E,
0x308D9C,
0x2C8E9B,
0x2C8E9B,
0x2B8B9B,
0x2B8699,
0x2D839A,
0x31839B,
0x30849E,
0x2D839C,
0x28849B,
0x25859B,
0x208597,
0x1F8695,
0x1D8493,
0x20849B,
0x1F7B8A,
0x388891,
0x509DA5,
0x559FAC,
0x5599AC,
0x599AAE,
0x5499A8,
0x519EA6,
0x509DA5,
0x5298A4,
0x5695A4,
0x5799A7,
0x5599A4,
0x5296A3,
0x5394A6,
0x5597A5,
0x5698A6,
0x5193A1,
0x5395A1,
0x5395A1,
0x51939F,
0x5696A2,
0x53939F,
0x5896A1,
0x55939E,
0x57939E,
0x57939E,
0x54909A,
0x558F9A,
0x58929D,
0x58929D,
0x59929D,
0x538C97,
0x598F99,
0x5A909A,
0x548893,
0x588C97,
0x5F919A,
0x578791,
0x598890,
0x5E8A93,
0x5F8C92,
0x5E8990,
0x5F878F,
0x60898F,
0x61888F,
0x60878E,
0x66898D,
0x628589,
0x67888D,
0x648688,
0x6A898B,
0x678587,
0x698586,
0x668081,
0x3A5C5E,
0x395B5D,
0x385B5F,
0x365D62,
0x355E64,
0x315E63,
0x305F65,
0x2F6065,
0x2F6065,
0x2E6067,
0x2C6167,
0x2B6269,
0x29626B,
0x29636E,
0x28646F,
0x26646F,
0x256571,
0x266672,
0x246672,
0x226874,
0x206874,
0x1F6974,
0x1E6A77,
0x1F6B79,
0x1E6C79,
0x1E6B7B,
0x1E6B7B,
0x1D6C7D,
0x1C6D7E,
0x1C6D7E,
0x1C6F81,
0x1C6F81,
0x1A6F83,
0x1B7084,
0x1C7185,
0x1B7285,
0x1C7386,
0x1C7386,
0x1C7488,
0x1C7488,
0x1B7689,
0x1C778A,
0x1B778C,
0x1B778C,
0x1A798D,
0x1A798D,
0x1B7A8E,
0x1C7B8F,
0x1C7B91,
0x1D7C92,
0x1E7D93,
0x1E7E94,
0x1E7E94,
0x1F7F95,
0x208096,
0x218197,
0x238399,
0x28889E,
0x2C8CA2,
0x2D8CA2,
0x2D8CA2,
0x2F8EA4,
0x2E8DA3,
0x2B8AA0,
0x2C8BA1,
0x3594AA,
0x308FA5,
0x3594AA,
0x2D8CA2,
0x3F9BB0,
0x3A96AB,
0x3A96AB,
0x3995AA,
0x3994A9,
0x3B96AB,
0x3A95AA,
0x409BB0,
0x3F9AAF,
0x439BAF,
0x429AAE,
0x469EB2,
0x449CB0,
0x459DB1,
0x439BAF,
0x449CB0,
0x479FB3,
0x48A0B4,
0x4CA4B8,
0x469EB2,
0x49A0B4,
0x4AA1B4,
0x4AA1B4,
0x489FB2,
0x489FB2,
0x4A9FB3,
0x4CA1B5,
0x4BA2B5,
0x49A0B3,
0x4BA0B4,
0x489DB1,
0x4BA0B4,
0x499EB2,
0x4CA1B5,
0x4BA0B4,
0x4CA1B5,
0x4BA0B4,
0x4AA1B5,
0x4BA2B6,
0x4BA2B6,
0x4AA1B5,
0x479EB2,
0x449BAF,
0x4F9FB6,
0x499BB1,
0x459CAF,
0x46A2B1,
0x44A1B0,
0x3D9DAB,
0x3E9EAC,
0x46A3B2,
0x4BA4B4,
0x499FB0,
0x4A9BAE,
0x4B99AD,
0x4B9AAB,
0x4D9CAB,
0x4F9EAD,
0x4F9EAB,
0x4A91A7,
0x4E99AE,
0x4192A5,
0x38909E,
0x439CAA,
0x4299AA,
0x3C92A3,
0x4697AC,
0x418FA6,
0x4090A9,
0x3B8FA9,
0x3690A8,
0x3090A6,
0x2A8FA1,
0x248E9E,
0x1F8999,
0x238092,
0x33818E,
0x77B6BD,
0x92C7CD,
0x91C3CC,
0x8FBFCD,
0x8FBDCC,
0x93C5CE,
0x84BEC0,
0x88C2C6,
0x90C0CA,
0x95BFCB,
0x96C1CA,
0x90BFC7,
0x8FBEC8,
0x96C1D1,
0x88B9C0,
0x83B4BB,
0x8EBFC6,
0x89BAC1,
0x89BAC1,
0x8EBFC6,
0x81B2B9,
0x86B5BB,
0x8BBAC0,
0x82B1B7,
0x89B6BC,
0x8DBABF,
0x86B1B7,
0x87B2B8,
0x8BB6BC,
0x84AFB5,
0x8BB3BB,
0x92BBC1,
0x91B8BF,
0x8EB5BC,
0x94B9C1,
0x94B9C1,
0x90B3B9,
0x8FB0B7,
0x96B7BE,
0x8EADB2,
0x8EABB1,
0x99B4BB,
0x9DB9BD,
0x96B2B6,
0x95AEB3,
0x9AB3B8,
0x94B0B4,
0x94ADB2,
0x9AB3B7,
0x95ADB1,
0x99AEB1,
0x99ADAE,
0xA5B7B9,
0xA6B8BA,
0x3C5B5D,
0x395B5D,
0x385B5D,
0x375D60,
0x355E62,
0x325D63,
0x325F65,
0x305F65,
0x2E5F64,
0x2D6064,
0x2E6067,
0x2B6269,
0x29626B,
0x28636B,
0x28646E,
0x28646F,
0x266471,
0x256571,
0x246672,
0x236772,
0x226874,
0x206874,
0x206A77,
0x1E6A77,
0x1E6A78,
0x1D6B78,
0x1E6B7B,
0x1D6C7B,
0x1D6C7D,
0x1C6D7E,
0x1B6E7E,
0x1C6F7F,
0x1A7081,
0x1B7182,
0x1B7084,
0x1C7185,
0x1B7285,
0x1C7386,
0x1B7486,
0x1C7488,
0x1C7488,
0x1B7689,
0x1B7689,
0x1B778C,
0x1B778C,
0x1B778C,
0x1A798D,
0x1B7A8E,
0x1B7A8E,
0x1C7B8F,
0x1D7C92,
0x1C7C92,
0x1D7D93,
0x1E7E94,
0x1F7F95,
0x208096,
0x1E7E94,
0x228298,
0x26869C,
0x27869C,
0x28879D,
0x2A899F,
0x2A899F,
0x28879D,
0x27869C,
0x2F8EA4,
0x2B8AA0,
0x308FA5,
0x29889E,
0x3695AB,
0x3490A7,
0x3490A5,
0x3490A5,
0x3490A5,
0x3590A5,
0x3691A6,
0x3994A9,
0x3A95AA,
0x3B96AB,
0x3A95AA,
0x3E96AA,
0x3D95A9,
0x3E96AA,
0x3C94A8,
0x3D95A9,
0x3F97AB,
0x4098AC,
0x439BAF,
0x4098AC,
0x4199AD,
0x429AAE,
0x429AAE,
0x4299AD,
0x4299AD,
0x439AAE,
0x449BAE,
0x449BAE,
0x439AAD,
0x439AAD,
0x439AAD,
0x439AAD,
0x439AAD,
0x439AAD,
0x439AAE,
0x459CB0,
0x449BAF,
0x449BAF,
0x459CB0,
0x459CB0,
0x439BAF,
0x4098AC,
0x3E96AA,
0x4599B3,
0x4195AF,
0x3E96AC,
0x3E9BAD,
0x3C9CAC,
0x3898A8,
0x3898A6,
0x3E9BAA,
0x3D96A8,
0x3E95A8,
0x4297AB,
0x4197A8,
0x3C93A4,
0x3894A1,
0x3795A1,
0x3595A1,
0x3C8FAD,
0x388CA8,
0x368DA8,
0x358FA7,
0x318DA4,
0x308CA1,
0x318CA1,
0x2E899E,
0x338BA3,
0x358DA5,
0x308AA2,
0x2E8AA1,
0x2A899F,
0x26879A,
0x228598,
0x1C7F92,
0x207B8D,
0x418997,
0x8FC4CC,
0x92BAC2,
0x85A9B5,
0x7FA3B1,
0x82A3B2,
0x87ADB6,
0x7AA7AA,
0x80ADB2,
0x86ACB5,
0x8CAAB5,
0x8CAAB5,
0x85A7B1,
0x86A8B4,
0x8EAFBE,
0x7FA5AE,
0x789EA7,
0x88AEB7,
0x81A7B0,
0x81A8AF,
0x88AFB6,
0x779EA5,
0x7FA4AC,
0x83A8AE,
0x698E94,
0x76999F,
0x84A7AD,
0x779AA0,
0x7E9FA6,
0x84A5AA,
0x6A8B90,
0x78979C,
0x85A4A9,
0x7E9DA2,
0x7B9A9F,
0x89A6AA,
0x85A2A6,
0x799599,
0x80999E,
0x8CA5A9,
0x748C90,
0x758A8F,
0x8FA4A9,
0x96A9AD,
0x84979B,
0x829599,
0x93A6AA,
0x81969B,
0x83969C,
0x899CA0,
0x889A9E,
0x88979A,
0x8E9C9F,
0x9DABAC,
0xA3AEB0,
0x3C5B5D,
0x3C5B5D,
0x395B5D,
0x375D60,
0x365D62,
0x345D63,
0x335E64,
0x325F65,
0x2F5E64,
0x2E5F64,
0x2D5F66,
0x2C6167,
0x2B6269,
0x29626B,
0x28626D,
0x28646E,
0x26646F,
0x266471,
0x266672,
0x236772,
0x226872,
0x206874,
0x1F6974,
0x206A77,
0x1E6A78,
0x1E6A78,
0x1D6A7A,
0x1D6A7A,
0x1D6C7B,
0x1B6C7D,
0x1C6D7E,
0x1B6E7E,
0x1C6F81,
0x1A7081,
0x1B7182,
0x1C7185,
0x1C7185,
0x1C7386,
0x1C7386,
0x1C7488,
0x1C7488,
0x1A7588,
0x1B7689,
0x1B768B,
0x1A768B,
0x1B778C,
0x1C788D,
0x1C788D,
0x1A798D,
0x1B7A8E,
0x1C7B91,
0x1C7B91,
0x1C7C92,
0x1D7D93,
0x1E7E94,
0x1F7F95,
0x1F7F95,
0x208096,
0x218197,
0x218197,
0x238298,
0x238298,
0x248399,
0x25849A,
0x27869C,
0x26859B,
0x27869C,
0x29889E,
0x2A899F,
0x29889E,
0x2A899F,
0x2D89A0,
0x2E8A9F,
0x308CA1,
0x2F8BA0,
0x328EA3,
0x318CA1,
0x348FA4,
0x338EA3,
0x348FA4,
0x378FA3,
0x368EA2,
0x3991A5,
0x3691A4,
0x3691A4,
0x3893A6,
0x3691A4,
0x3893A6,
0x3994A7,
0x3B93A7,
0x3B93A7,
0x3B93A7,
0x3C94A8,
0x3C94A8,
0x3C94A8,
0x3C94A8,
0x3C95A7,
0x3E95A8,
0x3D94A7,
0x3F96AA,
0x3D94A8,
0x3F96AA,
0x3C93A7,
0x3D94A8,
0x3D94A8,
0x3D94A8,
0x3B93A7,
0x3B93A7,
0x3B93A7,
0x3B93A7,
0x3B93A7,
0x3B93A7,
0x3A90AB,
0x3A91AC,
0x3791A9,
0x3591A6,
0x3291A5,
0x3392A4,
0x3491A3,
0x328EA3,
0x348FA4,
0x328CA4,
0x318BA3,
0x308CA5,
0x2D8BA4,
0x2A8DA4,
0x288FA2,
0x248DA0,
0x2588A7,
0x288BAA,
0x288AA7,
0x2486A1,
0x23859E,
0x24889F,
0x23899F,
0x20869C,
0x1E849A,
0x21859C,
0x218199,
0x218096,
0x237F96,
0x248097,
0x258196,
0x227E93,
0x1B7B91,
0x3F8A9D,
0x8FC0CE,
0x97BBC7,
0x99BAC9,
0x93B4C5,
0x97B8C9,
0x96BCC5,
0x94BFC5,
0x92BFC5,
0x92B6C2,
0x96B3C1,
0x9AB7C5,
0x96B6C3,
0x8FB3C1,
0x92B6C6,
0x96BAC6,
0x98BCC8,
0x92B6C2,
0x94B8C4,
0x94B9C2,
0x94B6C0,
0x9ABCC6,
0x98BAC4,
0x94B6C0,
0x95B7C0,
0x95B6BF,
0x97B8C1,
0x9ABBC4,
0x96B7C0,
0x95B3BD,
0x98B6C0,
0x98B7BA,
0x97B4B8,
0x94B1B5,
0x94B1B5,
0x98B4B8,
0x99B5B9,
0x99B2B7,
0x98B1B5,
0x99B1B5,
0x9BB3B7,
0x9EB3B8,
0x9CB1B6,
0x9CAFB3,
0x9CAFB3,
0x9CAFB3,
0x9BAEB2,
0x9DB0B4,
0x9CAEB2,
0x9EB0B4,
0xA1B0B3,
0x9EADB0,
0xA1AFB0,
0xA5B0B2,
0xA1ABAD};

	/*uint32_t shrek_size=24000;*/

	post_log("\n \t Starting LCD Post Diags\n");

	CPU_RMW_OR_32(ASIU_TOP_CLK_GATING_CTRL, (1 << ASIU_TOP_CLK_GATING_CTRL__LCD_CLK_GATE_EN));

	/*Setup for 800x480 display*/
	lcdParam.PPL = 800;
	lcdParam.HBP = 2;
	lcdParam.HFP = 167;
	lcdParam.HSW = 1;
	lcdParam.LPF = 480;
	lcdParam.VSW = 1;
	lcdParam.VBP = 33;
	lcdParam.VFP = 12;
	lcdParam.LED = 5;
	lcdParam.IHS = 1;
	lcdParam.IVS = 1;

	/*Give a valid DDR address.*/
	BufferAddr = 0xA0000000;

	/*first config the LCD with watermark level set to be 4 and 24 bits*/
	lcd_config(&lcdParam, BPP_24, clockDiv, BufferAddr, 0);
	address = BufferAddr;

	/* This may not be required for RGB panel. */
	CPU_RMW_OR_32(LCDControl, 1<<8);/*LCD Control Register BGR*/

	/*Put a white background*/
	while (address < BufferAddr + 0x180000) {
		CPU_WRITE_32(address, 0xFFFFFF); /*White*/
		address += 4;
	}

	i = 0; j = 0; k = 0; m = 0;
	address = BufferAddr;

	for (i = 0; i < 120; i++) {
		for (m = 0; m < 4; m++) {
			/*Scaling height by 4*/
			for (j = 0; j < 200; j++) {
				for (k = 0; k < 4; k++) {
					/*Scaling width by 4*/
					CPU_WRITE_32(address, shrek[200*i+j]);
					address += 4;
				}
			}
		}
	}

	lcd_power_on();

	post_log("\nPlease check the LCD panel for a Shrek image\n");
	post_getConfirmation("\n Do you see Shrek on the panel? (Y/N)");
	post_log("\nSuccessful, LCD diags done  !!!\n");
	return 0;
}

#endif /* CONFIG_POST & CONFIG_SYS_POST_LCD */
#endif

